#!/usr/bin/env ruby -Ku
# -*- coding: utf-8 -*-

# # SujicFE (Sujic Front End Tool)
# Author::    Ryo Takagi
# Copyright:: (C)2015-2019 Ryo Takagi.
# License::   See source.
#
# This file was created using yardoc from the source file
# __SujicFE.rb__. Enjoy! _Italic!_ __BOLD!__ Yeah!

require 'tk'
require 'tkextlib/tile'
require 'rexml/document'
require 'rexml/formatters/pretty'
require 'open3'
require 'rubygems'
require 'gettext'

include GetText
base_dir = File.dirname(__FILE__)
set_output_charset("UTF-8")
bindtextdomain("SujicFE", :path => File.join(base_dir, "locale"))




#= File class additional function
class File

  #== open file method with exception handling.
  def self.open_for_read (d)
    begin
      return File.open(d, "r")
    rescue Errno::ENOENT, Errno::EACCES
      return 0
    rescue ezzz
      p _("Unknown error reported...")
      exit
    end
  end

end


#= Integer class additional function
class Integer

  # Converting to string (as time)
  def to_s_as_time ()
    return "%02d:%02d:%02d" % [ self / 3600, self % 3600 / 60 ,
                                self % 3600 % 60 ]
  end

end


#= String class additional function
class String

  # Converting to integer (time.parse did not do the right job for me)
  def to_i_as_time
    if ( /(\d+):(\d+):(\d+)/ =~ self ) != nil
      return ( ($1.to_i() * 60 + $2.to_i()) * 60 + $3.to_i )
    elsif( /(\d+):(\d+)/ =~ self ) != nil
      return ($1.to_i() * 60 + $2.to_i())
    else
      return (self.to_i)
    end
  end

  # Check if this is float (from stackoverflow)
  def is_float?
    true if Float(self) rescue false
  end

  # Check if this is integer (from stackoverflow)
  def is_integer? 
    [                          # In descending order of likeliness:
      /^[0-9]$/,               # decimal incl. zero
      /^[-+]?[1-9]([0-9]*)?$/, # decimal
      /^0[0-7]+$/,             # octal
      /^0x[0-9A-Fa-f]+$/,      # hexadecimal
      /^0b[01]+$/              # binary
    ].each do |match_pattern|
      return true if self =~ match_pattern
    end
    return false
  end

  def is_i_as_time? 
    [                          # In descending order of likeliness:
      /(\d+):(\d+):(\d+)/,     # hh:mm:ss
      /(\d+):(\d+)/            # mm:ss
    ].each do |match_pattern|
      return true if self =~ match_pattern
    end
    return is_integer?
  end

end

#= class Processor: "THE PROCESSOR" that will perform all necessary jobs.
class Processor

  # Constant: Blank input file
  BLANKINPUT = <<-BLI_END
<?xml version='1.0' encoding='UTF-8'?> 
<rtss_input_data>
  <!-- Train types -->
  <!--##SUJIC_TRAINTYPE##-->
  <!-- Speed classes -->
  <!--##SUJIC_SPEEDCLASS##-->
  <!-- Stations and platforms -->
  <!--##SUJIC_STATION##-->
  <!-- Lines -->
  <!--##SUJIC_LINE##-->
  <!-- Basic schedule set: "RG" (regular) -->
  <create class='sujic_schedule_set'>
    <data class='x_schedule_set' cycletime='0' name='RG'>
      <train_graph
        hspace_name_mileage_traingraph='10.0'
        line_main_grid_stroke='black'
        line_main_grid_stroke_dasharray='none'
        line_main_grid_stroke_width='0.5'
        line_platform_stroke='black'
        line_platform_stroke_dasharray='4,1'
        line_platform_stroke_width='0.5'
        line_station_stroke='black'
        line_station_stroke_dasharray='none'
        line_station_stroke_width='0.5'
        line_sub_grid_stroke='black'
        line_sub_grid_stroke_dasharray='3,2'
        line_sub_grid_stroke_width='0.5'
        main_grid_interval_relative_to_sub='5'
        margin_bottom='15.0'
        margin_left='15.0'
        margin_right='15.0'
        margin_top='15.0'
        outfile='out.svg'
        out_from_time="10:00:00"
        out_to_time="11:00:00"
        pixels_per_kilometre='25.0'
        pixels_per_minute='12.0'
        showing_additional_cycles='5'
        sub_grid_interval_in_seconds='60.0'
        text_line_name_fill='black'
        text_line_name_font_family='sans-serif'
        text_line_name_font_size='15.0'
        text_line_name_font_weight='normal'
        text_line_name_lengthadjust='spacing'
        text_line_name_width='150.0'
        text_mileage_figure_fill='black'
        text_mileage_figure_font_family='sans-serif'
        text_mileage_figure_font_size='12.0'
        text_mileage_figure_font_weight='normal'
        text_mileage_figure_lengthadjust='spacing'
        text_mileage_figure_width='35.0'
        text_platform_name_fill='black'
        text_platform_name_font_family='sans-serif'
        text_platform_name_font_size='6.0'
        text_platform_name_font_weight='normal'
        text_station_name_fill='black'
        text_station_name_font_family='&apos;sans-serif&apos;'
        text_station_name_font_size='12.0'
        text_station_name_font_weight='normal'
        text_station_name_lengthadjust='spacing'
        text_station_name_width='100.0'
        vspace_around_traingraph='40.0'
        vspace_line_above='20.0'
        vspace_line_below='20.0'
        vspace_platform_platform='7.0'
        vspace_platform_station='15.0'
        vspace_station_platform='15.0' />
    </data>
  </create>
  <!--##SUJIC_SCHEDULE_SET##-->
  <!-- Passenger activity models -->
  <!--##SUJIC_PASSENGER_ACTIVITY##-->
  <!-- Passenger appearance model for the activity model "F1" -->
  <!--##SUJIC_PASSENGER_APPEARANCE##-->
  <!-- Evaluator -->
</rtss_input_data> 
BLI_END

  # Train graph default parameters
  def train_graph_defaults(_h_tgp)
    _h_tgp['text_line_name_width'] = 150.0
    _h_tgp['text_line_name_lengthadjust'] = 'spacing'
    _h_tgp['text_line_name_font_family'] = 'sans-serif'
    _h_tgp['text_line_name_font_weight'] = 'normal'
    _h_tgp['text_line_name_fill'] = 'black'
    _h_tgp['text_line_name_font_size'] = 15.0
    _h_tgp['text_platform_name_font_size'] = 6.0
    _h_tgp['text_platform_name_font_family'] = 'sans-serif'
    _h_tgp['text_platform_name_font_weight'] = 'normal'
    _h_tgp['text_platform_name_fill'] = 'black'
    _h_tgp['line_platform_stroke'] = 'black'
    _h_tgp['line_platform_stroke_width'] = '0.5'
    _h_tgp['line_platform_stroke_dasharray'] = '4,1'
    _h_tgp['text_station_name_font_size'] = 12.0
    _h_tgp['text_station_name_width'] = 100.0
    _h_tgp['text_station_name_lengthadjust'] = 'spacing'
    _h_tgp['text_station_name_font_family'] = 'sans-serif'
    _h_tgp['text_station_name_font_weight'] = 'normal'
    _h_tgp['text_station_name_fill'] = 'black'
    _h_tgp['text_mileage_figure_font_size'] = 12.0
    _h_tgp['text_mileage_figure_width'] = 35.0
    _h_tgp['text_mileage_figure_lengthadjust'] = 'spacing'
    _h_tgp['text_mileage_figure_font_family'] = 'sans-serif'
    _h_tgp['text_mileage_figure_font_weight'] = 'normal'
    _h_tgp['text_mileage_figure_fill'] = 'black'
    _h_tgp['line_station_stroke'] = 'black'
    _h_tgp['line_station_stroke_width'] = '0.5'
    _h_tgp['line_station_stroke_dasharray'] = 'none'
    _h_tgp['hspace_name_mileage_traingraph'] = 10.0
    _h_tgp['vspace_platform_platform'] = 7.0
    _h_tgp['vspace_station_platform'] = 15.0
    _h_tgp['vspace_platform_station'] = 15.0
    _h_tgp['vspace_line_above'] = 20.0
    _h_tgp['vspace_line_below'] = 20.0
    _h_tgp['vspace_around_traingraph'] = 40.0
    _h_tgp['margin_top'] = 15.0
    _h_tgp['margin_bottom'] = 15.0
    _h_tgp['margin_right'] = 15.0
    _h_tgp['margin_left'] = 15.0
    _h_tgp['pixels_per_kilometre'] = 25.0
    _h_tgp['pixels_per_minute'] = 12.0
    _h_tgp['showing_additional_cycles'] = 5
    _h_tgp['main_grid_interval_relative_to_sub'] = 5
    _h_tgp['sub_grid_interval_in_seconds'] = 60.0
    _h_tgp['line_main_grid_stroke'] = 'black'
    _h_tgp['line_main_grid_stroke_width'] = '0.5'
    _h_tgp['line_main_grid_stroke_dasharray'] = 'none'
    _h_tgp['line_sub_grid_stroke'] = 'black'
    _h_tgp['line_sub_grid_stroke_width'] = '0.5'
    _h_tgp['line_sub_grid_stroke_dasharray'] = '3,2'
  end

  # Cross reference checker: Check if deleting an object is safe or not.
  # @return [boolean] True if OK to delete the object.
  # @param p_win [TkToplevel] Parent window.
  # @param d_ck [Hash] Check if the element to be deleted is used somewhere
  #   else.  
  #   - `d_ck['nil']`: Contains a string expressing checked element in message
  #       (train type, etc.).  
  #   - `d_ck[ky]`: Contains an `arr` for `ky`  
  #   - `ky`: String expressing elements for cross reference to
  #       checked element.  
  #   - `arr`: array.  
  #   - `arr[n]`: array.  
  #   - `arr[n][0]`: XPath to elements containing attributes for
  #       x-ref check  
  #   - `arr[n][1]`: Hash (`[key] => val`).
  #       All attributes should be equal.  
  #   - `key` = array, `[0]` relative xpath from `tt_el`,
  #       `[1]` its attribute.  
  #   - `val` = array, `[0]` relative xpath from `arr[n][0]`,
  #       `[1]` its attribute.  
  #   - `arr[n][2]`: XPath to proper name of, OR lambda returning
  #       name of, element at `arr[n][1]`, `val[0]`. If XPath is specified,
  #       it is relative to one of the elements specified by `arr[n][0]`.  
  #   - `arr[n][3]`: Root element of the XPath specified in
  #       `arr[n][0]`. If nil or unspecified, `@doc` is used.  
  # @param tt_el [string] Element to be checked for cross reference.
  # @param l_el [lambda] Lambda returning string representing `tt_el`.
  def cross_reference_checker(p_win,d_ck,tt_el=nil,l_el=nil)
    if tt_el != nil && d_ck != nil
      d_ck.each do |key,val|
        next if key == nil
        val.each do |e_ca|
          if e_ca[3]
            mydoc = @doc.elements[e_ca[3]]
          else
            mydoc = @doc
          end
          mydoc.elements.each(e_ca[0]) do |e_ck|
            sw_false = false
            sw_attr = true
            e_ca[1].each do |an,vx|
              sw_false = true
              if sw_attr &&
                 tt_el.elements[an[0]].attributes[an[1]] ==
                 e_ck.elements[vx[0]].attributes[vx[1]]
                sw_attr = true
              else
                sw_attr = false
              end
            end
            if sw_false && sw_attr
              unless e_ca[2].respond_to? :call
                str_tsn = e_ck.elements[e_ca[2]].attributes['name']
              else
                str_tsn = e_ca[2].call(e_ck)
              end
              str_x = l_el.call(tt_el)
              sf_em =
                _("Cannot delete %3$s \"%4$s\" because it is " +
                  "referred to by %1$s \"%2$s\".")
              Tk.messageBox(
                "parent" => p_win,
                "title" => _("Error: cannot delete"),
                "message" => sprintf(sf_em, key, str_tsn, d_ck[nil], str_x))
              return false
            end
          end
        end
      end
    end
    return true
  end

  # Creator of the "editor select frame" window.
  # @return None.
  # @param parent_win [TkToplevel] Parent window.
  # @param es_win [TkToplevel] Window in which the parent frame resides.
  # @param fr_ul [TkFrame] Parent frame. The editor select frame will be
  #   created as its child frame.
  # @param _d [Hash] Data.  
  #   - `_d['new_b']`: REXML::Node. If new, add before this node.  
  #   - `_d['new_a']`: REXML::Node. If new, add after this node.  
  #   - `_d['new_e']`: REXML::Element. If new, add under this element.  
  #   - `_d['xpath_lb']`: XPath for populating listbox.  
  #   - `_d['xpath_root']`: Root element of xpath_lb.
  #     Nil if `@doc` is used.  
  #   - `_d['listlambda']`: lambda object returning string for listbox.  
  #   - `_d['lb_otext']`: text expressing the class of objs in listbox.  
  #   - `_d['xref_cd']`: data for cross reference checks.
  #     (see method `cross_reference_checker` for details)  
  #   - `_d['editor']` = `ed`: lambda object of the "editor" to invoke.
  #     Eight params: `ed.call(e_a, e_b, e_c, e_d, e_e, e_f, e_g, e_h)`  
  #   - `_d['deletor']` = `dt`: lambda object of the "deletor" to invoke.  
  #   - `_d['no_cpmv']`: Boolean.
  #     True if copy and move are to be prohibited.  
  #   - `e_a` through `e_h`: ?
  # @param e_parent [REXML::Element] Parent element.
  def editor_select_frame(parent_win,es_win,fr_ul,_d,e_parent=nil)
    esf1m = TkFrame.new(fr_ul).pack
    esf1l  = TkLabelFrame.new(esf1m) do
      text(_("Select action:"))
      pack(fill: :both)
    end
    esf1r = TkLabelFrame.new(esf1m) do
      text(_("Open the editing window"))
      pack(fill: :both)
    end

    eslb_f = TkFrame.new(fr_ul)
    eslb_f.pack('side'=>'top')
    eslb_sb = TkScrollbar.new(eslb_f)
    eslb = TkListbox.new(
      eslb_f, 'height' => 10, 'width' => 50, 'selectmode' => 'single',
      'state' => 'normal',
      'yscrollcommand' => proc{|*arg|eslb_sb.set(*arg)})
    eslb_sb.command(proc{|*arg| eslb.yview(*arg)})
    eslb_sb.pack('side' => 'right', 'fill' => 'y')
    eslb.pack('side' => 'left', 'fill' => 'both')

    ar_el = Array.new
    v_x = TkVariable.new('edit')
    v_bt = TkVariable.new(_("Start editing"))

    rb_new = TkRadioButton.new(esf1l) do
      text _('New')
      value "new"
      variable v_x
      select
      command {
        v_bt.set_value(_("Start editing"))
        eslb.state('disabled')
      }
      pack(side: :left)
    end
    rb_edt = TkRadioButton.new(esf1l) do
      text _('Edit')
      value "edit"
      variable v_x
      command {
        v_bt.set_value(_("Start editing"))
        eslb.state('normal')
      }
      pack(side: :left)
    end
    if _d['no_cpmv'] == nil || _d['no_cpmv'] == false
      rb_mov = TkRadioButton.new(esf1l) do
        text _('Move')
        value "move"
        variable v_x
        command {
          v_bt.set_value(_("Start moving"))
          eslb.state('normal')
        }
        pack(side: :left)
      end
      rb_cpy = TkRadioButton.new(esf1l) do
        text _('Copy')
        value "copy"
        variable v_x
        command {
          v_bt.set_value(_("Start editing"))
          eslb.state('normal')
        }
        pack(side: :left)
      end
    end
    rb_del = TkRadioButton.new(esf1l) do
      text _("Delete")
      value "delete"
      variable v_x
      command {
        v_bt.set_value(_("Delete now"))
        eslb.state('normal')
      }
      pack(side: :left)
    end

    l_btn = lambda {
      p "BUTTON CHECK, eslb.size = " + eslb.size.to_s + " ..."
      if eslb.size < 1
        rb_edt.state("disabled")
        if _d['no_cpmv'] == nil || _d['no_cpmv'] == false
          rb_mov.state("disabled")
          rb_cpy.state("disabled")
        end
        rb_del.state("disabled")
      else
        rb_edt.state("normal")
        if _d['no_cpmv'] == nil || _d['no_cpmv'] == false
          rb_mov.state("normal")
          rb_cpy.state("normal")
        end
        rb_del.state("normal")
      end
    }

    _d['xpath_root'] = @doc unless _d['xpath_root']
    populate_listbox(eslb,_d['xpath_lb'],_d['listlambda'],nil,ar_el,
                     _d['xpath_root'])
    eslb.state('disabled')
    l_btn.call

    btn_ttedt = TkButton.new(esf1r) do
      textvariable v_bt
      # Do the edit
      pack(fill: :x)
    end
    btn_ttedt.command do
      if eslb.curselection.size > 1
        _mbspfm = _("You have selected more than one %1$s in the list.") +
                  _("Only one %1$s may be selected at a time.")
        Tk.messageBox(
          "title" => _("Error: duplicate selection"),
          "message" => sprintf(_mbspfm,_d['lb_otext']))
      elsif v_x != "new" && eslb.size > 0 && eslb.curselection.size == 0
        _mbspfm = _("Please select one %1$s from the list.")
        Tk.messageBox(
          "title" => _("Error: none selected"),
          "message" => sprintf(_mbspfm,_d['lb_otext']))
      elsif ( ( eslb.size == 0 ) && ( v_x != "new" ) )
        _mbspfm = _("There is no %1$s in the list.")
        Tk.messageBox(
          "title" => _("Error: cannot proceed"),
          "message" => sprintf(_mbspfm,_d['lb_otext']))
      elsif _d['new_b'] == nil && _d['new_a'] == nil &&
            _d['new_e'] == nil && v_x == "new" && eslb.size == 0
        Tk.messageBox(
          "title" => _("Error: Unable to add new item"),
          "message" => sprintf(_("Error: Unable to add new %1$s"),
                               _d['lb_otext']))
      else
        e_cur = nil
        e_cur = ar_el[eslb.curselection[0]] unless
          eslb.size == 0 || v_x == "new"
        if v_x == "delete"
          # Deleting an item from the listbox. Check if this is referred
          # to by other object(s).
          if cross_reference_checker(
               es_win,_d['xref_cd'],e_cur,_d['listlambda'])
            str_msg2 = sprintf(
              _("Deleting the %1$s instance \"%2$s\". Are you sure?"),
              _d['lb_otext'],_d['listlambda'].call(e_cur))
            _mbspfm = _("Confirm deleting selected %1$s")
            answer2 = Tk.messageBox(
              "icon" => "error",
              "type" => "yesno",
              "title" => sprintf(_mbspfm,_d['lb_otext']),
              "message" => str_msg2 )
            if answer2 == "yes"
              if _d['deletor'] == nil
                e_cur.parent.delete_element(e_cur)
                if _d['xpath_root']
                  _xrt = _d['xpath_root']
                else
                  _xrt = @doc
                end
                populate_listbox(eslb,_d['xpath_lb'],_d['listlambda'],nil,
                                 ar_el,_xrt)
              else
                p "%%%%%%%%% Deletor calling, v_x = " + v_x + " %%%%%%%%%"
                (_d['deletor']).call(
                  es_win,eslb,ar_el,_d,v_x,l_btn,e_cur,e_parent)
              end
            end
          end
        else
          p "%%%%%%%%% Editor calling, v_x = " + v_x + " %%%%%%%%%"
          (_d['editor']).call(es_win,eslb,ar_el,_d,v_x,l_btn,e_cur,e_parent)
        end
      end
    end
  end

  # Insert position controller
  # Creating insert position controller radiobuttons.
  # @return [Hash] +retv+: Hash containing button objects and a TkVariable.
  #   - +\[nil\]+: a TkVariable object that shows the selection.
  #   - +\[\'lframe\'\]+: TkLabelFrame enclosing all items.
  #   - +\[\'before\'\]+: Button "Insert before this item".
  #   - +\[\'after\'\]+: Button "Insert after this item".
  # @param fr_ul [TkFrame] Parent frame. +retv \[\'lframe\'\]+ will be created
  #   under this frame.
  def insert_position_controller_radiobuttons(fr_ul)
    retv = Hash.new
    retv[nil] = TkVariable.new("before")
    retv['lframe'] = TkLabelFrame.new(fr_ul) do
      text(_("Select inserting position:"))
      pack(fill: :both)
    end
    retv['before'] = TkRadioButton.new(retv['lframe']) do
      text _('Insert before this item')
      value "before"
      variable retv[nil]
      select
      pack(side: :left)
    end
    retv['after'] = TkRadioButton.new(retv['lframe']) do
      text _('Insert after this item')
      value "after"
      variable retv[nil]
      pack(side: :left)
    end
    return retv
  end


  # "Pick one" window:
  # Creating a selector that "picks one" from the displayed listbox.
  # @return [Boolean] True if one item is successfully selected.
  # @param parent_win [TkToplevel] Parent window.
  # @param e_cur [REXML::Element] Current element (default selection).
  # @param _d [Hash] Data.
  #   - +_d \[:xpath_lb\]+: XPath for populating listbox.
  #   - +_d \[:listlambda\]+: Lambda object returning string for listbox.
  #   - +_d \[:lb_otext\]+: Text expressing the class of objs in listbox.
  #   - +_d \[:sw_nil\]+: True if selection of nil is allowed.
  #   - +_d \[:sw_new\]+: True if no default is selected.
  #   - +_d \[:slambda\]+ = +sl+: Lambda object to execute when selected.
  #     +sl.call(e, _org, _de, _dh)+. Returns true if successful;
  #     Takes four arguments as shown below.
  #   - +e+ (argument to +sl+): selected element.
  #   - +_org+ (argument to +sl+): +e_cur+.
  #   - +_de+ (argument to +sl+) = +_d+.
  #   - +_d+ (argument to +sl+) = +_h+.
  #   - +_h+: window (or class) specific data.
  #   - +l_ok+: Lambda object to check entries. True if OK to be in list.
  #     +l_ok.call(e)+ where +e+ is the entry.
  def pick_one_window(parent_win, e_cur, _d, _h, l_ok = nil)
    p "#### PICK ONE WINDOW ####"
    if _d[:sw_new] == true
      strx = "no default selection"
    elsif e_cur == nil
      strx = "the default selection of nil"
    else
      strx = "the default selection of " + _d[:listlambda].call(e_cur)
    end
    p "\"Pick one\" window for " + _d[:lb_otext] +
      " objects starting with " + strx + "..."

    # Create a child window and a listbox
    pick_one_w = open_editor_window(
      parent_win,
      _("\"Sujic\"") + ":: " +
      sprintf(_("Pick up one %1$s element"),_d[:lb_otext]))
    pow_lb_f = TkFrame.new(pick_one_w)
    pow_lb_f.pack('side'=>'top')
    pow_lb_sb = TkScrollbar.new(pow_lb_f)
    pow_lb = TkListbox.new(
      pow_lb_f, 'height' => 10, 'width' => 50, 'selectmode' => 'single',
      'yscrollcommand' => proc{|*arg|pow_lb_sb.set(*arg)})
    pow_lb_sb.command(proc{|*arg| pow_lb.yview(*arg)})
    pow_lb_sb.pack('side' => 'right', 'fill' => 'y')
    pow_lb.pack('side' => 'left', 'fill' => 'both')
    pow_el = Array.new

    # Populate listbox
    if _d[:sw_nil] && e_cur == nil
      e_s = "nil"
    else
      e_s = e_cur
    end
    populate_listbox(pow_lb, _d[:xpath_lb], _d[:listlambda], e_s, pow_el,
                     @doc,_d[:sw_nil],l_ok)

    # Enter button command
    l_sc = lambda {
      if pow_lb.curselection.size > 1
        _mbspfm = _("You have selected more than one %1$s in the list.")
        Tk.messageBox(
          "title" => _("Error: duplicate selection"),
          "message" => sprintf(_mbspfm,_d[:lb_otext]))
        return false
      elsif pow_lb.curselection.size < 1
        _mbspfm = _("No %1$s was selected from the list.")
        Tk.messageBox(
          "title" => _("Error: none selected"),
          "message" => sprintf(_mbspfm,_d[:lb_otext]))
        return false
      else
        e_sel = nil
        e_sel = pow_el[pow_lb.curselection[0]] if pow_lb.size != 0
        return _d[:slambda].call(e_sel,e_cur,_h)
      end
    }

    # Buttons to close this window
    p "### last_of_editor_window in pick_one_window"
    last_of_editor_window(
      parent_win,pick_one_w,nil,nil,nil,nil,nil,nil,l_sc,nil)
  end

  # Speed class data editor: Creating a window for speed class data editing.
  # @return None.
  # @param parent_win [TkToplevel] Parent window.
  # @param lbox [TkListbox] Listbox containing this train type.
  # @param ar_pel [xx] xx.
  # @param v_x [xx] xx.
  # @param l_btn [xx] xx.
  # @param e_cur [REXML::Element] Current element if it already exists.
  def speed_class_editor(parent_win, lbox, ar_pel, _de, v_x, l_btn, e_cur,
                         e_parent)
    # e_parent = nil
    p 'speed_class_editor start'

    #-----------------#
    # Initializing... #
    #-----------------#
    # Window open
    ln_edt = open_editor_window(
      parent_win, _("\"Sujic\"") + ":: " + _("Editing a speed class"))

    # Data extracting
    if v_x == "edit"
      e_xc = e_cur.deep_clone
      e_nc = e_cur
    elsif v_x == "copy"
      e_xc = e_cur.deep_clone
      e_xc.elements['data'].add_attribute('name','')
      e_nc = e_xc
    elsif v_x == "new"
      e_xc = REXML::Element.new('create')
      {'class'=>'sujic_speedclass'}.each{|_k,_v| e_xc.add_attribute(_k,_v)}
      e_xc.add_element('data',{'name' => '','class' => 'x_speedclass'})
      e_nc = e_xc
    else
      e_nc = e_cur
    end
    ln_wh = Hash.new

    l_discard_edit = lambda {
      if v_x == "edit"
        e_cur.parent.insert_after(e_cur,e_xc)
        e_cur.parent.delete(e_cur)
      end
    }

    # Tab representation (notebook widget)
    ln_wh[:ttn] = Tk::Tile::Notebook.new(ln_edt) do
      place('x'=>10, 'y'=>10)
    end
    ln_wh[:ttn].pack

    # Create a frame for selecting insertion position.
    # Unnecessary if v_x == "edit"
    if v_x != "edit"
      ln_wh[:pss] = TkFrame.new(ln_wh[:ttn])
      ln_wh[:ttn].add(ln_wh[:pss], :text => _('Where to insert'))
      # edit_select_window data
      h_pss = Hash.new
      h_pss_l01 = TkFrame.new(ln_wh[:pss]).pack(side: :top, fill: :x)
      h_pss = insert_position_controller_radiobuttons(h_pss_l01)
      h_pss_l02 = TkFrame.new(ln_wh[:pss]).pack(side: :top, fill: :x)
      ar_el = Array.new
      h_pss_lb_sb = TkScrollbar.new(h_pss_l02)
      h_pss_lb = TkListbox.new(
        h_pss_l02, 'height' => 10, 'width' => 50, 'selectmode' => 'single',
        'yscrollcommand' => proc{|*arg|h_pss_lb_sb.set(*arg)})
      h_pss_lb_sb.command(proc{|*arg| h_pss_lb.yview(*arg)})
      h_pss_lb_sb.pack('side' => 'right', 'fill' => 'y')
      h_pss_lb.pack('side' => 'left', 'fill' => 'both')
      populate_listbox(h_pss_lb,_de['xpath_lb'],_de['listlambda'],nil,ar_el,
                       @doc)
      if h_pss_lb.size < 1
        h_pss.each do |key,val|
          next if key == nil || key == "lframe"
          val.state("disabled")
        end
      end
    end

    # Create a frame for main editing (speed class name change, etc.)
    ln_wh[:lnm] = TkFrame.new(ln_wh[:ttn])

    # Create a frame for coupled trainsets
    ln_wh[:cts] = TkFrame.new(ln_wh[:ttn])

    # Adding frames to the notebook widget
    ln_wh[:ttn].add(ln_wh[:lnm], :text => _('General'))
    ln_wh[:ttn].add(ln_wh[:cts], :text => _('Coupled trainsets'))

    #--------------------------#
    # Populating general frame #
    #--------------------------#
    ln_wh[:lnm_l01] = TkFrame.new(ln_wh[:lnm]).pack(side: :top, fill: :x)
    ln_wh[:lnm_l01_lbl] = TkLabel.new(ln_wh[:lnm_l01])
    ln_wh[:lnm_l01_lbl].text=_("Speed class name: ")
    ln_wh[:lnm_l01_lbl].pack(side: :left)
    ln_wh[:lnm_l01_ent] = TkEntry.new(ln_wh[:lnm_l01])
    ln_wh[:lnm_l01_ent].width(45)
    ln_wh[:lnm_l01_ent].value = e_nc.elements['data'].attributes['name']
    ln_wh[:lnm_l01_ent].pack(side: :left, fill: :x)

    #------------------------------------#
    # Populating coupled trainsets frame #
    #------------------------------------#
    ln_wh[:uch] = Hash.new
    ln_wh[:uch]['new_b'] = e_nc.elements['data']
    ln_wh[:uch]['new_a'] = ln_wh[:uch]['new_e'] = nil
    ln_wh[:uch]['xpath_lb'] =
      'data[@name=\'' + e_nc.elements['data'].attributes['name'] +
      '\']/coupled_trainset'
    ln_wh[:uch]['xpath_root'] = e_nc
    ln_wh[:uch]['listlambda'] = lambda{|e|
      retv = ''
      if e == nil
        retv << "nil"
      else
        delim = ''
        e.elements.each do |e_c|
          retv << delim
          delim = '-'
          retv << e_c.attributes['name']
          if e_c.attributes['reverse'] == 'false'
            retv << '/f'
          else
            retv << '/r'
          end
        end
      end
      return retv
    }
    ln_wh[:uch]['lb_otext'] = _("Coupled trainset")
    ln_wh[:uch]['xref_cd'] = Hash.new
    ln_wh[:uch]['xref_cd'][nil] = _("Coupled trainset")
    ln_wh[:uch]['editor'] = lambda {|a,b,c,d,e,f,g,h|
      coupled_trainset_editor(a,b,c,d,e,f,g,h) }
    ln_wh[:uch]['deletor'] = lambda {|a,b,c,d,e,f,g,h|
      coupled_trainset_editor(a,b,c,d,e,f,g,h) }
    ln_wh[:uch]['no_cpmv'] = true
    ln_wh[:ucm] = TkFrame.new(ln_wh[:cts]).pack
    editor_select_frame(parent_win,ln_edt,ln_wh[:ucm],ln_wh[:uch],e_nc)

    # Enter button command
    l_sc = lambda {
      tt_xa = 'rtss_input_data/create[@class=\'sujic_speedclass\']/data['
      tt_xa += '@name=\'' + ln_wh[:lnm_l01_ent].value + '\']'
      if ln_wh[:lnm_l01_ent].value == "nil"
        Tk.messageBox(
          "title" =>_("\"Nil\" not allowed as speed class name"),
          "message" =>_("You cannot use \"nil\" as a name of a speed class."))
        return false
      elsif v_x != "move" && ln_wh[:lnm_l01_ent].value == ""
        Tk.messageBox(
          "title" => _("Speed class name not specified?"),
          "message" => _("Please specify a speed class name") )
        return false
      elsif ( v_x == "move" ) &&
            ( ln_wh[:lnm_l01_ent].value !=
              e_nc.elements['data'].attributes['name'] )
        Tk.messageBox(
          "title" => _("Name change not allowed"),
          "message" => _("Element name should not be changed when moving"))
        return false
      elsif ( v_x != "move" ) &&
            ( ln_wh[:lnm_l01_ent].value !=
              e_nc.elements['data'].attributes['name'] &&
              @doc.elements[tt_xa] != nil )
        Tk.messageBox(
          "title" => _("Name already in use"),
          "message" => _("This speed class name is already used: ") + "\n" +
                       ln_wh[:lnm_l01_ent].value )
        return false
      elsif ( v_x == "new" ) && ( h_pss_lb.size < 1 ) &&
            ( _de['new_b'] == nil )
        # Bug: first element insertion destination not set
        Tk.messageBox(
          "title" => _("Internal error") + " L00743",
          "message" =>
          sprintf(_("Internal error %1$s has happened. Report this as bug."),
                  "L00743"))
        return true
      elsif ( v_x == "move" ) && ( h_pss_lb.size < 1 )
        # Bug: move selected when no element is present
        Tk.messageBox(
          "title" => _("Internal error") + " L00751",
          "message" =>
          sprintf(_("Internal error %1$s has happened. Report this as bug."),
                  "L00751"))
        return true
      elsif ( v_x == "new" || v_x == "copy" ) &&
            h_pss_lb.curselection.size > 1
        # Curselection should be 1
        Tk.messageBox(
          "title" => _("Multiple insertion destinations"),
          "message" =>
          _("Select only one insertion destination element."))
        return false
      elsif ( ( v_x == "new" && h_pss_lb.size >= 1 ) || v_x == "copy" ) &&
            h_pss_lb.curselection.size < 1
        # Curselection should be 1
        Tk.messageBox(
          "title" => _("Select an insertion destination"),
          "message" => _("Select one insertion destination element " +
                         "from the list in tab \"Where to insert\"."))
        return false
      elsif e_nc.elements['data'].elements.to_a('coupled_trainset') == nil ||
            e_nc.elements['data'].elements.to_a('coupled_trainset').size < 1
        Tk.messageBox(
          "title" => _("No coupled trainset"),
          "message" => _("Coupled trainset could not be found."))
        return false
      else
        ## Checks before inserting...
        if e_nc.elements['data'].elements.to_a('coupled_trainset') != nil
          e_nc.elements['data'].elements.each('coupled_trainset') do |ctck|
            if ctck.elements.to_a('traintype') == nil ||
               ctck.elements.to_a('traintype').size < 1
              Tk.messageBox(
                "title" => _("Coupled trainset empty"),
                "message" => _("Empty coupled trainset element was found."))
              return false
            end
          end
        end
        ## Checks before inserting ended...
        if v_x == "new" || v_x == "copy"
          if ln_wh[:lnm_l01_ent].value !=
             e_nc.elements['data'].attributes['name']
            str_on = e_nc.elements['data'].attributes['name']
            e_nc.elements['data'].add_attribute(
              'name', ln_wh[:lnm_l01_ent].value)
          end
          p "###### -- insert -- ###### [v_x = " + v_x + "]"
          p "###### -- insert -- ###### [h_pss_lb.size = " +
            h_pss_lb.size.to_s + "]"
          if ( h_pss_lb.size < 1 )
            elm_ins = _de['new_b']
            sw_after = false
          else
            p "###### -- insert -- ###### [cursel = " +
              h_pss_lb.curselection[0].to_s + "]"
            sw_after = false if h_pss[nil] == "before"
            sw_after = true if h_pss[nil] == "after"
            elm_ins = ar_el[h_pss_lb.curselection[0]]
            p "###### -- insert -- ######"
            puts elm_ins
            p "###### -- insert after? -- ######"
            p "---- after" if sw_after
            p "---- before" unless sw_after
            p "###### -- insert end -- ######"
          end
          if sw_after
            elm_ins.parent.insert_after(elm_ins,e_nc)
          else
            elm_ins.parent.insert_before(elm_ins,e_nc)
          end
        elsif v_x == "move" && e_cur != ar_el[h_pss_lb.curselection[0]]
          elm_cut = e_cur.parent.delete(e_cur)
          p "#### ELM_CUT is not equal to E_CUR ####" if elm_cut != e_cur
          sw_after = false if h_pss[nil] == "before"
          sw_after = true if h_pss[nil] == "after"
          elm_ins = ar_el[h_pss_lb.curselection[0]]
          elm_ins.parent.insert_after(elm_ins, elm_cut) if sw_after
        elsif v_x != "move" ## Edit only
          if ln_wh[:lnm_l01_ent].value != e_nc.attributes['name']
            str_on = e_nc.attributes['name']
            e_nc.elements['data'].add_attribute(
              'name', ln_wh[:lnm_l01_ent].value)
            # [('rtss_input_data/create[@class=\'sujic_trainset\']/using[' +
            #   '@class=\'sujic_traintype\']'),
            #  ('rtss_input_data/create[@class=\'sujic_speedclass\']' +
            #   '/data/coupled_trainset/traintype')].each do |tt_xb|
            #   @doc.elements.each(tt_xb) do |elem|
            #     elem.add_attribute('name',tt_ah['name']) if
            #       elem.attributes['name'] == str_on
            #   end
            # end
          end
        end
        return true
      end
    }

    # Close this window
    last_of_editor_window(
      parent_win,ln_edt,_de['xpath_lb'],_de['xpath_root'],_de['listlambda'],
      l_btn,lbox,ar_pel,l_sc,v_x,l_discard_edit)
    p 'speed_class_editor end'

  end

  #================================================#
  # Creating a window for coupled trainset editing #
  #================================================#
  def coupled_trainset_editor(parent_win, lbox, ar_pel, _de, v_x,
                              l_btn, e_cur, e_parent)
    p "coupled_trainset_editor starting..."

    # v_x can be one of: "new", "edit", "delete"
    if v_x == "new" || v_x == "edit"
      coupled_trainset_editor_nwed(
        parent_win, lbox, ar_pel, _de, v_x, l_btn, e_cur, e_parent)
    elsif v_x == "delete"
      coupled_trainset_deletor(
        parent_win, lbox, ar_pel, _de, v_x, l_btn, e_cur, e_parent)
    else
      # Bug: move selected when no element is present
      Tk.messageBox(
        "title" => _("Internal error") + " L00860",
        "message" =>
        sprintf(_("Internal error %1$s has happened. Report this as bug."),
                 "L00860"))
    end
  end

  def coupled_trainset_deletor(
        parent_win, lbox, ar_pel, _de, v_x, l_btn, e_cur, e_parent)
    p "coupled_trainset_deletor starting..."

    # XRef checking: NONE

    # Delete and close this window
    e_cur.parent.delete_element(e_cur)
    if _de['xpath_root']
      _xrt = _de['xpath_root']
    else
      _xrt = @doc
    end
    populate_listbox(lbox,_de['xpath_lb'],_de['listlambda'],nil,ar_pel,_xrt)
    p 'coupled_trainset_deletor end'
  end

  def coupled_trainset_editor_nwed(parent_win, lbox, ar_pel,
                                   _de, v_x, l_btn, e_cur, e_parent)
    p "coupled_trainset_editor_nwed starting..."

    # Window open
    ln_edt = open_editor_window(
      parent_win, _("\"Sujic\"") + ":: " + _("Editing a coupled trainset"))
    # Data extracting
    if v_x == "new"
      e_xc = REXML::Element.new('coupled_trainset')
      e_nc = e_xc
    else # "edit"
      e_xc = e_cur.deep_clone
      e_nc = e_cur
    end
    l_discard_edit = lambda {
      if v_x == "edit"
        e_parent.elements['data'].insert_after(e_cur,e_xc)
        e_parent.elements['data'].delete(e_cur)
      end
    }
    ln_wh = Hash.new

    # Create a frame for train types
    ln_wh[:cts] = TkFrame.new(ln_edt).pack(side: :top, fill: :x)

    #-----------------------------#
    # Populating train type frame #
    #-----------------------------#
    ln_wh[:uch] = Hash.new
    ln_wh[:uch]['new_a'] = ln_wh[:uch]['new_b'] = nil
    ln_wh[:uch]['new_e'] = e_nc
    ln_wh[:uch]['xpath_lb'] = 'traintype'
    ln_wh[:uch]['xpath_root'] = e_nc
    ln_wh[:uch]['listlambda'] = lambda{|e|
      retv = ''
      if e == nil
        retv << "nil"
      else
        retv << e.attributes['name'] if e.attributes['name']
        retv << "nil" unless e.attributes['name']
        if e.attributes['reverse'] == 'false'
          retv << '/f'
        else
          retv << '/r'
        end
      end
      return retv
    }
    ln_wh[:uch]['lb_otext'] = _("Train type")
    ln_wh[:uch]['xref_cd'] = Hash.new
    ln_wh[:uch]['xref_cd'][nil] = _("Train type")
    ln_wh[:uch]['editor'] = lambda {|a,b,c,d,e,f,g,h|
      traintype_in_coupled_trainset_editor(a,b,c,d,e,f,g,h) }
    ln_wh[:uch]['deletor'] = lambda {|a,b,c,d,e,f,g,h|
      traintype_in_coupled_trainset_editor(a,b,c,d,e,f,g,h) }
    ln_wh[:uch]['no_cpmv'] = true
    ln_wh[:ucm] = TkFrame.new(ln_wh[:cts]).pack
    editor_select_frame(parent_win,ln_edt,ln_wh[:ucm],ln_wh[:uch],e_nc)

    # Enter button command
    l_sc = lambda {
      if e_nc.elements['traintype'] == nil ||
            e_nc.elements.size < 1
        Tk.messageBox(
          "title" => _("Coupled trainset empty"),
          "message" => _("Empty coupled trainset element was found."))
        return false
      else
        if v_x == "new"
          e_parent.elements['data'].add_element(e_nc)
        end
        return true
      end
    }

    # Close this window
    last_of_editor_window(
      parent_win,ln_edt,_de['xpath_lb'],_de['xpath_root'],_de['listlambda'],
      l_btn,lbox,ar_pel,l_sc,v_x,l_discard_edit)
    p 'coupled_trainset_editor end'

  end

  #=====================================================================#
  # Creating a train type selection window for coupled trainset editing #
  #=====================================================================#
  def traintype_in_coupled_trainset_editor(
        parent_win, lbox, ar_pel, _de, v_x, l_btn, e_cur, e_parent)
    p "traintype_in_coupled_trainset_editor starting..."
  
    if v_x == "new" || v_x == "edit"
      traintype_in_coupled_trainset_editor_nwed(
        parent_win, lbox, ar_pel, _de, v_x, l_btn, e_cur, e_parent)
    elsif v_x == "delete"
      traintype_in_coupled_trainset_deletor(
        parent_win, lbox, ar_pel, _de, v_x, l_btn, e_cur, e_parent)
    else
      # Bug: move selected when no element is present
      Tk.messageBox(
        "title" => _("Internal error") + " L00985",
        "message" =>
        sprintf(_("Internal error %1$s has happened. Report this as bug."),
                 "L00985"))
    end
    p "traintype_in_coupled_trainset_editor end"
  end

  def traintype_in_coupled_trainset_editor_nwed(
        parent_win, lbox, ar_pel, _de, v_x, l_btn, e_cur, e_parent)
    p "traintype_in_coupled_trainset_editor_nwed starting..."

    # Window open
    ln_edt = open_editor_window(
      parent_win,
      _("\"Sujic\"") + ":: " + _("Train type selector for coupled trainset"))
    # Data extracting
    e_xc = REXML::Element.new('traintype')
    if v_x == "new"
      e_nc = e_xc
    else
      e_cur.attributes.each do |ex|
        e_xc.add_attribute(ex.clone)
      end
      e_nc = e_cur
    end
    l_discard_edit = lambda {
      p "l_discard_edit for traintype_in_coupled_trainset_editor_nwed"
      p v_x.value
      if v_x == "edit"
        p e_cur
        p e_xc
        p e_parent
        e_parent.insert_after(e_cur,e_xc)
        e_parent.delete(e_cur)
      end
    }
    ln_wh = Hash.new

    # Create a frame for train types
    ln_wh[:cts] = TkFrame.new(ln_edt).pack(side: :top, fill: :x)

    #---------------------------------------#
    # Populating train type frame: pick one #
    #---------------------------------------#
    ln_wh[:_ah] = Hash.new
    ln_wh[:_ah][:nme] = e_nc.attributes['name']
    ln_wh[:cts01] = TkFrame.new(ln_wh[:cts]).pack(
      side: :top, fill: :x, anchor: :w)
    ln_wh[:cts01v] = TkVariable.new('orig')
    ln_wh[:cts01_lbl] = TkLabel.new(ln_wh[:cts01]).pack(side: :left)
    ln_wh[:cts01_lbl].text = _("Train type name")
    str_rb1 = "\"nil\""
    str_rb1 = e_nc.attributes['name'] if e_nc.attributes['name']
    p "--**--##"
    p str_rb1
    p "--**--##"
    ln_wh[:cts01_rb1] = TkRadioButton.new(ln_wh[:cts01]) do
      text str_rb1
      value "orig"
      variable ln_wh[:cts01v]
      select
      pack(side: :left)
    end
    ln_wh[:cts01_rb2] = TkRadioButton.new(ln_wh[:cts01]) do
      text _("Select another")
      value "another"
      variable ln_wh[:cts01v]
      pack(side: :left)
    end
    ln_wh[:cts02] = TkFrame.new(ln_wh[:cts]).pack(
      side: :top, fill: :x, anchor: :w)
    ln_wh[:cts02v] = TkVariable.new('normal')
    ln_wh[:cts02_lbl] = TkLabel.new(ln_wh[:cts02]).pack(side: :left)
    ln_wh[:cts02_lbl].text = _("Trainset orientation")
    ln_wh[:cts02_rb1] = TkRadioButton.new(ln_wh[:cts02]) do
      text _("normal")
      value "normal"
      variable ln_wh[:cts02v]
      select
      pack(side: :left)
    end
    ln_wh[:cts02_rb2] = TkRadioButton.new(ln_wh[:cts02]) do
      text _("reverse")
      value "reverse"
      variable ln_wh[:cts02v]
      pack(side: :left)
    end
    ln_wh[:cts01_rb2].command(
      proc{
        p "Radiobutton LSM01_RB2"
        _lsm01_2h = Hash.new
        _lsm01_2h[:xpath_lb] =
          'rtss_input_data/create[@class=\'sujic_traintype\']'
        _lsm01_2h[:listlambda] = lambda{|e|
          e.elements['data'].attributes['name']}
        _lsm01_2h[:lb_otext] = _("Train type")
        _lsm01_2h[:sw_nil] = false
        _lsm01_2h[:slambda] = lambda {|e,_org,_dx|
          if e == nil
            Tk.messageBox(
              "title" => _("Internal error") + " L01090",
              "message" =>
              sprintf(_("Internal error %1$s has happened. " +
                        "Report this as bug."), "L01090"))
            return false
          elsif e != _org
            _dx[:_ah][:nme] = e.elements['data'].attributes['name']
            _dx[:cts01_rb1].text(e.elements['data'].attributes['name'])
            _dx[:cts01_rb1].select
            return true
          end }
        stn_c = nil
        stn_c = @doc.elements[
          'rtss_input_data/create[@class=\'sujic_traintype\']/data[@name=\'' +
          str_rb1 + '\']/..'] unless e_nc.attributes['name']
        pick_one_window(ln_edt, stn_c, _lsm01_2h, ln_wh)
      })

    # Enter button command
    l_sc = lambda {
      if ln_wh[:_ah][:nme] == "nil"
        Tk.messageBox(
          "title" => _("\"Nil\" not allowed as train type name"),
          "message" => _("You cannot use \"nil\" as a name of a train type."))
        return false
      else
        if v_x == "new"
          e_parent.add_element(e_nc)
        end
        e_nc.add_attribute('name', ln_wh[:_ah][:nme])
        if ln_wh[:cts02v] == 'normal'
          e_nc.add_attribute('reverse', 'false')
        else
          e_nc.add_attribute('reverse', 'true')
        end
        return true
      end
    }

    # Close this window
    last_of_editor_window(
      parent_win, ln_edt, _de['xpath_lb'], _de['xpath_root'],
      _de['listlambda'], l_btn, lbox, ar_pel, l_sc, v_x, l_discard_edit)
    p 'traintype_in_coupled_trainset_editor_nwed end'
  end

  def traintype_in_coupled_trainset_deletor(
        parent_win, lbox, ar_pel, _de, v_x, l_btn, e_cur, e_parent)
    p "traintype_in_coupled_trainset_deletor starting..."

    # XRef checking: NONE

    # Delete and close this window
    e_cur.parent.delete_element(e_cur)
    if _de['xpath_root']
      _xrt = _de['xpath_root']
    else
      _xrt = @doc
    end
    populate_listbox(lbox,_de['xpath_lb'],_de['listlambda'],nil,ar_pel,_xrt)
    p 'traintype_in_coupled_trainset_deletor end'
  end

  # Train type data editor: Creating a window for train type data editing.
  # @return None.
  # @param parent_win [TkToplevel] Parent window.
  # @param lbox [TkListbox] Listbox containing this train type.
  # @param ar_pel [xx] xx.
  # @param v_x [xx] xx.
  # @param l_btn [xx] xx.
  # @param e_cur [REXML::Element] Current element if it already exists.
  def train_type_editor(parent_win, lbox, ar_pel, _de, v_x, l_btn, e_cur,
                        e_parent)
    # e_parent = nil
    p 'train_type_editor start'

    # Data extracting ...
    tt_ah = Hash.new
    elm_c = nil
    elm_c = e_cur.elements['data'] if e_cur != nil
    if v_x == "edit" || v_x == "copy" || v_x == "move"
      tt_ah['name'] = elm_c.attributes['name']
      tt_ah['seatings'] = elm_c.attributes['seatings']
      tt_ah['length'] = elm_c.attributes['length']
      tt_ah['coaches'] = elm_c.attributes['coaches']
      tt_ah['capacity'] = elm_c.attributes['capacity']
    else
      tt_ah['name'] = ""
      tt_ah['seatings'] = 50
      tt_ah['length'] = 20
      tt_ah['coaches'] = 1
      tt_ah['capacity'] = 150
    end

    #-----------------------------#
    # Creating a child window ... #
    #-----------------------------#
    # Window open
    tt_edt = open_editor_window(
      parent_win, _("\"Sujic\"") + ":: " + _("Editing a train type"))
    # Frame
    tt_l01 = TkFrame.new(tt_edt).pack(side: :top, fill: :x)
    # Buttons and entry widgets
    tt_l01_lbl = TkLabel.new(tt_l01)
    tt_l01_lbl.text=_("Train type name: ")
    tt_l01_lbl.pack(side: :left)
    tt_l01_ent=TkEntry.new(tt_l01)
    tt_l01_ent.width(45)
    tt_l01_ent.value=tt_ah['name']
    tt_l01_ent.pack(side: :left, fill: :x)
    if v_x != "move"
      tt_l02 = TkFrame.new(tt_edt).pack(side: :top, anchor: :e)
      tt_l02_lbl = TkLabel.new(tt_l02)
      tt_l02_lbl.text=_("Seating capacity [persons]: ")
      tt_l02_lbl.pack(side: :left)
      tt_l02_ent=TkEntry.new(tt_l02)
      tt_l02_ent.width(40)
      tt_l02_ent.value=tt_ah['seatings']
      tt_l02_ent.pack(side: :right)
      tt_l03 = TkFrame.new(tt_edt).pack(side: :top, anchor: :e)
      tt_l03_lbl = TkLabel.new(tt_l03)
      tt_l03_lbl.text=_("Capacity [persons]: ")
      tt_l03_lbl.pack(side: :left)
      tt_l03_ent=TkEntry.new(tt_l03)
      tt_l03_ent.width(40)
      tt_l03_ent.value=tt_ah['capacity']
      tt_l03_ent.pack(side: :right)
      tt_l04 = TkFrame.new(tt_edt).pack(side: :top, anchor: :e)
      tt_l04_lbl = TkLabel.new(tt_l04)
      tt_l04_lbl.text=_("Number of coaches: ")
      tt_l04_lbl.pack(side: :left)
      tt_l04_ent=TkEntry.new(tt_l04)
      tt_l04_ent.width(40)
      tt_l04_ent.value=tt_ah['coaches']
      tt_l04_ent.pack(side: :right)
      tt_l05 = TkFrame.new(tt_edt).pack(side: :top, anchor: :e)
      tt_l05_lbl = TkLabel.new(tt_l05)
      tt_l05_lbl.text=_("Trainset length [m]: ")
      tt_l05_lbl.pack(side: :left)
      tt_l05_ent=TkEntry.new(tt_l05)
      tt_l05_ent.width(40)
      tt_l05_ent.value=tt_ah['length']
      tt_l05_ent.pack(side: :right)
    end
    # Listbox for specifying where to insert
    if v_x != "edit"
      tt_l06 = TkFrame.new(tt_edt).pack(side: :top, fill: :x)
      pcrbs = Hash.new
      pcrbs = insert_position_controller_radiobuttons(tt_l06)
      tt_l07 = TkFrame.new(tt_edt).pack(side: :top, fill: :x)
      ar_el = Array.new
      lb_sb = TkScrollbar.new(tt_l07)
      lb = TkListbox.new(
        tt_l07, 'height' => 10, 'width' => 50, 'selectmode' => 'single',
        'yscrollcommand' => proc{|*arg|lb_sb.set(*arg)})
      lb_sb.command(proc{|*arg| lb.yview(*arg)})
      lb_sb.pack('side' => 'right', 'fill' => 'y')
      lb.pack('side' => 'left', 'fill' => 'both')
      populate_listbox(lb,_de['xpath_lb'],_de['listlambda'],nil,ar_el,@doc)
      if lb.size < 1
        pcrbs.each do |key,val|
          next if key == nil || key == "lframe"
          val.state("disabled")
        end
      end
    end
    # Enter button command
    l_sc = lambda {
      tt_xa = 'rtss_input_data/create[@class=\'sujic_traintype\']/data['
      tt_xa += '@name=\'' + tt_l01_ent.value + '\']'
      if tt_l01_ent.value == "nil"
        Tk.messageBox(
          "title" => _("\"Nil\" not allowed as train type name"),
          "message" => _("You cannot use \"nil\" as a name of a train type."))
        return false
      elsif v_x != "move" && tt_l01_ent.value == ""
        Tk.messageBox(
          "title" => _("Train type name not specified?"),
          "message" => _("Please specify a train type name.") )
        return false
      elsif ( v_x == "move" ) &&
            ( tt_l01_ent.value != elm_c.attributes['name'] )
        Tk.messageBox(
          "title" => _("Name change not allowed"),
          "message" => _("Element name should not be changed when moving"))
        return false
      elsif ( v_x != "move" ) &&
            ( ( elm_c != nil ) &&
              ( tt_l01_ent.value != elm_c.attributes['name'] ) ||
              ( elm_c == nil ) && ( v_x == "new" ) ) &&
            ( @doc.elements[tt_xa] != nil )
        Tk.messageBox(
          "title" => _("Name already in use"),
          "message" => _("This train type name is already used: ") + "\n" +
                       tt_l01_ent.value )
        return false
      elsif ( v_x != "move" ) && ( ! tt_l02_ent.value.is_integer? )
        # Value check (1) l02_ent: seatings = integer
        Tk.messageBox(
          "title" => _("Seating capacity must be an integer"),
          "message" =>
          _("Seating capacity of a trainset must be an integer figure.") )
        return false
      elsif ( v_x != "move" ) && ( ! tt_l03_ent.value.is_integer? )
        # Value check (2) l03_ent: capacity = integer
        Tk.messageBox(
          "title" => _("Capacity must be an integer"),
          "message" =>
          _("Capacity of a trainset must be an integer figure.") )
        return false
      elsif ( v_x != "move" ) && ( ! tt_l04_ent.value.is_integer? )
        # Value check (3) l04_ent: coaches = integer
        Tk.messageBox(
          "title" => _("Number of coaches must be an integer"),
          "message" =>
          _("Number of coaches in a trainset must be an integer figure.") )
        return false
      elsif ( v_x != "move" ) && ( ! tt_l05_ent.value.is_float? )
        # Value check (4) l05_ent: length = float
        Tk.messageBox(
          "title" => _("Trainset length must be a real number"),
          "message" =>
          _("Trainset length must be a real number (Unit: m).") )
        return false
      elsif ( v_x == "new" ) && ( lb.size < 1 ) &&
            ( _de['new_b'] == nil )
        # Bug: first element insertion destination not set
        Tk.messageBox(
          "title" => _("Internal error") + " L01299",
          "message" =>
          sprintf(_("Internal error %1$s has happened. Report this as bug."),
                  "L01299"))
        return true
      elsif ( v_x == "move" ) && ( lb.size < 1 )
        # Bug: move selected when no element is present
        Tk.messageBox(
          "title" => _("Internal error") + " L01307",
          "message" =>
          sprintf(_("Internal error %1$s has happened. Report this as bug."),
                  "L01307"))
        return true
      elsif ( v_x == "new" || v_x == "copy" ) && lb.curselection.size > 1
        # Curselection should be 1
        Tk.messageBox(
          "title" => _("Multiple insertion destinations"),
          "message" =>
          _("Select only one insertion destination element."))
        return false
      elsif ( ( v_x == "new" && lb.size >= 1 ) || v_x == "copy" ) &&
            lb.curselection.size < 1
        # Curselection should be 1
        Tk.messageBox(
          "title" => _("Select an insertion destination"),
          "message" =>
          _("Select one insertion destination element from the list below."))
        return false
      else
        if v_x != "move"
          tt_ah['name'] = tt_l01_ent.value
          tt_ah['seatings'] = tt_l02_ent.value.to_i
          tt_ah['length'] = tt_l05_ent.value.to_f
          tt_ah['coaches'] = tt_l04_ent.value.to_i
          tt_ah['capacity'] = tt_l03_ent.value.to_i
        end
        if v_x == "new" || v_x == "copy"
          # Create an element and insert it
          n_e = REXML::Element.new('create')
          { 'class' => 'sujic_traintype' }.each { |key, val|
            n_e.add_attribute(key, val) }
          p "###### -- insert -- ###### [v_x = " + v_x + "]"
          p "###### -- insert -- ###### [lb.size = " + lb.size.to_s + "]"
          if ( lb.size < 1 )
            elm_ins = _de['new_b']
            sw_after = false
          else
            p "###### -- insert -- ###### [cursel = " +
              lb.curselection[0].to_s + "]"
            sw_after = false if pcrbs[nil] == "before"
            sw_after = true if pcrbs[nil] == "after"
            elm_ins = ar_el[lb.curselection[0]]
            p "###### -- insert -- ######"
            puts elm_ins
            p "###### -- insert after? -- ######"
            p "---- after" if sw_after
            p "---- before" unless sw_after
            p "###### -- insert end -- ######"
          end
          if sw_after
            elm_ins.parent.insert_after(elm_ins,n_e)
          else
            elm_ins.parent.insert_before(elm_ins,n_e)
          end
          n_e.add_element('data',{'name' => tt_ah['name'],
                                  'class' => 'x_traintype',
                                  'seatings' => tt_ah['seatings'].to_s,
                                  'length' => tt_ah['length'].to_s,
                                  'coaches' => tt_ah['coaches'].to_s,
                                  'capacity' => tt_ah['capacity'].to_s})
        elsif v_x == "move" && e_cur != ar_el[lb.curselection[0]]
          elm_cut = e_cur.parent.delete(e_cur)
          p "#### ELM_CUT is not equal to E_CUR ####" if elm_cut != e_cur
          sw_after = false if pcrbs[nil] == "before"
          sw_after = true if pcrbs[nil] == "after"
          elm_ins = ar_el[lb.curselection[0]]
          elm_ins.parent.insert_after(elm_ins, elm_cut) if sw_after
        elsif v_x != "move" ## Edit only
          tt_ah.each{|key, val|
            elm_c.add_attribute(key, val.to_s) if key != "name"}
          if tt_ah['name'] != elm_c.attributes['name']
            str_on = elm_c.attributes['name']
            elm_c.add_attribute('name', tt_ah['name'])
            [('rtss_input_data/create[@class=\'sujic_trainset\']/using[' +
              '@class=\'sujic_traintype\']'),
             ('rtss_input_data/create[@class=\'sujic_speedclass\']' +
              '/data/coupled_trainset/traintype')].each do |tt_xb|
              @doc.elements.each(tt_xb) do |elem|
                elem.add_attribute('name',tt_ah['name']) if
                  elem.attributes['name'] == str_on
              end
            end
          end
        end
        return true
      end
    }
    # Button to close this window
    p "### last_of_editor_window in train_type_editor"
    last_of_editor_window(
      parent_win,tt_edt,_de['xpath_lb'],nil,_de['listlambda'],l_btn,lbox,
      ar_pel,l_sc,v_x)
  end

  # Train data editor.
  # @return None.
  # @param parent_win [TkToplevel] Parent window.
  # @param lbox [TkListbox] Listbox to be updated after editing is done.
  # @param ar_pel [xx] xx
  # @param _de [xx] xx
  # @param v_x [TkVariable] xx
  # @param l_btn [xx] xx
  # @param e_cur [REXML::Element] Current element.
  # @param e_parent [REXML::Element] Parent element of +e_cur+. In this case
  #   it will be the <create> element for the schedule set.
  def train_editor(parent_win, lbox, ar_pel, _de, v_x, l_btn, e_cur,
                   e_parent)
    if e_cur
      p "train_editor start, tr|ss = \"" +
        e_cur.elements['data'].attributes['name'] +
        "\"|\"" + e_parent.elements['data'].attributes['name'] + "\""
    else
      p "train_editor start, tr|ss = \"nil\"|\"" +
        e_parent.elements['data'].attributes['name'] + "\""
    end

    #---------------------------------------------#
    # Creating a child window and populate it ... #
    #---------------------------------------------#
    t_ewn = open_editor_window(
      parent_win, _("\"Sujic\"") + ":: " + _("Editing a train"))

    # Data extracting
    if v_x == "edit"
      e_xc = e_cur.deep_clone
      e_nc = e_cur
    elsif v_x == "copy"
      e_xc = e_cur.deep_clone
      e_xc.elements['data'].add_attribute('name','')
      e_nc = e_xc
    elsif v_x == "new"
      e_xc = REXML::Element.new('create')
      {'class'=>'sujic_train'}.each{|_k,_v| e_xc.add_attribute(_k,_v)}
      e_xc.add_element('under',{'name' => @ss_name,
                                'class' => 'sujic_schedule_set'})
      e_xc.add_element('data',{'name' => '','class' => 'x_train'})
      e_nc = e_xc
    else
      e_nc = e_cur
    end
    ln_wh = Hash.new

    l_discard_edit = lambda {
      if v_x == "edit"
        e_cur.parent.insert_after(e_cur,e_xc)
        e_cur.parent.delete(e_cur)
      end
    }

    # Tab representation (notebook widget)
    ln_wh[:ttn] = Tk::Tile::Notebook.new(t_ewn) do
      place('x'=>10, 'y'=>10)
    end
    ln_wh[:ttn].pack

    # Create a frame for selecting insertion position.
    # Unnecessary if v_x == "edit"
    if v_x != "edit"
      ln_wh[:pss] = TkFrame.new(ln_wh[:ttn])
      ln_wh[:ttn].add(ln_wh[:pss], :text => _('Where to insert'))
      # edit_select_window data
      h_pss = Hash.new
      h_pss_l01 = TkFrame.new(ln_wh[:pss]).pack(side: :top, fill: :x)
      h_pss = insert_position_controller_radiobuttons(h_pss_l01)
      h_pss_l02 = TkFrame.new(ln_wh[:pss]).pack(side: :top, fill: :x)
      ar_el = Array.new
      h_pss_lb_sb = TkScrollbar.new(h_pss_l02)
      h_pss_lb = TkListbox.new(
        h_pss_l02, 'height' => 10, 'width' => 50, 'selectmode' => 'single',
        'yscrollcommand' => proc{|*arg|h_pss_lb_sb.set(*arg)})
      h_pss_lb_sb.command(proc{|*arg| h_pss_lb.yview(*arg)})
      h_pss_lb_sb.pack('side' => 'right', 'fill' => 'y')
      h_pss_lb.pack('side' => 'left', 'fill' => 'both')
      populate_listbox(h_pss_lb,_de['xpath_lb'],_de['listlambda'],nil,ar_el,
                       @doc)
      if h_pss_lb.size < 1
        h_pss.each do |key,val|
          next if key == nil || key == "lframe"
          val.state("disabled")
        end
      end
    end

    # Create a frame for main editing (train name change, etc.)
    ln_wh[:lnm] = TkFrame.new(ln_wh[:ttn])

    # Create a frame for list of times
    ln_wh[:stu] = TkFrame.new(ln_wh[:ttn])

    # Adding frames to the notebook widget
    ln_wh[:ttn].add(ln_wh[:lnm], :text => _('General'))
    ln_wh[:ttn].add(ln_wh[:stu], :text => _('Times list'))

    #--------------------------#
    # Populating general frame #
    #--------------------------#
    ln_wh[:lnm_l01] = TkFrame.new(ln_wh[:lnm]).pack(side: :top, fill: :x)
    ln_wh[:lnm_l01_lbl] = TkLabel.new(ln_wh[:lnm_l01])
    ln_wh[:lnm_l01_lbl].text=_("Train name: ")
    ln_wh[:lnm_l01_lbl].pack(side: :left)
    ln_wh[:lnm_l01_ent] = TkEntry.new(ln_wh[:lnm_l01])
    ln_wh[:lnm_l01_ent].width(45)
    ln_wh[:lnm_l01_ent].value = e_nc.elements['data'].attributes['name']
    ln_wh[:lnm_l01_ent].pack(side: :left, fill: :x)
    ln_wh[:lnm_l02] = TkFrame.new(ln_wh[:lnm]).pack(side: :top, fill: :x)
    ln_wh[:lnm_l02_lbl] = TkLabel.new(ln_wh[:lnm_l02])
    ln_wh[:lnm_l02_lbl].text=_("Speed class: ")
    ln_wh[:lnm_l02_lbl].pack(side: :left)
    ln_wh[:lnm_l02v] = TkVariable.new('orig')
    ln_wh[:lnm_l02tv] = TkVariable.new(
      (e_nc.elements['data'].attributes['speedclass'] ?
         e_nc.elements['data'].attributes['speedclass'] : 'nil'))
    ln_wh[:lnm_l02_rb1] = TkRadioButton.new(ln_wh[:lnm_l02]) do
      textvariable ln_wh[:lnm_l02tv]
      value "orig"
      variable ln_wh[:lnm_l02v]
      pack(side: :left)
    end
    ln_wh[:lnm_l02_rb2] = TkRadioButton.new(ln_wh[:lnm_l02]) do
      text _("Select another")
      value "another"
      variable ln_wh[:lnm_l02v]
      pack(side: :left)
    end

    ## (c1) Radiobutton [:lnm_l02_rb2] command.
    stn_c = (ln_wh[:lnm_l02tv] != "nil" ?
               @doc.elements[
               'rtss_input_data/create[@class=\'sujic_speedclass\']/data[' +
               '@name=\'' + ln_wh[:lnm_l02tv] + '\']/..'] : nil)
    ln_wh[:lnm_l02_rb2].command(
      proc{
        p "Radiobutton LNM02_RB2"
        _lnm02_2h = Hash.new
        _lnm02_2h[:xpath_lb] =
          'rtss_input_data/create[@class=\'sujic_speedclass\']'
        _lnm02_2h[:listlambda] = lambda{|e|
          e.elements['data'].attributes['name']}
        _lnm02_2h[:lb_otext] = _("Speed class")
        _lnm02_2h[:sw_nil] = true
        _lnm02_2h[:slambda] = lambda {|e,_org,_dx|
          p "*-*-*-*-*=*-*-*-*-*=*-*-*-*-*=*-*-*-*-*=*-*-*-*-*=*-*-*-*-*"
          print e
          p ""
          p "*-*-*-*-*=*-*-*-*-*=*-*-*-*-*=*-*-*-*-*=*-*-*-*-*=*-*-*-*-*"
          if e != _org
            if e
              e_nc.elements['data'].add_attribute(
                'speedclass',e.elements['data'].attributes['name'])
            else
              e_nc.elements['data'].add_attribute('speedclass','nil')
            end
            _dx[:lnm_l02tv].set_value(
              e_nc.elements['data'].attributes['speedclass'])
            stn_c = e
          end
          _dx[:lnm_l02v].set_value('orig')
          return true }
        pick_one_window(t_ewn, stn_c, _lnm02_2h, ln_wh)
      })

    ln_wh[:lnm_l03] = TkFrame.new(ln_wh[:lnm]).pack(side: :top, fill: :x)
    ln_wh[:lnm_l03_lbl] = TkLabel.new(ln_wh[:lnm_l03])
    ln_wh[:lnm_l03_lbl].text=_("Running direction: ")
    ln_wh[:lnm_l03_lbl].pack(side: :left)
    e_nc.elements['data'].add_attribute(
      'down','true') unless e_nc.elements['data'].attributes['down']
    str_dn = e_nc.elements['data'].attributes['down'].downcase
    if str_dn == "true" || str_dn == "t" || str_dn == "yes" || str_dn == "y"
      e_nc.elements['data'].add_attribute('down','true')
    else
      e_nc.elements['data'].add_attribute('down','false')
    end
    ln_wh[:lnm_l03v] = TkVariable.new(
      e_nc.elements['data'].attributes['down'])
    ln_wh[:lnm_l03_rb1] = TkRadioButton.new(ln_wh[:lnm_l03]) do
      text _("Down")
      value "true"
      variable ln_wh[:lnm_l03v]
      pack(side: :left)
    end
    ln_wh[:lnm_l03_rb2] = TkRadioButton.new(ln_wh[:lnm_l03]) do
      text _("Up")
      value "false"
      variable ln_wh[:lnm_l03v]
      pack(side: :left)
    end

    ## (c2) Radiobutton [:lnm_l03_rb1] command.
    ln_wh[:lnm_l03_rb1].command(
      proc{p "Radiobutton LNM03_RB1"
        e_nc.elements['data'].add_attribute('down','true')})

    ## (c3) Radiobutton [:lnm_l03_rb2] command.
    ln_wh[:lnm_l03_rb2].command(
      proc{p "Radiobutton LNM03_RB2"
        e_nc.elements['data'].add_attribute('down','false')})

    ln_wh[:lnm_l04] = TkFrame.new(ln_wh[:lnm]).pack(side: :top, fill: :x)
    ln_wh[:lnm_l04_lbl] = TkLabel.new(ln_wh[:lnm_l04])
    ln_wh[:lnm_l04_lbl].text=_(
      "Use \"nil\" speed class to register timings?: ")
    ln_wh[:lnm_l04_lbl].pack(side: :left)
    e_nc.elements['data'].add_attribute(
      'nilstd','false') unless e_nc.elements['data'].attributes['nilstd']
    str_ns = e_nc.elements['data'].attributes['nilstd'].downcase
    if str_ns == "true" || str_ns == "t" || str_ns == "yes" || str_ns == "y"
      e_nc.elements['data'].add_attribute('nilstd','true')
    else
      e_nc.elements['data'].add_attribute('nilstd','false')
    end
    ln_wh[:lnm_l04v] = TkVariable.new(
      e_nc.elements['data'].attributes['nilstd'])
    ln_wh[:lnm_l04_rb1] = TkRadioButton.new(ln_wh[:lnm_l04]) do
      text _("Yes")
      value "true"
      variable ln_wh[:lnm_l04v]
      pack(side: :left)
    end
    ln_wh[:lnm_l04_rb2] = TkRadioButton.new(ln_wh[:lnm_l04]) do
      text _("No")
      value "false"
      variable ln_wh[:lnm_l04v]
      pack(side: :left)
    end

    ## (c4) Radiobutton [:lnm_l04_rb1] command.
    ln_wh[:lnm_l04_rb1].command(
      proc{p "Radiobutton LNM04_RB1"
        e_nc.elements['data'].add_attribute('nilstd','true')})

    ## (c5) Radiobutton [:lnm_l04_rb2] command.
    ln_wh[:lnm_l04_rb1].command(
      proc{p "Radiobutton LNM04_RB2"
        e_nc.elements['data'].add_attribute('nilstd','false')})

    #------------------------------#
    # Populating train times frame #
    #------------------------------#
    ln_wh[:uch] = Hash.new
    ln_wh[:uch][:deptcond] = false
    ln_wh[:uch]['new_b'] = e_nc.elements['data']
    ln_wh[:uch]['new_a'] = ln_wh[:uch]['new_e'] = nil
    # ln_wh[:uch]['xpath_lb'] =
    #   'rtss_input_data/create[@class=\'sujic_line\']/data[@name=\'' +
    #   e_nc.elements['data'].attributes['name'] +
    #   '\']/../using[@type=\'station\']'
    ln_wh[:uch]['xpath_lb'] = 'data/train_time'
    ln_wh[:uch]['xpath_root'] = e_nc
    ln_wh[:uch]['listlambda'] = lambda{|e|
      print e
      p "============================================"
      str_k = e.attributes['line'] + "/" + e.attributes['station'] +
              "::" + e.attributes['platform'] + "[" +
              e.attributes['status'] + "] -- " + e.attributes['time']
      p (e_parent.elements['data'].attributes['name'] ?
           e_parent.elements['data'].attributes['name'] : "nil") + "::" +
        (e_nc.attributes['name'] ? e_nc.attributes['name'] : "nil") +
        " list-> " + str_k
      return str_k}
    ln_wh[:uch]['lb_otext'] = _("Train times")
    ln_wh[:uch]['xref_cd'] = Hash.new
    ln_wh[:uch]['xref_cd'][nil] = _("Train time")
    ln_wh[:uch]['editor'] = lambda {|a,b,c,d,e,f,g,h|
      train_time_editor(a,b,c,d,e,f,g,h) }
    ln_wh[:uch]['deletor'] = lambda {|a,b,c,d,e,f,g,h|
      train_time_editor(a,b,c,d,e,f,g,h) }
    ln_wh[:uch]['no_cpmv'] = true
    ln_wh[:ucm] = TkFrame.new(ln_wh[:stu]).pack
    editor_select_frame(parent_win,t_ewn,ln_wh[:ucm],ln_wh[:uch],e_nc)

    l_sc = lambda {
      tt_xa = 'rtss_input_data/create[@class=\'sujic_train\']/under[@name=\''
      tt_xa += @ss_name + '\']/../data[@name=\''
      tt_xa += ln_wh[:lnm_l01_ent].value + '\']'
      if ln_wh[:lnm_l01_ent].value == "nil"
        Tk.messageBox(
          "title" => _("\"Nil\" not allowed as train name"),
          "message" => _("You cannot use \"nil\" as a name of a train.") )
        return false
      elsif v_x != "move" && ln_wh[:lnm_l01_ent].value == ""
        Tk.messageBox(
          "title" => _("Train name not specified?"),
          "message" => _("Please specify a train name") )
        return false
      elsif ( v_x == "move" ) &&
            ( ln_wh[:lnm_l01_ent].value !=
              e_nc.elements['data'].attributes['name'] )
        Tk.messageBox(
          "title" => _("Name change not allowed"),
          "message" => _("Element name should not be changed when moving"))
        return false
      elsif ( v_x != "move" ) &&
            ( ln_wh[:lnm_l01_ent].value !=
              e_nc.elements['data'].attributes['name'] &&
              @doc.elements[tt_xa] != nil )
        Tk.messageBox(
          "title" => _("Name already in use"),
          "message" => _("This train name is already used: ") + "\n" +
                       ln_wh[:lnm_l01_ent].value )
        return false
      elsif ( v_x == "new" ) && ( h_pss_lb.size < 1 ) &&
            ( _de['new_a'] == nil )
        # Bug: first element insertion destination not set
        Tk.messageBox(
          "title" => _("Internal error") + " L01812",
          "message" =>
          sprintf(_("Internal error %1$s has happened. Report this as bug."),
                  "L01812"))
        return true
      elsif ( v_x == "move" ) && ( h_pss_lb.size < 1 )
        # Bug: move selected when no element is present
        Tk.messageBox(
          "title" => _("Internal error") + " L01820",
          "message" =>
          sprintf(_("Internal error %1$s has happened. Report this as bug."),
                  "L01820"))
        return true
      elsif ( v_x == "new" || v_x == "copy" ) &&
            h_pss_lb.curselection.size > 1
        # Curselection should be 1
        Tk.messageBox(
          "title" => _("Multiple insertion destinations"),
          "message" =>
          _("Select only one insertion destination element."))
        return false
      elsif ( ( v_x == "new" && h_pss_lb.size >= 1 ) || v_x == "copy" ) &&
            h_pss_lb.curselection.size < 1
        # Curselection should be 1
        Tk.messageBox(
          "title" => _("Select an insertion destination"),
          "message" => _("Select one insertion destination element " +
                         "from the list in tab \"Where to insert\"."))
        return false
      elsif ln_wh[:uch][:deptcond]
        Tk.messageBox(
          "title" => _("Train run not complete"),
          "message" => _("The last run of this train is not complete. " +
                         "Specify arrival properly."))
        return false
      else
        ## Checks before inserting...
        # if e_nc.elements['data'].elements.to_a('coupled_trainset') != nil
        #   e_nc.elements['data'].elements.each('coupled_trainset') do |ctck|
        #     if ctck.elements.to_a('traintype') == nil ||
        #        ctck.elements.to_a('traintype').size < 1
        #       Tk.messageBox(
        #         "title" => _("Coupled trainset empty"),
        #         "message" => _("Empty coupled trainset element was found."))
        #       return false
        #     end
        #   end
        # end
        ## Checks before inserting ended...
        if v_x == "new" || v_x == "copy"
          if ln_wh[:lnm_l01_ent].value !=
             e_nc.elements['data'].attributes['name']
            str_on = e_nc.elements['data'].attributes['name']
            e_nc.elements['data'].add_attribute(
              'name', ln_wh[:lnm_l01_ent].value)
          end
          p "###### -- insert -- ###### [v_x = " + v_x + "]"
          p "###### -- insert -- ###### [h_pss_lb.size = " +
            h_pss_lb.size.to_s + "]"
          if ( h_pss_lb.size < 1 )
            elm_ins = _de['new_a']
            sw_after = true
          else
            p "###### -- insert -- ###### [cursel = " +
              h_pss_lb.curselection[0].to_s + "]"
            sw_after = false if h_pss[nil] == "before"
            sw_after = true if h_pss[nil] == "after"
            elm_ins = ar_el[h_pss_lb.curselection[0]]
            p "###### -- insert -- ######"
            puts elm_ins
            p "###### -- insert after? -- ######"
            p "---- after" if sw_after
            p "---- before" unless sw_after
            p "###### -- insert end -- ######"
          end
          if sw_after
            elm_ins.parent.insert_after(elm_ins,e_nc)
          else
            elm_ins.parent.insert_before(elm_ins,e_nc)
          end
        elsif v_x == "move" && e_cur != ar_el[h_pss_lb.curselection[0]]
          elm_cut = e_cur.parent.delete(e_cur)
          p "#### ELM_CUT is not equal to E_CUR ####" if elm_cut != e_cur
          sw_after = false if h_pss[nil] == "before"
          sw_after = true if h_pss[nil] == "after"
          elm_ins = ar_el[h_pss_lb.curselection[0]]
          elm_ins.parent.insert_after(elm_ins, elm_cut) if sw_after
        elsif v_x != "move" ## Edit only
          if ln_wh[:lnm_l01_ent].value != e_nc.attributes['name']
            str_on = e_nc.attributes['name']
            e_nc.elements['data'].add_attribute(
              'name', ln_wh[:lnm_l01_ent].value)
            # [('rtss_input_data/create[@class=\'sujic_trainset\']/using[' +
            #   '@class=\'sujic_traintype\']'),
            #  ('rtss_input_data/create[@class=\'sujic_speedclass\']' +
            #   '/data/coupled_trainset/traintype')].each do |tt_xb|
            #   @doc.elements.each(tt_xb) do |elem|
            #     elem.add_attribute('name',tt_ah['name']) if
            #       elem.attributes['name'] == str_on
            #   end
            # end
          end
        end
        return true
      end
    }

    # Close this window
    last_of_editor_window(
      parent_win, t_ewn,_de['xpath_lb'],_de['xpath_root'],_de['listlambda'],
      l_btn, lbox, ar_pel, l_sc, v_x, l_discard_edit)
    p 'train_editor end'
  end


  #==========================================#
  # Creating a window for train time editing #
  #==========================================#
  def train_time_editor(
        parent_win, lbox, ar_pel, _de, v_x, l_btn, e_cur, e_parent)
    p "train_time_editor starting..."
    p "*-*-*-*-*=*-*-*-*-*=*-*-*-*-*=*-*-*-*-*=*-*-*-*-*=*-*-*-*-*"
    print e_parent
    p ""
    print e_cur
    p ""
    print _de
    p ""
    p e_parent
    p e_cur
    p "*-*-*-*-*=*-*-*-*-*=*-*-*-*-*=*-*-*-*-*=*-*-*-*-*=*-*-*-*-*"

    if v_x == "new" || v_x == "edit"
      train_time_editor_nwed(parent_win, lbox, ar_pel,
                              _de, v_x, l_btn, e_cur, e_parent)
    elsif v_x == "delete"
      train_time_deletor(parent_win, lbox, ar_pel,
                         _de, v_x, l_btn, e_cur, e_parent)
    else
      # Bug: move selected when no element is present
      Tk.messageBox(
        "title" => _("Internal error") + " L01818",
        "message" =>
        sprintf(_("Internal error %1$s has happened. Report this as bug."),
                 "L01818"))
    end
  end

  def train_time_editor_nwed(
        parent_win, lbox, ar_pel, _de, v_x, l_btn, e_cur, e_parent)
    p "train_time_editor_nwed starting..."
    p "===========-----============"
    print _de[:deptcond]
    p ""
    print _de[:preventry]
    p ""
    p "===========-----============"

    if v_x == "edit"
      Tk.messageBox(
        "message" => _("Train time editor for editing now being built...") )
      return
    end
    
    # Create a child window
    lsn_ew = open_editor_window(
      parent_win, _("\"Sujic\"") + ":: " + _("Editing a train time"))

    # Data extracting.
    # e_parent: <create class="sujic_train" ...>...</create>
    # e_cur: <train_time> (only set when v_x == "edit", nil if v_x == "new")
    lsn_h = Hash.new
    if v_x == "edit"
      lsn_h[:e_xc] = e_cur.deep_clone
      lsn_h[:e_nc] = e_cur
    else ## v_x == "new"
      lsn_h[:e_nc] = e_parent.elements['data'].add_element(
        'train_time',{'in_service'=>'true'})
      lsn_h[:e_xc] = nil
    end
    l_discard_edit = lambda {
      lsn_h[:e_nc].parent.insert_after(
        lsn_h[:e_nc],lsn_h[:e_xc]) if lsn_h[:e_xc]
      lsn_h[:e_nc].parent.delete(lsn_h[:e_nc]) if lsn_h[:e_nc]
    }

    # Tab representation (notebook widget)
    lsn_h[:ttn] = Tk::Tile::Notebook.new(lsn_ew) do
      place('x'=>10, 'y'=>10)
    end
    lsn_h[:ttn].pack

    # Create a frame for main editing
    lsn_h[:lsm] = TkFrame.new(lsn_h[:ttn])

    # Create a frame for settings
    lsn_h[:stg] = TkFrame.new(lsn_h[:ttn])

    # Adding frames to the notebook widget
    lsn_h[:ttn].add(lsn_h[:lsm], :text => _('General'))
    lsn_h[:ttn].add(lsn_h[:stg], :text => _('Settings'))

    #-----------------------#
    # Populating main frame #
    #-----------------------#
    ## (0) Data extracting
    lsn_h[:enc_isv] =
      (lsn_h[:e_nc].attributes['in_service'] == 'true' ? true : false)
    lsn_h[:enc_l] = lsn_h[:e_nc].attributes['line']
    lsn_h[:enc_s] = lsn_h[:e_nc].attributes['station']
    lsn_h[:enc_p] = lsn_h[:e_nc].attributes['platform']
    lsn_h[:enc_r] = lsn_h[:e_nc].attributes['road']
    lsn_h[:enc_adp] = lsn_h[:e_nc].attributes['status']
    lsn_h[:enc_t] = lsn_h[:e_nc].attributes['time']

    ## (1) Line 1: Station
    lsn_h[:lsm01] = TkFrame.new(lsn_h[:lsm]).pack(
      side: :top, fill: :x, anchor: :w)
    lsn_h[:lsm01v] = TkVariable.new('orig')
    lsn_h[:lsm01_lbl] = TkLabel.new(lsn_h[:lsm01]).pack(side: :left)
    lsn_h[:lsm01_lbl].text = _('Station: ')
    sw_dn = (e_parent.elements['data'].attributes['down'] == 'true')
    sw_stn_nsel = false
    if _de[:preventry]
      e_fs = @doc.elements[
        'rtss_input_data/create[@class=\'sujic_station\']/data[@name=\'' +
        _de[:preventry].attributes['station'] + '\']']
      if e_fs && _de[:deptcond]
        e_fp = e_fs.elements[
          'physical_platforms/physical_platform[@name=\'' +
          _de[:preventry].attributes['platform'] + '\']']
        if e_fp && _de[:preventry].attributes['line'] == 'nil'
          e_fr = e_fs.elements[
            'shunting_movements/shunt[@from=\'' + e_fp.attributes['name'] +
            '\']']
          if e_fr
            str_stf = e_fs.attributes['name']
            lsn_h[:enc_s] = str_stf
            lsn_h[:e_nc].add_attribute('station', str_stf)
            sw_stn_nsel = true
          end
        elsif e_fp
          e_fe = @doc.elements[
            'rtss_input_data/create[@class=\'sujic_line\']/data[@name=\'' +
            _de[:preventry].attributes['line'] + '\']/line_element[@' +
            (sw_dn ? 'from=\'' : 'to=\'') + e_fs.attributes['name'] + '\']']
          if e_fe
            e_fr = e_fe.elements[
              'roads/road[@name=\'' + _de[:preventry].attributes['road'] +
              '\']']
            if e_fr
              n_c = e_fr.attributes[(sw_dn ? 'from' : 'to') + '_connector']
              print n_c
              p ""
              e_fcn = e_fs.elements[
                'connectors/connector[@name=\'' + n_c +
                '\']/platform[@name=\'' + e_fp.attributes['name'] + '\']']
              print e_fcn
              p ""
              if e_fcn.attributes[(e_fcn.parent.attributes['side'] == 'down' ?
                                     'down' : 'up') + '_movement'] == 'true'
                str_stf = e_fe.attributes[(sw_dn ? 'to' : 'from')]
                lsn_h[:enc_s] = str_stf
                lsn_h[:e_nc].add_attribute('station', str_stf)
                sw_stn_nsel = true
              end
            end
          end
        end
      elsif e_fs
        str_stf = e_fs.attributes['name']
        lsn_h[:enc_s] = str_stf
        lsn_h[:e_nc].add_attribute('station', str_stf)
        sw_stn_nsel = true
      end
    end
    unless sw_stn_nsel
      if lsn_h[:enc_s] && lsn_h[:enc_s] != 'nil'
        str_stf = lsn_h[:enc_s]
      else
        str_stf = _("(To be selected)")
      end
    end
    lsn_h[:lsm01_rb1] = TkRadioButton.new(lsn_h[:lsm01]) do
      text str_stf
      value "orig"
      variable lsn_h[:lsm01v]
      pack(side: :left)
    end
    lsn_h[:lsm01_rb2] = TkRadioButton.new(lsn_h[:lsm01]) do
      text _("Select another")
      value "another"
      variable lsn_h[:lsm01v]
      pack(side: :left)
    end
    if sw_stn_nsel
      lsn_h[:lsm01_rb2].state = "disabled"
    end

    ## (2) Line 2: DEP/ARR/PAS 
    lsn_h[:lsm02] = TkFrame.new(lsn_h[:lsm]).pack(side: :top, fill: :x,
                                                  anchor: :w)
    if lsn_h[:enc_adp]
      if lsn_h[:enc_adp] == "ARR"
        str_x = "ARR"
      elsif lsn_h[:enc_adp] == "DEP"
        str_x = "DEP"
      elsif lsn_h[:enc_adp] == "PAS"
        str_x = "DEP"
      else
        str_x = (_de[:deptcond] ? "ARR" : "DEP")
      end
    else
      str_x = (_de[:deptcond] ? "ARR" : "DEP")
    end
    lsn_h[:e_nc].add_attribute('status',str_x)
    lsn_h[:enc_adp] = lsn_h[:e_nc].attributes['status']

    lsn_h[:lsm02v] = TkVariable.new(str_x)
    lsn_h[:lsm02_lbl] = TkLabel.new(lsn_h[:lsm02]).pack(side: :left)
    lsn_h[:lsm02_lbl].text = _("Event status: ")
    lsn_h[:lsm02_rb1] = TkRadioButton.new(lsn_h[:lsm02]) do
      text _("Departure")
      value "DEP"
      variable lsn_h[:lsm02v]
      pack(side: :left)
    end
    lsn_h[:lsm02_rb2] = TkRadioButton.new(lsn_h[:lsm02]) do
      text _("Arrival")
      value "ARR"
      variable lsn_h[:lsm02v]
      pack(side: :left)
    end
    lsn_h[:lsm02_rb3] = TkRadioButton.new(lsn_h[:lsm02]) do
      text _("Passage")
      value "PAS"
      variable lsn_h[:lsm02v]
      pack(side: :left)
    end
    if _de[:deptcond]
      lsn_h[:lsm02_rb1].state = "disabled"
      lsn_h[:lsm02_rb2].state = "normal"
      lsn_h[:lsm02_rb3].state = "normal"
    else
      lsn_h[:lsm02_rb1].state = "normal"
      lsn_h[:lsm02_rb2].state = "disabled"
      lsn_h[:lsm02_rb3].state = "disabled"
    end

    ## (2x) Line 2x: Platform
    lsn_h[:lsm02x] = TkFrame.new(lsn_h[:lsm]).pack(
      side: :top, fill: :x, anchor: :w)
    lsn_h[:lsm02xv] = TkVariable.new('orig')
    lsn_h[:lsm02x_lbl] = TkLabel.new(lsn_h[:lsm02x]).pack(side: :left)
    lsn_h[:lsm02x_lbl].text = _("Physical platform: ")
    sw_plt_nsel = false
    if _de[:preventry] && (! _de[:deptcond])
      e_fs = @doc.elements[
        'rtss_input_data/create[@class=\'sujic_station\']/data[@name=\'' +
        _de[:preventry].attributes['station'] + '\']']
      if e_fs
        e_fp = e_fs.elements[
          'physical_platforms/physical_platform[@name=\'' +
          _de[:preventry].attributes['platform'] + '\']']
        if e_fp
          str_stf = e_fp.attributes['name']
          lsn_h[:enc_p] = str_stf
          lsn_h[:e_nc].add_attribute('platform', str_stf)
          sw_plt_nsel = true
        end
      end
    end
    unless sw_plt_nsel
      if lsn_h[:enc_p] && lsn_h[:enc_p] != 'nil'
        str_stf = lsn_h[:enc_p]
      else
        str_stf = _("(To be selected)")
        lsn_h[:enc_p] = 'nil'
        lsn_h[:e_nc].add_attribute('platform',lsn_h[:enc_p])
      end
    end
    lsn_h[:lsm02x_rb1] = TkRadioButton.new(lsn_h[:lsm02x]) do
      text str_stf
      value "orig"
      variable lsn_h[:lsm02xv]
      pack(side: :left)
    end
    lsn_h[:lsm02x_rb2] = TkRadioButton.new(lsn_h[:lsm02x]) do
      text _("Select another")
      value "another"
      variable lsn_h[:lsm02xv]
      pack(side: :left)
    end

    ## (2y) Line 2y: Line
    lsn_h[:lsm02y] = TkFrame.new(lsn_h[:lsm]).pack(
      side: :top, fill: :x, anchor: :w)
    lsn_h[:lsm02yv] = TkVariable.new('orig')
    lsn_h[:lsm02y_lbl] = TkLabel.new(lsn_h[:lsm02y]).pack(side: :left)
    lsn_h[:lsm02y_lbl].text = _("Line: ")
    if _de[:preventry] && _de[:deptcond] && sw_stn_nsel
      p "DEPTCOND && PREVENTRY"
      str_stf = lsn_h[:enc_l] = _de[:preventry].attributes['line']
      lsn_h[:e_nc].add_attribute('line',lsn_h[:enc_l])
    elsif lsn_h[:enc_l] && lsn_h[:enc_l] != 'nil'
      str_stf = lsn_h[:enc_l]
    else
      str_stf = _("(To be selected)")
      lsn_h[:enc_l] = 'nil'
      lsn_h[:e_nc].add_attribute('line',lsn_h[:enc_l])
    end
    lsn_h[:lsm02z_tvr] = TkVariable.new(
      ((lsn_h[:enc_l] == 'nil') ? (_("Shunting to: ")) : (_("Road: "))))
    lsn_h[:lsm02y_rb1] = TkRadioButton.new(lsn_h[:lsm02y]) do
      text str_stf
      value "orig"
      variable lsn_h[:lsm02yv]
      pack(side: :left)
    end
    lsn_h[:lsm02y_rb2] = TkRadioButton.new(lsn_h[:lsm02y]) do
      text _("Select another")
      value "another"
      variable lsn_h[:lsm02yv]
      pack(side: :left)
    end

    ## (2z) Line 2z: Road or shunting destination
    lsn_h[:lsm02z] = TkFrame.new(lsn_h[:lsm]).pack(
      side: :top, fill: :x, anchor: :w)
    lsn_h[:lsm02zv] = TkVariable.new('orig')
    lsn_h[:lsm02z_lbl] = TkLabel.new(lsn_h[:lsm02z]).pack(side: :left)
    lsn_h[:lsm02z_lbl].textvariable = lsn_h[:lsm02z_tvr]
    if lsn_h[:enc_r] && lsn_h[:enc_r] != 'nil'
      str_stf = lsn_h[:enc_r]
    else
      str_stf = _("(To be selected)")
      lsn_h[:enc_r] = 'nil'
      lsn_h[:e_nc].add_attribute('road',lsn_h[:enc_r])
    end
    lsn_h[:lsm02z_rb1] = TkRadioButton.new(lsn_h[:lsm02z]) do
      text str_stf
      value "orig"
      variable lsn_h[:lsm02zv]
      pack(side: :left)
    end
    lsn_h[:lsm02z_rb2] = TkRadioButton.new(lsn_h[:lsm02z]) do
      text _("Select another")
      value "another"
      variable lsn_h[:lsm02zv]
      pack(side: :left)
    end

    ## (3) Line 3: Time
    lsn_h[:lsm03] = TkFrame.new(lsn_h[:lsm]).pack(
      side: :top, fill: :x, anchor: :w)
    lsn_h[:lsm03_btn] = TkButton.new(lsn_h[:lsm03]).pack(side: :right)
    lsn_h[:lsm03_lbl] = TkLabel.new(lsn_h[:lsm03]).pack(side: :left)
    lsn_h[:lsm03_lbl].text = _("Time: ")
    lsn_h[:lsm03_ent]=TkEntry.new(lsn_h[:lsm03])
    lsn_h[:lsm03_ent].width(25)
    if lsn_h[:enc_t] && lsn_h[:enc_t].is_i_as_time?
      str_stf = lsn_h[:enc_t]
    else
      str_stf = '00:00:00'
      lsn_h[:e_nc].add_attribute('time',str_stf)
      lsn_h[:enc_t] = str_stf
    end
    lsn_h[:lsm03_ent].value = str_stf
    lsn_h[:lsm03_ent_stt] = "readonly"
    lsn_h[:lsm03_ent].state(lsn_h[:lsm03_ent_stt])
    lsn_h[:lsm03_btn].text=_("Change")
    lsn_h[:lsm03_ent].pack(anchor: :c, fill: :both)
    lsn_h[:lsm03_btn].command {
      if lsn_h[:lsm03_ent_stt] == "readonly"
        lsn_h[:lsm03_ent_stt] = "normal"
        lsn_h[:lsm03_ent].state(lsn_h[:lsm03_ent_stt])
        return
      elsif !(lsn_h[:lsm03_ent].value.is_i_as_time?)
        Tk.messageBox(
          "title" => _("Time not in proper format"),
          "message" =>
          _("The input value is not in proper format of time " +
            "(e.g. \"hh:mm:ss\")") )
        lsn_h[:lsm03_ent].value= lsn_h[:enc_t]
        lsn_h[:lsm03_ent_stt] = "readonly"
        lsn_h[:lsm03_ent].state(lsn_h[:lsm03_ent_stt])
        return
      else
        lsn_h[:e_nc].add_attribute('time',lsn_h[:lsm03_ent].value)
        lsn_h[:lsm03_ent_stt] = "readonly"
        lsn_h[:lsm03_ent].state(lsn_h[:lsm03_ent_stt])
        return
      end
    }

    ## (3x) Line 3x: In service?
    lsn_h[:lsm03x] = TkFrame.new(lsn_h[:lsm]).pack(
      side: :top, fill: :x, anchor: :w)
    lsn_h[:lsm03xv] = TkVariable.new((lsn_h[:enc_isv] ? 'true' : 'false'))
    lsn_h[:lsm03x_lbl] = TkLabel.new(lsn_h[:lsm03x]).pack(side: :left)
    lsn_h[:lsm03x_lbl].text = _("In service?")
    lsn_h[:lsm03x_rb1] = TkRadioButton.new(lsn_h[:lsm03x]) do
      text _("Yes")
      value "true"
      variable lsn_h[:lsm03xv]
      pack(side: :left)
    end
    lsn_h[:lsm03x_rb2] = TkRadioButton.new(lsn_h[:lsm03x]) do
      text _("No")
      value "false"
      variable lsn_h[:lsm03xv]
      pack(side: :left)
    end

    ## (c1) Radiobutton [:lsm01_rb2] command.
    stn_c = (lsn_h[:enc_s] ?
               @doc.elements[
               'rtss_input_data/create[@class=\'sujic_station\']/data[' +
               '@name=\'' + lsn_h[:enc_s] + '\']/..'] : nil)
    lsn_h[:lsm01_rb2].command(
      proc{
        p "Radiobutton LSM01_RB2"
        _lsm01_2h = Hash.new
        _lsm01_2h[:xpath_lb] =
          'rtss_input_data/create[@class=\'sujic_station\']'
        _lsm01_2h[:listlambda] = lambda{|e|
          e.elements['data'].attributes['name']}
        _lsm01_2h[:lb_otext] = _("Station")
        _lsm01_2h[:sw_nil] = false
        _lsm01_2h[:slambda] = lambda {|e,_org,_dx|
          if e == nil
            Tk.messageBox(
              "title" => _("Internal error") + " L03467",
              "message" =>
              sprintf(_("Internal error %1$s has happened. " +
                        "Report this as bug."), "L03467"))
            return false
          elsif e != _org
            unless _dx[:e_nc] &&
                   _dx[:e_nc].elements[
                     '../train_time[@station=\'' +
                     e.elements['data'].attributes['name'] +
                     '\' and @status=\'' + _dx[:enc_adp] + '\']'] == nil
              Tk.messageBox(
                "title" =>
                _("Error: Station and status already in this train"),
                "message" =>
                sprintf(_("Station %1$s (%2$s) is already in this train"),
                        e.elements['data'].attributes['name'],_dx[:enc_adp]))
              return false
            end
            stn_c = e
            _dx[:e_nc].add_attribute(
              'station',e.elements['data'].attributes['name'])
            _dx[:enc_s] = _dx[:e_nc].attributes['station']
            _dx[:lsm01_rb1].text(e.elements['data'].attributes['name'])
            _dx[:lsm01_rb1].select
            _dx[:e_nc].add_attribute('platform','nil')
            _dx[:lsm02x_rb1].text(_('(To be selected)'))
            _dx[:lsm02x_rb1].select
            _dx[:enc_p] = 'nil'
            _dx[:e_nc].add_attribute('line','nil')
            _dx[:lsm02y_rb1].text(_('(To be selected)'))
            _dx[:lsm02y_rb1].select
            _dx[:enc_l] = 'nil'
            _dx[:e_nc].add_attribute('road','nil')
            _dx[:lsm02y_rb1].text(_('(To be selected)'))
            _dx[:lsm02y_rb1].select
            _dx[:enc_r] = 'nil'
            return true
          else
            _dx[:lsm01_rb1].text(e.elements['data'].attributes['name'])
            return true
          end }
        pick_one_window(lsn_ew, stn_c, _lsm01_2h, lsn_h)
      })

    ## (c2-2) Radiobutton [:lsm02_rb2] (ARR) command.
    lsn_h[:lsm02_rb2].command(
      proc{
        p "Radiobutton LSM02_RB2"
        lsn_h[:enc_l] = _de[:preventry].attributes['line']
        lsn_h[:e_nc].add_attribute('line',lsn_h[:enc_l])
        lsn_h[:lsm02y_rb1].text(lsn_h[:enc_l])
        lsn_h[:lsm02y_rb1].select
        lsn_h[:lsm02y_rb2].state = "disabled"
        lsn_h[:lsm02z_tvr].set_value(
          (lsn_h[:enc_l] == 'nil' ? _("Shunting to: ") : _("Road: ")))
        lsn_h[:e_nc].add_attribute('road','nil')
        lsn_h[:lsm02z_rb1].text(_('(To be selected)'))
        lsn_h[:lsm02z_rb1].select
        lsn_h[:enc_r] = 'nil'
        lsn_h[:lsm02z_rb2].state = "disabled"
        return true
      })

    ## (c2-3) Radiobutton [:lsm02_rb3] (PAS) command.
    lsn_h[:lsm02_rb3].command(
      proc{
        p "Radiobutton LSM02_RB3"
        lsn_h[:e_nc].add_attribute('line','nil')
        lsn_h[:lsm02y_rb1].text(_('(To be selected)'))
        lsn_h[:lsm02y_rb1].select
        lsn_h[:lsm02y_rb2].state = "normal"
        lsn_h[:enc_l] = 'nil'
        lsn_h[:lsm02z_tvr].set_value(
          (lsn_h[:enc_l] == 'nil' ? _("Shunting to: ") : _("Road: ")))
        lsn_h[:e_nc].add_attribute('road','nil')
        lsn_h[:lsm02z_rb1].text(_('(To be selected)'))
        lsn_h[:lsm02z_rb1].select
        lsn_h[:enc_r] = 'nil'
        lsn_h[:lsm02z_rb2].state = "normal"
        return true
      })

    ## (c1x) Radiobutton [:lsm02x_rb2] command.
    str_xpth =
      (( lsn_h[:enc_s] && lsn_h[:enc_s] != 'nil' &&
         lsn_h[:enc_p] && lsn_h[:enc_p] != 'nil' ) ?
         ('rtss_input_data/create[@class=\'sujic_station\']' +
          '/data[@name=\'' +  lsn_h[:enc_s] + '\']/physical_platforms/' +
          'physical_platform[@name=\'' + lsn_h[:enc_p] + '\']') : nil)
    p "======~~~~~~======______======~~~~~~======"
    p str_xpth
    p "======______======~~~~~~======______======"
    plt_c = (str_xpth ? @doc.elements[str_xpth] : nil)
    lsn_h[:lsm02x_rb2].command(
      proc{
        p "Radiobutton LSM02X_RB2"
        _lsm02x_2h = Hash.new
        _lsm02x_2h[:xpath_lb] =
          'rtss_input_data/create[@class=\'sujic_station\']/data[@name=\'' +
          lsn_h[:enc_s] + '\']/physical_platforms/physical_platform'
        _lsm02x_2h[:listlambda] = lambda{|e|
          lsn_h[:enc_s] + '::' + e.attributes['name']}
        _lsm02x_2h[:lb_otext] = _("Physical platform")
        _lsm02x_2h[:sw_nil] = false
        _lsm02x_2h[:l_ok] = lambda {|e|
          return true unless sw_stn_nsel
          if _de[:preventry].attributes['line'] == 'nil'
            Tk.messageBox(
              "title" => "Under construction",
              "message" => "This part is under construction. Sorry.")
            return false
          else
            _02x2h_r = @doc.elements[
              'rtss_input_data/create[@class=\'sujic_line\']/data[' +
              '@name=\'' + _de[:preventry].attributes['line'] + '\']/' +
              'line_element[@' + (sw_dn ? 'from=\'' : 'to=\'') +
              _de[:preventry].attributes['station'] + '\']/roads/road[' +
              '@name=\'' + _de[:preventry].attributes['road'] + '\']']
            return false unless _02x2h_r
            _02x2h_c = @doc.elements[
              'rtss_input_data/create[@class=\'sujic_station\']/data[@name=' +
              '\'' + lsn_h[:enc_s] + '\']/connectors/connector[@name=\'' +
              _02x2h_r.attributes[(sw_dn ? 'to' : 'from') + '_connector'] +
              '\']']
            return false unless _02x2h_c
            _02x2h_p = _02x2h_c.elements[
              'platform[@name=\'' + e.attributes['name'] + '\']']
            return false unless _02x2h_p
            return (_02x2h_p.attributes[
                     ((_02x2h_c.attributes['side'] == 'down') ?
                        'up' : 'down') + '_movement'] == 'true')
          end}
        _lsm02x_2h[:slambda] = lambda {|e,_org,_dx|
          if e == nil
            Tk.messageBox(
              "title" => _("Internal error") + " L02486",
              "message" =>
              sprintf(_("Internal error %1$s has happened. " +
                        "Report this as bug."), "L02486"))
            return false
          elsif e != _org
            plt_c = e
            _dx[:e_nc].add_attribute('platform',e.attributes['name'])
            _dx[:enc_p] = _dx[:e_nc].attributes['platform']
            _dx[:lsm02x_rb1].text(_lsm02x_2h[:listlambda].call(e))
            _dx[:lsm02x_rb1].select
            return true
          else
            _dx[:lsm02x_rb1].text(_dx[:enc_p])
            return true
          end }
        pick_one_window(lsn_ew, plt_c, _lsm02x_2h, lsn_h, _lsm02x_2h[:l_ok])
      })

    ## (c1y) Radiobutton [:lsm02y_rb2] command.
    lin_c = (lsn_h[:enc_s] && lsn_h[:enc_s] != 'nil' &&
             lsn_h[:enc_l] && lsn_h[:enc_l] != 'nil' ?
               @doc.elements[
               'rtss_input_data/create[@class=\'sujic_line\']/data[' +
               '@name=\'' + lsn_h[:enc_l] + '\']/..'] : nil)
    lsn_h[:lsm02y_rb2].command(
      proc{
        p "Radiobutton LSM02Y_RB2"
        _lsm02y_2h = Hash.new
        _lsm02y_2h[:xpath_lb] =
          'rtss_input_data/create[@class=\'sujic_line\']/using[@name=\'' +
          lsn_h[:enc_s] + '\']/..'
        _lsm02y_2h[:listlambda] = lambda{|e|
          e.elements['data'].attributes['name']}
        _lsm02y_2h[:lb_otext] = _("Line")
        _lsm02y_2h[:sw_nil] = true
        _lsm02y_2h[:slambda] = lambda {|e,_org,_dx|
          lin_c = e
          if e
            _dx[:e_nc].add_attribute(
              'line',e.elements['data'].attributes['name'])
          else
            _dx[:e_nc].add_attribute('line','nil')
          end
          _dx[:enc_l] = _dx[:e_nc].attributes['line']
          _dx[:lsm02z_tvr].set_value(
            (_dx[:enc_l] == 'nil' ? _("Shunting to: ") : _("Road: ")))
          _dx[:lsm02y_rb1].text(e.elements['data'].attributes['name'])
          _dx[:lsm02y_rb1].select
          return true }
        pick_one_window(lsn_ew, lin_c, _lsm02y_2h, lsn_h)
      })

    ## (c1z) Radiobutton [:lsm02z_rb2] command.
    rd_c = (lsn_h[:enc_s] && lsn_h[:enc_s] != 'nil' &&
             lsn_h[:enc_l] && lsn_h[:enc_l] != 'nil' &&
             lsn_h[:enc_r] && lsn_h[:enc_r] != 'nil' ?
               @doc.elements[
               'rtss_input_data/create[@class=\'sujic_line\']/data[' +
               '@name=\'' + lsn_h[:enc_l] + '\']/line_element[@' +
               (e_parent.elements['data'].attributes['down'] == 'true' ?
                  'from' : 'to') + '=\'' + lsn_h[:enc_s] +
               '\']/roads/road[@name=\'' + lsn_h[:enc_r] + '\']'] : nil)
    lsn_h[:lsm02z_rb2].command(
      proc{
        p "Radiobutton LSM02Z_RB2"
        _lsm02z_2h = Hash.new
        _lsm02z_2h[:xpath_lb] =
          'rtss_input_data/create[@class=\'sujic_line\']/data[' +
          '@name=\'' + lsn_h[:enc_l] + '\']/line_element[@' +
          (e_parent.elements['data'].attributes['down'] == 'true' ?
             'from' : 'to') + '=\'' + lsn_h[:enc_s] + '\']/roads/road'
        _lsm02z_2h[:listlambda] = lambda{|e|e.attributes['name']}
        _lsm02z_2h[:lb_otext] = _("Road")
        _lsm02z_2h[:sw_nil] = false
        _lsm02z_2h[:l_ok] = lambda {|e|
          n_c = e.attributes[
            (e_parent.elements['data'].attributes['down'] == 'true' ?
               'from_connector' : 'to_connector')]
          return false if n_c == nil
          e_cn = @doc.elements[
            'rtss_input_data/create[@class=\'sujic_station\']/data[@name=\'' +
            lsn_h[:enc_s] + '\']/connectors/connector[@name=\'' + n_c +
            '\']/platform[@name=\'' + lsn_h[:enc_p] + '\']']
          return false if e_cn == nil
          an_c = (e_cn.parent.attributes['side'] == 'down' ? 'down' : 'up')
          return e_cn.attributes[(an_c + '_movement')] == 'true'
        }
        _lsm02z_2h[:slambda] = lambda {|e,_org,_dx|
          if e == nil
            Tk.messageBox(
              "title" => _("Internal error") + " L02635",
              "message" =>
              sprintf(_("Internal error %1$s has happened. " +
                        "Report this as bug."), "L02635"))
            return false
          elsif e != _org
            rd_c = e
            _dx[:e_nc].add_attribute('road',e.attributes['name'])
            _dx[:enc_r] = _dx[:e_nc].attributes['road']
            _dx[:lsm02z_rb1].text(e.attributes['name'])
            _dx[:lsm02z_rb1].select
            return true
          end }
        pick_one_window(lsn_ew, rd_c, _lsm02z_2h, lsn_h, _lsm02z_2h[:l_ok])
      })

    ## (c3x-1) Radiobutton [:lsm03x_rb1] command.
    lsn_h[:lsm03x_rb1].command(
      proc{
        p "Radiobutton LSM03X_RB1"
        lsn_h[:e_nc].add_attribute('in_service','true')
      })

    ## (c3x-2) Radiobutton [:lsm03x_rb2] command.
    lsn_h[:lsm03x_rb2].command(
      proc{
        p "Radiobutton LSM03X_RB2"
        lsn_h[:e_nc].add_attribute('in_service','false')
      })

    if sw_stn_nsel && ( lsn_h[:enc_adp] == "ARR" )
      lsn_h[:lsm02y_rb2].state = "disabled"
      lsn_h[:lsm02z_rb2].state = "disabled"
    end
    if sw_plt_nsel
      lsn_h[:lsm02x_rb2].state = "disabled"
    end

    #---------------------------#
    # Populating settings frame #
    #---------------------------#
    lsn_h[:ttn].itemconfigure(lsn_h[:stg],:state => "disabled")

    #----------------#
    # Check and exit #
    #----------------#
    l_sc = lambda {
      if ! _de[:deptcond]
        _de[:deptcond] = true
      elsif lsn_h[:lsm02v] == "ARR"
        _de[:deptcond] = false
      end
      _de[:preventry] = lsn_h[:e_nc]
      p "===*****===*****===*****==="
      print _de[:preventry]
      print '\n'
      p "===*****===*****===*****==="
      return true
    }
    
    # Close this window
    last_of_editor_window(
      parent_win,lsn_ew,_de['xpath_lb'],_de['xpath_root'],_de['listlambda'],
      l_btn,lbox,ar_pel,l_sc,v_x,l_discard_edit)
    p 'train_editor end'

  end

  # Trainset data editor.
  # @return None.
  # @param parent_win [TkToplevel] Parent window.
  # @param lbox [TkListbox] Listbox to be updated after editing is done.
  # @param ar_pel [xx] xx
  # @param _de [xx] xx
  # @param v_x [TkVariable] xx
  # @param l_btn [xx] xx
  # @param e_cur [REXML::Element] Current element.
  # @param e_parent [REXML::Element] Parent element of +e_cur+. In this case
  #   it will be the <create> element for the schedule set.
  def trainset_editor(parent_win, lbox, ar_pel, _de, v_x, l_btn, e_cur,
                      e_parent)
    if e_cur
      p "trainset_editor start, ts|ss = \"" +
        e_cur.elements['data'].attributes['name'] +
        "\"|\"" + e_parent.elements['data'].attributes['name'] + "\""
    else
      p "trainset_editor start, ts|ss = \"nil\"|\"" +
        e_parent.elements['data'].attributes['name'] + "\""
    end

    #---------------------------------------------#
    # Creating a child window and populate it ... #
    #---------------------------------------------#
    t_ewn = open_editor_window(
      parent_win, _("\"Sujic\"") + ":: " + _("Editing a trainset"))

    # Data extracting
    if v_x == "edit"
      e_xc = e_cur.deep_clone
      e_nc = e_cur
    elsif v_x == "copy"
      e_xc = e_cur.deep_clone
      e_xc.elements['data'].add_attribute('name','')
      e_nc = e_xc
    elsif v_x == "new"
      e_xc = REXML::Element.new('create')
      {'class'=>'sujic_trainset'}.each{|_k,_v| e_xc.add_attribute(_k,_v)}
      e_xc.add_element('under',{'name' => @ss_name,
                                'class' => 'sujic_schedule_set'})
      e_xc.add_element('using',{'name' => 'nil', 'type' =>'type',
                                'class' => 'sujic_traintype'})
      e_xc.add_element('data',{'name' => '','class' => 'x_trainset'})
      e_nc = e_xc
    else
      e_nc = e_cur
    end
    ln_wh = Hash.new

    l_discard_edit = lambda {
      if v_x == "edit"
        e_cur.parent.insert_after(e_cur,e_xc)
        e_cur.parent.delete(e_cur)
      end
    }

    # Tab representation (notebook widget)
    ln_wh[:ttn] = Tk::Tile::Notebook.new(t_ewn) do
      place('x'=>10, 'y'=>10)
    end
    ln_wh[:ttn].pack

    # Create a frame for selecting insertion position.
    # Unnecessary if v_x == "edit"
    if v_x != "edit"
      ln_wh[:pss] = TkFrame.new(ln_wh[:ttn])
      ln_wh[:ttn].add(ln_wh[:pss], :text => _('Where to insert'))
      # edit_select_window data
      h_pss = Hash.new
      h_pss_l01 = TkFrame.new(ln_wh[:pss]).pack(side: :top, fill: :x)
      h_pss = insert_position_controller_radiobuttons(h_pss_l01)
      h_pss_l02 = TkFrame.new(ln_wh[:pss]).pack(side: :top, fill: :x)
      ar_el = Array.new
      h_pss_lb_sb = TkScrollbar.new(h_pss_l02)
      h_pss_lb = TkListbox.new(
        h_pss_l02, 'height' => 10, 'width' => 50, 'selectmode' => 'single',
        'yscrollcommand' => proc{|*arg|h_pss_lb_sb.set(*arg)})
      h_pss_lb_sb.command(proc{|*arg| h_pss_lb.yview(*arg)})
      h_pss_lb_sb.pack('side' => 'right', 'fill' => 'y')
      h_pss_lb.pack('side' => 'left', 'fill' => 'both')
      populate_listbox(h_pss_lb,_de['xpath_lb'],_de['listlambda'],nil,ar_el,
                       @doc)
      if h_pss_lb.size < 1
        h_pss.each do |key,val|
          next if key == nil || key == "lframe"
          val.state("disabled")
        end
      end
    end

    # Create a frame for main editing (train name change, etc.)
    ln_wh[:lnm] = TkFrame.new(ln_wh[:ttn])

    # Create a frame for list of times
    ln_wh[:stu] = TkFrame.new(ln_wh[:ttn])

    # Adding frames to the notebook widget
    ln_wh[:ttn].add(ln_wh[:lnm], :text => _('General'))
    ln_wh[:ttn].add(ln_wh[:stu], :text => _('Times list'))

    #--------------------------#
    # Populating general frame #
    #--------------------------#
    ln_wh[:lnm_l01] = TkFrame.new(ln_wh[:lnm]).pack(side: :top, fill: :x)
    ln_wh[:lnm_l01_lbl] = TkLabel.new(ln_wh[:lnm_l01])
    ln_wh[:lnm_l01_lbl].text=_("Trainset name: ")
    ln_wh[:lnm_l01_lbl].pack(side: :left)
    ln_wh[:lnm_l01_ent] = TkEntry.new(ln_wh[:lnm_l01])
    ln_wh[:lnm_l01_ent].width(45)
    ln_wh[:lnm_l01_ent].value = e_nc.elements['data'].attributes['name']
    ln_wh[:lnm_l01_ent].pack(side: :left, fill: :x)
    ln_wh[:lnm_l02] = TkFrame.new(ln_wh[:lnm]).pack(side: :top, fill: :x)
    ln_wh[:lnm_l02_lbl] = TkLabel.new(ln_wh[:lnm_l02])
    ln_wh[:lnm_l02_lbl].text=_("Train type: ")
    ln_wh[:lnm_l02_lbl].pack(side: :left)
    ln_wh[:lnm_l02v] = TkVariable.new('orig')
    ln_wh[:lnm_l02tv] = TkVariable.new(
      e_nc.elements['using'].attributes['name'])
    ln_wh[:lnm_l02_rb1] = TkRadioButton.new(ln_wh[:lnm_l02]) do
      textvariable ln_wh[:lnm_l02tv]
      value "orig"
      variable ln_wh[:lnm_l02v]
      pack(side: :left)
    end
    ln_wh[:lnm_l02_rb2] = TkRadioButton.new(ln_wh[:lnm_l02]) do
      text _("Select another")
      value "another"
      variable ln_wh[:lnm_l02v]
      pack(side: :left)
    end

    ## (c1) Radiobutton [:lnm_l02_rb2] command.
    stn_c = (ln_wh[:lnm_l02tv] != "nil" ?
               @doc.elements[
               'rtss_input_data/create[@class=\'sujic_traintype\']/data[' +
               '@name=\'' + ln_wh[:lnm_l02tv] + '\']/..'] : nil)
    ln_wh[:lnm_l02_rb2].command(
      proc{
        p "Radiobutton LNM02_RB2"
        _lnm02_2h = Hash.new
        _lnm02_2h[:xpath_lb] =
          'rtss_input_data/create[@class=\'sujic_traintype\']'
        _lnm02_2h[:listlambda] = lambda{|e|
          e.elements['data'].attributes['name']}
        _lnm02_2h[:lb_otext] = _("Train type")
        _lnm02_2h[:sw_nil] = false
        _lnm02_2h[:slambda] = lambda {|e,_org,_dx|
          puts "======= Traintype selected ======="
          p e
          puts "======= End traintype selected ======="
          if e
            e_nc.elements['using'].add_attribute(
              'name',e.elements['data'].attributes['name'])
          else
            e_nc.elements['using'].add_attribute('name','nil')
          end
          _dx[:lnm_l02tv].set_value(
            e_nc.elements['using'].attributes['name'])
          stn_c = e
          _dx[:lnm_l02v].set_value('orig')
          return true }
        pick_one_window(t_ewn, stn_c, _lnm02_2h, ln_wh)
      })

    #---------------------------------#
    # Populating trainset times frame #
    #---------------------------------#
    ln_wh[:uch] = Hash.new
    ln_wh[:uch][:deptcond] = false
    ln_wh[:uch]['new_b'] = e_nc.elements['data']
    ln_wh[:uch]['new_a'] = ln_wh[:uch]['new_e'] = nil
    # ln_wh[:uch]['xpath_lb'] =
    #   'rtss_input_data/create[@class=\'sujic_line\']/data[@name=\'' +
    #   e_nc.elements['data'].attributes['name'] +
    #   '\']/../using[@type=\'station\']'
    ln_wh[:uch]['xpath_lb'] = 'data/train_time'
    ln_wh[:uch]['xpath_root'] = e_nc
    ln_wh[:uch]['listlambda'] = lambda{|e|
      puts e
      puts "============================================"
      str_k = e.attributes['train'] + "@" + e.attributes['station'] +
              ":=:" + e.attributes['platform'] + "[" +
              e.attributes['status'] + "]"
      puts (e_parent.elements['data'].attributes['name'] ?
              e_parent.elements['data'].attributes['name'] : "nil") + "::" +
           (e_nc.attributes['name'] ? e_nc.attributes['name'] : "nil") +
           " list-> " + str_k
      return str_k}
    ln_wh[:uch]['lb_otext'] = _("Trainset times")
    ln_wh[:uch]['xref_cd'] = Hash.new
    ln_wh[:uch]['xref_cd'][nil] = _("Trainset time")
    ln_wh[:uch]['editor'] = lambda {|a,b,c,d,e,f,g,h|
      trainset_time_editor(a,b,c,d,e,f,g,h) }
    ln_wh[:uch]['deletor'] = lambda {|a,b,c,d,e,f,g,h|
      trainset_time_editor(a,b,c,d,e,f,g,h) }
    ln_wh[:uch]['no_cpmv'] = true
    ln_wh[:ucm] = TkFrame.new(ln_wh[:stu]).pack
    editor_select_frame(parent_win,t_ewn,ln_wh[:ucm],ln_wh[:uch],e_nc)

    l_sc = lambda {
      tt_xa = 'rtss_input_data/create[@class=\'sujic_trainset\']/under'
      tt_xa += '[@name=\'' + @ss_name + '\']/../data[@name=\''
      tt_xa += ln_wh[:lnm_l01_ent].value + '\']'
      if ln_wh[:lnm_l01_ent].value == "nil"
        Tk.messageBox(
          "title" => _("\"Nil\" not allowed as trainset name"),
          "message" => _("You cannot use \"nil\" as a name of a trainset.") )
        return false
      elsif v_x != "move" && ln_wh[:lnm_l01_ent].value == ""
        Tk.messageBox(
          "title" => _("Trainset name not specified?"),
          "message" => _("Please specify a trainset name") )
        return false
      elsif ( v_x == "move" ) &&
            ( ln_wh[:lnm_l01_ent].value !=
              e_nc.elements['data'].attributes['name'] )
        Tk.messageBox(
          "title" => _("Name change not allowed"),
          "message" => _("Element name should not be changed when moving"))
        return false
      elsif ( v_x != "move" ) &&
            ( ln_wh[:lnm_l01_ent].value !=
              e_nc.elements['data'].attributes['name'] &&
              @doc.elements[tt_xa] != nil )
        Tk.messageBox(
          "title" => _("Name already in use"),
          "message" => _("This trainset name is already used: ") + "\n" +
                       ln_wh[:lnm_l01_ent].value )
        return false
      elsif ( v_x == "new" ) && ( h_pss_lb.size < 1 ) &&
            ( _de['new_a'] == nil )
        # Bug: first element insertion destination not set
        Tk.messageBox(
          "title" => _("Internal error") + " L03049",
          "message" =>
          sprintf(_("Internal error %1$s has happened. Report this as bug."),
                  "L03049"))
        return true
      elsif ( v_x == "move" ) && ( h_pss_lb.size < 1 )
        # Bug: move selected when no element is present
        Tk.messageBox(
          "title" => _("Internal error") + " L03057",
          "message" =>
          sprintf(_("Internal error %1$s has happened. Report this as bug."),
                  "L03057"))
        return true
      elsif ( v_x == "new" || v_x == "copy" ) &&
            h_pss_lb.curselection.size > 1
        # Curselection should be 1
        Tk.messageBox(
          "title" => _("Multiple insertion destinations"),
          "message" =>
          _("Select only one insertion destination element."))
        return false
      elsif ( ( v_x == "new" && h_pss_lb.size >= 1 ) || v_x == "copy" ) &&
            h_pss_lb.curselection.size < 1
        # Curselection should be 1
        Tk.messageBox(
          "title" => _("Select an insertion destination"),
          "message" => _("Select one insertion destination element " +
                         "from the list in tab \"Where to insert\"."))
        return false
      elsif ln_wh[:uch][:deptcond]
        Tk.messageBox(
          "title" => _("Train run not complete"),
          "message" => _("The last run of this trainset is not complete. " +
                         "Specify arrival properly."))
        return false
      else
        ## Checks before inserting...
        # if e_nc.elements['data'].elements.to_a('coupled_trainset') != nil
        #   e_nc.elements['data'].elements.each('coupled_trainset') do |ctck|
        #     if ctck.elements.to_a('traintype') == nil ||
        #        ctck.elements.to_a('traintype').size < 1
        #       Tk.messageBox(
        #         "title" => _("Coupled trainset empty"),
        #         "message" => _("Empty coupled trainset element was found."))
        #       return false
        #     end
        #   end
        # end
        ## Checks before inserting ended...
        if v_x == "new" || v_x == "copy"
          if ln_wh[:lnm_l01_ent].value !=
             e_nc.elements['data'].attributes['name']
            str_on = e_nc.elements['data'].attributes['name']
            e_nc.elements['data'].add_attribute(
              'name', ln_wh[:lnm_l01_ent].value)
          end
          p "###### -- insert -- ###### [v_x = " + v_x + "]"
          p "###### -- insert -- ###### [h_pss_lb.size = " +
            h_pss_lb.size.to_s + "]"
          if ( h_pss_lb.size < 1 )
            elm_ins = @doc.elements[_de['new_a']]
            sw_after = true
            p "###### -- insert -- ###### ******"
            p _de['new_a']
            p "###### -- insert -- ###### **//**"
          else
            p "###### -- insert -- ###### [cursel = " +
              h_pss_lb.curselection[0].to_s + "]"
            sw_after = false if h_pss[nil] == "before"
            sw_after = true if h_pss[nil] == "after"
            elm_ins = ar_el[h_pss_lb.curselection[0]]
            p "###### -- insert -- ######"
            puts elm_ins
            p "###### -- insert after? -- ######"
            p "---- after" if sw_after
            p "---- before" unless sw_after
            p "###### -- insert end -- ######"
          end
          if sw_after
            elm_ins.parent.insert_after(elm_ins,e_nc)
          else
            elm_ins.parent.insert_before(elm_ins,e_nc)
          end
        elsif v_x == "move" && e_cur != ar_el[h_pss_lb.curselection[0]]
          elm_cut = e_cur.parent.delete(e_cur)
          p "#### ELM_CUT is not equal to E_CUR ####" if elm_cut != e_cur
          sw_after = false if h_pss[nil] == "before"
          sw_after = true if h_pss[nil] == "after"
          elm_ins = ar_el[h_pss_lb.curselection[0]]
          elm_ins.parent.insert_after(elm_ins, elm_cut) if sw_after
        elsif v_x != "move" ## Edit only
          if ln_wh[:lnm_l01_ent].value != e_nc.attributes['name']
            str_on = e_nc.attributes['name']
            e_nc.elements['data'].add_attribute(
              'name', ln_wh[:lnm_l01_ent].value)
            # [('rtss_input_data/create[@class=\'sujic_trainset\']/using[' +
            #   '@class=\'sujic_traintype\']'),
            #  ('rtss_input_data/create[@class=\'sujic_speedclass\']' +
            #   '/data/coupled_trainset/traintype')].each do |tt_xb|
            #   @doc.elements.each(tt_xb) do |elem|
            #     elem.add_attribute('name',tt_ah['name']) if
            #       elem.attributes['name'] == str_on
            #   end
            # end
          end
        end
        return true
      end
    }

    # Close this window
    last_of_editor_window(
      parent_win, t_ewn,_de['xpath_lb'],_de['xpath_root'],_de['listlambda'],
      l_btn, lbox, ar_pel, l_sc, v_x, l_discard_edit)
    p 'trainset_editor end'
  end


  #==========================================#
  # Creating a window for train time editing #
  #==========================================#
  def trainset_time_editor(
        parent_win, lbox, ar_pel, _de, v_x, l_btn, e_cur, e_parent)
    puts "trainset_time_editor starting..."
    puts "*****Trainset_Time_Editor*****"
    puts e_parent
    puts "*****Trainset_Time_Editor: e_parent*****"
    puts e_cur
    puts "*****Trainset_Time_Editor: e_cur*****"
    puts _de
    puts "*****Trainset_Time_Editor: _de*****"

    if v_x == "new" || v_x == "edit"
      trainset_time_editor_nwed(parent_win, lbox, ar_pel,
                                _de, v_x, l_btn, e_cur, e_parent)
    elsif v_x == "delete"
      trainset_time_deletor(parent_win, lbox, ar_pel,
                            _de, v_x, l_btn, e_cur, e_parent)
    else
      # Bug: move selected when no element is present
      Tk.messageBox(
        "title" => _("Internal error") + " L03187",
        "message" =>
        sprintf(_("Internal error %1$s has happened. Report this as bug."),
                 "L03187"))
    end
  end

  def trainset_time_editor_nwed(
        parent_win, lbox, ar_pel, _de, v_x, l_btn, e_cur, e_parent)
    puts "train_time_editor_nwed starting..."
    puts "=====TrainTimeEditorNewEdt====="
    p _de[:deptcond]
    puts "=====TrainTimeEditorNewEdt deptcond====="
    p _de[:preventry]
    puts "=====TrainTimeEditorNewEdt preventry====="

    if v_x == "edit"
      Tk.messageBox(
        "message"=>_("Trainset time editor for editing now being built..."))
      return
    end
    
    # Create a child window
    lsn_ew = open_editor_window(
      parent_win, _("\"Sujic\"") + ":: " + _("Editing a trainset time"))

    # Data extracting.
    # e_parent: <create class="sujic_trainset" ...>...</create>
    # e_cur: <train_time> (only set when v_x == "edit", nil if v_x == "new")
    lsn_h = Hash.new
    if v_x == "edit"
      lsn_h[:e_xc] = e_cur.deep_clone
      lsn_h[:e_nc] = e_cur
    else ## v_x == "new"
      lsn_h[:e_nc] = e_parent.elements['data'].add_element('train_time')
      lsn_h[:e_xc] = nil
    end
    l_discard_edit = lambda {
      lsn_h[:e_nc].parent.insert_after(
        lsn_h[:e_nc],lsn_h[:e_xc]) if lsn_h[:e_xc]
      lsn_h[:e_nc].parent.delete(lsn_h[:e_nc]) if lsn_h[:e_nc]
    }

    # Tab representation (notebook widget)
    lsn_h[:ttn] = Tk::Tile::Notebook.new(lsn_ew) do
      place('x'=>10, 'y'=>10)
    end
    lsn_h[:ttn].pack

    # Create a frame for main editing
    lsn_h[:lsm] = TkFrame.new(lsn_h[:ttn])

    # Create a frame for settings
    lsn_h[:stg] = TkFrame.new(lsn_h[:ttn])

    # Adding frames to the notebook widget
    lsn_h[:ttn].add(lsn_h[:lsm], :text => _('General'))
    lsn_h[:ttn].add(lsn_h[:stg], :text => _('Settings'))

    #-----------------------#
    # Populating main frame #
    #-----------------------#
    ## (0) Data extracting
    lsn_h[:enc_tr] = lsn_h[:e_nc].attributes['train']
    lsn_h[:enc_isv] =
      (lsn_h[:e_nc].attributes['in_service'] ?
         (lsn_h[:e_nc].attributes['in_service'] == 'true' ? 'true' :
            (lsn_h[:e_nc].attributes['in_service'] == 'nil' ? 'nil' :
               'false')) : 'nil')

    ## (0x) Line 0x: Train
    ##   if _de[:deptcond] == true
    ##     Use previous train
    ##   elsif _de[:preventry]
    ##     Select from trains departing from the same physical platform
    ##   else
    ##     Select freely from all trains in the Schedule Set @ss_name
    ##   end
    lsn_h[:lsm00x] = TkFrame.new(lsn_h[:lsm]).pack(
      side: :top, fill: :x, anchor: :w)
    lsn_h[:lsm00xv] = TkVariable.new('orig')
    lsn_h[:lsm00x_lbl] = TkLabel.new(lsn_h[:lsm00x]).pack(side: :left)
    lsn_h[:lsm00x_lbl].text = _('Train: ')
    if _de[:deptcond]
      puts "TsTEne: train name: DEPTCOND"
      str_stf = lsn_h[:enc_tr] = _de[:preventry].attributes['train']
      lsn_h[:e_nc].add_attribute('train',lsn_h[:enc_tr])
    elsif lsn_h[:enc_tr] && lsn_h[:enc_tr] != 'nil'
      str_stf = lsn_h[:enc_tr]
    else
      str_stf = _("(To be selected)")
      lsn_h[:enc_tr] = 'nil'
      lsn_h[:e_nc].add_attribute('train',lsn_h[:enc_tr])
    end
    lsn_h[:lsm00x_rb1] = TkRadioButton.new(lsn_h[:lsm00x]) do
      text str_stf
      value "orig"
      variable lsn_h[:lsm00xv]
      pack(side: :left)
    end
    lsn_h[:lsm00x_rb2] = TkRadioButton.new(lsn_h[:lsm00x]) do
      text _("Select another")
      value "another"
      variable lsn_h[:lsm00xv]
      pack(side: :left)
    end

    ## (1) Line 1: Train time
    ##   if _de[:deptcond] == true
    ##     Select from arrivals in the specified train after _de[:preventry]
    ##   elsif _de[:preventry]
    ##     Select the departure of the specified train from same platform
    ##     (one choice only but still the user has to select it...)
    ##   else
    ##     Select freely from all departures in the specified train
    ##   end
    lsn_h[:lsm01] = TkFrame.new(lsn_h[:lsm]).pack(
      side: :top, fill: :x, anchor: :w)
    lsn_h[:lsm01v] = TkVariable.new('orig')
    lsn_h[:lsm01_lbl] = TkLabel.new(lsn_h[:lsm01]).pack(side: :left)
    lsn_h[:lsm01_lbl].text = _('Train time: ')
    if lsn_h[:e_nc] && lsn_h[:e_nc].attributes['train'] &&
       lsn_h[:e_nc].attributes['station'] &&
       lsn_h[:e_nc].attributes['platform'] &&
       lsn_h[:e_nc].attributes['status']
      str_stf = lsn_h[:e_nc].attributes['train'] + '@' +
                lsn_h[:e_nc].attributes['station'] + ':=:' +
                lsn_h[:e_nc].attributes['platform'] + '[' +
                lsn_h[:e_nc].attributes['status'] + ']'
    else
      str_stf = _("(To be selected)")
    end
    lsn_h[:lsm01_rb1] = TkRadioButton.new(lsn_h[:lsm01]) do
      text str_stf
      value "orig"
      variable lsn_h[:lsm01v]
      pack(side: :left)
    end
    lsn_h[:lsm01_rb2] = TkRadioButton.new(lsn_h[:lsm01]) do
      text _("Select another")
      value "another"
      variable lsn_h[:lsm01v]
      pack(side: :left)
    end

    ## (2x) Line 2x: Platform
    ##   In all cases, select from the list of logical platforms that belong
    ##   to the same physical platform.
    lsn_h[:lsm02x] = TkFrame.new(lsn_h[:lsm]).pack(
      side: :top, fill: :x, anchor: :w)
    lsn_h[:lsm02xv] = TkVariable.new('orig')
    lsn_h[:lsm02x_lbl] = TkLabel.new(lsn_h[:lsm02x]).pack(side: :left)
    lsn_h[:lsm02x_lbl].text = _("Logical platform: ")
    if lsn_h[:e_nc] && lsn_h[:e_nc].attributes['platform']
      str_stf = lsn_h[:e_nc].attributes['platform']
    else
      str_stf = _("(To be selected)")
    end
    lsn_h[:lsm02x_rb1] = TkRadioButton.new(lsn_h[:lsm02x]) do
      text str_stf
      value "orig"
      variable lsn_h[:lsm02xv]
      pack(side: :left)
    end
    lsn_h[:lsm02x_rb2] = TkRadioButton.new(lsn_h[:lsm02x]) do
      text _("Select another")
      value "another"
      variable lsn_h[:lsm02xv]
      pack(side: :left)
    end

    ## (3x) Line 3x: In service?
    lsn_h[:lsm03x] = TkFrame.new(lsn_h[:lsm]).pack(
      side: :top, fill: :x, anchor: :w)
    lsn_h[:lsm03xv] = TkVariable.new(lsn_h[:enc_isv])
    lsn_h[:lsm03x_lbl] = TkLabel.new(lsn_h[:lsm03x]).pack(side: :left)
    lsn_h[:lsm03x_lbl].text = _("In service?")
    lsn_h[:lsm03x_rb3] = TkRadioButton.new(lsn_h[:lsm03x]) do
      text _("Use train's setting")
      value "nil"
      variable lsn_h[:lsm03xv]
      pack(side: :left)
    end
    lsn_h[:lsm03x_rb1] = TkRadioButton.new(lsn_h[:lsm03x]) do
      text _("Yes")
      value "true"
      variable lsn_h[:lsm03xv]
      pack(side: :left)
    end
    lsn_h[:lsm03x_rb2] = TkRadioButton.new(lsn_h[:lsm03x]) do
      text _("No")
      value "false"
      variable lsn_h[:lsm03xv]
      pack(side: :left)
    end

    ## (c0x) Radiobutton [:lsm00x_rb2] command.
    ##   if _de[:deptcond]
    ##     Use previous train
    ##   elsif _de[:preventry]
    ##     Select from trains departing from the same physical platform
    ##   else
    ##     Select freely from all trains in the Schedule Set @ss_name
    ##   end
    lin_c = (lsn_h[:enc_tr] && lsn_h[:enc_tr] != 'nil' ?
               @doc.elements[
               'rtss_input_data/create[@class=\'sujic_train\']/data[' +
               '@name=\'' + lsn_h[:enc_tr] + '\']/..'] : nil)
    lsn_h[:lsm00x_rb2].command(
      proc{
        puts "Radiobutton LSM00X_RB2"
        _lsm00x_2h = Hash.new
        _lsm00x_2h[:xpath_lb] =
          'rtss_input_data/create[@class=\'sujic_train\']/under[@name=\'' +
          @ss_name + '\']/..'
        _lsm00x_2h[:listlambda] = lambda{|e|
          e.elements['data'].attributes['name']}
        _lsm00x_2h[:lb_otext] = _("Train")
        _lsm00x_2h[:sw_nil] = false
        _lsm00x_2h[:l_ok] = lambda {|e|
          return true unless _de[:preventry]
          _00x2h_tr = @doc.elements[
            'rtss_input_data/create[@class=\'sujic_train\']/under[@class=' +
            '\'sujic_schedule_set\' and @name=\'' + @ss_name + '\']/../data' +
            '[@name=\'' + _de[:preventry].attributes['train'] + '\']/..']
          return false unless _00x2h_tr
          if _de[:deptcond]
            return true if e == _00x2h_tr
          else
            _00x2h_s = @doc.elements[
              'rtss_input_data/create[@class=\'sujic_station\']/data[' +
              '@name=\'' + _de[:preventry].attributes['station'] + '\']/..']
            return false unless _00x2h_s
            _00x2h_lp = _00x2h_s.elements[
              'data/platforms/platform[@name=\'' +
              _de[:preventry].attributes['platform'] + '\']']
            return false unless _00x2h_lp
            _00x2h_pp = _00x2h_s.elements[
              'data/physical_platforms/physical_platform[@name=\'' +
              _00x2h_lp.attributes['physical'] + '\']']
            return false unless _00x2h_pp
            _00x2h_tr.elements['data'].get_elements('train_time').each {|ee|
              if _00x2h_s.elements['data'].attributes['name'] ==
                 ee.attributes['station'] &&
                 _00x2h_pp.attributes['name'] == ee.attributes['platform']
                return true
              end }
          end
          return false }
        _lsm00x_2h[:slambda] = lambda {|e,_org,_dx|
          lin_c = e
          if e
            _dx[:e_nc].add_attribute(
              'train',e.elements['data'].attributes['name'])
          else
            _dx[:e_nc].add_attribute('train','nil')
          end
          _dx[:enc_tr] = _dx[:e_nc].attributes['train']
          _dx[:lsm00x_rb1].text(e.elements['data'].attributes['name'])
          _dx[:lsm00x_rb1].select
          _dx[:lsm01_rb1].text = _("(To be selected)")
          return true }
        pick_one_window(lsn_ew, lin_c, _lsm00x_2h, lsn_h, _lsm00x_2h[:l_ok])
      })

    ## (c1) Radiobutton [:lsm01_rb2] command.
    ##   if _de[:deptcond] == true
    ##     Select from arrivals in the specified train after _de[:preventry]
    ##   elsif _de[:preventry]
    ##     Select the departure of the specified train from same platform
    ##     (one choice only but still the user has to select it...)
    ##   else
    ##     Select freely from all departures in the specified train
    ##   end
    if _de[:preventry]
      stn_c_lp = @doc.elements[
        'rtss_input_data/create[@class=\'sujic_train\']/data[@name=\'' +
        _de[:preventry].attributes['station'] + '\']/platforms/platform[' +
        '@name=\'' + _de[:preventry].attributes['platform'] + '\']']
      stn_c = @doc.elements[
        'rtss_input_data/create[@class=\'sujic_train\']/using[@name=\'' +
        @ss_name + '\']/../data[@name=\'' + _de[:preventry].attributes[
          'train'] + '\']/train_time[@station=\'' +
        _de[:preventry].attributes['station'] + '\' and @platform=\'' +
        (stn_c_lp ? stn_c_lp.attributes['physical'] : 'nil') +
        '\' and @status=\'' + _de[:preventry].attributes['status'] + '\']']
    else
      stn_c = nil
    end
    lsn_h[:lsm01_rb2].command(
      proc{
        p "Radiobutton LSM01_RB2"
        _lsm01h = Hash.new
        _lsm01h[:xpath_lb] =
          'rtss_input_data/create[@class=\'sujic_train\']/under[@name=\'' +
          @ss_name + '\']/../data[@name=\'' + lsn_h[:enc_tr] +
          '\']/train_time'
        _lsm01h[:listlambda] = lambda{|e|
          print e
          puts "\n=====TsTEne 01 RB2 list====="
          str_k = e.attributes['line'] + "/" + e.attributes['station'] +
                  "::" + e.attributes['platform'] + "[" +
                  e.attributes['status'] + "] -- " + e.attributes['time']
          p " list-> " + str_k
          return str_k}
        _lsm01h[:lb_otext] = _("Station")
        _lsm01h[:sw_nil] = false
        _lsm01h[:l_ok] = lambda {|e|
          if _de[:preventry]
            _01o_sd = @doc.elements[
              'rtss_input_data/create[@class=\'sujic_station\']/data[' +
              '@name=\'' + _de[:preventry].attributes['station'] + '\']']
            return false unless _01o_sd
          else
            _01o_sd = nil
          end
          if (!_de[:deptcond]) && (! _de[:preventry])
            return true if e.attributes['status'] == 'DEP'
          else
            _01o_trs = @doc.elements[
              'rtss_input_data/create[@class=\'sujic_train\']/under[@class=' +
              '\'sujic_schedule_set\' and @name=\'' + @ss_name + '\']/..' +
              '/data[@name=\'' + lsn_h[:enc_tr] + '\']/..']
            return false unless _01o_trs
            if _de[:deptcond]
              sw_dept_found = false
              _01o_lp = _01o_sd.elements[
                'platforms/platform[@name=\'' + _de[:preventry].attributes[
                  'platform'] + '\']']
              return false unless _01o_lp
              _01o_pp = _01o_sd.elements[
                'physical_platforms/physical_platform[@name=\'' +
                _01o_lp.attributes['physical'] + '\']']
              return false unless _01o_pp
              _01o_trs.elements['data'].get_elements('train_time').each {|ee|
                return false if
                  (! sw_dept_found) &&
                  ee.attributes['station'] == e.attributes['station'] &&
                  ee.attributes['platform'] == e.attributes['platform'] &&
                  ee.attributes['status'] == e.attributes['status']
                sw_dept_found = true if
                  (! sw_dept_found) &&
                  ee.attributes['station'] == _01o_sd.attributes['name'] &&
                  ee.attributes['platform'] == _01o_pp.attributes['name'] &&
                  ee.attributes['status'] ==
                  _de[:preventry].attributes['status']
                return true if
                  sw_dept_found &&
                  ee.attributes['station'] == e.attributes['station'] &&
                  ee.attributes['platform'] == e.attributes['platform'] &&
                  ee.attributes['status'] == e.attributes['status'] &&
                  e.attributes['status'] == 'ARR'}
            else
              _01o_lp = _01o_sd.elements[
                'platforms/platform[@name=\'' + _de[:preventry].attributes[
                  'platform'] + '\']']
              return false unless _01o_lp
              _01o_pp = _01o_sd.elements[
                'physical_platforms/physical_platform[@name=\'' +
                _01o_lp.attributes['physical'] + '\']']
              return false unless _01o_pp
              _01o_trs.elements['data'].get_elements('train_time').each {|ee|
                return true if
                  lsn_h[:e_nc] && e.attributes['station'] ==
                                  _de[:preventry].attributes['station'] &&
                  e.attributes['platform'] == _01o_pp.attributes['name'] &&
                  e.attributes['status'] == 'DEP'}
            end
            return false
          end }
        _lsm01h[:slambda] = lambda {|e,_org,_dx|
          if e == nil
            Tk.messageBox(
              "title" => _("Internal error") + " L03473",
              "message" =>
              sprintf(_("Internal error %1$s has happened. " +
                        "Report this as bug."), "L03473"))
            return false
          end
          if e != stn_c
            stn_c = e
            _dx[:e_nc].add_attribute('station',e.attributes['station'])
            _dx[:e_nc].add_attribute('platform','nil')
            _dx[:e_nc].add_attribute('status',e.attributes['status'])
          end
          _01o_trs = @doc.elements[
            'rtss_input_data/create[@class=\'sujic_train\']/under[@class=' +
            '\'sujic_schedule_set\' and @name=\'' + @ss_name + '\']/..' +
            '/data[@name=\'' + lsn_h[:enc_tr] + '\']']
          if _01o_trs
            _01o_trst = _01o_trs.elements[
              'train_time[@station=\'' + lsn_h[:e_nc].attributes['station'] +
              '\' and @status=\'' + lsn_h[:e_nc].attributes['status'] + '\']']
            if _01o_trst
              _01o_strpp = _01o_trst.attributes['platform']
            else
              _01o_strpp = "(nil)"
            end
          else
            _01o_strpp = "(nil)"
          end
          _dx[:lsm01_rb1].text =
            lsn_h[:e_nc].attributes['train'] + '@' +
            lsn_h[:e_nc].attributes['station'] + '::' + _01o_strpp + '[' +
            lsn_h[:e_nc].attributes['status'] + ']'
          _dx[:lsm01_rb1].select
          return true
        }
        pick_one_window(lsn_ew, stn_c, _lsm01h, lsn_h, _lsm01h[:l_ok])
      })

    ## (c2x) Radiobutton [:lsm02x_rb2] command.
    ##   In all cases, select from the list of logical platforms that belong
    ##   to the same physical platform.
    str_xpth = 
      (( lsn_h[:e_nc].attributes['station'] &&
         lsn_h[:e_nc].attributes['station'] != 'nil' &&
         lsn_h[:e_nc].attributes['platform'] &&
         lsn_h[:e_nc].attributes['platform'] != 'nil' ) ?
         ('rtss_input_data/create[@class=\'sujic_station\']/data[@name=\'' +
          lsn_h[:e_nc].attributes['station'] + '\']/platforms/platform[' +
          '@name=\'' + lsn_h[:e_nc].attributes['platform'] + '\']') : nil)
    p "======~~~~~~======______======~~~~~~======"
    p str_xpth
    p "======______======~~~~~~======______======"
    plt_c = (str_xpth ? @doc.elements[str_xpth] : nil)
    lsn_h[:lsm02x_rb2].command(
      proc{
        p "Radiobutton LSM02X_RB2"
        _lsm02x_2h = Hash.new
        _lsm02x_2h[:xpath_lb] =
          'rtss_input_data/create[@class=\'sujic_station\']/data[@name=\'' +
          lsn_h[:e_nc].attributes['station'] + '\']/platforms/platform'
        _lsm02x_2h[:listlambda] = lambda{|e|
          lsn_h[:e_nc].attributes['station'] + ':=:' +
            e.attributes['name']}
        _lsm02x_2h[:lb_otext] = _("Logical platform")
        _lsm02x_2h[:sw_nil] = false
        _lsm02x_2h[:l_ok] = lambda {|e|
          _02x_trs = @doc.elements[
            'rtss_input_data/create[@class=\'sujic_train\']/under[@class=' +
            '\'sujic_schedule_set\' and @name=\'' + @ss_name + '\']/..' +
            '/data[@name=\'' + lsn_h[:enc_tr] + '\']']
          return false unless _02x_trs
          _02x_trst = _02x_trs.elements[
            'train_time[@station=\'' + lsn_h[:e_nc].attributes['station'] +
            '\' and @status=\'' + lsn_h[:e_nc].attributes['status'] + '\']']
          return false unless _02x_trst
          return (e.attributes['physical'] ==
                  _02x_trst.attributes['platform'])
        }
        _lsm02x_2h[:slambda] = lambda {|e,_org,_dx|
          if e == nil
            Tk.messageBox(
              "title" => _("Internal error") + " L02486",
              "message" =>
              sprintf(_("Internal error %1$s has happened. " +
                        "Report this as bug."), "L02486"))
            return false
          elsif e != _org
            plt_c = e
            _dx[:e_nc].add_attribute('platform',e.attributes['name'])
            _dx[:enc_p] = _dx[:e_nc].attributes['platform']
            _dx[:lsm02x_rb1].text(_lsm02x_2h[:listlambda].call(e))
            _dx[:lsm02x_rb1].select
            return true
          else
            _dx[:lsm02x_rb1].text(_dx[:enc_p])
            return true
          end }
        pick_one_window(lsn_ew, plt_c, _lsm02x_2h, lsn_h, _lsm02x_2h[:l_ok])
      })

    if _de[:deptcond]
      lsn_h[:lsm00x_rb2].state = "disabled"
    end

    #---------------------------#
    # Populating settings frame #
    #---------------------------#
    lsn_h[:ttn].itemconfigure(lsn_h[:stg],:state => "disabled")

    #----------------#
    # Check and exit #
    #----------------#
    l_sc = lambda {
      if ! _de[:deptcond]
        _de[:deptcond] = true
      elsif lsn_h[:e_nc].attributes['status'] == 'ARR'
        _de[:deptcond] = false
      end
      _de[:preventry] = lsn_h[:e_nc]
      p "===*****===*****===*****==="
      print _de[:preventry]
      print '\n'
      p "===*****===*****===*****==="
      return true
    }
    
    # Close this window
    last_of_editor_window(
      parent_win,lsn_ew,_de['xpath_lb'],_de['xpath_root'],_de['listlambda'],
      l_btn,lbox,ar_pel,l_sc,v_x,l_discard_edit)
    puts 'trainset_time_editor_nwed end'

  end

  # Passenger activity data editor.
  # @return None.
  # @param parent_win [TkToplevel] Parent window.
  # @param lbox [TkListbox] Listbox to be updated after editing is done.
  # @param ar_pel [xx] xx
  # @param _de [xx] xx
  # @param v_x [TkVariable] xx
  # @param l_btn [xx] xx
  # @param e_cur [REXML::Element] Current element.
  # @param e_parent [REXML::Element] Parent element of +e_cur+. In this case
  #   it will be the <create> element for the schedule set.
  def passenger_activity_editor(
        parent_win, lbox, ar_pel, _de, v_x, l_btn, e_cur, e_parent)
    p "passenger_activity_editor start"

    #---------------------------------------------#
    # Creating a child window and populate it ... #
    #---------------------------------------------#
    t_ewn = open_editor_window(
      parent_win,
      _("\"Sujic\"") + ":: " + _("Editing a passenger activity model"))

    # Data extracting
    if v_x == "edit"
      e_xc = e_cur.deep_clone
      e_nc = e_cur
    elsif v_x == "copy"
      e_xc = e_cur.deep_clone
      e_xc.elements['data'].add_attribute('name','')
      e_nc = e_xc
    elsif v_x == "new"
      e_xc = REXML::Element.new('create')
      {'class'=>'sujic_passenger_activity'}.each{|_k,_v|
        e_xc.add_attribute(_k,_v)}
      e_xc.add_element('data',{'name'=>'','class'=>'x_passenger_activity_f1'})
      e_nc = e_xc
    else
      e_nc = e_cur
    end
    ln_wh = Hash.new
    ln_wh[:ptn_rv] = TkVariable.new(e_nc.elements['data'].attributes['class'])

    l_discard_edit = lambda {
      if v_x == "edit"
        e_cur.parent.insert_after(e_cur,e_xc)
        e_cur.parent.delete(e_cur)
      end
    }

    # Tab representation (notebook widget)
    ln_wh[:ttn] = Tk::Tile::Notebook.new(t_ewn) do
      place('x'=>10, 'y'=>10)
    end
    ln_wh[:ttn].pack

    # Create a frame for selecting insertion position.
    # Unnecessary if v_x == "edit"
    if v_x != "edit"
      ln_wh[:pss] = TkFrame.new(ln_wh[:ttn])
      ln_wh[:ttn].add(ln_wh[:pss], :text => _('Where to insert'))
      # edit_select_window data
      h_pss = Hash.new
      h_pss_l01 = TkFrame.new(ln_wh[:pss]).pack(side: :top, fill: :x)
      h_pss = insert_position_controller_radiobuttons(h_pss_l01)
      h_pss_l02 = TkFrame.new(ln_wh[:pss]).pack(side: :top, fill: :x)
      ar_el = Array.new
      h_pss_lb_sb = TkScrollbar.new(h_pss_l02)
      h_pss_lb = TkListbox.new(
        h_pss_l02, 'height' => 10, 'width' => 50, 'selectmode' => 'single',
        'yscrollcommand' => proc{|*arg|h_pss_lb_sb.set(*arg)})
      h_pss_lb_sb.command(proc{|*arg| h_pss_lb.yview(*arg)})
      h_pss_lb_sb.pack('side' => 'right', 'fill' => 'y')
      h_pss_lb.pack('side' => 'left', 'fill' => 'both')
      populate_listbox(h_pss_lb,_de['xpath_lb'],_de['listlambda'],nil,ar_el,
                       @doc)
      if h_pss_lb.size < 1
        h_pss.each do |key,val|
          next if key == nil || key == "lframe"
          val.state("disabled")
        end
      end
    end

    # Create a frame for main editing (train name change, etc.)
    ln_wh[:lnm] = TkFrame.new(ln_wh[:ttn])

    # Create a frame for list of models
    ln_wh[:ptn] = TkFrame.new(ln_wh[:ttn])

    # Adding frames to the notebook widget
    ln_wh[:ttn].add(ln_wh[:lnm], :text => _('General'))
    ln_wh[:ttn].add(ln_wh[:ptn], :text => _('Model class'))

    #--------------------------#
    # Populating general frame #
    #--------------------------#
    ln_wh[:lnm_l01] = TkFrame.new(ln_wh[:lnm]).pack(side: :top, fill: :x)
    ln_wh[:lnm_l01_lbl] = TkLabel.new(ln_wh[:lnm_l01])
    ln_wh[:lnm_l01_lbl].text=_("Model name: ")
    ln_wh[:lnm_l01_lbl].pack(side: :left)
    ln_wh[:lnm_l01_ent] = TkEntry.new(ln_wh[:lnm_l01])
    ln_wh[:lnm_l01_ent].width(45)
    ln_wh[:lnm_l01_ent].value = e_nc.elements['data'].attributes['name']
    ln_wh[:lnm_l01_ent].pack(side: :left, fill: :x)

    #------------------------------#
    # Populating model class frame #
    #------------------------------#
    ln_wh[:ptn_ttn] = Tk::Tile::Notebook.new(ln_wh[:ptn]) do
      pack(fill: :x)
    end
    ln_wh[:ptn_ttn_p1] = TkFrame.new(ln_wh[:ptn_ttn])
    ln_wh[:ptn_ttn_p2] = TkFrame.new(ln_wh[:ptn_ttn])
    ln_wh[:ptn_ttn].add(ln_wh[:ptn_ttn_p1], :text => '1')
    ln_wh[:ptn_ttn].add(ln_wh[:ptn_ttn_p2], :text => '2')
    ln_wh[:ptn_ttn].itemconfigure(ln_wh[:ptn_ttn_p2], :state => 'disabled')
    ln_wh[:ptn_l_f1] = TkFrame.new(ln_wh[:ptn_ttn_p1]).pack(
      side: :top, fill: :x)
    ln_wh[:ptn_l_f1_rb] = TkRadioButton.new(ln_wh[:ptn_l_f1]) do
      text _("F1")
      value "x_passenger_activity_f1"
      variable ln_wh[:ptn_rv]
      command(
        proc{
          puts "PasAct: F1"
          e_nc.elements['data'].add_attribute(
            'class','x_passenger_activity_f1')
        })
      pack(side: :left)
    end
    ln_wh[:ptn_l_f2] = TkFrame.new(ln_wh[:ptn_ttn_p1]).pack(
      side: :top, fill: :x)
    ln_wh[:ptn_l_f2_rb] = TkRadioButton.new(ln_wh[:ptn_l_f2]) do
      text _("F2")
      value "x_passenger_activity_f2"
      variable ln_wh[:ptn_rv]
      command(
        proc{
          puts "PasAct: F2"
          e_nc.elements['data'].add_attribute(
            'class','x_passenger_activity_f2')
        })
      pack(side: :left)
    end

    l_sc = lambda {
      tt_xa = 'rtss_input_data/create[@class=\'sujic_passenger_activity\']'
      tt_xa += '/data[@name=\'' + ln_wh[:lnm_l01_ent].value + '\']'
      if ln_wh[:lnm_l01_ent].value == "nil"
        Tk.messageBox(
          "title" =>
          _("\"Nil\" not allowed as passenger activity model name"),
          "message" =>
          _("You cannot use \"nil\" as a name of a passenger activity model."))
        return false
      elsif v_x != "move" && ln_wh[:lnm_l01_ent].value == ""
        Tk.messageBox(
          "title" => _("Passenger activity model name not specified?"),
          "message" => _("Please specify a passenger activity model name."))
        return false
      elsif ( v_x == "move" ) &&
            ( ln_wh[:lnm_l01_ent].value !=
              e_nc.elements['data'].attributes['name'] )
        Tk.messageBox(
          "title" => _("Name change not allowed"),
          "message" => _("Element name should not be changed when moving"))
        return false
      elsif ( v_x != "move" ) &&
            ( ln_wh[:lnm_l01_ent].value !=
              e_nc.elements['data'].attributes['name'] &&
              @doc.elements[tt_xa] != nil )
        Tk.messageBox(
          "title" => _("Name already in use"),
          "message" =>
          _("This passenger activity model name is already used: ") + "\n" +
          ln_wh[:lnm_l01_ent].value )
        return false
      elsif ( v_x == "new" ) && ( h_pss_lb.size < 1 ) &&
            ( _de['new_b'] == nil )
        # Bug: first element insertion destination not set
        Tk.messageBox(
          "title" => _("Internal error") + " L03697",
          "message" =>
          sprintf(_("Internal error %1$s has happened. Report this as bug."),
                  "L03697"))
        return true
      elsif ( v_x == "move" ) && ( h_pss_lb.size < 1 )
        # Bug: move selected when no element is present
        Tk.messageBox(
          "title" => _("Internal error") + " L03705",
          "message" =>
          sprintf(_("Internal error %1$s has happened. Report this as bug."),
                  "L03705"))
        return true
      elsif ( v_x == "new" || v_x == "copy" ) &&
            h_pss_lb.curselection.size > 1
        # Curselection should be 1
        Tk.messageBox(
          "title" => _("Multiple insertion destinations"),
          "message" =>
          _("Select only one insertion destination element."))
        return false
      elsif ( ( v_x == "new" && h_pss_lb.size >= 1 ) || v_x == "copy" ) &&
            h_pss_lb.curselection.size < 1
        # Curselection should be 1
        Tk.messageBox(
          "title" => _("Select an insertion destination"),
          "message" => _("Select one insertion destination element " +
                         "from the list in tab \"Where to insert\"."))
        return false
      else
        ## Checks before inserting...
        # if e_nc.elements['data'].elements.to_a('coupled_trainset') != nil
        #   e_nc.elements['data'].elements.each('coupled_trainset') do |ctck|
        #     if ctck.elements.to_a('traintype') == nil ||
        #        ctck.elements.to_a('traintype').size < 1
        #       Tk.messageBox(
        #         "title" => _("Coupled trainset empty"),
        #         "message" => _("Empty coupled trainset element was found."))
        #       return false
        #     end
        #   end
        # end
        ## Checks before inserting ended...
        if v_x == "new" || v_x == "copy"
          if ln_wh[:lnm_l01_ent].value !=
             e_nc.elements['data'].attributes['name']
            str_on = e_nc.elements['data'].attributes['name']
            e_nc.elements['data'].add_attribute(
              'name', ln_wh[:lnm_l01_ent].value)
          end
          p "###### -- insert -- ###### [v_x = " + v_x + "]"
          p "###### -- insert -- ###### [h_pss_lb.size = " +
            h_pss_lb.size.to_s + "]"
          if ( h_pss_lb.size < 1 )
            elm_ins = _de['new_b']
            sw_after = false
          else
            p "###### -- insert -- ###### [cursel = " +
              h_pss_lb.curselection[0].to_s + "]"
            sw_after = false if h_pss[nil] == "before"
            sw_after = true if h_pss[nil] == "after"
            elm_ins = ar_el[h_pss_lb.curselection[0]]
            p "###### -- insert -- ######"
            puts elm_ins
            p "###### -- insert after? -- ######"
            p "---- after" if sw_after
            p "---- before" unless sw_after
            p "###### -- insert end -- ######"
          end
          if sw_after
            elm_ins.parent.insert_after(elm_ins,e_nc)
          else
            elm_ins.parent.insert_before(elm_ins,e_nc)
          end
        elsif v_x == "move" && e_cur != ar_el[h_pss_lb.curselection[0]]
          elm_cut = e_cur.parent.delete(e_cur)
          p "#### ELM_CUT is not equal to E_CUR ####" if elm_cut != e_cur
          sw_after = false if h_pss[nil] == "before"
          sw_after = true if h_pss[nil] == "after"
          elm_ins = ar_el[h_pss_lb.curselection[0]]
          elm_ins.parent.insert_after(elm_ins, elm_cut) if sw_after
        elsif v_x != "move" ## Edit only
          if ln_wh[:lnm_l01_ent].value != e_nc.attributes['name']
            str_on = e_nc.attributes['name']
            e_nc.elements['data'].add_attribute(
              'name', ln_wh[:lnm_l01_ent].value)
            # [('rtss_input_data/create[@class=\'sujic_trainset\']/using[' +
            #   '@class=\'sujic_traintype\']'),
            #  ('rtss_input_data/create[@class=\'sujic_speedclass\']' +
            #   '/data/coupled_trainset/traintype')].each do |tt_xb|
            #   @doc.elements.each(tt_xb) do |elem|
            #     elem.add_attribute('name',tt_ah['name']) if
            #       elem.attributes['name'] == str_on
            #   end
            # end
          end
        end
        return true
      end
    }

    # Close this window
    last_of_editor_window(
      parent_win, t_ewn,_de['xpath_lb'],_de['xpath_root'],_de['listlambda'],
      l_btn, lbox, ar_pel, l_sc, v_x, l_discard_edit)
    p 'passenger_activity_editor end'
  end

  # Passenger appearance model editor.
  # @return None.
  # @param parent_win [TkToplevel] Parent window.
  # @param lbox [TkListbox] Listbox to be updated after editing is done.
  # @param ar_pel [xx] xx
  # @param _de [xx] xx
  # @param v_x [TkVariable] xx
  # @param l_btn [xx] xx
  # @param e_cur [REXML::Element] Current element.
  # @param e_parent [REXML::Element] Parent element of +e_cur+. In this case
  #   it will be the <create> element for the schedule set.
  def passenger_appearance_model_editor(
        parent_win, lbox, ar_pel, _de, v_x, l_btn, e_cur, e_parent)
    p "passenger_appearance_model_editor start"

    #---------------------------------------------#
    # Creating a child window and populate it ... #
    #---------------------------------------------#
    t_ewn = open_editor_window(
      parent_win,
      _("\"Sujic\"") + ":: " + _("Editing a passenger appearance model"))

    # Data extracting
    if v_x == "edit"
      e_xc = e_cur.deep_clone
      e_nc = e_cur
    elsif v_x == "copy"
      e_xc = e_cur.deep_clone
      e_xc.elements['data'].add_attribute('name','')
      e_nc = e_xc
    elsif v_x == "new"
      e_xc = REXML::Element.new('create')
      {'class'=>'sujic_passenger_appearance_model'}.each{|_k,_v|
        e_xc.add_attribute(_k,_v)}
      e_xc.add_element(
        'under',{'name'=>'nil','class'=>'sujic_passenger_activity'})
      e_xc.add_element(
        'data',{'name'=>'','class'=>'f1_constant_rate_inflow',
                'cycletime'=>'1:00:00'})
      e_nc = e_xc
    else
      e_nc = e_cur
    end
    ln_wh = Hash.new
    ln_wh[:ptn_rv] = TkVariable.new(e_nc.elements['data'].attributes['class'])

    l_discard_edit = lambda {
      if v_x == "edit"
        e_cur.parent.insert_after(e_cur,e_xc)
        e_cur.parent.delete(e_cur)
      end
    }

    # Tab representation (notebook widget)
    ln_wh[:ttn] = Tk::Tile::Notebook.new(t_ewn) do
      place('x'=>10, 'y'=>10)
    end
    ln_wh[:ttn].pack

    # Create a frame for selecting insertion position.
    # Unnecessary if v_x == "edit"
    if v_x != "edit"
      ln_wh[:pss] = TkFrame.new(ln_wh[:ttn])
      ln_wh[:ttn].add(ln_wh[:pss], :text => _('Where to insert'))
      # edit_select_window data
      h_pss = Hash.new
      h_pss_l01 = TkFrame.new(ln_wh[:pss]).pack(side: :top, fill: :x)
      h_pss = insert_position_controller_radiobuttons(h_pss_l01)
      h_pss_l02 = TkFrame.new(ln_wh[:pss]).pack(side: :top, fill: :x)
      ar_el = Array.new
      h_pss_lb_sb = TkScrollbar.new(h_pss_l02)
      h_pss_lb = TkListbox.new(
        h_pss_l02, 'height' => 10, 'width' => 50, 'selectmode' => 'single',
        'yscrollcommand' => proc{|*arg|h_pss_lb_sb.set(*arg)})
      h_pss_lb_sb.command(proc{|*arg| h_pss_lb.yview(*arg)})
      h_pss_lb_sb.pack('side' => 'right', 'fill' => 'y')
      h_pss_lb.pack('side' => 'left', 'fill' => 'both')
      populate_listbox(h_pss_lb,_de['xpath_lb'],_de['listlambda'],nil,ar_el,
                       @doc)
      if h_pss_lb.size < 1
        h_pss.each do |key,val|
          next if key == nil || key == "lframe"
          val.state("disabled")
        end
      end
    end

    # Create a frame for main editing (train name change, etc.)
    ln_wh[:lnm] = TkFrame.new(ln_wh[:ttn])

    # Create a frame for list of models
    ln_wh[:ptn] = TkFrame.new(ln_wh[:ttn])

    # Create a frame for list of settings
    ln_wh[:stn] = TkFrame.new(ln_wh[:ttn])

    # Adding frames to the notebook widget
    ln_wh[:ttn].add(ln_wh[:lnm], :text => _('General'))
    ln_wh[:ttn].add(ln_wh[:ptn], :text => _('Model class'))
    ln_wh[:ttn].add(ln_wh[:stn], :text => _('Model settings'))

    #--------------------------#
    # Populating general frame #
    #--------------------------#
    ln_wh[:lnm_l01] = TkFrame.new(ln_wh[:lnm]).pack(side: :top, fill: :x)
    ln_wh[:lnm_l01_lbl] = TkLabel.new(ln_wh[:lnm_l01])
    ln_wh[:lnm_l01_lbl].text=_("Model name: ")
    ln_wh[:lnm_l01_lbl].pack(side: :left)
    ln_wh[:lnm_l01_ent] = TkEntry.new(ln_wh[:lnm_l01])
    ln_wh[:lnm_l01_ent].width(45)
    ln_wh[:lnm_l01_ent].value = e_nc.elements['data'].attributes['name']
    ln_wh[:lnm_l01_ent].pack(side: :left, fill: :x)
    ln_wh[:lnm_l02] = TkFrame.new(ln_wh[:lnm]).pack(side: :top, fill: :x)
    ln_wh[:lnm_l02_lbl] = TkLabel.new(ln_wh[:lnm_l02])
    ln_wh[:lnm_l02_lbl].text=_("Passenger activity: ")
    ln_wh[:lnm_l02_lbl].pack(side: :left)
    ln_wh[:lnm_l02v] = TkVariable.new('orig')
    p "*****XXXXX*****"
    p e_nc.elements['under'].attributes['name']
    p "*****XXXXX*****"
    ln_wh[:lnm_l02tv] = TkVariable.new(
      e_nc.elements['under'].attributes['name'])
    if ln_wh[:lnm_l02tv].value == 'nil'
      ln_wh[:lnm_l02tv].set_value(_("(To be selected)"))
    end
    ln_wh[:lnm_l02_rb1] = TkRadioButton.new(ln_wh[:lnm_l02]) do
      textvariable ln_wh[:lnm_l02tv]
      value "orig"
      variable ln_wh[:lnm_l02v]
      pack(side: :left)
    end
    ln_wh[:lnm_l02_rb2] = TkRadioButton.new(ln_wh[:lnm_l02]) do
      text _("Select another")
      value "another"
      variable ln_wh[:lnm_l02v]
      pack(side: :left)
    end

    #------------------------------#
    # Populating model class frame #
    #------------------------------#
    ln_wh[:ptn_ttn] = Tk::Tile::Notebook.new(ln_wh[:ptn]) do
      pack(fill: :x)
    end
    ln_wh[:ptn_ttn_p1] = TkFrame.new(ln_wh[:ptn_ttn])
    ln_wh[:ptn_ttn_p2] = TkFrame.new(ln_wh[:ptn_ttn])
    ln_wh[:ptn_ttn].add(ln_wh[:ptn_ttn_p1], :text => '1')
    ln_wh[:ptn_ttn].add(ln_wh[:ptn_ttn_p2], :text => '2')
    ln_wh[:ptn_ttn].itemconfigure(ln_wh[:ptn_ttn_p2], :state => 'disabled')
    ln_wh[:ptn_l_f1] = TkFrame.new(ln_wh[:ptn_ttn_p1]).pack(
      side: :top, fill: :x)
    ln_wh[:ptn_l_f1_rb] = TkRadioButton.new(ln_wh[:ptn_l_f1]) do
      text _("F1 constant rate inflow")
      value "f1_constant_rate_inflow"
      variable ln_wh[:ptn_rv]
      pack(side: :left)
    end
    ln_wh[:ptn_l_f2] = TkFrame.new(ln_wh[:ptn_ttn_p1]).pack(
      side: :top, fill: :x)
    ln_wh[:ptn_l_f2_rb] = TkRadioButton.new(ln_wh[:ptn_l_f2]) do
      text _("F2 constant rate outflow")
      value "f2"
      variable ln_wh[:ptn_rv]
      pack(side: :left)
    end

    #---------------------------------#
    # Populating model settings frame #
    #---------------------------------#
    ln_wh[:stn_df] = nil

    ## (X1) Setting up settings frame for "f1_constant_rate_inflow"
    ln_wh[:stn_1cri] = lambda{
      if ln_wh[:stn_df]
        ln_wh[:stn_df].destroy
      end
      ln_wh[:stn_df] = TkFrame.new(ln_wh[:stn]).pack(fill: :x)
      ln_wh[:s_ct] = TkFrame.new(ln_wh[:stn_df]).pack(fill: :both)
      ln_wh[:s_ct_lbl] = TkLabel.new(ln_wh[:s_ct])
      ln_wh[:s_ct_lbl].text=_("Cycle time for OD: ")
      ln_wh[:s_ct_lbl].pack(side: :left)
      ln_wh[:s_ct_ent]=TkEntry.new(ln_wh[:s_ct])
      ln_wh[:s_ct_ent].width(25)
      ln_wh[:s_ct_ent].value= e_nc.elements['data'].attributes['cycletime']
      ln_wh[:s_ct_ent_stt] = "readonly"
      ln_wh[:s_ct_ent].state(ln_wh[:s_ct_ent_stt])
      ln_wh[:s_ct_btn]=TkButton.new(ln_wh[:s_ct])
      ln_wh[:s_ct_btn].text=_("Change")
      ln_wh[:s_ct_btn].pack(side: :right)
      ln_wh[:s_ct_ent].pack(anchor: :c, fill: :both)
      ln_wh[:s_ct_btn].command {
        if ln_wh[:s_ct_ent_stt] == "readonly"
          ln_wh[:s_ct_ent_stt] = "normal"
          ln_wh[:s_ct_ent].state(ln_wh[:s_ct_ent_stt])
          return
        elsif !(ln_wh[:s_ct_ent].value.is_i_as_time?)
          Tk.messageBox(
            "title" => _("Time not in proper format"),
            "message" =>
            _("The input value is not in proper format of time " +
              "(e.g. \"hh:mm:ss\")") )
          ln_wh[:s_ct_ent].value=e_nc.elements['data'].attributes['cycletime']
          ln_wh[:s_ct_ent_stt] = "readonly"
          ln_wh[:s_ct_ent].state(ln_wh[:s_ct_ent_stt])
          return
        else
          e_nc.elements['data'].add_attribute(
            'cycletime', ln_wh[:s_ct_ent].value)
          ln_wh[:s_ct_ent_stt] = "readonly"
          ln_wh[:s_ct_ent].state(ln_wh[:s_ct_ent_stt])
          return
        end
      }
      ln_wh[:s_od_btn]=TkButton.new(ln_wh[:stn_df]).pack(fill: :both)
      ln_wh[:s_od_btn].text=_("Edit OD")
      ln_wh[:s_od_btn].command {od_editor(t_ewn,e_nc.elements['data'])}
    }

    ## (X2) Setting up settings frame for "f2_constant_rate_outflow"
    ln_wh[:stn_2cro] = lambda{
      if ln_wh[:stn_df]
        ln_wh[:stn_df].destroy
      end
      ln_wh[:stn_df] = TkFrame.new(ln_wh[:stn]).pack(fill: :x)
      ln_wh[:s_ct] = TkFrame.new(ln_wh[:stn_df]).pack(fill: :both)
      ln_wh[:s_ct_lbl] = TkLabel.new(ln_wh[:s_ct])
      ln_wh[:s_ct_lbl].text=_("Cycle time for OD: ")
      ln_wh[:s_ct_lbl].pack(side: :left)
      ln_wh[:s_ct_ent]=TkEntry.new(ln_wh[:s_ct])
      ln_wh[:s_ct_ent].width(25)
      ln_wh[:s_ct_ent].value= e_nc.elements['data'].attributes['cycletime']
      ln_wh[:s_ct_ent_stt] = "readonly"
      ln_wh[:s_ct_ent].state(ln_wh[:s_ct_ent_stt])
      ln_wh[:s_ct_btn]=TkButton.new(ln_wh[:s_ct])
      ln_wh[:s_ct_btn].text=_("Change")
      ln_wh[:s_ct_btn].pack(side: :right)
      ln_wh[:s_ct_ent].pack(anchor: :c, fill: :both)
      ln_wh[:s_ct_btn].command {
        if ln_wh[:s_ct_ent_stt] == "readonly"
          ln_wh[:s_ct_ent_stt] = "normal"
          ln_wh[:s_ct_ent].state(ln_wh[:s_ct_ent_stt])
          return
        elsif !(ln_wh[:s_ct_ent].value.is_i_as_time?)
          Tk.messageBox(
            "title" => _("Time not in proper format"),
            "message" =>
            _("The input value is not in proper format of time " +
              "(e.g. \"hh:mm:ss\")") )
          ln_wh[:s_ct_ent].value=e_nc.elements['data'].attributes['cycletime']
          ln_wh[:s_ct_ent_stt] = "readonly"
          ln_wh[:s_ct_ent].state(ln_wh[:s_ct_ent_stt])
          return
        else
          e_nc.elements['data'].add_attribute(
            'cycletime', ln_wh[:s_ct_ent].value)
          ln_wh[:s_ct_ent_stt] = "readonly"
          ln_wh[:s_ct_ent].state(ln_wh[:s_ct_ent_stt])
          return
        end
      }
      ln_wh[:s_od_btn]=TkButton.new(ln_wh[:stn_df]).pack(fill: :both)
      ln_wh[:s_od_btn].text=_("Edit OD")
      ln_wh[:s_od_btn].command {od_editor(t_ewn,e_nc.elements['data'])}
    }

    ## (c1) Radiobutton [:lnm_l02_rb2] command.
    stn_c =
      (ln_wh[:lnm_l02tv] != "nil" ?
         @doc.elements[
         'rtss_input_data/create[@class=\'sujic_passenger_activity\']/' +
         'data[@name=\'' + ln_wh[:lnm_l02tv] + '\']/..'] : nil)
    ln_wh[:lnm_l02_rb2].command(
      proc{
        p "Radiobutton LNM02_RB2"
        _lnm02_2h = Hash.new
        _lnm02_2h[:xpath_lb] =
          'rtss_input_data/create[@class=\'sujic_passenger_activity\']'
        _lnm02_2h[:listlambda] = lambda{|e|
          e.elements['data'].attributes['class'] + '::' +
            e.elements['data'].attributes['name']}
        _lnm02_2h[:lb_otext] = _("Passenger activity")
        _lnm02_2h[:sw_nil] = false
        _lnm02_2h[:slambda] = lambda {|e,_org,_dx|
          puts "======= PasAct selected ======="
          p e
          puts "======= End PasAct selected ======="
          if e
            e_nc.elements['under'].add_attribute(
              'name',e.elements['data'].attributes['name'])
          else
            e_nc.elements['under'].add_attribute('name','nil')
          end
          _dx[:lnm_l02tv].set_value(e_nc.elements['under'].attributes['name'])
          stn_c = e
          _dx[:lnm_l02v].set_value('orig')
          if e.elements['data'].attributes['class'] ==
             'x_passenger_activity_f1'
            ln_wh[:stn_1cri].call
            unless ln_wh[:ptn_rv] == 'f1_constant_rate_inflow'
              ln_wh[:ptn_rv] = 'f1_constant_rate_inflow'
            end
            ln_wh[:ptn_l_f1_rb].state('normal')
            ln_wh[:ptn_l_f2_rb].state('disabled')
          elsif e.elements['data'].attributes['class'] ==
                'x_passenger_activity_f2'
            ln_wh[:stn_2cro].call
            unless ln_wh[:ptn_rv] == 'f2_constant_rate_outflow'
              ln_wh[:ptn_rv] = 'f2_constant_rate_outflow'
            end
            ln_wh[:ptn_l_f1_rb].state('disabled')
            ln_wh[:ptn_l_f2_rb].state('normal')
          end
          return true }
        pick_one_window(t_ewn, stn_c, _lnm02_2h, ln_wh)
      })

    ## (c2) Radiobutton [:ptn_l_f1_rb] command.
    ln_wh[:ptn_l_f1_rb].command(
      proc{
        puts "PasApp: F1 constant rate inflow"
        e_nc.elements['data'].add_attribute(
          'class','f1_constant_rate_inflow')
        ln_wh[:stn_1cri].call })

    ## (c3) Radiobutton [:ptn_l_f2_rb] command.
    ln_wh[:ptn_l_f2_rb].command(
      proc{
        puts "PasApp: F2 constant rate outflow"
        e_nc.elements['data'].add_attribute(
          'class','f2_constant_rate_outflow')
        ln_wh[:stn_2cro].call })

    l_sc = lambda {
      tt_xa = 'rtss_input_data/create[@class=\'sujic_passenger_appearance'
      tt_xa += '_model\']/data[@name=\'' + ln_wh[:lnm_l01_ent].value + '\']'
      if ln_wh[:lnm_l01_ent].value == "nil"
        Tk.messageBox(
          "title" =>
          _("\"Nil\" not allowed as passenger appearance model name"),
          "message" =>
          _("You cannot use \"nil\" as a name of a " +
            "passenger appearance model."))
        return false
      elsif v_x != "move" && ln_wh[:lnm_l01_ent].value == ""
        Tk.messageBox(
          "title" => _("Passenger appearance model name not specified?"),
          "message" => _("Please specify a passenger appearance model name."))
        return false
      elsif ( v_x == "move" ) &&
            ( ln_wh[:lnm_l01_ent].value !=
              e_nc.elements['data'].attributes['name'] )
        Tk.messageBox(
          "title" => _("Name change not allowed"),
          "message" => _("Element name should not be changed when moving"))
        return false
      elsif ( v_x != "move" ) &&
            ( ln_wh[:lnm_l01_ent].value !=
              e_nc.elements['data'].attributes['name'] &&
              @doc.elements[tt_xa] != nil )
        Tk.messageBox(
          "title" => _("Name already in use"),
          "message" =>
          _("This passenger appearance model name is already used: ") + "\n" +
          ln_wh[:lnm_l01_ent].value )
        return false
      elsif ( v_x == "new" ) && ( h_pss_lb.size < 1 ) &&
            ( _de['new_b'] == nil )
        # Bug: first element insertion destination not set
        Tk.messageBox(
          "title" => _("Internal error") + " L04150",
          "message" =>
          sprintf(_("Internal error %1$s has happened. Report this as bug."),
                  "L04150"))
        return true
      elsif ( v_x == "move" ) && ( h_pss_lb.size < 1 )
        # Bug: move selected when no element is present
        Tk.messageBox(
          "title" => _("Internal error") + " L04158",
          "message" =>
          sprintf(_("Internal error %1$s has happened. Report this as bug."),
                  "L04158"))
        return true
      elsif ( v_x == "new" || v_x == "copy" ) &&
            h_pss_lb.curselection.size > 1
        # Curselection should be 1
        Tk.messageBox(
          "title" => _("Multiple insertion destinations"),
          "message" =>
          _("Select only one insertion destination element."))
        return false
      elsif ( ( v_x == "new" && h_pss_lb.size >= 1 ) || v_x == "copy" ) &&
            h_pss_lb.curselection.size < 1
        # Curselection should be 1
        Tk.messageBox(
          "title" => _("Select an insertion destination"),
          "message" => _("Select one insertion destination element " +
                         "from the list in tab \"Where to insert\"."))
        return false
      else
        ## Checks before inserting...
        ## Checks before inserting ended...
        if v_x == "new" || v_x == "copy"
          if ln_wh[:lnm_l01_ent].value !=
             e_nc.elements['data'].attributes['name']
            str_on = e_nc.elements['data'].attributes['name']
            e_nc.elements['data'].add_attribute(
              'name', ln_wh[:lnm_l01_ent].value)
          end
          p "###### -- insert -- ###### [v_x = " + v_x + "]"
          p "###### -- insert -- ###### [h_pss_lb.size = " +
            h_pss_lb.size.to_s + "]"
          if ( h_pss_lb.size < 1 )
            elm_ins = _de['new_b']
            sw_after = false
          else
            p "###### -- insert -- ###### [cursel = " +
              h_pss_lb.curselection[0].to_s + "]"
            sw_after = false if h_pss[nil] == "before"
            sw_after = true if h_pss[nil] == "after"
            elm_ins = ar_el[h_pss_lb.curselection[0]]
            p "###### -- insert -- ######"
            puts elm_ins
            p "###### -- insert after? -- ######"
            p "---- after" if sw_after
            p "---- before" unless sw_after
            p "###### -- insert end -- ######"
          end
          if sw_after
            elm_ins.parent.insert_after(elm_ins,e_nc)
          else
            elm_ins.parent.insert_before(elm_ins,e_nc)
          end
        elsif v_x == "move" && e_cur != ar_el[h_pss_lb.curselection[0]]
          elm_cut = e_cur.parent.delete(e_cur)
          p "#### ELM_CUT is not equal to E_CUR ####" if elm_cut != e_cur
          sw_after = false if h_pss[nil] == "before"
          sw_after = true if h_pss[nil] == "after"
          elm_ins = ar_el[h_pss_lb.curselection[0]]
          elm_ins.parent.insert_after(elm_ins, elm_cut) if sw_after
        elsif v_x != "move" ## Edit only
          if ln_wh[:lnm_l01_ent].value != e_nc.attributes['name']
            str_on = e_nc.attributes['name']
            e_nc.elements['data'].add_attribute(
              'name', ln_wh[:lnm_l01_ent].value)
          end
        end
        return true
      end
    }

    # Close this window
    last_of_editor_window(
      parent_win, t_ewn,_de['xpath_lb'],_de['xpath_root'],_de['listlambda'],
      l_btn, lbox, ar_pel, l_sc, v_x, l_discard_edit)
    p 'passenger_appearance_model_editor end'
  end


  # Create new window for OD data editing.
  # @return No return.
  def od_editor (parent_win, dt_obj)

    #-----------------#
    # Initializing... #
    #-----------------#
    # Create a child window
    es_win = open_editor_window(
      parent_win, _("\"Sujic\"") + ":: " + _("Editing OD data"))
    es_wl = TkFrame.new(es_win).pack(side: :top, fill: :x)

    # Create a frame for selecting Station objects
    es_ft = TkFrame.new(es_wl).pack(side: :left, fill: :x)
    es_fa = Array.new
    es_ta = Array.new
    sw_done = false
    l_sf_create = lambda{
      #-------------------------#
      # Populating "from" frame #
      #-------------------------#
      f_l01 = TkLabel.new(es_ft) do
        text(_("Select \"from\" stations:"))
        pack(side: :top, fill: :x)
      end
      f_l02 = TkFrame.new(es_ft).pack(side: :top, fill: :x)
      ar_el_f = Array.new
      f_lb_sb = TkScrollbar.new(f_l02)
      f_lb = TkListbox.new(
        f_l02, 'height' => 10, 'width' => 50, 'selectmode' => 'extended',
        'yscrollcommand' => proc{|*arg|f_lb_sb.set(*arg)})
      f_lb_sb.command(proc{|*arg| f_lb.yview(*arg)})
      f_lb_sb.pack('side' => 'right', 'fill' => 'y')
      f_lb.pack('side' => 'left', 'fill' => 'both')
      f_lb_xpath = 'rtss_input_data/create[@class=\'sujic_station\']'
      f_lb_llmbd = lambda{|e| e.elements['data'].attributes['name']}
      populate_listbox(f_lb,f_lb_xpath,f_lb_llmbd,nil,ar_el_f,@doc)
      f_bn = TkButton.new(es_ft).pack(side: :top, fill: :x)
      f_bn.text=_("Select these \"from\" stations")
      f_bn.command {
        if f_lb.curselection.size < 1
          # Curselection should be 1 or more
          Tk.messageBox(
            "title" => _("Select \"from\" stations"),
            "message" =>
            _("Select one or more \"from\" stations."))
          return false
        else
          f_lb.curselection.each do |x_a|
            p "From selection <== " +
              ar_el_f[x_a].elements['data'].attributes['name']
            es_fa.push(ar_el_f[x_a])
          end
          #-----------------------#
          # Populating "to" frame #
          #-----------------------#
          t_l01 = TkLabel.new(es_ft) do
            text(_("Select \"to\" stations:"))
            pack(side: :top, fill: :x)
          end
          f_l01.destroy
          t_l02 = TkFrame.new(es_ft).pack(side: :top, fill: :x)
          ar_el_t = Array.new
          t_lb_sb = TkScrollbar.new(t_l02)
          t_lb = TkListbox.new(
            t_l02, 'height' => 10, 'width' => 50, 'selectmode' => 'extended',
            'yscrollcommand' => proc{|*arg|t_lb_sb.set(*arg)})
          t_lb_sb.command(proc{|*arg| t_lb.yview(*arg)})
          t_lb_sb.pack('side' => 'right', 'fill' => 'y')
          t_lb.pack('side' => 'left', 'fill' => 'both')
          t_lb_xpath = 'rtss_input_data/create[@class=\'sujic_station\']'
          t_lb_llmbd = lambda{|e| e.elements['data'].attributes['name']}
          populate_listbox(t_lb,t_lb_xpath,t_lb_llmbd,nil,ar_el_t,@doc)
          f_l02.destroy
          t_bn = TkButton.new(es_ft).pack(side: :top, fill: :both)
          t_bn.text=_("Select these \"to\" stations")
          t_bn.command {
            if sw_done
              sw_done = false
              t_bn.destroy
              t_l02.destroy
              l_sf_create.call
            else
              if t_lb.curselection.size < 1
                # Curselection should be 1 or more
                Tk.messageBox(
                  "title" => _("Select \"to\" stations"),
                  "message" =>
                  _("Select one or more \"to\" stations."))
                return false
              else
                t_lb.curselection.each do |x_a|
                  p "To selection <== " +
                    ar_el_t[x_a].elements['data'].attributes['name']
                  es_ta.push(ar_el_t[x_a])
                end
              end
              od_table_editor(es_win,es_fa,es_ta,dt_obj)
              t_bn.text=_("Re-select stations")
              sw_done = true
              t_l01.destroy
              t_lb.erase
            end}
          f_bn.destroy
        end}}
    l_sf_create.call
    

    # Close this window with tabs
    last_of_editor_window(
      parent_win,es_win, nil, nil, nil, nil, nil, nil, nil, nil)

  end



  # Create new window for OD data editing.
  # @return No return.
  def od_table_editor (parent_win, ar_f, ar_t, dt_obj)

    #-----------------#
    # Initializing... #
    #-----------------#
    # Create a child window
    es_win = open_editor_window(
      parent_win, _("\"Sujic\"") + ":: " + _("Editing OD table"))
    es_fs = TkFrame.new(es_win).pack(fill: :x, side: :top)

    #-------------------------#
    # Populating "from" frame #
    #-------------------------#
    cols = Array.new
    cols.push(Array.new)
    obj_x = TkLabel.new(es_fs) do
      text(_("From \\ To"))
      grid('row'=>0, 'column'=>0)
    end
    cols[0].push(obj_x)
    i = 0
    ar_f.each do |f_st|
      i += 1
      obj_x = TkLabel.new(es_fs) do
        text(f_st.elements['data'].attributes['name'])
        grid('row'=>i, 'column'=>0)
      end
      cols[0].push(obj_x)
    end
    i = 0
    ar_t.each do |t_st|
      i += 1
      cols.push(Array.new)
      obj_x = TkLabel.new(es_fs) do
        text(t_st.elements['data'].attributes['name'])
        grid('row'=>0, 'column'=>i)
      end
      cols[i].push(obj_x)
    end
    i = 0
    ar_f.each do |f_st|
      i += 1
      j = 0
      ar_t.each do |t_st|
        j += 1
        if f_st == t_st
          obj_x = TkLabel.new(es_fs) do
            text('---')
            grid('row'=>i, 'column'=>j)
          end
          cols[i].push(obj_x)
        else
          obj_x = TkEntry.new(es_fs).grid('row'=>i,'column'=>j)
          obj_x.width(8)
          cols[i].push(obj_x)
          od_ent = dt_obj.elements[
            'od_data[@from=\'' + f_st.elements['data'].attributes['name'] +
            '\' and @to=\'' + t_st.elements['data'].attributes['name'] +
            '\']']
          if od_ent
            obj_x.value=od_ent.attributes['demand']
          else
            obj_x.value='0'
          end
        end
      end
    end
      
    l_sc = lambda {
      i = 0
      ar_f.each do |f_st|
        i += 1
        j = 0
        ar_t.each do |t_st|
          j += 1
          if f_st != t_st
            od_ent = dt_obj.elements[
              'od_data[@from=\'' + f_st.elements['data'].attributes['name'] +
              '\' and @to=\'' + t_st.elements['data'].attributes['name'] +
              '\']']
            if cols[i][j].value == "0" && od_ent
              dt_obj.delete(od_ent)
            elsif od_ent
              od_ent.add_attribute('demand', cols[i][j].value)
            else
              dt_obj.add_element(
                'od_data',
                {'from'=>f_st.elements['data'].attributes['name'],
                 'to'=>t_st.elements['data'].attributes['name'],
                 'class'=>'x_od_data', 'demand'=>cols[i][j].value})
            end
          end
        end
      end
      return true
    }

    # Close this window with tabs
    last_of_editor_window(
      parent_win,es_win, nil, nil, nil, nil, nil, nil, l_sc, nil)

  end

  # Evaluator data editor: Creating a window for evaluator data editing.
  # @return None.
  # @param parent_win [TkToplevel] Parent window.
  # @param lbox [TkListbox] Listbox containing this train type.
  # @param ar_pel [xx] xx.
  # @param v_x [xx] xx.
  # @param l_btn [xx] xx.
  # @param e_cur [REXML::Element] Current element if it already exists.
  def evaluator_editor(
        parent_win, lbox, ar_pel, _de, v_x, l_btn, e_cur, e_parent)
    # e_parent = nil
    p 'evaluator_editor start'

    # Data extracting ...
    tt_ah = Hash.new
    elm_c = nil
    elm_c = e_cur.elements['data'] if e_cur != nil
    if v_x == "edit" || v_x == "copy" || v_x == "move"
      tt_ah['name'] = elm_c.attributes['name']
      tt_ah['currency'] = elm_c.attributes['currency']
      tt_ah['eval_1'] = elm_c.attributes['eval_1']
      tt_ah['eval_2'] = elm_c.attributes['eval_2']
      tt_ah['eval_3'] = elm_c.attributes['eval_3']
      tt_ah['eval_4'] = elm_c.attributes['eval_4']
      tt_ah['eval_5'] = elm_c.attributes['eval_5']
      tt_ah['evals_per_time'] = elm_c.attributes['evals_per_time']
      tt_ah['most_crowded_runs_list'] =
        elm_c.attributes['most_crowded_runs_list']
      tt_ah['outfile'] = elm_c.attributes['outfile']
      if elm_c.attributes['token_routes_out']
        tt_ah['token_routes_out'] =
          elm_c.attributes['token_routes_out'].downcase
      else
        tt_ah['token_routes_out'] = "i"
      end
      if elm_c.attributes['branch_tokens_out']
        tt_ah['branch_tokens_out'] =
          elm_c.attributes['branch_tokens_out'].downcase
      else
        tt_ah['branch_tokens_out'] = "i"
      end
        
    else
      tt_ah['name'] = ""
      tt_ah['currency'] = _("Yen")
      tt_ah['eval_1'] = "35"
      tt_ah['eval_2'] = "300"
      tt_ah['eval_3'] = "5"
      tt_ah['eval_4'] = "0"
      tt_ah['eval_5'] = "0"
      tt_ah['evals_per_time'] = "1:00"
      tt_ah['most_crowded_runs_list'] = "10"
      tt_ah['outfile'] = "out.ods"
      tt_ah['token_routes_out'] = "i"
      tt_ah['branch_tokens_out'] = "i"
    end

    #-----------------------------#
    # Creating a child window ... #
    #-----------------------------#
    # Window open
    tt_edt = open_editor_window(
      parent_win, _("\"Sujic\"") + ":: " + _("Editing evaluator parameters"))
    
    # Entry widgets
    tt_l01 = TkFrame.new(tt_edt).pack(side: :top, fill: :x)
    tt_l01_lbl = TkLabel.new(tt_l01)
    tt_l01_lbl.text=_("Evaluator name: ")
    tt_l01_lbl.pack(side: :left)
    tt_l01_ent=TkEntry.new(tt_l01)
    tt_l01_ent.width(25)
    tt_l01_ent.value=tt_ah['name']
    tt_l01_ent.pack(fill: :x)
    if v_x != "move"
      tt_l02 = TkFrame.new(tt_edt).pack(side: :top, anchor: :e)
      tt_l02_lbl = TkLabel.new(tt_l02)
      tt_l02_lbl.text=_("Currency: ")
      tt_l02_lbl.pack(side: :left)
      tt_l02_ent=TkEntry.new(tt_l02)
      tt_l02_ent.width(15)
      tt_l02_ent.value=tt_ah['currency']
      tt_l02_ent.pack(side: :left)
      tt_l02_btn=TkButton.new(tt_l02)
      tt_l02_btn.text=_("(?)")
      tt_l02_btn.pack(side: :left)
      tt_l02_btn.command {
        Tk.messageBox(
          "title" =>
          _("Currency (defaults to Yen)"),
          "message" =>
          _("Currency is the numerator of the unit of " +
            "all evaluation parameters. " +
            "Except parameter 2, the unit of these parameters are " +
            "this currency divided by the unit time " +
            "as specified elsewhere."))
      }
      tt_lp1 = TkFrame.new(tt_edt).pack(side: :top, anchor: :e)
      tt_lp1_lbl = TkLabel.new(tt_lp1)
      tt_lp1_lbl.text=_("Evaluation parameter 1: ")
      tt_lp1_lbl.pack(side: :left)
      tt_lp1_ent=TkEntry.new(tt_lp1)
      tt_lp1_ent.width(15)
      tt_lp1_ent.value=tt_ah['eval_1']
      tt_lp1_ent.pack(side: :left)
      tt_lp1_btn=TkButton.new(tt_lp1)
      tt_lp1_btn.text=_("(?)")
      tt_lp1_btn.pack(side: :left)
      tt_lp1_btn.command {
        Tk.messageBox(
          "title" =>
          _("Evaluation parameter 1 (defaults to 35)"),
          "message" =>
          _("This parameter is the disbenefit per unit time during which " +
            "a passenger travels. Its unit is the currency per unit time. " +
            "Both the currency and the unit travel time are " +
            "specified elsewhere."))
      }
      tt_lp2 = TkFrame.new(tt_edt).pack(side: :top, anchor: :e)
      tt_lp2_lbl = TkLabel.new(tt_lp2)
      tt_lp2_lbl.text=_("Evaluation parameter 2: ")
      tt_lp2_lbl.pack(side: :left)
      tt_lp2_ent=TkEntry.new(tt_lp2)
      tt_lp2_ent.width(15)
      tt_lp2_ent.value=tt_ah['eval_2']
      tt_lp2_ent.pack(side: :left)
      tt_lp2_btn=TkButton.new(tt_lp2)
      tt_lp2_btn.text=_("(?)")
      tt_lp2_btn.pack(side: :left)
      tt_lp2_btn.command {
        Tk.messageBox(
          "title" =>
          _("Evaluation parameter 2 (defaults to 300)"),
          "message" =>
          _("This parameter is the disbenefit per transfer from one train " +
            "to another of a passenger during his/her travel. Its unit is " +
            "the currency per transfer occasion. " +
            "The currency is specified elsewhere."))
      }
      tt_lp3 = TkFrame.new(tt_edt).pack(side: :top, anchor: :e)
      tt_lp3_lbl = TkLabel.new(tt_lp3)
      tt_lp3_lbl.text=_("Evaluation parameter 3: ")
      tt_lp3_lbl.pack(side: :left)
      tt_lp3_ent=TkEntry.new(tt_lp3)
      tt_lp3_ent.width(15)
      tt_lp3_ent.value=tt_ah['eval_3']
      tt_lp3_ent.pack(side: :left)
      tt_lp3_btn=TkButton.new(tt_lp3)
      tt_lp3_btn.text=_("(?)")
      tt_lp3_btn.pack(side: :left)
      tt_lp3_btn.command {
        Tk.messageBox(
          "title" =>
          _("Evaluation parameter 3 (defaults to 5)"),
          "message" =>
          _("This parameter is the disbenefit per unit time during which " +
            "a passenger stands in a train, i.e. the passenger could not " +
            "find a seat in the train. Its unit is " +
            "the currency per unit time. Both the currency and " +
            "the unit time are specified elsewhere."))
      }
      tt_lp4 = TkFrame.new(tt_edt).pack(side: :top, anchor: :e)
      tt_lp4_lbl = TkLabel.new(tt_lp4)
      tt_lp4_lbl.text=_("Evaluation parameter 4: ")
      tt_lp4_lbl.pack(side: :left)
      tt_lp4_ent=TkEntry.new(tt_lp4)
      tt_lp4_ent.width(15)
      tt_lp4_ent.value=tt_ah['eval_4']
      tt_lp4_ent.pack(side: :left)
      tt_lp4_btn=TkButton.new(tt_lp4)
      tt_lp4_btn.text=_("(?)")
      tt_lp4_btn.pack(side: :left)
      tt_lp4_btn.command {
        Tk.messageBox(
          "title" =>
          _("Evaluation parameter 4 (defaults to 0)"),
          "message" =>
          _("This parameter is the disbenefit per unit time during which " +
            "a passenger walks between platforms in a station. " +
            "Its unit is the currency per unit time. Both the currency " +
            "and the unit time are specified elsewhere."))
      }
      tt_lp5 = TkFrame.new(tt_edt).pack(side: :top, anchor: :e)
      tt_lp5_lbl = TkLabel.new(tt_lp5)
      tt_lp5_lbl.text=_("Evaluation parameter 5: ")
      tt_lp5_lbl.pack(side: :left)
      tt_lp5_ent=TkEntry.new(tt_lp5)
      tt_lp5_ent.width(15)
      tt_lp5_ent.value=tt_ah['eval_5']
      tt_lp5_ent.pack(side: :left)
      tt_lp5_btn=TkButton.new(tt_lp5)
      tt_lp5_btn.text=_("(?)")
      tt_lp5_btn.pack(side: :left)
      tt_lp5_btn.command {
        Tk.messageBox(
          "title" =>
          _("Evaluation parameter 5 (defaults to 0)"),
          "message" =>
          _("This parameter is the disbenefit per unit time during which " +
            "a passenger waits for the next train at one of the platforms " +
            "in a station. Its unit is the currency per unit time. " +
            "Both the currency and the unit time are specified elsewhere."))
      }
      tt_l04 = TkFrame.new(tt_edt).pack(side: :top, anchor: :e)
      tt_l04_lbl = TkLabel.new(tt_l04)
      tt_l04_lbl.text=_("Unit time of parameters: ")
      tt_l04_lbl.pack(side: :left)
      tt_l04_ent=TkEntry.new(tt_l04)
      tt_l04_ent.width(15)
      tt_l04_ent.value=tt_ah['evals_per_time']
      tt_l04_ent.pack(side: :left)
      tt_l04_btn=TkButton.new(tt_l04)
      tt_l04_btn.text=_("(?)")
      tt_l04_btn.pack(side: :left)
      tt_l04_btn.command {
        Tk.messageBox(
          "title" =>
          _("Unit time of parameters (defaults to 1:00)"),
          "message" =>
          _("Unit time is the denominator of the unit of " +
            "all evaluation parameters except parameter 2. " +
            "The unit of these parameters are " +
            "the currency as specified elsewhere divided by this unit time."))
      }
      tt_l05 = TkFrame.new(tt_edt).pack(side: :top, anchor: :e)
      tt_l05_lbl = TkLabel.new(tt_l05)
      tt_l05_lbl.text=_("Most crowded runs list size: ")
      tt_l05_lbl.pack(side: :left)
      tt_l05_ent=TkEntry.new(tt_l05)
      tt_l05_ent.width(15)
      tt_l05_ent.value=tt_ah['most_crowded_runs_list']
      tt_l05_ent.pack(side: :left)
      tt_l05_btn=TkButton.new(tt_l05)
      tt_l05_btn.text=_("(?)")
      tt_l05_btn.pack(side: :left)
      tt_l05_btn.command {
        Tk.messageBox(
          "title" =>
          _("Most crowded runs list size (defaults to 10)"),
          "message" =>
          _("This is the (maximum) size of the \"most crowded runs list\" " +
            "in the simulation output file."))
      }
      tt_l05y1 = TkFrame.new(tt_edt).pack(side: :top, fill: :x)
      tt_l05y1_v = TkVariable.new(tt_ah['token_routes_out'])
      tt_l05y1_lbl = TkLabel.new(tt_l05y1)
      tt_l05y1_lbl.text=_("Show passenger token routes: ")
      tt_l05y1_lbl.pack(side: :left)
      tt_l05y1_rb_y=TkRadioButton.new(tt_l05y1) do
        text _("Yes")
        value "i"
        variable tt_l05y1_v
        command(
          proc{
            puts "eval 04742: Show token routes: Yes"
            tt_ah['token_routes_out'] = "i"
          })
        pack(side: :left)
      end
      tt_l05y1_rb_n=TkRadioButton.new(tt_l05y1) do
        text _("No")
        value "n"
        variable tt_l05y1_v
        command(
          proc{
            puts "eval 04755: Show token routes: No"
            tt_ah['token_routes_out'] = "n"
          })
        pack(side: :left)
      end
      tt_l05y2 = TkFrame.new(tt_edt).pack(side: :top, fill: :x)
      tt_l05y2_v = TkVariable.new(tt_ah['branch_tokens_out'])
      tt_l05y2_lbl = TkLabel.new(tt_l05y2)
      tt_l05y2_lbl.text=_("Show branch passenger tokens: ")
      tt_l05y2_lbl.pack(side: :left)
      tt_l05y2_rb_y=TkRadioButton.new(tt_l05y2) do
        text _("Yes")
        value "i"
        variable tt_l05y2_v
        command(
          proc{
            puts "eval 04771: Show branch tokens: Yes"
            tt_ah['branch_tokens_out'] = "i"
          })
        pack(side: :left)
      end
      tt_l05y2_rb_n=TkRadioButton.new(tt_l05y2) do
        text _("No")
        value "n"
        variable tt_l05y2_v
        command(
          proc{
            puts "eval 04782: Show branch tokens: No"
            tt_ah['branch_tokens_out'] = "n"
          })
        pack(side: :left)
      end
      tt_l05x = TkFrame.new(tt_edt).pack(side: :top, fill: :x)
      tt_l05x_lbl = TkLabel.new(tt_l05x)
      tt_l05x_lbl.text=_("File name: ")
      tt_l05x_lbl.pack(side: :left)
      tt_l05x_ent=TkEntry.new(tt_l05x)
      tt_l05x_ent.width(40)
      tt_l05x_ent.value=tt_ah['outfile']
      tt_l05x_ent.state="readonly"
      tt_l05x_ent.pack(fill: :x)
      tt_l05x_btn=TkButton.new(tt_edt)
      tt_l05x_btn.text=_("Choose simulation output file")
      tt_l05x_btn.pack(fill: :x)
      tt_l05x_btn.command {
        str_tg=Tk.getSaveFile(
          'title' => _('Choose simulation output file name'),
          'defaultextension' => 'ods',
          'filetypes' => "{ODS {.ods}}" ) ;
        tt_l05x_ent.state="normal"
        tt_l05x_ent.value=str_tg unless str_tg.empty?
        tt_l05x_ent.state="readonly"
      }
    end
    # Listbox for specifying where to insert
    if v_x != "edit"
      tt_l06 = TkFrame.new(tt_edt).pack(side: :top, fill: :x)
      pcrbs = Hash.new
      pcrbs = insert_position_controller_radiobuttons(tt_l06)
      tt_l07 = TkFrame.new(tt_edt).pack(side: :top, fill: :x)
      ar_el = Array.new
      lb_sb = TkScrollbar.new(tt_l07)
      lb = TkListbox.new(
        tt_l07, 'height' => 10, 'width' => 50, 'selectmode' => 'single',
        'yscrollcommand' => proc{|*arg|lb_sb.set(*arg)})
      lb_sb.command(proc{|*arg| lb.yview(*arg)})
      lb_sb.pack('side' => 'right', 'fill' => 'y')
      lb.pack('side' => 'left', 'fill' => 'both')
      populate_listbox(lb,_de['xpath_lb'],_de['listlambda'],nil,ar_el,@doc)
      if lb.size < 1
        pcrbs.each do |key,val|
          next if key == nil || key == "lframe"
          val.state("disabled")
        end
      end
    end
    # Enter button command
    l_sc = lambda {
      tt_xa = 'rtss_input_data/create[@class=\'sujic_traintype\']/data['
      tt_xa += '@name=\'' + tt_l01_ent.value + '\']'
      if tt_l01_ent.value == "nil"
        Tk.messageBox(
          "title" => _("\"Nil\" not allowed as evaluator name"),
          "message" => _("You cannot use \"nil\" as a name of an evaluator."))
        return false
      elsif v_x != "move" && tt_l01_ent.value == ""
        Tk.messageBox(
          "title" => _("Evaluator name not specified?"),
          "message" => _("Please specify an evaluator name.") )
        return false
      elsif ( v_x == "move" ) &&
            ( tt_l01_ent.value != elm_c.attributes['name'] )
        Tk.messageBox(
          "title" => _("Name change not allowed"),
          "message" => _("Element name should not be changed when moving"))
        return false
      elsif ( v_x != "move" ) &&
            ( ( elm_c != nil ) &&
              ( tt_l01_ent.value != elm_c.attributes['name'] ) ||
              ( elm_c == nil ) && ( v_x == "new" ) ) &&
            ( @doc.elements[tt_xa] != nil )
        Tk.messageBox(
          "title" => _("Name already in use"),
          "message" => _("This evaluator name is already used: ") + "\n" +
                       tt_l01_ent.value )
        return false
      elsif ( v_x != "move" ) && ( ! tt_lp1_ent.value.is_float? )
        # Value check (1) lp1_ent: param 1 = float
        Tk.messageBox(
          "title" => _("Evaluation parameter 1 must be a real number"),
          "message" =>
          _("Evaluation parameter 1 must be a real number.") )
        return false
      elsif ( v_x != "move" ) && ( ! tt_lp1_ent.value.is_float? )
        # Value check (1) lp1_ent: param 1 = float
        Tk.messageBox(
          "title" => _("Evaluation parameter 1 must be a real number"),
          "message" =>
          _("Evaluation parameter 1 must be a real number.") )
        return false
      elsif ( v_x != "move" ) && ( ! tt_lp2_ent.value.is_float? )
        # Value check (2) lp1_ent: param 2 = float
        Tk.messageBox(
          "title" => _("Evaluation parameter 2 must be a real number"),
          "message" =>
          _("Evaluation parameter 2 must be a real number.") )
        return false
      elsif ( v_x != "move" ) && ( ! tt_lp3_ent.value.is_float? )
        # Value check (3) lp3_ent: param 3 = float
        Tk.messageBox(
          "title" => _("Evaluation parameter 3 must be a real number"),
          "message" =>
          _("Evaluation parameter 3 must be a real number.") )
        return false
      elsif ( v_x != "move" ) && ( ! tt_lp4_ent.value.is_float? )
        # Value check (4) lp4_ent: param 4 = float
        Tk.messageBox(
          "title" => _("Evaluation parameter 4 must be a real number"),
          "message" =>
          _("Evaluation parameter 4 must be a real number.") )
        return false
      elsif ( v_x != "move" ) && ( ! tt_lp5_ent.value.is_float? )
        # Value check (5) lp5_ent: param 5 = float
        Tk.messageBox(
          "title" => _("Evaluation parameter 5 must be a real number"),
          "message" =>
          _("Evaluation parameter 5 must be a real number.") )
        return false
      elsif ( v_x != "move" ) && ( ! tt_l04_ent.value.is_i_as_time? )
        # Value check (6) l04_ent: evals_per_time = time
        Tk.messageBox(
          "title" => _("Unit time not in proper time format"),
          "message" =>
          _("Unit time is not in proper time format " +
            "(e.g. xx:yy for xx minutes and yy seconds).") )
        return false
      elsif ( v_x != "move" ) && ( ! tt_l05_ent.value.is_integer? )
        # Value check (7) l05_ent: Most crowded runs = integer
        Tk.messageBox(
          "title" => _("Most crowded runs list size must be an integer"),
          "message" =>
          _("Most crowded runs list size must be an integer figure.") )
        return false
      elsif ( v_x == "new" ) && ( lb.size < 1 ) &&
            ( _de['new_e'] == nil )
        # Bug: first element insertion destination not set
        Tk.messageBox(
          "title" => _("Internal error") + " L04750",
          "message" =>
          sprintf(_("Internal error %1$s has happened. Report this as bug."),
                  "L04750"))
        return true
      elsif ( v_x == "move" ) && ( lb.size < 1 )
        # Bug: move selected when no element is present
        Tk.messageBox(
          "title" => _("Internal error") + " L04758",
          "message" =>
          sprintf(_("Internal error %1$s has happened. Report this as bug."),
                  "L04758"))
        return true
      elsif ( v_x == "new" || v_x == "copy" ) && lb.curselection.size > 1
        # Curselection should be 1
        Tk.messageBox(
          "title" => _("Multiple insertion destinations"),
          "message" =>
          _("Select only one insertion destination element."))
        return false
      elsif ( ( v_x == "new" && lb.size >= 1 ) || v_x == "copy" ) &&
            lb.curselection.size < 1
        # Curselection should be 1
        Tk.messageBox(
          "title" => _("Select an insertion destination"),
          "message" =>
          _("Select one insertion destination element from the list below."))
        return false
      else
        if v_x != "move"
          tt_ah['name'] = tt_l01_ent.value
          tt_ah['currency'] = tt_l02_ent.value
          tt_ah['eval_1'] = tt_lp1_ent.value
          tt_ah['eval_2'] = tt_lp2_ent.value
          tt_ah['eval_3'] = tt_lp3_ent.value
          tt_ah['eval_4'] = tt_lp4_ent.value
          tt_ah['eval_5'] = tt_lp5_ent.value
          tt_ah['evals_per_time'] = tt_l04_ent.value
          tt_ah['most_crowded_runs_list'] = tt_l05_ent.value
          tt_ah['outfile'] = tt_l05x_ent.value
        end
        if v_x == "new" || v_x == "copy"
          # Create an element and insert it
          n_e = REXML::Element.new('create')
          { 'class' => 'sujic_evaluator' }.each { |key, val|
            n_e.add_attribute(key, val) }
          p "###### -- insert -- ###### [v_x = " + v_x + "]"
          p "###### -- insert -- ###### [lb.size = " + lb.size.to_s + "]"
          if ( lb.size < 1 )
            _de['new_e'].add_element(n_e)
          else
            p "###### -- insert -- ###### [cursel = " +
              lb.curselection[0].to_s + "]"
            sw_after = false if pcrbs[nil] == "before"
            sw_after = true if pcrbs[nil] == "after"
            elm_ins = ar_el[lb.curselection[0]]
            p "###### -- insert -- ######"
            puts elm_ins
            p "###### -- insert after? -- ######"
            p "---- after" if sw_after
            p "---- before" unless sw_after
            p "###### -- insert end -- ######"
            if sw_after
              elm_ins.parent.insert_after(elm_ins,n_e)
            else
              elm_ins.parent.insert_before(elm_ins,n_e)
            end
          end
          n_e.add_element(
            'data',
            {'name' => tt_ah['name'], 'class' => 'old_sujic_evaluator',
             'currency' => tt_ah['currency'], 'eval_1' => tt_ah['eval_1'],
             'eval_2' => tt_ah['eval_2'], 'eval_3' => tt_ah['eval_3'],
             'eval_4' => tt_ah['eval_4'], 'eval_5' => tt_ah['eval_5'],
             'evals_per_time' => tt_ah['evals_per_time'],
             'most_crowded_runs_list' => tt_ah['most_crowded_runs_list'],
             'token_routes_out' => tt_ah['token_routes_out'],
             'branch_tokens_out' => tt_ah['branch_tokens_out'],
             'outfile' => tt_ah['outfile']})
        elsif v_x == "move" && e_cur != ar_el[lb.curselection[0]]
          elm_cut = e_cur.parent.delete(e_cur)
          p "#### ELM_CUT is not equal to E_CUR ####" if elm_cut != e_cur
          sw_after = false if pcrbs[nil] == "before"
          sw_after = true if pcrbs[nil] == "after"
          elm_ins = ar_el[lb.curselection[0]]
          elm_ins.parent.insert_after(elm_ins, elm_cut) if sw_after
        elsif v_x != "move" ## Edit only
          tt_ah.each{|key, val|
            elm_c.add_attribute(key, val.to_s) if key != "name"}
          if tt_ah['name'] != elm_c.attributes['name']
            elm_c.add_attribute('name', tt_ah['name'])
          end
        end
        return true
      end
    }
    # Button to close this window
    p "### last_of_editor_window in train_type_editor"
    last_of_editor_window(
      parent_win,tt_edt,_de['xpath_lb'],nil,_de['listlambda'],l_btn,lbox,
      ar_pel,l_sc,v_x)
  end

  # Read cycle time.
  # @return [integer] Cycle time.
  # @param ss_obj [REXML::Element] Element containing cycle time as the
  #   value of attribute 'cycletime' to its child element 'data'.
  def read_cycle_time(ss_obj)
    @cycle_time = ss_obj.elements['data'].attributes['cycletime'].to_i_as_time
    p "Cycle time: " + @cycle_time.to_s
  end

  # SVG subprogram: trains (1) List all train times, assuming +@cycle_time+
  # is set.
  # @return [boolean] True if successful.
  def check_train_time_list()
    if @train_times.has_key?(@ss_name)
      @train_times.delete(@ss_name)
    end
    @train_times[@ss_name] = Hash.new
    tr_xp = 'rtss_input_data/create[@class=\'sujic_train\']/under['
    tr_xp += '@name=\''
    tr_xp += @ss_name
    tr_xp += '\']/../data/'
    @doc.elements.each(tr_xp) do |elem|
      tr_n_str = elem.attributes['name']
      p "check_train_time_list checking train " + tr_n_str
      @train_times[@ss_name][tr_n_str] = Hash.new
      if elem.attributes['stroke'] == nil
        elem.add_attribute('stroke','black')
      end
      if elem.attributes['stroke-width'] == nil
        elem.add_attribute('stroke-width','1')
      end
      if elem.attributes['stroke-dasharray'] == nil
        elem.add_attribute('stroke-dasharray','none')
      end
      @train_times[@ss_name][tr_n_str]['data'] = {
        'stroke' => elem.attributes['stroke'],
        'stroke-width' => elem.attributes['stroke-width'],
        'stroke-dasharray' => elem.attributes['stroke-dasharray'] }
      @train_times[@ss_name][tr_n_str]['times'] = Hash.new
      sw_cd_diff = false
      cd_diff = 0
      elem.elements.each do |e_c|
        if e_c.name != 'train_time'
          Tk.messageBox( "title" => _("Wrong XML?"),
                         "message" => (
                           "エラー::\n<train_time> でない要素が列車 \"" +
                           tr_n_str + "\" にみつかりました。"))
          return false
        end
        c_del = 0
        time_t = e_c.attributes['time'].to_i_as_time
        while time_t >= @cycle_time do
          c_del += 1
          time_t -= @cycle_time
        end
        while time_t < 0 do
          c_del -= 1
          time_t += @cycle_time
        end
        if sw_cd_diff
          c_del -= cd_diff
        else
          cd_diff = c_del
          c_del = 0
          sw_cd_diff = true
        end
        if !(@train_times[@ss_name][tr_n_str]['times']
              .has_key?([c_del,time_t]))
          @train_times[@ss_name][tr_n_str]['times'][[c_del,time_t]] =
            Array.new
        end
        @train_times[@ss_name][tr_n_str]['times'][[c_del,time_t]] <<
        { 'line' => e_c.attributes['line'],
          'station' => e_c.attributes['station'],
          'platform' => e_c.attributes['platform'],
          'status' => e_c.attributes['status'],
          'in_service' => !! e_c.attributes['in_service'],
          'road' => e_c.attributes['road'] ,
          'trainset_found' => false }
        p " " + e_c.attributes['station'] + "::" +
          e_c.attributes['platform'] + "/" + e_c.attributes['status'] +
          " == [" + time_t.to_s_as_time + "/" + c_del.to_s + "]"
      end
    end
    return true
  end

  # SVG subprogram: trains (2) List all trainset times.
  # Assume @cycle_time and @train_times are set.
  # @return [boolean] True when successful.
  def check_trainset_time_list()
    if @trainset_times.has_key?(@ss_name)
      @trainset_times.delete(@ss_name)
    end
    @trainset_times[@ss_name] = Hash.new
    @trainset_times[@ss_name][nil] = { 'minimum_cycle_delay' => 0 ,
        'maximum_cycle_delay' => 0 }
    tr_xp = 'rtss_input_data/create[@class=\'sujic_trainset\']/under['
    tr_xp += '@name=\''
    tr_xp += @ss_name
    tr_xp += '\']/../'
    @doc.elements.each(tr_xp) do |elem|
      cd_diff = 0
      prev_train = "nil"
      prev_station = ""
      prev_platform = ""
      prev_status = ""
      c_del = 0
      time_t = 0
      prev_c_del = 0
      prev_time_t = 0
      _line = ""
      tr_n_str = elem.elements['data'].attributes['name']
      p "check_trainset_time_list checking trainset " + tr_n_str
      @trainset_times[@ss_name][tr_n_str] = Hash.new
      if elem.elements['data'].attributes['stroke'] == nil
        elem.elements['data'].add_attribute('stroke','orangered')
      end
      if elem.elements['data'].attributes['stroke-width'] == nil
        elem.elements['data'].add_attribute('stroke-width','0.5')
      end
      if elem.elements['data'].attributes['stroke-dasharray'] == nil
        elem.elements['data'].add_attribute('stroke-dasharray','none')
      end
      if elem.elements['data'].attributes['initial_mark_x_offset'] == nil
        elem.elements['data'].add_attribute(
          'initial_mark_x_offset','0')
      end
      if elem.elements['data'].attributes['initial_mark_y_offset'] == nil
        elem.elements['data'].add_attribute(
          'initial_mark_y_offset','5')
      end
      if elem.elements['data'].attributes[
           'initial_mark_horizontal_separation'] == nil
        elem.elements['data'].add_attribute(
          'initial_mark_horizontal_separation','1')
      end
      if elem.elements['data'].attributes['initial_mark_font_size'] == nil
        elem.elements['data'].add_attribute(
          'initial_mark_font_size','9')
      end
      if elem.elements['data'].attributes['initial_mark_font_family'] == nil
        elem.elements['data'].add_attribute(
          'initial_mark_font_family','sans-serif')
      end
      if elem.elements['data'].attributes['initial_mark_font_weight'] == nil
        elem.elements['data'].add_attribute(
          'initial_mark_font_weight','normal')
      end
      if elem.elements['data'].attributes['initial_mark_font_fill'] == nil
        elem.elements['data'].add_attribute(
          'initial_mark_font_fill','orangered')
      end
      if elem.elements['data'].attributes['final_mark_x_offset'] == nil
        elem.elements['data'].add_attribute(
          'final_mark_x_offset','0')
      end
      if elem.elements['data'].attributes['final_mark_y_offset'] == nil
        elem.elements['data'].add_attribute(
          'final_mark_y_offset','5')
      end
      if elem.elements['data'].attributes[
           'final_mark_horizontal_separation'] == nil
        elem.elements['data'].add_attribute(
          'final_mark_horizontal_separation','1')
      end
      if elem.elements['data'].attributes['final_mark_font_size'] == nil
        elem.elements['data'].add_attribute(
          'final_mark_font_size','9')
      end
      if elem.elements['data'].attributes['final_mark_font_family'] == nil
        elem.elements['data'].add_attribute(
          'final_mark_font_family','sans-serif')
      end
      if elem.elements['data'].attributes['final_mark_font_weight'] == nil
        elem.elements['data'].add_attribute(
          'final_mark_font_weight','normal')
      end
      if elem.elements['data'].attributes['final_mark_font_fill'] == nil
        elem.elements['data'].add_attribute(
          'initial_mark_font_fill','orangered')
      end
      @trainset_times[@ss_name][tr_n_str]['data'] = {
        'stroke' => elem.elements['data'].attributes['stroke'],
        'stroke-width' =>
        elem.elements['data'].attributes['stroke-width'],
        'stroke-dasharray' =>
        elem.elements['data'].attributes['stroke-dasharray'],
        'initial_mark_x_offset' =>
        elem.elements['data'].attributes['initial_mark_x_offset'].to_f,
        'initial_mark_y_offset' =>
        elem.elements['data'].attributes['initial_mark_y_offset'].to_f,
        'initial_mark_horizontal_separation' =>
        elem.elements['data'].attributes[
          'initial_mark_horizontal_separation'].to_f,
        'initial_mark_font_size' =>
        elem.elements['data'].attributes['initial_mark_font_size'].to_f,
        'initial_mark_font_family' =>
        elem.elements['data'].attributes['initial_mark_font_family'],
        'initial_mark_font_weight' =>
        elem.elements['data'].attributes['initial_mark_font_weight'],
        'initial_mark_font_fill' =>
        elem.elements['data'].attributes['initial_mark_font_fill'],
        'final_mark_x_offset' =>
        elem.elements['data'].attributes['final_mark_x_offset'].to_f,
        'final_mark_y_offset' =>
        elem.elements['data'].attributes['final_mark_y_offset'].to_f,
        'final_mark_horizontal_separation' =>
        elem.elements['data'].attributes[
          'final_mark_horizontal_separation'].to_f,
        'final_mark_font_size' =>
        elem.elements['data'].attributes['final_mark_font_size'],
        'final_mark_font_family' =>
        elem.elements['data'].attributes['final_mark_font_family'],
        'final_mark_font_weight' =>
        elem.elements['data'].attributes['final_mark_font_weight'],
        'final_mark_font_fill' =>
        elem.elements['data'].attributes['final_mark_font_fill'] }
      @trainset_times[@ss_name][tr_n_str]['times'] = Hash.new
      elem.elements['data'].elements.each do |e_c|
        if e_c.name != 'train_time'
          Tk.messageBox( "title" => "Wrong XML?",
                         "message" => (
                           "エラー::\n<train_time> でない要素が編成 \"" +
                           tr_n_str + "\" にみつかりました。"))
          return false
        end
        #..........................#
        # Physical platform search #
        #..........................#
        pplat_xp = 'rtss_input_data/create[@class=\'sujic_station\']'
        pplat_xp += '/data[@name=\''
        pplat_xp += e_c.attributes['station']
        pplat_xp += '\']/platforms/platform[@name=\''
        pplat_xp += e_c.attributes['platform']
        pplat_xp += '\']'
        sw_pplat = true
        pplat_name = ""
        @doc.elements.each(pplat_xp) do |pp_e|
          if pp_e.attributes['physical'] == "nil"
            Tk.messageBox( "title" => "Wrong XML?",
                           "message" => (
                             "エラー::\n編成 \"" +
                             tr_n_str + "\" において駅 \"" +
                             e_c.attributes['station'] + "\", 論理ホーム \"" +
                             e_c.attributes['platform'] + "\" に対応する" +
                           "物理ホームが設定されていませんでした。" ))
                             
            return false
          end
          sw_pplat = false
          pplat_name = pp_e.attributes['physical']
          break
        end
        if sw_pplat
          Tk.messageBox( "title" => "Wrong XML?",
                         "message" => (
                           "エラー::\n編成 \"" + tr_n_str +
                           "\" において <train_time> 要素に指定された, 駅 \"" +
                           e_c.attributes['station'] +
                           "\" もしくは同駅構内の論理ホーム \"" +
                           e_c.attributes['platform'] +
                           "\" が設定されていませんでした。" ))
          return false
        end
        p e_c.attributes['train']
        x = @train_times[@ss_name][e_c.attributes['train']]['times']
        if prev_train == e_c.attributes['train']
          sw_prev = false
          sw_break = false
          x.each { |key, val|
            val.each do |v_i|
              if v_i['station'] == prev_station &&
                 v_i['platform'] == prev_platform &&
                 v_i['status'] == prev_status
                sw_prev = true
              elsif sw_prev
                c_del = key[0]
                time_t = key[1]
                c_del -= cd_diff
                if v_i['station'] == e_c.attributes['station'] &&
                   v_i['platform'] == pplat_name &&
                   v_i['status'] == e_c.attributes['status']
                  sw_break = true
                end
                if !(@trainset_times[@ss_name][tr_n_str]['times']
                      .has_key?([c_del,time_t]))
                  @trainset_times[@ss_name][tr_n_str][
                    'times'][[c_del,time_t]] = Array.new
                end
                @trainset_times[@ss_name][tr_n_str][
                  'times'][[c_del,time_t]] <<
                { 'line' => v_i['line'],
                  'station' => v_i['station'],
                  'platform' => v_i['platform'],
                  'train' => e_c.attributes['train'],
                  'status' => v_i['status'],
                  'in_service' => !! v_i['in_service'],
                  'road' => v_i['road'] }
                p " " + v_i['station'] + "::" + v_i['platform'] + "/" +
                  v_i['status'] + " == [" +
                  time_t.to_s_as_time + "/" + c_del.to_s + "]"
                if @trainset_times[@ss_name][nil][
                    'minimum_cycle_delay'] > c_del
                  @trainset_times[@ss_name][nil][
                    'minimum_cycle_delay'] = c_del
                end
                if @trainset_times[@ss_name][nil][
                    'maximum_cycle_delay'] < c_del
                  @trainset_times[@ss_name][nil][
                    'maximum_cycle_delay'] = c_del
                end
                v_i['trainset_found'] = true
              end
              if sw_break
                break
              end
            end
            if sw_break
              break
            end
          }
          if !sw_break
            Tk.messageBox( "title" => "Wrong XML?",
                           "message" => (
                             "エラー::\n編成 \"" + tr_n_str +
                             "\" において，" +
                             "対応する列車時刻情報がみつかりませんでした。(264)"
                           ))
            return false
          end
        else
          if prev_train != "nil" && e_c.attributes['status'] != "ARR"
            Tk.messageBox( "title" => "Wrong XML?",
                           "message" => (
                             "エラー::\n編成 \"" + tr_n_str +
                             "\" において，列車名が " +
                             "DEP 時と直後の ARR 時で異なっています。(273)"))
            return false
          end
          sw_break = false
          x.each { |key, val|
            val.each do |v_i|
              p v_i['station'] + " <=> " + e_c.attributes['station']
              p v_i['platform'] + " <=> " + pplat_name
              p v_i['status'] + " <=> " + e_c.attributes['status']
              p "*******************************"
              if v_i['station'] == e_c.attributes['station'] &&
                 v_i['platform'] == pplat_name &&
                 v_i['status'] == e_c.attributes['status']
                c_del = key[0]
                time_t = key[1]
                _line = v_i['line']
                v_i['trainset_found'] = true
                sw_break = true
                p "Break initiated"
                break
              end
            end
            break if sw_break
          }
          if !sw_break
            Tk.messageBox( "title" => "Wrong XML?",
                           "message" => (
                             "エラー::\n編成 \"" + tr_n_str +
                             "\" において，" +
                             "対応する列車時刻情報がみつかりませんでした。(298)"
                           ))
            return false
          end
          c_del_preserve = c_del
          if prev_train != "nil" 
            if prev_time_t < time_t
              c_del = prev_c_del + 1
            else
              c_del = prev_c_del
            end
          else
            c_del = 0
          end
          cd_diff = c_del_preserve - c_del
          if !(@trainset_times[@ss_name][tr_n_str]['times']
                .has_key?([c_del,time_t]))
            @trainset_times[@ss_name][tr_n_str]['times'][[c_del,time_t]] =
              Array.new
          end
          @trainset_times[@ss_name][tr_n_str]['times'][[c_del,time_t]] <<
          { 'line' => _line, 'station' => e_c.attributes['station'],
            'platform' => pplat_name,
            'train' => e_c.attributes['train'],
            'status' => e_c.attributes['status'],
            'in_service' => !! e_c.attributes['in_service'],
            'road' => e_c.attributes['road'] }
          p " " + e_c.attributes['station'] + "::" +
            pplat_name + "/" + e_c.attributes['status'] +
            " == [" + time_t.to_s_as_time + "/" + c_del.to_s + "]"
          if @trainset_times[@ss_name][nil]['minimum_cycle_delay'] > c_del
            @trainset_times[@ss_name][nil]['minimum_cycle_delay'] = c_del
          end
          if @trainset_times[@ss_name][nil]['maximum_cycle_delay'] < c_del
            @trainset_times[@ss_name][nil]['maximum_cycle_delay'] = c_del
          end
        end
        prev_c_del = c_del
        prev_time_t = time_t
        prev_train = e_c.attributes['train']
        prev_station = e_c.attributes['station']
        prev_platform = pplat_name
        prev_status = e_c.attributes['status']
      end
    end
    @train_times[@ss_name].each { |x_key,x_val|
      next if x_key == nil
      x_val['times'].each { |key, val|
        val.each do |v_i|
          if !(v_i['trainset_found'])
            Tk.messageBox( "title" => "Wrong XML?",
                           "message" => (
                             "エラー::\n列車 \"" + x_key +
                             "\" において，" +
                             "対応する編成情報がない駅がみつかりました。(350)\n" +
                             "(" + v_i['station'] + "駅, Plat " +
                             v_i['platform'] + ": " + v_i['status'] ))
            return false
          end
        end
      }
    }
    return true
  end

  # SVG subprogram: drawing trains.
  # If a train enters the view zone at beginning edge of train graph...
  # - Cycle time: Tc
  # - Station A: height a, time/cd Ta/Da (outside view zone)
  # - Station B: height b, time/cd Tb/Db (inside view zone)
  # - Starting position: height b-(b-a)*(Db*Tc+Tb)/((Db-Da)*Tc+(Tb-Ta))
  # If a train exits the view zone at ending edge of train graph...
  # - Cycle time: Tc
  # - Station A: height a, time/cd Ta/Da (inside view zone)
  # - Station B: height b, time/cd Tb/Db (outside view zone)
  # - Starting position: height b-(b-a)*(Db*Tc+Tb)/((Db-Da)*Tc+(Tb-Ta))
  # Train label:
  # - If train in "down" direction, @text_train_label_path_offset is
  #   ADDED to horizontal position of the textPath. If "up" otherwise.
  # @return [boolean] True if successful.
  # @param tgo_root [xx] xx
  def draw_trains_for_svg(tgo_root)
    @trainset_times[@ss_name].each { |key, val|
      next if key == nil
      if !draw_trainset_for_svg(tgo_root,key,val,0)
        return false
      end
      if !draw_trainset_for_svg(tgo_root,key,val,1)
        return false
      end
      if !draw_trainset_for_svg(tgo_root,key,val,2)
        return false
      end
      if !draw_trainset_for_svg(tgo_root,key,val,3)
        return false
      end
      if !draw_trainset_for_svg(tgo_root,key,val,4)
        return false
      end
    }
    return true
  end

  def draw_trainset_for_svg(tgo_root,key,val,cd_st)
    p "Trainset " + key + "/" + cd_st.to_s + " printing"
    tm_so = val['times'].sort
    c_elm = REXML::Element.new('g')
    tgo_root.add_element(c_elm)
    { 'id' => key + "/" + cd_st.to_s, 'stroke' => val['data']['stroke'],
      'stroke_width' => val['data']['stroke-width'],
      'stroke_dasharray' => val['data']['stroke-dasharray']
    }.each { |key_a, val_a| c_elm.add_attribute(key_a, val_a) }

    #---------------------------------#
    # Scan trainset from tm_so[0][0]. #
    #---------------------------------#
    i_x = 0
    i_y = 0
    j_x = i_x
    j_y = i_y + 1
    if j_y == tm_so[j_x][1].size
      j_y = 0
      j_x += 1
      if j_x == tm_so.size
        Tk.messageBox( "title" => "Wrong XML?",
                       "message" => (
                         "エラー::\n編成 \"" + key +
                         "\" には <train_time> がひとつしかない?"))
        return false
      end
    end
    k_x = 0
    k_y = 0
    sw_last = false
    loop do
      #.............................#
      # j_x, j_y and k_x, k_y check #
      #.............................#
      tm_so_i = tm_so[i_x][1][i_y]
      tm_so_j = tm_so[j_x][1][j_y]
      tm_st_i = tm_so[i_x][0]
      tm_st_j = tm_so[j_x][0]
      k_x = j_x
      k_y = j_y + 1
      if k_y == tm_so[k_x][1].size
        k_y = 0
        k_x += 1
        if k_x == tm_so.size
          sw_last = true
        end
      end

      #...................#
      # search @line_data #
      #...................#
      line_block_id = 0
      l01_y1 = 0.0
      l01_y2 = 0.0
      l_x1 = (@tg_params['margin_left'] +
              @tg_params['text_station_name_width'] +
              @tg_params['text_mileage_figure_width'] +
              (@tg_params['hspace_name_mileage_traingraph'] * 2) +
              (((tm_st_i[0] + cd_st) * @cycle_time + tm_st_i[1]) *
               @tg_params['pixels_per_minute'] / 60.0))
      l_x2 = (@tg_params['margin_left'] +
              @tg_params['text_station_name_width'] +
              @tg_params['text_mileage_figure_width'] +
              (@tg_params['hspace_name_mileage_traingraph'] * 2) +
              (((tm_st_j[0] + cd_st) * @cycle_time + tm_st_j[1]) *
               @tg_params['pixels_per_minute'] / 60.0))
      while line_block_id < @line_data[tm_so_i['line']].size
        if (@line_data[tm_so_i['line']][line_block_id]['stations'].
             has_key?(tm_so_i['station']) &&
            @line_data[tm_so_i['line']][line_block_id]['stations'].
             has_key?(tm_so_j['station']))
          l01_y1 = @line_data[tm_so_i['line']][line_block_id]['stations'][
            tm_so_i['station']][1]
          l01_y2 = @line_data[tm_so_i['line']][line_block_id]['stations'][
            tm_so_j['station']][1]
          break
        end
        line_block_id += 1
      end
      p "Stn " + tm_so_i['station'] + " Plat " +
        tm_so_i['platform'] + "[" + tm_st_i[1].to_s + "/" +
        tm_st_i[0].to_s + "] --> Stn " +
        tm_so_j['station'] + " Plat " +
        tm_so_j['platform'] + + "[" + tm_st_j[1].to_s + "/" +
        tm_st_j[0].to_s + "]: (" + l_x1.to_s + "," + l01_y1.to_s +
        ") -- (" + l_x2.to_s + "," + l01_y2.to_s + ")"
      direc = 1.0
      if l01_y1 < l01_y2
        direc = -1.0
      end
      if (i_x == 0) && (i_y == 0)
        gc_elm = REXML::Element.new('circle')
        c_elm.add_element(gc_elm)
        { 'cx' => (l_x1 + @trainset_times[@ss_name][key]['data'][
                     'initial_mark_x_offset']).to_s,
          'cy' => (l01_y1 + direc * @trainset_times[@ss_name][key]['data'][
                              'initial_mark_y_offset'] +
                   direc * 5).to_s,
          'r' => '5', 'fill' => 'none',
          'stroke' => @trainset_times[@ss_name][key]['data']['stroke'],
          'stroke-width' => @trainset_times[@ss_name][key]['data']['stroke'],
          'stroke-dasharray' =>
          @trainset_times[@ss_name][key]['data']['stroke-dasharray']
        }.each { |key_a, val_a| gc_elm.add_attribute(key_a, val_a) }
        gc_elm = REXML::Element.new('line')
        c_elm.add_element(gc_elm)
        { 'x1' => (l_x1 + @trainset_times[@ss_name][key]['data'][
                     'initial_mark_x_offset']).to_s,
          'y1' => (l01_y1 +
                   (direc * @trainset_times[@ss_name][key]['data'][
                              'initial_mark_y_offset'])).to_s,
          'x2' => l_x1.to_s, 'y2' => l01_y1.to_s,
          'stroke' => @trainset_times[@ss_name][key]['data']['stroke'],
          'stroke-width' => @trainset_times[@ss_name][key]['data']['stroke'],
          'stroke-dasharray' =>
          @trainset_times[@ss_name][key]['data']['stroke-dasharray']
        }.each { |key_a, val_a| gc_elm.add_attribute(key_a, val_a) }
      end
      gc_elm = REXML::Element.new('line')
      c_elm.add_element(gc_elm)
      { 'x1' => l_x1.to_s, 'y1' => l01_y1.to_s,
        'x2' => l_x2.to_s, 'y2' => l01_y2.to_s,
        'stroke' => @trainset_times[@ss_name][key]['data']['stroke'],
        'stroke-width' => @trainset_times[@ss_name][key]['data']['stroke'],
        'stroke-dasharray' =>
        @trainset_times[@ss_name][key]['data']['stroke-dasharray']
      }.each { |key_a, val_a| gc_elm.add_attribute(key_a, val_a) }
      if sw_last
        gc_elm = REXML::Element.new('line')
        c_elm.add_element(gc_elm)
        { 'x1' => (l_x2 + @trainset_times[@ss_name][key]['data'][
                     'final_mark_x_offset']).to_s,
          'y1' => (l01_y2 -
                   (direc * @trainset_times[@ss_name][key]['data'][
                      'final_mark_y_offset'])).to_s,
          'x2' => (l_x2 + @trainset_times[@ss_name][key]['data'][
                      'final_mark_x_offset'] - 5).to_s,
          'y2' => (l01_y2 -
                   (direc * @trainset_times[@ss_name][key]['data'][
                      'final_mark_y_offset']) -
                   (direc * 10)).to_s,
           'stroke' => @trainset_times[@ss_name][key]['data']['stroke'],
           'stroke-width' =>
           @trainset_times[@ss_name][key]['data']['stroke-width'],
           'stroke-dasharray' =>
           @trainset_times[@ss_name][key]['data']['stroke-dasharray']
         }.each { |key_a, val_a| gc_elm.add_attribute(key_a, val_a) }
        gc_elm = REXML::Element.new('line')
        c_elm.add_element(gc_elm)
        { 'x1' => (l_x2 + @trainset_times[@ss_name][key]['data'][
                     'final_mark_x_offset']).to_s,
          'y1' => (l01_y2 - direc * @trainset_times[@ss_name][key]['data'][
                              'final_mark_y_offset']).to_s,
          'x2' => (l_x2 + @trainset_times[@ss_name][key]['data'][
                     'final_mark_x_offset'] + 5).to_s,
          'y2' => (l01_y2 - direc * @trainset_times[@ss_name][key]['data'][
                              'final_mark_y_offset'] -
                   direc * 10).to_s,
          'r' => '5', 'fill' => 'none',
          'stroke' => @trainset_times[@ss_name][key]['data']['stroke'],
          'stroke-width' => @trainset_times[@ss_name][key]['data']['stroke'],
          'stroke-dasharray' =>
          @trainset_times[@ss_name][key]['data']['stroke-dasharray']
        }.each { |key_a, val_a| gc_elm.add_attribute(key_a, val_a) }
        gc_elm = REXML::Element.new('line')
        c_elm.add_element(gc_elm)
        { 'x1' => (l_x2 + @trainset_times[@ss_name][key]['data'][
                     'final_mark_x_offset'] - 5).to_s,
          'y1' => (l01_y2 - direc * @trainset_times[@ss_name][key]['data'][
                              'final_mark_y_offset'] -
                   direc * 10).to_s,
          'x2' => (l_x2 + @trainset_times[@ss_name][key]['data'][
                     'final_mark_x_offset'] + 5).to_s,
          'y2' => (l01_y2 - direc * @trainset_times[@ss_name][key]['data'][
                              'final_mark_y_offset'] -
                   direc * 10).to_s,
          'r' => '5', 'fill' => 'none',
          'stroke' => @trainset_times[@ss_name][key]['data']['stroke'],
          'stroke-width' => @trainset_times[@ss_name][key]['data']['stroke'],
          'stroke-dasharray' =>
          @trainset_times[@ss_name][key]['data']['stroke-dasharray']
        }.each { |key_a, val_a| gc_elm.add_attribute(key_a, val_a) }
        gc_elm = REXML::Element.new('line')
        c_elm.add_element(gc_elm)
        { 'x1' => (l_x2 + @trainset_times[@ss_name][key]['data'][
                     'final_mark_x_offset']).to_s,
          'y1' => (l01_y2 - direc * @trainset_times[@ss_name][key]['data'][
                              'final_mark_y_offset']).to_s,
          'x2' => l_x2.to_s, 'y2' => l01_y2.to_s,
          'stroke' => @trainset_times[@ss_name][key]['data']['stroke'],
          'stroke-width' => @trainset_times[@ss_name][key]['data']['stroke'],
          'stroke-dasharray' =>
          @trainset_times[@ss_name][key]['data']['stroke-dasharray']
        }.each { |key_a, val_a| gc_elm.add_attribute(key_a, val_a) }
        break
      end
      i_x = j_x
      i_y = j_y
      j_x = k_x
      j_y = k_y
    end
    return true
  end

  #============================#
  # SVG subprogram: grid lines #
  #============================#
  def draw_grid_lines_for_svg(tgo_root)
    ## Assume @cycle_time is set
    time_t = 0.0
    line_count = 0
    while time_t <=
          (@trainset_times[@ss_name][nil]['maximum_cycle_delay'] -
           @trainset_times[@ss_name][nil]['minimum_cycle_delay'] +
           1 + @tg_params['showing_additional_cycles']) * @cycle_time do
      if line_count == @tg_params['main_grid_interval_relative_to_sub']
        line_count = 0
      end
      c_elm = REXML::Element.new('line')
      attr = {
        'x1' =>
        (@tg_params['margin_left'] + @tg_params['text_station_name_width'] +
         @tg_params['text_mileage_figure_width'] +
         (@tg_params['hspace_name_mileage_traingraph'] * 2) +
         (time_t * @tg_params['pixels_per_minute'] / 60.0)).to_s,
        'x2' =>
        (@tg_params['margin_left'] + @tg_params['text_station_name_width'] +
         @tg_params['text_mileage_figure_width'] +
         (@tg_params['hspace_name_mileage_traingraph'] * 2) +
         (time_t * @tg_params['pixels_per_minute'] / 60.0)).to_s,
        'y1' => @line_data[nil]['height_top'],
        'y2' => @line_data[nil]['height_bottom'] }
      if line_count == 0
        attr['stroke'] = @tg_params['line_main_grid_stroke']
        attr['stroke-width'] = @tg_params['line_main_grid_stroke_width']
        attr['stroke-dasharray'] =
          @tg_params['line_main_grid_stroke_dasharray']
      else
        attr['stroke'] = @tg_params['line_sub_grid_stroke']
        attr['stroke-width'] = @tg_params['line_sub_grid_stroke_width']
        attr['stroke-dasharray'] =
          @tg_params['line_sub_grid_stroke_dasharray']
      end
      attr.each { |key, val| c_elm.add_attribute(key, val) }
      tgo_root.add_element(c_elm)
      time_t += @tg_params['sub_grid_interval_in_seconds']
      line_count += 1
    end
  end

  #===================================================#
  # SVG subprogram: platform name and horizontal line #
  #===================================================#
  def svg_write_platform_names_and_lines(tgo_root,ldata)
    if ldata.size > 0
      for ipl in ldata do
        @height += @next_vspace_to_add
        if @sw_line_height_to_set
          @line_data[nil]['height_top'] = @height
          @sw_line_height_to_set = false
        end
        @line_data[nil]['height_bottom'] = @height
        ipl[1]=@height
        @next_vspace_to_add = @tg_params['vspace_platform_platform']
        c_elm = REXML::Element.new('text')
        { 'x' => (@tg_params['margin_left'] +
                  @tg_params['text_station_name_width']).to_s,
          'y' => @height.to_s, 'text-anchor' => 'end',
          'font-family' => @tg_params['text_platform_name_font_family'],
          'font-size' => @tg_params['text_platform_name_font_size'].to_s,
          'font-weight' => @tg_params['text_platform_name_font_weight'],
          'fill' => @tg_params['text_platform_name_fill'] }.each { |key, val|
          c_elm.add_attribute(key, val) }
        tgo_root.add_element(c_elm)
        c_elm.add_text(ipl[0])
        cdelay_base =
          @trainset_times[@ss_name][nil]['maximum_cycle_delay'] -
          @trainset_times[@ss_name][nil]['minimum_cycle_delay'] + 1
        c_elm = REXML::Element.new('line')
        { 'x1' => (@tg_params['margin_left'] +
                   @tg_params['text_station_name_width'] +
                   @tg_params['hspace_name_mileage_traingraph']).to_s,
          'x2' => (@width - @tg_params['margin_right']).to_s,
          'y1' => @height.to_s, 'y2' => @height.to_s,
          'stroke' => @tg_params['line_platform_stroke'],
          'stroke-width' => @tg_params['line_platform_stroke_width'],
          'stroke-dasharray' => @tg_params['line_platform_stroke_dasharray']
        }.each { |key, val| c_elm.add_attribute(key, val) }
        tgo_root.add_element(c_elm)
      end
      @next_vspace_to_add = @tg_params['vspace_platform_station']
    end
  end

  #==============================#
  # Line parsing for SVG drawing #
  #==============================#
  def parse_line_for_svg()
    @line_data.clear
    @line_data = { nil => { 'total_length' => 0.0, 'line_separations' => 0,
                            'section_separations' => 0 , 'height_top' => 0,
                            'height_bottom' => 0 }}
    line_xp = 'rtss_input_data/create[@class=\'sujic_line\']/data'
    @doc.elements.each(line_xp) do |d_elem|
      l_name=d_elem.attributes['name']
      p "line " + l_name + " parsing starts..."
      l_e_xp = 'rtss_input_data/create[@class=\'sujic_line\']/data[@name=\''
      l_e_xp += l_name + '\']/line_element'
      @line_data[l_name] = Array.new
      first = true
      size_m1 = 0
      position_start = 0.0
      first_to_show = false
      section_open = false
      @doc.elements.each(l_e_xp) do |elem|
        show_plat=!!(elem.elements['diagram_info'].attributes[
                      'show_from_platforms']!='false')
        if show_plat || first
          if first
            first=false
            if show_plat
              first_to_show = true
            end
          else
            p "section ends, length = " +
              @line_data[l_name][size_m1]['length'].to_s
            @line_data[nil]['total_length'] +=
              @line_data[l_name][size_m1]['length']
            size_m1 += 1
            @line_data[nil]['section_separations'] += 1
            section_open = false
          end
          if elem.attributes['to'] != 'nil'
            p elem.attributes['from'] + ": starting a section at " +
              position_start.to_s
            @line_data[l_name][size_m1] = {
              'start' => elem.attributes['from'],
              'startpos' => position_start, 'length' => 0.0,
              'stations' => { elem.attributes['from'] => [position_start] },
              'start_plats' => Array.new, 'end_plats' => Array.new}
            section_open = true
          end
          if show_plat
            stn_xp = 'rtss_input_data/create[@class=\'sujic_station\']'
            stn_xp += '/data[@name=\''
            stn_xp += elem.attributes['from']
            stn_xp += '\']/physical_platforms/physical_platform'
            @doc.elements.each(stn_xp) do |e_s|
              p elem.attributes['from'] + " plat name: " +
                e_s.attributes['name']
              if elem.attributes['to'] != 'nil'
                @line_data[l_name][size_m1]['start_plats'] <<
                  [e_s.attributes['name'],0.0]
              else
                @line_data[l_name][-1]['end_plats'] <<
                  [e_s.attributes['name'],0.0]
              end
            end
          end
        end
        if elem.attributes['to'] != 'nil'
          i_s_dist = elem.elements['diagram_info']
                     .attributes['inter_station_distance'].to_f
          p "i_s_dist = " + i_s_dist.to_s
          @line_data[l_name][size_m1]['end'] = elem.attributes['to']
          position_start += i_s_dist
          @line_data[l_name][size_m1]['endpos'] = position_start
          p " " + elem.attributes['to'] + " at " + position_start.to_s
          @line_data[l_name][size_m1]['length'] += i_s_dist
          @line_data[l_name][size_m1]['stations'][elem.attributes['to']] = 
            [@line_data[l_name][size_m1]['endpos']]
        end
      end
      if section_open
        p "section ends, length = " +
          @line_data[l_name][-1]['length'].to_s
        @line_data[nil]['total_length'] += @line_data[l_name][-1]['length']
        section_open = false
      end
      p "line ends, last section length = " +
        @line_data[l_name][-1]['length'].to_s
      @line_data[nil]['line_separations'] += 1
      @line_data[nil]['section_separations'] -= 1
    end
    @line_data[nil]['line_separations'] -= 1
    return true
  end

  #======================#
  # Output XML data file #
  #======================#
  def output_xml_data_file (outfile,xmldoc)
    iox=File.open(outfile,"w")
    pretty_formatter=REXML::Formatters::Pretty.new
    pretty_formatter.write(xmldoc, iox)
    iox.close
  end

  #============================================#
  # Populate the last part of an editor window #
  #============================================#
  def last_of_editor_window (
        parent_win, st_edt, xpth_lbx, _xrt, _listl, l_btn, lbox, ar_el,
        l_sc, v_x, l_discard_edit = nil)
    if _xrt == nil
      docroot = @doc
    else
      docroot = _xrt
    end
    if l_sc != nil
      btn_sc = TkButton.new(st_edt) do
        text(_("OK --- Reflect changes and close the window"))
        pack(fill: :x)
      end
      txt_dc = _("Cancel --- Discard changes and close this window")
    else
      txt_dc = _("Close the window")
    end
    btn_dc = TkButton.new(st_edt) do
      text(txt_dc)
      pack(fill: :x)
    end
    l_cls = lambda {
      p "editor closing..."
      st_edt.destroy
      parent_win.grab_set
      if xpth_lbx != nil
        lbox.state('normal') if (v_x == "new")
        populate_listbox(lbox, xpth_lbx, _listl, nil, ar_el, docroot)
        lbox.state('disabled') if (v_x == "new")
        l_btn.call if l_btn != nil
      end
    }
    l_wm = lambda {
      _mbspfm = _("Discard edits?")
      _str_m = _("Really discard all edits?")
      ans_ixml = Tk.messageBox(
        "icon" => "warning",
        "type" => "okcancel",
        "default" => "cancel",
        "title" => sprintf(_mbspfm),
        "message" => _str_m )
      ans_ixml == "ok" }
    if l_sc == nil
      btn_dc.command ( proc do l_cls.call; end )
    else
      btn_sc.command ( proc do l_cls.call if l_sc.call; end )
      btn_dc.command (
        proc do
          if l_wm.call
            l_discard_edit.call if l_discard_edit != nil
            l_cls.call
          end
        end )
    end
    st_edt.protocol(
      "WM_DELETE_WINDOW",
      proc {
        if l_wm.call
          l_discard_edit.call if l_discard_edit != nil
          l_cls.call
        end
      })
  end

  #=======================#
  # Open an editor window #
  #=======================#
  def open_editor_window (parent_win, title_str)
    # Create a child window
    st_edt = TkToplevel.new(parent_win, title: title_str)

    # Prohibit operations on the parent window
    st_edt.grab_set

    # Return child window
    return st_edt
  end

  #===============================================#
  # Listbox creation for top level windows (tabs) #
  #===============================================#
  def populate_toplevel_listbox (lbx, cls_a, sel_str)
    p 'populate_toplevel_listbox start'

    # Empty listbox
    lbx.clear()

    # Pick up necessary elements and put the string to the listbox
    sel = -1
    pthx = 'rtss_input_data/create[@class=\'' + cls_a[0] + '\']'
    for idx in 1...cls_a.size do
      pthx += '/under[@class=\'' + cls_a[idx][0] + '\' and @name=\'' +
              cls_a[idx][1] + '\']'
    end
    for idx in 1...cls_a.size do
      pthx += '/..'
    end
    @doc.elements.each( pthx ) do |element|
      p "List insertion: " + element.attributes['class'] + "::" +
        element.elements['data'].attributes['name']
      if element.elements['data'].attributes['name'] == sel_str
        sel = lbx.size()
      end
      lbx.insert 'end', element.elements['data'].attributes['name']
    end
    if sel >= 0
      lbx.selection_set(sel)
    end
    p 'populate_toplevel_listbox end'
  end

  # Listbox creation
  # @return None.
  # @param lbx
  def populate_listbox(lbx, xpth_lbx, _listl, e_sel, ar_el, _doc,
                       sw_nil=false, l_ok = nil)
    p 'populate_listbox start'
    p xpth_lbx

    _listl = lambda {|elm| elm.attributes['name']} if _listl == nil

    # Empty listbox
    lbx.clear()
    ar_el.clear() if ar_el != nil

    # Pick up necessary elements and put the string to the listbox
    sel = -1
    if sw_nil == true
      sel = lbx.size() if e_sel == "nil"
      lbx.insert 'end', "nil"
      ar_el.push(nil) if ar_el != nil
    end
    _doc.elements.each( xpth_lbx ) do |element|
      sw_ok = true
      if l_ok
        sw_ok = l_ok.call(element)
      end
      if sw_ok
        p "List insertion: " + _listl.call(element)
        sel = lbx.size() if element == e_sel
        lbx.insert 'end', _listl.call(element)
        ar_el.push element if ar_el != nil
      else
        p "List insertion refused by l_ok: " + _listl.call(element)
      end
    end
    lbx.selection_set(sel) if sel >= 0
    p 'populate_listbox end'
  end

  # Station data editor: Creating a window for station data editing
  # @return None.
  # @param parent_win [TkToplevel] Parent window.
  # @param lbox [TkListbox] Listbox containing this train type.
  # @param ar_pel [xx] xx.
  # @param v_x [xx] xx.
  # @param l_btn [xx] xx.
  # @param e_cur [REXML::Element] Current element if it already exists.
  def station_editor (parent_win, lbox, ar_pel, _de, v_x, l_btn, e_cur,
                      e_parent)
    # e_parent = nil
    p 'station_editor start'

    #-----------------#
    # Initializing... #
    #-----------------#
    st_edt=open_editor_window(
      parent_win, _("\"Sujic\"") + ":: " + _("Editing a station"))

    # Data extracting
    if v_x == "edit"
      e_xc = e_cur.deep_clone
      e_nc = e_cur
    elsif v_x == "copy"
      e_xc = e_cur.deep_clone
      e_xc.elements['data'].add_attribute('name','')
      e_nc = e_xc
    elsif v_x == "new"
      e_xc = REXML::Element.new('create')
      {'class'=>'sujic_station'}.each{|_k,_v| e_xc.add_attribute(_k,_v)}
      e_nc = e_xc
      e_ncd = e_xc.add_element('data',{'name'=>'','class'=>'x_station'})
      e_ncdpp = e_ncd.add_element('physical_platforms')
      e_ncdcn = e_ncd.add_element('connectors')
      e_ncdsm = e_ncd.add_element('shunting_movements')
      e_ncdeh = e_ncd.add_element('event_headways')
      e_ncdlp = e_ncd.add_element('platforms')
      e_ncdit = e_ncd.add_element('intratimes')
    else
      e_nc = e_cur
    end
    st_wh = Hash.new

    l_discard_edit = lambda {
      if v_x == "edit"
        e_cur.parent.insert_after(e_cur,e_xc)
        e_cur.parent.delete(e_cur)
      end
    }

    # Tab representation (notebook widget)
    st_wh[:ttn] = Tk::Tile::Notebook.new(st_edt) do
      place('x' => 10, 'y' => 10)
    end
    st_wh[:ttn].pack

    # Create a frame for selecting insertion position.
    # Unnecessary if v_x == "edit"
    if v_x != "edit"
      st_wh[:pss] = TkFrame.new(st_wh[:ttn])
      st_wh[:ttn].add(st_wh[:pss], :text => _('Where to insert'))
      # edit_select_window data
      h_pss = Hash.new
      h_pss_l01 = TkFrame.new(st_wh[:pss]).pack(side: :top, fill: :x)
      h_pss = insert_position_controller_radiobuttons(h_pss_l01)
      h_pss_l02 = TkFrame.new(st_wh[:pss]).pack(side: :top, fill: :x)
      ar_el = Array.new
      h_pss_lb_sb = TkScrollbar.new(h_pss_l02)
      h_pss_lb = TkListbox.new(
        h_pss_l02, 'height' => 10, 'width' => 50, 'selectmode' => 'single',
        'yscrollcommand' => proc{|*arg|h_pss_lb_sb.set(*arg)})
      h_pss_lb_sb.command(proc{|*arg| h_pss_lb.yview(*arg)})
      h_pss_lb_sb.pack('side' => 'right', 'fill' => 'y')
      h_pss_lb.pack('side' => 'left', 'fill' => 'both')
      populate_listbox(h_pss_lb,_de['xpath_lb'],_de['listlambda'],nil,ar_el,
                       @doc)
      if h_pss_lb.size < 1
        h_pss.each do |key,val|
          next if key == nil || key == "lframe"
          val.state("disabled")
        end
      end
    end

    # Create frames for physical platform (:ppl), connector (:con), shunting
    # movement (:shm), event_headway (:ehw), logical platform (:lpl) and
    # intratime (:itm) objects. Unnecessary when v_x == "move"
    if v_x != "move"
      st_wh[:gen] = TkFrame.new(st_wh[:ttn])
      st_wh[:ttn].add(st_wh[:gen], :text => _('General'))
      st_wh[:ptn] = TkFrame.new(st_wh[:ttn])
      st_wh[:ttn].add(st_wh[:ptn], :text => _('Patterns'))
      st_wh[:pcn] = Tk::Tile::Notebook.new(st_wh[:ptn]) do
        pack(fill: :x)
      end
      st_wh[:pcn].pack
      st_wh[:pcp] = TkFrame.new(st_wh[:pcn])
      st_wh[:pcn].add(st_wh[:pcp], :text => _('Departing/arriving platforms'))
      st_wh[:pcu] = TkFrame.new(st_wh[:pcn])
      st_wh[:pcn].add(st_wh[:pcu], :text => _('Up side'))
      st_wh[:pcd] = TkFrame.new(st_wh[:pcn])
      st_wh[:pcn].add(st_wh[:pcd], :text => _('Down side'))
      st_wh[:plf] = TkFrame.new(st_wh[:ttn])
      st_wh[:ttn].add(st_wh[:plf], :text => _('Platforms'))
      st_wh[:pln] = Tk::Tile::Notebook.new(st_wh[:plf]) do
        place('x' => 10, 'y' => 10)
      end
      st_wh[:pln].pack
      st_wh[:ppl] = TkFrame.new(st_wh[:pln])
      st_wh[:pln].add(st_wh[:ppl], :text => _('Physical'))
      st_wh[:lpl] = TkFrame.new(st_wh[:pln])
      st_wh[:pln].add(st_wh[:lpl], :text => _('Logical'))
      st_wh[:tpf] = TkFrame.new(st_wh[:ttn])
      st_wh[:ttn].add(st_wh[:tpf], :text => _('Train paths'))
      st_wh[:tpn] = Tk::Tile::Notebook.new(st_wh[:tpf]) do
        place('x' => 10, 'y' => 10)
      end
      st_wh[:tpn].pack
      st_wh[:con] = TkFrame.new(st_wh[:tpn])
      st_wh[:tpn].add(st_wh[:con], :text => _('Interstation connectors'))
      st_wh[:shm] = TkFrame.new(st_wh[:tpn])
      st_wh[:tpn].add(st_wh[:shm], :text => _('Shunting movements'))
      st_wh[:tmf] = TkFrame.new(st_wh[:ttn])
      st_wh[:ttn].add(st_wh[:tmf], :text => _('Times'))
      st_wh[:tmn] = Tk::Tile::Notebook.new(st_wh[:tmf]) do
        place('x' => 10, 'y' => 10)
      end
      st_wh[:tmn].pack
      st_wh[:ehw] = TkFrame.new(st_wh[:tmn])
      st_wh[:tmn].add(st_wh[:ehw], :text => _('Event headways'))
      st_wh[:itm] = TkFrame.new(st_wh[:tmn])
      st_wh[:tmn].add(st_wh[:itm], :text => _('Transfer times'))
      # change tab states
      if v_x == "new"
        st_wh[:ttn].itemconfigure(st_wh[:ptn],:state => "normal")
        st_wh[:ttn].itemconfigure(st_wh[:plf],:state => "disabled")
        st_wh[:ttn].itemconfigure(st_wh[:tpf],:state => "disabled")
        st_wh[:ttn].itemconfigure(st_wh[:tmf],:state => "disabled")
      else
        st_wh[:ttn].itemconfigure(st_wh[:ptn],:state => "disabled")
        st_wh[:ttn].itemconfigure(st_wh[:plf],:state => "normal")
        st_wh[:ttn].itemconfigure(st_wh[:tpf],:state => "normal")
        st_wh[:ttn].itemconfigure(st_wh[:tmf],:state => "normal")
      end
      # edit_select_window data
      h_ppl = Hash.new
      h_cnl = Hash.new
      h_sml = Hash.new
      h_ehl = Hash.new
      h_lpl = Hash.new
      h_itl = Hash.new

      #----------------------------#
      # Populating "general" frame #
      #----------------------------#
      st_wh[:gen_l01] = TkFrame.new(st_wh[:gen]).pack(side: :top, fill: :x)
      st_wh[:gen_l01_lbl] = TkLabel.new(st_wh[:gen_l01])
      st_wh[:gen_l01_lbl].text=_("Station name: ")
      st_wh[:gen_l01_lbl].pack(side: :left)
      st_wh[:gen_l01_ent] = TkEntry.new(st_wh[:gen_l01])
      st_wh[:gen_l01_ent].width(45)
      st_wh[:gen_l01_ent].value = e_nc.elements['data'].attributes['name']
      st_wh[:gen_l01_ent].pack(side: :left, fill: :x)
      st_wh[:gen_l02] = TkFrame.new(st_wh[:gen]).pack(side: :top, fill: :x)
      st_wh[:gen_l02_lbl] = TkLabel.new(st_wh[:gen_l02])
      st_wh[:gen_l02_lbl].text = _("Use patterns?")
      st_wh[:gen_l02_lbl].pack(side: :left)
      st_wh[:gen_l02_v] = TkVariable.new('yes')
      if e_nc.elements['data'].attributes['h_plat'] == nil ||
         e_nc.elements['data'].attributes['h_plat'] == ''
        str_h_p = '2_2s'
      else
        str_h_p = e_nc.elements['data'].attributes['h_plat']
      end
      st_wh[:ptp_v] = TkVariable.new(str_h_p)
      if e_nc.elements['data'].attributes['h_up'] == nil ||
         e_nc.elements['data'].attributes['h_up'] == ''
        str_h_p = 'stop'
      else
        str_h_p = e_nc.elements['data'].attributes['h_up']
      end
      st_wh[:ptu_v] = TkVariable.new(str_h_p)
      if e_nc.elements['data'].attributes['h_down'] == nil ||
         e_nc.elements['data'].attributes['h_down'] == ''
        str_h_p = 'stop'
      else
        str_h_p = e_nc.elements['data'].attributes['h_down']
      end
      st_wh[:ptd_v] = TkVariable.new(str_h_p)
      st_wh[:gen_l02_rb1] = TkRadioButton.new(st_wh[:gen_l02]) do
        text _("Yes")
        value "yes"
        variable st_wh[:gen_l02_v]
        select
        command(
          proc{
            puts "st_wh 2014: Stn_Use_Pattern: Yes, P:" + st_wh[:ptp_v] +
                 "|U:" + st_wh[:ptu_v] + "|D:" + st_wh[:ptd_v]
            st_wh[:ttn].itemconfigure(st_wh[:ptn],:state => "normal")
            st_wh[:ttn].itemconfigure(st_wh[:plf],:state => "disabled")
            st_wh[:ttn].itemconfigure(st_wh[:tpf],:state => "disabled")
            st_wh[:ttn].itemconfigure(st_wh[:tmf],:state => "disabled")
          })
        pack(side: :left)
      end
      st_wh[:gen_l02_rb2] = TkRadioButton.new(st_wh[:gen_l02]) do
        text _("No")
        value "no"
        variable st_wh[:gen_l02_v]
        command(
          proc{
            puts "st_wh 2015: Stn_NOT_Use_Patterns"
            st_wh[:ttn].itemconfigure(st_wh[:ptn],:state => "disabled")
            st_wh[:ttn].itemconfigure(st_wh[:plf],:state => "normal")
            st_wh[:ttn].itemconfigure(st_wh[:tpf],:state => "normal")
            st_wh[:ttn].itemconfigure(st_wh[:tmf],:state => "normal")
          })
        pack(side: :left)
      end

      #------------------------------------------------#
      # Populating patterns frame: platforms child tab #
      #------------------------------------------------#
      st_wh[:ptp_l01] = TkFrame.new(st_wh[:pcp]).pack(side: :top, fill: :x)
      st_wh[:ptp_l01_lbl] = TkLabel.new(st_wh[:ptp_l01])
      st_wh[:ptp_l01_lbl].text =
        _("Select one departing / arriving platforms data set:")
      st_wh[:ptp_l01_lbl].pack(side: :left)
      st_wh[:ptp_l02] = TkFrame.new(st_wh[:pcp]).pack(side: :top, fill: :x)
      st_wh[:ptp_ttn] = Tk::Tile::Notebook.new(st_wh[:ptp_l02]) do
        place('x'=>10, 'y'=>10)
      end
      st_wh[:ptp_ttn].pack(side: :top, anchor: :nw, fill: :both)
      st_wh[:ptp_ttn_p1] = TkFrame.new(st_wh[:ptp_ttn])
      st_wh[:ptp_ttn_p2] = TkFrame.new(st_wh[:ptp_ttn])
      st_wh[:ptp_ttn].add(st_wh[:ptp_ttn_p1], :text => '1')
      st_wh[:ptp_ttn].add(st_wh[:ptp_ttn_p2], :text => '2')
      st_wh[:ptp_ttn].itemconfigure(st_wh[:ptp_ttn_p2], :state => 'disabled')
      st_wh[:ptp_l_2_2s] = TkFrame.new(st_wh[:ptp_ttn_p1]).pack(
        side: :top, fill: :x)
      st_wh[:ptp_l_2_2s_rb1] = TkRadioButton.new(st_wh[:ptp_l_2_2s]) do
        text _("Two side platforms")
        value "2_2s"
        variable st_wh[:ptp_v]
        command(
          proc{
            puts "st_wh 2047: Stn_Pattern: P:" + st_wh[:ptp_v] +
                 "|U:" + st_wh[:ptu_v] + "|D:" + st_wh[:ptd_v]
          })
        pack(side: :left)
      end
      st_wh[:ptp_l_2_1i] = TkFrame.new(st_wh[:ptp_ttn_p1]).pack(
        side: :top, fill: :x)
      st_wh[:ptp_l_2_1i_rb1] = TkRadioButton.new(st_wh[:ptp_l_2_1i]) do
        text _("One island platform w/ two platforms (tracks)")
        value "2_1i"
        variable st_wh[:ptp_v]
        command(
          proc{
            puts "st_wh 2058: Stn_Pattern: P:" + st_wh[:ptp_v] +
                 "|U:" + st_wh[:ptu_v] + "|D:" + st_wh[:ptd_v]
          })
        pack(side: :left)
      end
      st_wh[:ptp_l_4_2i] = TkFrame.new(st_wh[:ptp_ttn_p1]).pack(
        side: :top, fill: :x)
      st_wh[:ptp_l_4_2i_rb1] = TkRadioButton.new(st_wh[:ptp_l_4_2i]) do
        text _("Two island platforms w/ four platforms")
        value "4_2i"
        variable st_wh[:ptp_v]
        command(
          proc{
            puts "st_wh 2069: Stn_Pattern: P:" + st_wh[:ptp_v] +
                 "|U:" + st_wh[:ptu_v] + "|D:" + st_wh[:ptd_v]
          })
        pack(side: :left)
      end
      st_wh[:ptp_l_4_2s] = TkFrame.new(st_wh[:ptp_ttn_p1]).pack(
        side: :top, fill: :x)
      st_wh[:ptp_l_4_2s_rb1] = TkRadioButton.new(st_wh[:ptp_l_4_2s]) do
        text _("Two side platforms w/ two passing loops")
        value "4_2s"
        variable st_wh[:ptp_v]
        command(
          proc{
            puts "st_wh 2080: Stn_Pattern: P:" + st_wh[:ptp_v] +
                 "|U:" + st_wh[:ptu_v] + "|D:" + st_wh[:ptd_v]
          })
        pack(side: :left)
      end
      st_wh[:ptp_l_3_2s] = TkFrame.new(st_wh[:ptp_ttn_p1]).pack(
        side: :top, fill: :x)
      st_wh[:ptp_l_3_2s_rb1] = TkRadioButton.new(st_wh[:ptp_l_3_2s]) do
        text _("Two side platforms w/ one common loop")
        value "3_2s"
        variable st_wh[:ptp_v]
        command(
          proc{
            puts "st_wh 2091: Stn_Pattern: P:" + st_wh[:ptp_v] +
                 "|U:" + st_wh[:ptu_v] + "|D:" + st_wh[:ptd_v]
          })
        pack(side: :left)
      end
      st_wh[:ptp_l_2_2s2t] = TkFrame.new(st_wh[:ptp_ttn_p1]).pack(
        side: :top, fill: :x)
      st_wh[:ptp_l_2_2s2t_rb1] = TkRadioButton.new(st_wh[:ptp_l_2_2s2t]) do
        text _("Two side platforms w/ two tail tracks")
        value "2_2s2t"
        variable st_wh[:ptp_v]
        command(
          proc{
            puts "st_wh 2102: Stn_Pattern: P:" + st_wh[:ptp_v] +
                 "|U:" + st_wh[:ptu_v] + "|D:" + st_wh[:ptd_v]
          })
        pack(side: :left)
      end

      #-----------------------------------------------------------#
      # Populating patterns frame: Up side / Down side child tabs #
      #-----------------------------------------------------------#
      st_wh[:ptu_l01] = TkFrame.new(st_wh[:pcu]).pack(side: :top, fill: :x)
      st_wh[:ptu_l01_lbl] = TkLabel.new(st_wh[:ptu_l01])
      st_wh[:ptu_l01_lbl].text =
        _("Select one up side connectors data set:")
      st_wh[:ptu_l01_lbl].pack(side: :left)
      st_wh[:ptu_l02] = TkFrame.new(st_wh[:pcu]).pack(side: :top, fill: :x)
      st_wh[:ptu_ttn] = Tk::Tile::Notebook.new(st_wh[:ptu_l02]) do
        place('x'=>10, 'y'=>10)
      end
      st_wh[:ptu_ttn].pack(side: :top, anchor: :nw, fill: :both)
      st_wh[:ptu_ttn_p1] = TkFrame.new(st_wh[:ptu_ttn])
      st_wh[:ptu_ttn_p2] = TkFrame.new(st_wh[:ptu_ttn])
      st_wh[:ptu_ttn].add(st_wh[:ptu_ttn_p1], :text => '1')
      st_wh[:ptu_ttn].add(st_wh[:ptu_ttn_p2], :text => '2')
      st_wh[:ptu_ttn].itemconfigure(st_wh[:ptu_ttn_p2], :state => 'disabled')
      st_wh[:ptd_l01] = TkFrame.new(st_wh[:pcd]).pack(side: :top, fill: :x)
      st_wh[:ptd_l01_lbl] = TkLabel.new(st_wh[:ptd_l01])
      st_wh[:ptd_l01_lbl].text =
        _("Select one down side connectors data set:")
      st_wh[:ptd_l01_lbl].pack(side: :left)
      st_wh[:ptd_l02] = TkFrame.new(st_wh[:pcd]).pack(side: :top, fill: :x)
      st_wh[:ptd_ttn] = Tk::Tile::Notebook.new(st_wh[:ptd_l02]) do
        place('x'=>10, 'y'=>10)
      end
      st_wh[:ptd_ttn].pack(side: :top, anchor: :nw, fill: :both)
      st_wh[:ptd_ttn_p1] = TkFrame.new(st_wh[:ptd_ttn])
      st_wh[:ptd_ttn_p2] = TkFrame.new(st_wh[:ptd_ttn])
      st_wh[:ptd_ttn].add(st_wh[:ptd_ttn_p1], :text => '1')
      st_wh[:ptd_ttn].add(st_wh[:ptd_ttn_p2], :text => '2')
      st_wh[:ptd_ttn].itemconfigure(st_wh[:ptd_ttn_p2], :state => 'disabled')
      st_wh[:ptu_l_stop] = TkFrame.new(st_wh[:ptu_ttn_p1]).pack(
        side: :top, fill: :x)
      st_wh[:ptu_l_stop_rb1] = TkRadioButton.new(st_wh[:ptu_l_stop]) do
        text _("Buffer stop(s) only")
        value "stop"
        variable st_wh[:ptu_v]
        command(
          proc{
            puts "st_wh 2129: Stn_Pattern: P:" + st_wh[:ptp_v] +
                 "|U:" + st_wh[:ptu_v] + "|D:" + st_wh[:ptd_v]
          })
        pack(side: :left)
      end
      st_wh[:ptd_l_stop] = TkFrame.new(st_wh[:ptd_ttn_p1]).pack(
        side: :top, fill: :x)
      st_wh[:ptd_l_stop_rb1] = TkRadioButton.new(st_wh[:ptd_l_stop]) do
        text _("Buffer stop(s) only")
        value "stop"
        variable st_wh[:ptd_v]
        command(
          proc{
            puts "st_wh 2140: Stn_Pattern: P:" + st_wh[:ptp_v] +
                 "|U:" + st_wh[:ptu_v] + "|D:" + st_wh[:ptd_v]
          })
        pack(side: :left)
      end
      st_wh[:ptu_l_2_wx] = TkFrame.new(st_wh[:ptu_ttn_p1]).pack(
        side: :top, fill: :x)
      st_wh[:ptu_l_2_wx_rb1] = TkRadioButton.new(st_wh[:ptu_l_2_wx]) do
        text _("Double track with scissors crossings")
        value "2_wx"
        variable st_wh[:ptu_v]
        command(
          proc{
            puts "st_wh 2151: Stn_Pattern: P:" + st_wh[:ptp_v] +
                 "|U:" + st_wh[:ptu_v] + "|D:" + st_wh[:ptd_v]
          })
        pack(side: :left)
      end
      st_wh[:ptd_l_2_wx] = TkFrame.new(st_wh[:ptd_ttn_p1]).pack(
        side: :top, fill: :x)
      st_wh[:ptd_l_2_wx_rb1] = TkRadioButton.new(st_wh[:ptd_l_2_wx]) do
        text _("Double track with scissors crossings")
        value "2_wx"
        variable st_wh[:ptd_v]
        command(
          proc{
            puts "st_wh 2162: Stn_Pattern: P:" + st_wh[:ptp_v] +
                 "|U:" + st_wh[:ptu_v] + "|D:" + st_wh[:ptd_v]
          })
        pack(side: :left)
      end
      st_wh[:ptu_l_2_nco] = TkFrame.new(st_wh[:ptu_ttn_p1]).pack(
        side: :top, fill: :x)
      st_wh[:ptu_l_2_nco_rb1] = TkRadioButton.new(st_wh[:ptu_l_2_nco]) do
        text _("Double track w/o crossovers")
        value "2_nco"
        variable st_wh[:ptu_v]
        command(
          proc{
            puts "st_wh 2956: Stn_Pattern: P:" + st_wh[:ptp_v] +
                 "|U:" + st_wh[:ptu_v] + "|D:" + st_wh[:ptd_v]
          })
        pack(side: :left)
      end
      st_wh[:ptd_l_2_nco] = TkFrame.new(st_wh[:ptd_ttn_p1]).pack(
        side: :top, fill: :x)
      st_wh[:ptd_l_2_nco_rb1] = TkRadioButton.new(st_wh[:ptd_l_2_nco]) do
        text _("Double track w/o crossovers")
        value "2_nco"
        variable st_wh[:ptd_v]
        command(
          proc{
            puts "st_wh 2969: Stn_Pattern: P:" + st_wh[:ptp_v] +
                 "|U:" + st_wh[:ptu_v] + "|D:" + st_wh[:ptd_v]
          })
        pack(side: :left)
      end

      #------------------------------------#
      # Populating physical platform frame #
      #------------------------------------#
      h_ppl['new_b'] = h_ppl['new_a'] = nil
      h_ppl['new_e'] = e_nc.elements['data/physical_platforms']
      h_ppl['xpath_lb'] = 'data/physical_platforms/physical_platform'
      h_ppl['xpath_root'] = e_nc
      # h_ppl['xpath_lb'] =
      #   'rtss_input_data/create[@class=\'sujic_station\']/data[@name=\'' +
      #   ( e_nc.elements['data'].attributes['name'] == nil ? "nil" :
      #       e_nc.elements['data'].attributes['name'] ) + 
      #   '\']/physical_platforms/physical_platform'
      h_ppl['listlambda'] = lambda{|e|
        e.elements['../..'].attributes['name'] + "::" + e.attributes['name']}
      h_ppl['lb_otext'] = _("Physical platform")
      h_ppl['xref_cd'] = Hash.new
      h_ppl['xref_cd'][nil] = _("Physical platform")
      _pp_key2 = _("Interstation connector")
      # _pp2_val0 =
      #   'rtss_input_data/create[@class=\'sujic_station\']/data[@name=\'' +
      #   ( e_nc.elements['data'].attributes['name'] == nil ? "nil" :
      #       e_nc.elements['data'].attributes['name'] ) +
      #   '\']/connectors/connector/platform'
      _pp2_val0 = 'data/connectors/connector/platform'
      _pp2_h = {['.','name'] => ['.','name']}

      h_ppl['xref_cd'][_pp_key2] = [[_pp2_val0,_pp2_h,'../',e_nc]]
      _pp_key4 = _("Event headway")
      # _pp4_val0 =
      #   'rtss_input_data/create[@class=\'sujic_station\']/data[@name=\'' +
      #   ( e_nc.elements['data'].attributes['name'] == nil ? "nil" :
      #       e_nc.elements['data'].attributes['name'] ) +
      #   '\']/event_headways/event_headway'
      _pp4_val0 = 'data/event_headways/event_headway'
      _pp4_1h = {['.','name'] => ['from_group/event','platform']}
      _pp4_2h = {['.','name'] => ['from_group/event','from']}
      _pp4_3h = {['.','name'] => ['from_group/event','to']}
      _pp4_4h = {['.','name'] => ['to_group/event','platform']}
      _pp4_5h = {['.','name'] => ['to_group/event','from']}
      _pp4_6h = {['.','name'] => ['to_group/event','to']}
      h_ehl['listlambda'] = lambda{|e|
        e_f = e.elements['from_group/event[1]']
        e_t = e.elements['to_group/event[1]']
        e.elements['../..'].attributes['name'] + "::E:" +
          e_f.attributes['platform'] + "::" + e_f.attributes['status'] + "<" +
          e_f.attributes['direction'] + ">==" + e_t.attributes['platform'] +
          "::" + e_t.attributes['status'] + "<" +
          e_t.attributes['direction'] + ">"}
      l_eh_pp = h_ehl['listlambda']
      h_ppl['xref_cd'][_pp_key4] =
        [[_pp4_val0,_pp4_1h,l_eh_pp,e_nc],[_pp4_val0,_pp4_2h,l_eh_pp,e_nc],
         [_pp4_val0,_pp4_3h,l_eh_pp,e_nc],[_pp4_val0,_pp4_4h,l_eh_pp,e_nc],
         [_pp4_val0,_pp4_5h,l_eh_pp,e_nc],[_pp4_val0,_pp4_6h,l_eh_pp,e_nc]]
      _pp_key1 = _("Line")
      _pp1_val00 = 'rtss_input_data/create[@class=\'sujic_line\']/data' +
                   '/line_element/run_time_constraints/run_time_constraint'
      _pp1_0h = {['../..','name'] => ['../..','from'],
                 ['.','name'] => ['.','from_platform']}
      _pp1_val10 = 'rtss_input_data/create[@class=\'sujic_line\']/data' +
                   '/line_element/run_time_constraints/run_time_constraint'
      _pp1_1h = {['../..','name'] => ['../..','to'],
                 ['.','name'] => ['.','to_platform']}
      h_ppl['xref_cd'][_pp_key1] = [[_pp1_val00,_pp1_0h,'../../..'],
                                    [_pp1_val10,_pp1_1h,'../../..']]
      _pp_key3 = _("Train")
      _pp3_val0 = 'rtss_input_data/create[@class=\'sujic_train\']/data' +
                  '/train_time'
      _pp3_h = {['../..','name'] => ['.','station'],
                ['.','name'] => ['.','platform']}
      h_ppl['xref_cd'][_pp_key3] = [[_pp3_val0,_pp3_h,'../']]
      h_ppl['editor'] = lambda { |a,b,c,d,e,f,g,h|
        physical_platform_editor(a,b,c,d,e,f,g,h) }
      pp_f1m = TkFrame.new(st_wh[:ppl]).pack
      editor_select_frame(parent_win,st_edt,pp_f1m,h_ppl)

      #-----------------------------------------#
      # Populating interstation connector frame #
      #-----------------------------------------#
      h_cnl['new_b'] = h_cnl['new_a'] = nil
      h_cnl['new_e'] = e_nc.elements['data/connectors']
      # h_cnl['xpath_lb'] =
      #   'rtss_input_data/create[@class=\'sujic_station\']/data[@name=\'' +
      #   ( e_nc.elements['data'].attributes['name'] == nil ? "nil" :
      #       e_nc.elements['data'].attributes['name'] ) +
      #   '\']/connectors/connector'
      h_cnl['xpath_lb'] = 'data/connectors/connector'
      h_cnl['xpath_root'] = e_nc
      h_cnl['listlambda'] = lambda{|e|
        sw_up = true if e.attributes['side'] == 'up'
        sw_up = false if e.attributes['side'] == 'down'
        s_x = ""
        s_x += "<<" + e.attributes['name'] + ":" if sw_up
        s_x += e.elements['../..'].attributes['name']
        s_x += ":" + e.attributes['name'] + ">>" unless sw_up
        next s_x}
      h_cnl['lb_otext'] = _("Interstation connector")
      h_cnl['xref_cd'] = Hash.new
      h_cnl['xref_cd'][nil] = _("Interstation connector")
      _cn_key1 = _("Line")
      _cn1_val00 = 'rtss_input_data/create[@class=\'sujic_line\']/data' +
                   '/line_element/roads/road'
      _cn1_0h = {['../..','name'] => ['../..','from'],
                 ['.','name'] => ['.','from_connector']}
      _cn1_val10 = 'rtss_input_data/create[@class=\'sujic_line\']/data' +
                   '/line_element/run_time_constraints/run_time_constraint'
      _cn1_1h = {['../..','name'] => ['../..','to'],
                 ['.','name'] => ['.','to_connector']}
      h_cnl['xref_cd'][_cn_key1] = [[_cn1_val00,_cn1_0h,'.'],
                                    [_cn1_val10,_cn1_1h,'.']]
      _cn_key4 = _("Event headway")
      # _cn4_val0 =
      #   'rtss_input_data/create[@class=\'sujic_station\']/data[@name=\'' +
      #   ( e_nc.elements['data'].attributes['name'] == nil ? "nil" :
      #       e_nc.elements['data'].attributes['name'] ) +
      #   '\']/event_headways/event_headway'
      _cn4_val0 = 'data/event_headways/event_headway'
      _cn4_1h = {['.','name'] => ['from_group/event','from_connector']}
      _cn4_2h = {['.','name'] => ['from_group/event','to_connector']}
      _cn4_3h = {['.','name'] => ['to_group/event','from_connector']}
      _cn4_4h = {['.','name'] => ['to_group/event','to_connector']}
      l_eh_cn = h_ehl['listlambda']
      h_cnl['xref_cd'][_cn_key4] =
        [[_cn4_val0,_cn4_1h,l_eh_cn,e_nc],[_cn4_val0,_cn4_2h,l_eh_cn,e_nc],
         [_cn4_val0,_cn4_3h,l_eh_cn,e_nc],[_cn4_val0,_cn4_4h,l_eh_cn,e_nc]]
      h_cnl['editor'] = lambda { |a,b,c,d,e,f,g,h|
        connector_editor(a,b,c,d,e,f,g,h) }
      cn_f1m = TkFrame.new(st_wh[:con]).pack
      editor_select_frame(parent_win,st_edt,cn_f1m,h_cnl,e_nc)

      #------------------------------------#
      # Populating shunting movement frame #
      #------------------------------------#
      h_sml['new_b'] = h_sml['new_a'] = nil
      h_sml['new_e'] = e_nc.elements['data/shunting_movements']
      # h_sml['xpath_lb'] =
      #   'rtss_input_data/create[@class=\'sujic_station\']/data[@name=\'' +
      #   ( e_nc.elements['data'].attributes['name'] == nil ? "nil" :
      #       e_nc.elements['data'].attributes['name'] ) +
      #   '\']/shunting_movements/shunt'
      h_sml['xpath_lb'] = 'data/shunting_movements/shunt'
      h_sml['xpath_root'] = e_nc
      h_sml['listlambda'] = lambda{|e|
        e.elements['../..'].attributes['name'] + "::S:" +
          e.attributes['from'] + "->" + e.attributes['to']}
      h_sml['lb_otext'] = _("Shunting movement")
      h_sml['xref_cd'] = Hash.new
      h_sml['xref_cd'][nil] = _("Shunting movement")
      h_sml['editor'] = lambda { |a,b,c,d,e,f,g,h|
        connector_editor(a,b,c,d,e,f,g,h) }
      sm_f1m = TkFrame.new(st_wh[:shm]).pack
      editor_select_frame(parent_win,st_edt,sm_f1m,h_sml,e_nc)

      #--------------------------------#
      # Populating event headway frame #
      #--------------------------------#
      h_ehl['new_b'] = h_ehl['new_a'] = nil
      h_ehl['new_e'] = e_nc.elements['data/event_headways']
      # h_ehl['xpath_lb'] =
      #   'rtss_input_data/create[@class=\'sujic_station\']/data[@name=\'' +
      #   ( e_nc.elements['data'].attributes['name'] == nil ? "nil" :
      #       e_nc.elements['data'].attributes['name'] ) +
      #   '\']/event_headways/event_headway'
      h_ehl['xpath_lb'] = 'data/event_headways/event_headway'
      h_ehl['xpath_root'] = e_nc
      ## h_ehl['listlambda'] defined at physical platform part
      h_ehl['lb_otext'] = _("Event headway")
      h_ehl['xref_cd'] = Hash.new
      h_ehl['xref_cd'][nil] = _("Event headway")
      h_ehl['editor'] = lambda { |a,b,c,d,e,f,g,h|
        connector_editor(a,b,c,d,e,f,g,h) }
      eh_f1m = TkFrame.new(st_wh[:ehw]).pack
      editor_select_frame(parent_win,st_edt,eh_f1m,h_ehl,e_nc)

      #-------------------------------------#
      # Populating (logical) platform frame #
      #-------------------------------------#
      h_lpl['new_b'] = h_lpl['new_a'] = nil
      h_lpl['new_e'] = e_nc.elements['data/platforms']
      # h_lpl['xpath_lb'] =
      #   'rtss_input_data/create[@class=\'sujic_station\']/data[@name=\'' +
      #   ( e_nc.elements['data'].attributes['name'] == nil ? "nil" :
      #       e_nc.elements['data'].attributes['name'] ) +
      #   '\']/platforms/platform'
      h_lpl['xpath_lb'] = 'data/platforms/platform'
      h_lpl['xpath_root'] = e_nc
      h_lpl['listlambda'] = lambda{|e|
        e.elements['../..'].attributes['name'] + ":=:" + e.attributes['name']}
      h_lpl['lb_otext'] = _("Logical platform")
      h_lpl['xref_cd'] = Hash.new
      h_lpl['xref_cd'][nil] = _("Logical platform")
      _lp_key1 = _("Transfer time")
      # _lp1_val0 =
      #   'rtss_input_data/create[@class=\'sujic_station\']/data[@name=\'' +
      #   ( e_nc.elements['data'].attributes['name'] == nil ? "nil" :
      #       e_nc.elements['data'].attributes['name'] ) +
      #   '\']/intratimes/intratime'
      _lp1_val0 = 'data/intratimes/intratime'
      _lp1_1h = {['.','name'] => ['.','from']}
      _lp1_2h = {['.','name'] => ['.','to']}
      h_itl['listlambda'] = lambda{|e|
        e.elements['../..'].attributes['name'] + "::I:" +
          e.attributes['from'] + "=>" + e.attributes['to']}
      l_it_lp = h_itl['listlambda']
      h_lpl['xref_cd'][_lp_key1] =
        [[_lp1_val0,_lp1_1h,l_it_lp,e_nc],[_lp1_val0,_lp1_2h,l_it_lp,e_nc]]
      _lp_key2 = _("Trainset")
      _lp2_val0 = 'rtss_input_data/create[@class=\'sujic_trainset\']/data' +
                  '/train_time'
      _lp2_h = {['../..','name'] => ['.','station'],
                ['.','name'] => ['.','platform']}
      h_lpl['xref_cd'][_lp_key2] = [[_lp2_val0,_lp2_h,'../']]
      h_lpl['editor'] = lambda { |a,b,c,d,e,f,g,h|
        platform_editor(a,b,c,d,e,f,g,h) }
      lp_f1m = TkFrame.new(st_wh[:lpl]).pack
      editor_select_frame(parent_win,st_edt,lp_f1m,h_lpl,e_nc)

      #----------------------------#
      # Populating intratime frame #
      #----------------------------#
      h_itl['new_b'] = h_itl['new_a'] = nil
      h_itl['new_e'] = e_nc.elements['data/intratimes']
      # h_itl['xpath_lb'] =
      #   'rtss_input_data/create[@class=\'sujic_station\']/data[@name=\'' +
      #   ( e_nc.elements['data'].attributes['name'] == nil ? "nil" :
      #       e_nc.elements['data'].attributes['name'] ) +
      #   '\']/intratimes/intratime'
      h_itl['xpath_lb'] = 'data/intratimes/intratime'
      h_itl['xpath_root'] = e_nc
      ## h_itl['listlambda'] defined at logical platform part
      h_itl['lb_otext'] = _("Transfer time")
      h_itl['xref_cd'] = Hash.new
      h_itl['xref_cd'][nil] = _("Transfer time")
      h_itl['editor'] = lambda { |a,b,c,d,e,f,g,h|
        intratime_editor(a,b,c,d,e,f,g,h) }
      it_f1m = TkFrame.new(st_wh[:itm]).pack
      editor_select_frame(parent_win,st_edt,it_f1m,h_itl,e_nc)
    end

    l_sc = lambda {
      tt_xa = 'rtss_input_data/create[@class=\'sujic_station\']/data['
      tt_xa += '@name=\'' + st_wh[:gen_l01_ent].value + '\']'
      if st_wh[:gen_l01_ent].value == "nil"
        Tk.messageBox(
          "title" => _("\"Nil\" not allowed as station name"),
          "message" => _("You cannot use \"nil\" as a name of a station."))
        return false
      elsif v_x != "move" && st_wh[:gen_l01_ent].value == ""
        Tk.messageBox(
          "title" => _("Station name not specified?"),
          "message" => _("Please specify a station name") )
        return false
      elsif ( v_x == "move" ) && ( st_wh[:gen_l01_ent] != nil ) &&
            ( st_wh[:gen_l01_ent].value !=
              e_nc.elements['data'].attributes['name'] )
        Tk.messageBox(
          "title" => _("Name change not allowed"),
          "message" => _("Element name should not be changed when moving"))
        return false
      elsif ( v_x != "move" ) &&
            ( st_wh[:gen_l01_ent].value != e_nc.attributes['name'] ) &&
            ( @doc.elements[tt_xa] != nil )
        Tk.messageBox(
          "title" => _("Name already in use"),
          "message" => _("This station name is already used: ") + "\n" +
                       st_wh[:gen_l01_ent].value )
        return false
      elsif ( v_x == "new" ) && ( h_pss_lb.size < 1 ) &&
            ( _de['new_b'] == nil )
        # Bug: first element insertion destination not set
        Tk.messageBox(
          "title" => _("Internal error") + " L03225",
          "message" =>
          sprintf(_("Internal error %1$s has happened. Report this as bug."),
                  "L03225"))
        return false
      elsif ( v_x == "move" ) && ( h_pss_lb.size < 1 )
        # Bug: move selected when no element is present
        Tk.messageBox(
          "title" => _("Internal error") + " L03232",
          "message" =>
          sprintf(_("Internal error %1$s has happened. Report this as bug."),
                  "L03232"))
        return false
      elsif ( v_x == "new" || v_x == "copy" ) &&
            h_pss_lb.curselection.size > 1
        # Curselection should be 1
        Tk.messageBox(
          "title" => _("Multiple insertion destinations"),
          "message" =>
          _("Select only one insertion destination element."))
        return false
      elsif ( ( v_x == "new" && h_pss_lb.size >= 1 ) || v_x == "copy" ) &&
            h_pss_lb.curselection.size < 1
        # Curselection should be 1
        Tk.messageBox(
          "title" => _("Select an insertion destination"),
          "message" =>
          _("Select one insertion destination element from the list below."))
        return false
      else
        ## Checks before inserting...
        ## Checks before inserting ended...
        if st_wh[:gen_l02_v] == 'yes'
          ## Using patterns ...
          ## Discarding current contents in elements <physical_platforms>,
          ## <connectors>, <shunting_movements>, <event_headways>,
          ## <platforms> and <intratimes>
          discard_current_contents_in_SFC_lambda = lambda {
            ['physical_platforms','connectors','shunting_movements',
             'event_headways','platforms','intratimes'].each do |tg|
              e_nc.elements.each('data/' + tg) do |e_del|
                e_del.each do |x|
                  x.parent.delete(x)
                end
              end
            end
          }
          show_unable_combin_msg_in_SFC_lambda = lambda {
            res = Tk.messageBox(
              "parent" => st_edt,
              "title" => _(
                "Error: Unavailable pattern combination for station"),
              "message" => sprintf(
                _("Error: This combination of patterns for %1$s \"%2$s\"" +
                  " is either not yet ready or unavailable.") + "\n" +
                _("Departing/arriving platforms") + ": %3$s\n" +
                _("Up side") + ": %4$s\n" + _("Down side") + ": %5$s",
                _de['lb_otext'], st_wh[:gen_l01_ent].value,
                st_wh[:ptp_v], st_wh[:ptu_v], st_wh[:ptd_v]))
            return false
          }
          create_plat12_in_SFC_lambda = lambda {
            e_nc.elements['data/physical_platforms'].add_element(
              'physical_platform',
              {'class'=>'x_physical_platform','name'=>'1'})
            e_nc.elements['data/physical_platforms'].add_element(
              'physical_platform',
              {'class'=>'x_physical_platform','name'=>'2'})
            e_nc.elements['data/platforms'].add_element(
              'platform',
              {'class'=>'x_platform','name'=>'1','physical'=>'1'})
            e_nc.elements['data/platforms'].add_element(
              'platform',
              {'class'=>'x_platform','name'=>'2','physical'=>'2'})
            e_nc.elements['data/platforms'].add_element(
              'platform',
              {'class'=>'x_platform','name'=>'E','physical'=>'nil',
               'entrance'=>'true','exit'=>'true'})
            e_nc.elements['data/intratimes'].add_element(
              'intratime',
              {'class'=>'x_intra_sta','from'=>'1','to'=>'E','time'=>'0'})
            e_nc.elements['data/intratimes'].add_element(
              'intratime',
              {'class'=>'x_intra_sta','from'=>'E','to'=>'1','time'=>'0'})
            e_nc.elements['data/intratimes'].add_element(
              'intratime',
              {'class'=>'x_intra_sta','from'=>'2','to'=>'E','time'=>'0'})
            e_nc.elements['data/intratimes'].add_element(
              'intratime',
              {'class'=>'x_intra_sta','from'=>'E','to'=>'2','time'=>'0'})
          }
          create_plat34_after12_in_SFC_lambda = lambda {
            e_nc.elements['data/physical_platforms'].add_element(
              'physical_platform',
              {'class'=>'x_physical_platform','name'=>'3'})
            e_nc.elements['data/physical_platforms'].add_element(
              'physical_platform',
              {'class'=>'x_physical_platform','name'=>'4'})
            e_nc.elements['data/platforms'].add_element(
              'platform',
              {'class'=>'x_platform','name'=>'3','physical'=>'3'})
            e_nc.elements['data/platforms'].add_element(
              'platform',
              {'class'=>'x_platform','name'=>'4','physical'=>'4'})
            e_nc.elements['data/intratimes'].add_element(
              'intratime',
              {'class'=>'x_intra_sta','from'=>'3','to'=>'E','time'=>'0'})
            e_nc.elements['data/intratimes'].add_element(
              'intratime',
              {'class'=>'x_intra_sta','from'=>'E','to'=>'3','time'=>'0'})
            e_nc.elements['data/intratimes'].add_element(
              'intratime',
              {'class'=>'x_intra_sta','from'=>'4','to'=>'E','time'=>'0'})
            e_nc.elements['data/intratimes'].add_element(
              'intratime',
              {'class'=>'x_intra_sta','from'=>'E','to'=>'4','time'=>'0'})
          }
          create_down_con_plat12_wx_in_SFC_lambda = lambda {
            e_x = e_nc.elements['data/connectors'].add_element(
              'connector',{'name'=>'DD','side'=>'down'})
            e_x.add_element(
              'platform',
              {'name'=>'1','down_movement'=>'true','up_movement'=>'false'})
            e_x.add_element(
              'platform',
              {'name'=>'2','down_movement'=>'true','up_movement'=>'false'})
            e_x = e_nc.elements['data/connectors'].add_element(
              'connector',{'name'=>'DU','side'=>'down'})
            e_x.add_element(
              'platform',
              {'name'=>'1','up_movement'=>'true','down_movement'=>'false'})
            e_x.add_element(
              'platform',
              {'name'=>'2','up_movement'=>'true','down_movement'=>'false'})
          }
          create_up_con_plat12_nco_in_SFC_lambda = lambda {
            e_x = e_nc.elements['data/connectors'].add_element(
              'connector',{'name'=>'UD','side'=>'up'})
            e_x.add_element(
              'platform',
              {'name'=>'1','down_movement'=>'true','up_movement'=>'false'})
            e_x = e_nc.elements['data/connectors'].add_element(
              'connector',{'name'=>'UU','side'=>'up'})
            e_x.add_element(
              'platform',
              {'name'=>'2','up_movement'=>'true','down_movement'=>'false'})
          }
          create_down_con_plat12_nco_in_SFC_lambda = lambda {
            e_x = e_nc.elements['data/connectors'].add_element(
              'connector',{'name'=>'DD','side'=>'down'})
            e_x.add_element(
              'platform',
              {'name'=>'1','down_movement'=>'true','up_movement'=>'false'})
            e_x = e_nc.elements['data/connectors'].add_element(
              'connector',{'name'=>'DU','side'=>'down'})
            e_x.add_element(
              'platform',
              {'name'=>'2','up_movement'=>'true','down_movement'=>'false'})
          }
          create_up_con_plat12_wx_in_SFC_lambda = lambda {
            e_x = e_nc.elements['data/connectors'].add_element(
              'connector',{'name'=>'UD','side'=>'up'})
            e_x.add_element(
              'platform',
              {'name'=>'1','down_movement'=>'true','up_movement'=>'false'})
            e_x.add_element(
              'platform',
              {'name'=>'2','down_movement'=>'true','up_movement'=>'false'})
            e_x = e_nc.elements['data/connectors'].add_element(
              'connector',{'name'=>'UU','side'=>'up'})
            e_x.add_element(
              'platform',
              {'name'=>'1','up_movement'=>'true','down_movement'=>'false'})
            e_x.add_element(
              'platform',
              {'name'=>'2','up_movement'=>'true','down_movement'=>'false'})
          }
          create_down_con_plat1234_nco_in_SFC_lambda = lambda {
            e_x = e_nc.elements['data/connectors'].add_element(
              'connector',{'name'=>'DD','side'=>'down'})
            e_x.add_element(
              'platform',
              {'name'=>'1','down_movement'=>'true','up_movement'=>'false'})
            e_x.add_element(
              'platform',
              {'name'=>'2','down_movement'=>'true','up_movement'=>'false'})
            e_x = e_nc.elements['data/connectors'].add_element(
              'connector',{'name'=>'DU','side'=>'down'})
            e_x.add_element(
              'platform',
              {'name'=>'3','up_movement'=>'true','down_movement'=>'false'})
            e_x.add_element(
              'platform',
              {'name'=>'4','up_movement'=>'true','down_movement'=>'false'})
          }
          create_up_con_plat1234_nco_in_SFC_lambda = lambda {
            e_x = e_nc.elements['data/connectors'].add_element(
              'connector',{'name'=>'UD','side'=>'up'})
            e_x.add_element(
              'platform',
              {'name'=>'1','down_movement'=>'true','up_movement'=>'false'})
            e_x.add_element(
              'platform',
              {'name'=>'2','down_movement'=>'true','up_movement'=>'false'})
            e_x = e_nc.elements['data/connectors'].add_element(
              'connector',{'name'=>'UU','side'=>'up'})
            e_x.add_element(
              'platform',
              {'name'=>'3','up_movement'=>'true','down_movement'=>'false'})
            e_x.add_element(
              'platform',
              {'name'=>'4','up_movement'=>'true','down_movement'=>'false'})
          }
          if st_wh[:ptp_v] == "2_1i" || st_wh[:ptp_v] == "2_2s"
            if st_wh[:ptu_v] == "stop"
              if st_wh[:ptd_v] == "2_wx"
                discard_current_contents_in_SFC_lambda.call
                create_plat12_in_SFC_lambda.call
                create_down_con_plat12_wx_in_SFC_lambda.call
              elsif st_wh[:ptd_v] == "2_nco"
                discard_current_contents_in_SFC_lambda.call
                create_plat12_in_SFC_lambda.call
                create_down_con_plat12_nco_in_SFC_lambda.call
              else
                return show_unable_combin_msg_in_SFC_lambda.call
              end
            elsif st_wh[:ptu_v] == "2_wx"
              if st_wh[:ptd_v] == "2_wx"
                discard_current_contents_in_SFC_lambda.call
                create_plat12_in_SFC_lambda.call
                create_up_con_plat12_wx_in_SFC_lambda.call
                create_down_con_plat12_wx_in_SFC_lambda.call
              elsif st_wh[:ptd_v] == "2_nco"
                discard_current_contents_in_SFC_lambda.call
                create_plat12_in_SFC_lambda.call
                create_up_con_plat12_wx_in_SFC_lambda.call
                create_down_con_plat12_nco_in_SFC_lambda.call
              elsif st_wh[:ptd_v] == "stop"
                discard_current_contents_in_SFC_lambda.call
                create_plat12_in_SFC_lambda.call
                create_up_con_plat12_wx_in_SFC_lambda.call
              else
                return show_unable_combin_msg_in_SFC_lambda.call
              end
            elsif st_wh[:ptu_v] == "2_nco"
              if st_wh[:ptd_v] == "2_wx"
                discard_current_contents_in_SFC_lambda.call
                create_plat12_in_SFC_lambda.call
                create_up_con_plat12_nco_in_SFC_lambda.call
                create_down_con_plat12_wx_in_SFC_lambda.call
              elsif st_wh[:ptd_v] == "2_nco"
                discard_current_contents_in_SFC_lambda.call
                create_plat12_in_SFC_lambda.call
                create_up_con_plat12_nco_in_SFC_lambda.call
                create_down_con_plat12_nco_in_SFC_lambda.call
              else
                return show_unable_combin_msg_in_SFC_lambda.call
              end
            else
              return show_unable_combin_msg_in_SFC_lambda.call
            end
          elsif st_wh[:ptp_v] == "4_2i" && st_wh[:ptu_v] == "2_nco" &&
                st_wh[:ptd_v] == "2_nco"
            discard_current_contents_in_SFC_lambda.call
            create_plat12_in_SFC_lambda.call
            create_plat34_after12_in_SFC_lambda.call
            create_down_con_plat1234_nco_in_SFC_lambda.call
            create_up_con_plat1234_nco_in_SFC_lambda.call
          else
            return show_unable_combin_msg_in_SFC_lambda.call
          end
        end
        if v_x == "new" || v_x == "copy"
          # Insert edited e_nc
          if st_wh[:gen_l01_ent].value !=
             e_nc.elements['data'].attributes['name']
            str_on = e_nc.elements['data'].attributes['name']
            e_nc.elements['data'].add_attribute(
              'name', st_wh[:gen_l01_ent].value)
          end
          p "###### -- insert -- ###### [v_x = " + v_x + "]"
          p "###### -- insert -- ###### [h_pss_lb.size = " +
            h_pss_lb.size.to_s + "]"
          if ( h_pss_lb.size < 1 )
            elm_ins = _de['new_b']
            sw_after = false
          else
            p "###### -- insert -- ###### [cursel = " +
              h_pss_lb.curselection[0].to_s + "]"
            sw_after = false if h_pss[nil] == "before"
            sw_after = true if h_pss[nil] == "after"
            elm_ins = ar_el[h_pss_lb.curselection[0]]
            p "###### -- insert -- ######"
            puts elm_ins
            p "###### -- insert after? -- ######"
            p "---- after" if sw_after
            p "---- before" unless sw_after
            p "###### -- insert end -- ######"
          end
          if sw_after
            elm_ins.parent.insert_after(elm_ins,e_nc)
          else
            elm_ins.parent.insert_before(elm_ins,e_nc)
          end
          # n_e.add_element('data',{'name' => tt_ah['name'],
          #                         'class' => 'x_traintype',
          #                         'seatings' => tt_ah['seatings'].to_s,
          #                         'length' => tt_ah['length'].to_s,
          #                         'coaches' => tt_ah['coaches'].to_s,
          #                         'capacity' => tt_ah['capacity'].to_s})
        elsif v_x == "move" && e_cur != ar_el[h_pss_lb.curselection[0]]
          elm_cut = e_cur.parent.delete(e_cur)
          p "#### ELM_CUT is not equal to E_CUR ####" if elm_cut != e_cur
          sw_after = false if pcrbs[nil] == "before"
          sw_after = true if pcrbs[nil] == "after"
          elm_ins = ar_el[h_pss_lb.curselection[0]]
          elm_ins.parent.insert_after(elm_ins, elm_cut) if sw_after
        elsif v_x != "move" ## Edit only
          tt_ah.each{|key, val|
            elm_c.add_attribute(key, val.to_s) if key != "name"}
          if tt_ah['name'] != elm_c.attributes['name']
            str_on = elm_c.attributes['name']
            elm_c.add_attribute('name', tt_ah['name'])
            [('rtss_input_data/create[@class=\'sujic_trainset\']/using[' +
              '@class=\'sujic_traintype\']'),
             ('rtss_input_data/create[@class=\'sujic_speedclass\']' +
              '/data/coupled_trainset/traintype')].each do |tt_xb|
              @doc.elements.each(tt_xb) do |elem|
                elem.add_attribute('name',tt_ah['name']) if
                  elem.attributes['name'] == str_on
              end
            end
          end
        end
      end
    }

    # Close this window
    last_of_editor_window(
      parent_win,st_edt,_de['xpath_lb'],nil,_de['listlambda'],l_btn,lbox,
      ar_pel,l_sc,v_x,l_discard_edit)
    p 'station_editor end'

  end

  #===================#
  # Line data editing #
  #===================#
  def line_editor(parent_win, lbox, ar_pel, _de, v_x, l_btn, e_cur,
                  e_parent)
    # e_parent: nil
    p "line_editor starting..."

    #-----------------#
    # Initializing... #
    #-----------------#
    # Window open
    ln_edt = open_editor_window(
      parent_win, _("\"Sujic\"") + ":: " + _("Editing a line"))

    # Data extracting
    if v_x == "edit"
      e_xc = e_cur.deep_clone
      e_nc = e_cur
    elsif v_x == "copy"
      e_xc = e_cur.deep_clone
      e_xc.elements['data'].add_attribute('name','')
      e_nc = e_xc
    elsif v_x == "new"
      e_xc = REXML::Element.new('create')
      {'class'=>'sujic_line'}.each{|_k,_v| e_xc.add_attribute(_k,_v)}
      e_xc.add_element('data',{'name' => '','class' => 'x_line'})
      e_nc = e_xc
    else
      e_nc = e_cur
    end
    ln_wh = Hash.new

    l_discard_edit = lambda {
      if v_x == "edit"
        e_cur.parent.insert_after(e_cur,e_xc)
        e_cur.parent.delete(e_cur)
      end
    }

    # Tab representation (notebook widget)
    ln_wh[:ttn] = Tk::Tile::Notebook.new(ln_edt) do
      place('x'=>10, 'y'=>10)
    end
    ln_wh[:ttn].pack

    # Create a frame for selecting insertion position.
    # Unnecessary if v_x == "edit"
    if v_x != "edit"
      ln_wh[:pss] = TkFrame.new(ln_wh[:ttn])
      ln_wh[:ttn].add(ln_wh[:pss], :text => _('Where to insert'))
      # edit_select_window data
      h_pss = Hash.new
      h_pss_l01 = TkFrame.new(ln_wh[:pss]).pack(side: :top, fill: :x)
      h_pss = insert_position_controller_radiobuttons(h_pss_l01)
      h_pss_l02 = TkFrame.new(ln_wh[:pss]).pack(side: :top, fill: :x)
      ar_el = Array.new
      h_pss_lb_sb = TkScrollbar.new(h_pss_l02)
      h_pss_lb = TkListbox.new(
        h_pss_l02, 'height' => 10, 'width' => 50, 'selectmode' => 'single',
        'yscrollcommand' => proc{|*arg|h_pss_lb_sb.set(*arg)})
      h_pss_lb_sb.command(proc{|*arg| h_pss_lb.yview(*arg)})
      h_pss_lb_sb.pack('side' => 'right', 'fill' => 'y')
      h_pss_lb.pack('side' => 'left', 'fill' => 'both')
      populate_listbox(h_pss_lb,_de['xpath_lb'],_de['listlambda'],nil,ar_el,
                       @doc)
      if h_pss_lb.size < 1
        h_pss.each do |key,val|
          next if key == nil || key == "lframe"
          val.state("disabled")
        end
      end
    end

    # Create a frame for main editing (line name change, etc.)
    ln_wh[:lnm] = TkFrame.new(ln_wh[:ttn])

    # Create a frame for "using" station objects
    ln_wh[:stu] = TkFrame.new(ln_wh[:ttn])

    # Adding frames to the notebook widget
    ln_wh[:ttn].add(ln_wh[:lnm], :text => _('General'))
    ln_wh[:ttn].add(ln_wh[:stu], :text => _('Stations list'))

    #--------------------------#
    # Populating general frame #
    #--------------------------#
    ln_wh[:lnm_l01] = TkFrame.new(ln_wh[:lnm]).pack(side: :top, fill: :x)
    ln_wh[:lnm_l01_lbl] = TkLabel.new(ln_wh[:lnm_l01])
    ln_wh[:lnm_l01_lbl].text=_("Line name: ")
    ln_wh[:lnm_l01_lbl].pack(side: :left)
    ln_wh[:lnm_l01_ent] = TkEntry.new(ln_wh[:lnm_l01])
    ln_wh[:lnm_l01_ent].width(45)
    ln_wh[:lnm_l01_ent].value = e_nc.elements['data'].attributes['name']
    ln_wh[:lnm_l01_ent].pack(side: :left, fill: :x)

    #----------------------------------#
    # Populating "using" station frame #
    #----------------------------------#
    ln_wh[:uch] = Hash.new
    ln_wh[:uch]['new_b'] = e_nc.elements['data']
    ln_wh[:uch]['new_a'] = ln_wh[:uch]['new_e'] = nil
    # ln_wh[:uch]['xpath_lb'] =
    #   'rtss_input_data/create[@class=\'sujic_line\']/data[@name=\'' +
    #   e_nc.elements['data'].attributes['name'] +
    #   '\']/../using[@type=\'station\']'
    ln_wh[:uch]['xpath_lb'] = 'using[@type=\'station\']'
    ln_wh[:uch]['xpath_root'] = e_nc
    ln_wh[:uch]['listlambda'] = lambda{|e|
      if e == nil
        "nil"
      else
        e.attributes['name']
      end }
    ln_wh[:uch]['lb_otext'] = _("Station")
    ln_wh[:uch]['xref_cd'] = Hash.new
    ln_wh[:uch]['xref_cd'][nil] = _("Station")
    ln_wh[:uch]['editor'] = lambda {|a,b,c,d,e,f,g,h|
      line_element_editor(a,b,c,d,e,f,g,h) }
    ln_wh[:uch]['deletor'] = lambda {|a,b,c,d,e,f,g,h|
      line_element_editor(a,b,c,d,e,f,g,h) }
    ln_wh[:uch]['no_cpmv'] = true
    ln_wh[:ucm] = TkFrame.new(ln_wh[:stu]).pack
    editor_select_frame(parent_win,ln_edt,ln_wh[:ucm],ln_wh[:uch],e_nc)

    l_sc = lambda {
      tt_xa = 'rtss_input_data/create[@class=\'sujic_line\']/data['
      tt_xa += '@name=\'' + ln_wh[:lnm_l01_ent].value + '\']'
      if ln_wh[:lnm_l01_ent].value == "nil"
        Tk.messageBox(
          "parent" => ln_edt,
          "title" => _("\"Nil\" not allowed as line name"),
          "message" => _("You cannot use \"nil\" as a name of a line.") )
        return false
      elsif v_x != "move" && ln_wh[:lnm_l01_ent].value == ""
        Tk.messageBox(
          "parent" => ln_edt,
          "title" => _("Line name not specified?"),
          "message" => _("Please specify a line name.") )
        return false
      elsif ( v_x == "move" ) &&
            ( ln_wh[:lnm_l01_ent].value !=
              e_nc.elements['data'].attributes['name'] )
        Tk.messageBox(
          "parent" => ln_edt,
          "title" => _("Name change not allowed"),
          "message" => _("Element name should not be changed when moving"))
        return false
      elsif ( v_x != "move" ) &&
            ( ln_wh[:lnm_l01_ent].value !=
              e_nc.elements['data'].attributes['name'] &&
              @doc.elements[tt_xa] != nil )
        Tk.messageBox(
          "parent" => ln_edt,
          "title" => _("Name already in use"),
          "message" => _("This line name is already used: ") + "\n" +
                       ln_wh[:lnm_l01_ent].value )
        return false
      elsif ( v_x == "new" ) && ( h_pss_lb.size < 1 ) &&
            ( _de['new_b'] == nil )
        # Bug: first element insertion destination not set
        Tk.messageBox(
          "parent" => ln_edt,
          "title" => _("Internal error") + " L03574",
          "message" =>
          sprintf(_("Internal error %1$s has happened. Report this as bug."),
                  "L03574"))
        return true
      elsif ( v_x == "move" ) && ( h_pss_lb.size < 1 )
        # Bug: move selected when no element is present
        Tk.messageBox(
          "parent" => ln_edt,
          "title" => _("Internal error") + " L03582",
          "message" =>
          sprintf(_("Internal error %1$s has happened. Report this as bug."),
                  "L03582"))
        return true
      elsif ( v_x == "new" || v_x == "copy" ) &&
            h_pss_lb.curselection.size > 1
        # Curselection should be 1
        Tk.messageBox(
          "parent" => ln_edt,
          "title" => _("Multiple insertion destinations"),
          "message" =>
          _("Select only one insertion destination element."))
        return false
      elsif ( ( v_x == "new" && h_pss_lb.size >= 1 ) || v_x == "copy" ) &&
            h_pss_lb.curselection.size < 1
        # Curselection should be 1
        Tk.messageBox(
          "parent" => ln_edt,
          "title" => _("Select an insertion destination"),
          "message" => _("Select one insertion destination element " +
                         "from the list in tab \"Where to insert\"."))
        return false
      elsif e_nc.elements.to_a('using') == nil ||
            e_nc.elements.to_a('using').size < 2
        Tk.messageBox(
          "parent" => ln_edt,
          "title" => _("Insufficient station(s)"),
          "message" => _("Not enough number of station(s) could be found."))
        return false
      else
        ## Checks before inserting...
        # if e_nc.elements['data'].elements.to_a('coupled_trainset') != nil
        #   e_nc.elements['data'].elements.each('coupled_trainset') do |ctck|
        #     if ctck.elements.to_a('traintype') == nil ||
        #        ctck.elements.to_a('traintype').size < 1
        #       Tk.messageBox(
        #         "title" => _("Coupled trainset empty"),
        #         "message" => _("Empty coupled trainset element was found."))
        #       return false
        #     end
        #   end
        # end
        ## Checks before inserting ended...
        if v_x == "new" || v_x == "copy"
          if ln_wh[:lnm_l01_ent].value !=
             e_nc.elements['data'].attributes['name']
            str_on = e_nc.elements['data'].attributes['name']
            e_nc.elements['data'].add_attribute(
              'name', ln_wh[:lnm_l01_ent].value)
          end
          p "###### -- insert -- ###### [v_x = " + v_x + "]"
          p "###### -- insert -- ###### [h_pss_lb.size = " +
            h_pss_lb.size.to_s + "]"
          if ( h_pss_lb.size < 1 )
            elm_ins = _de['new_b']
            sw_after = false
          else
            p "###### -- insert -- ###### [cursel = " +
              h_pss_lb.curselection[0].to_s + "]"
            sw_after = false if h_pss[nil] == "before"
            sw_after = true if h_pss[nil] == "after"
            elm_ins = ar_el[h_pss_lb.curselection[0]]
            p "###### -- insert -- ######"
            puts elm_ins
            p "###### -- insert after? -- ######"
            p "---- after" if sw_after
            p "---- before" unless sw_after
            p "###### -- insert end -- ######"
          end
          if sw_after
            elm_ins.parent.insert_after(elm_ins,e_nc)
          else
            elm_ins.parent.insert_before(elm_ins,e_nc)
          end
        elsif v_x == "move" && e_cur != ar_el[h_pss_lb.curselection[0]]
          elm_cut = e_cur.parent.delete(e_cur)
          p "#### ELM_CUT is not equal to E_CUR ####" if elm_cut != e_cur
          sw_after = false if h_pss[nil] == "before"
          sw_after = true if h_pss[nil] == "after"
          elm_ins = ar_el[h_pss_lb.curselection[0]]
          elm_ins.parent.insert_after(elm_ins, elm_cut) if sw_after
        elsif v_x != "move" ## Edit only
          if ln_wh[:lnm_l01_ent].value != e_nc.attributes['name']
            str_on = e_nc.attributes['name']
            e_nc.elements['data'].add_attribute(
              'name', ln_wh[:lnm_l01_ent].value)
            # [('rtss_input_data/create[@class=\'sujic_trainset\']/using[' +
            #   '@class=\'sujic_traintype\']'),
            #  ('rtss_input_data/create[@class=\'sujic_speedclass\']' +
            #   '/data/coupled_trainset/traintype')].each do |tt_xb|
            #   @doc.elements.each(tt_xb) do |elem|
            #     elem.add_attribute('name',tt_ah['name']) if
            #       elem.attributes['name'] == str_on
            #   end
            # end
          end
        end
        return true
      end
    }

    # Close this window
    last_of_editor_window(
      parent_win,ln_edt,_de['xpath_lb'],_de['xpath_root'],_de['listlambda'],
      l_btn,lbox,ar_pel,l_sc,v_x,l_discard_edit)
    p 'line_editor end'

  end

  #=========================================#
  # Creating a window for line name editing #
  #=========================================#
  def line_element_editor(
        parent_win, lbox, ar_pel, _de, v_x, l_btn, e_cur, e_parent)
    p "line_element_editor starting..."

    if v_x == "new" || v_x == "edit"
      line_element_editor_nwed(parent_win, lbox, ar_pel,
                              _de, v_x, l_btn, e_cur, e_parent)
    elsif v_x == "delete"
      line_element_deletor(parent_win, lbox, ar_pel,
                           _de, v_x, l_btn, e_cur, e_parent)
    else
      # Bug: move selected when no element is present
      Tk.messageBox(
        "parent" => parent_win,
        "title" => _("Internal error") + " L03363",
        "message" =>
        sprintf(_("Internal error %1$s has happened. Report this as bug."),
                 "L03363"))
    end
  end

  def line_element_editor_nwed(
        parent_win, lbox, ar_pel, _de, v_x, l_btn, e_cur, e_parent)
    p "line_element_editor_nwed starting..."

    if v_x == "edit"
      Tk.messageBox(
        "parent" => parent_win,
        "message" => _("Line element editor for section from \"%1$s\" " +
                       "to \"%2$s\" now being built...") )
      return
    end
    
    # Create a child window
    lsn_ew = open_editor_window(
      parent_win, _("\"Sujic\"") + ":: " + _("Editing a line element"))

    # Data extracting.
    # e_parent: <create class="sujic_line" ...>...</create>
    # e_cur: <using> (only set when v_x == "edit", nil if v_x == "new")
    lsn_h = Hash.new

    ## lsn_h[:e_001]: first station if set, nil otherwise.
    lsn_h[:e_001] = e_parent.elements['using[1]']
    if v_x == "edit"
      lsn_h[:e_xc] = e_cur.deep_clone
      lsn_h[:e_nc] = e_cur
      lsn_h[:lef_nc] = e_cur.elements[
        '../data/line_element[@from=\'' + e_cur.attributes['name'] + '\']']
      lsn_h[:lef_xc] = lsn_h[:lef_nc].deep_clone
      lsn_h[:let_nc] = e_cur.elements[
        '../data/line_element[@to=\'' + e_cur.attributes['name'] + '\']']
      if lsn_h[:let_nc]
        lsn_h[:let_xc] = lsn_h[:let_nc].deep_clone
      else
        lsn_h[:let_xc] = nil ## when first station selected on straight line
      end
    else ## v_x == "new"
      ## lsn_h[:e_nc]: Current elem. Simply deleted when edit is discarded.
      lsn_h[:e_nc] = REXML::Element.new('using')
      lsn_h[:e_nc].add_attributes({'class'=>'sujic_station',
                                   'type'=>'station'})
      ## lsn_h[:e_xc]: Element before editing for preservation in case
      ## edit is discarded.
      lsn_h[:e_xc] = nil

      ## e_ins_x: Elem with line name as its attribute 'name'.
      e_ins_x = e_parent.elements['data']

      ## Inserting lsn_h[:e_nc]
      e_parent.insert_before(e_ins_x,lsn_h[:e_nc])

      ## Setting let_nc and let_xc. _xc is for preservation, _nc is for edit.
      ## let_nc: line element with "to" set to the new station the operator
      ##   intends to add to the line.
      ## let_001: line element with "to" set to first station or "nil",
      ##   which will mean its "from" is set to the last station. Must be
      ##   found when e_001 is non-nil. Then, let_nc = let_001, and its
      ##   copy is preserved as let_xc.
      if lsn_h[:e_001]
        ## If a station is already in the list, get the LAST line element.
        ## Find a line element with @to set to the name of lsn_h[:e_001].
        lsn_h[:let_001] = lsn_h[:e_nc].elements[
          '../data/line_element[@to=\'' + lsn_h[:e_001].attributes['name'] +
          '\']']
        unless lsn_h[:let_001]
          ## If nil, find the line element with @to set to nil.
          lsn_h[:let_001] = lsn_h[:e_nc].elements[
            '../data/line_element[@to=\'nil\']']
        end

        ## lsn_h[:let_xc]: Previous data of the "next" (last) line element
        ## for preservation in case edit is discarded.
        lsn_h[:let_nc] = lsn_h[:let_001]
        lsn_h[:let_xc] = lsn_h[:let_001].deep_clone
      else
        ## If this is the first station to add, let_xc = let_nc = nil
        lsn_h[:let_nc] = lsn_h[:let_xc] = nil
      end

      ## lef_nc: line element with "from" set to the new station the
      ##   operator intends to add to the line. This must never be "nil".
      if lsn_h[:let_nc]
        ## For second station and onwards --- copy let_nc.
        lsn_h[:lef_nc] = lsn_h[:let_nc].deep_clone
        lsn_h[:e_nc].elements['../data'].add_element(lsn_h[:lef_nc])
        lsn_h[:let_nc].add_attribute('to','nil')
        # lsn_h[:lef_nc].elements['diagram_info'].add_element(
        #   'from_position',
        #   lsn_h[:lef_nc].elements['diagram_info'].attributes['to_position'])
        # lsn_h[:lef_nc].elements['diagram_info'].add_element(
        #   'to_position',
        #   (lsn_h[:lef_nc].elements['diagram_info'].attributes[
        #     'from_position'].to_f +
        #    lsn_h[:lef_nc].elements['diagram_info'].attributes[
        #      'inter_station_distance'].to_f).to_s)
      else
        ## For first station -- create a new one.
        lsn_h[:lef_nc] = e_parent.elements['data'].add_element('line_element')
        lsn_h[:lef_nc].add_element(
          'diagram_info',
          {'from_position'=>'0.0','inter_station_distance'=>'0.0',
           'to_position'=>'0.0'})
        lsn_h[:lef_nc].add_element(
          'roads',
          {'pattern'=>'double','signal_pattern'=>'uni'})
        lsn_h[:lef_nc].add_element('run_time_constraints')
      end
      lsn_h[:lef_nc].add_attribute('from','nil')
      lsn_h[:lef_nc].add_attribute('to','nil')
      lsn_h[:lef_xc] = nil
    end
    l_discard_edit = lambda {
      lsn_h[:e_nc].parent.insert_after(
        lsn_h[:e_nc],lsn_h[:e_xc]) if lsn_h[:e_xc]
      lsn_h[:e_nc].parent.delete(lsn_h[:e_nc]) if lsn_h[:e_nc]
      lsn_h[:lef_nc].parent.insert_after(
        lsn_h[:lef_nc],lsn_h[:lef_xc]) if lsn_h[:lef_xc]
      lsn_h[:lef_nc].parent.delete(lsn_h[:lef_nc]) if lsn_h[:lef_nc]
      lsn_h[:let_nc].parent.insert_after(
        lsn_h[:let_nc],lsn_h[:let_xc]) if lsn_h[:let_xc]
      lsn_h[:let_nc].parent.delete(lsn_h[:let_nc]) if lsn_h[:let_nc]
    }

    # Tab representation (notebook widget)
    lsn_h[:ttn] = Tk::Tile::Notebook.new(lsn_ew) do
      place('x'=>10, 'y'=>10)
    end
    lsn_h[:ttn].pack

    # Create a frame for main editing
    #  (line name change, train graph info, etc.)
    lsn_h[:lsm] = TkFrame.new(lsn_h[:ttn])

    # Create a frame for patterns
    lsn_h[:ptn] = TkFrame.new(lsn_h[:ttn])

    # Create a frame for roads
    lsn_h[:rds] = TkFrame.new(lsn_h[:ttn])

    # Create a frame for run time constraints
    lsn_h[:rtc] = TkFrame.new(lsn_h[:ttn])

    # Adding frames to the notebook widget
    lsn_h[:ttn].add(lsn_h[:lsm], :text => _('General'))
    lsn_h[:ttn].add(lsn_h[:ptn], :text => _('Patterns'))
    lsn_h[:ttn].add(lsn_h[:rds], :text => _('Roads'))
    lsn_h[:ttn].add(lsn_h[:rtc], :text => _('Run time constraints'))

    #-----------------------#
    # Populating main frame #
    #-----------------------#
    ## (0) Data extracting
    lsn_h[:_ah] = Hash.new
    lsn_h[:e_prv] = lsn_h[:e_nc].previous_element
    lsn_h[:e_nxt] = lsn_h[:e_nc].next_element
    lsn_h[:e_nxt] = nil unless lsn_h[:e_nxt].name == "using"
    if lsn_h[:e_nc].attributes['name']
      lsn_h[:_ah][:e_nc] = @doc.elements[
        ('rtss_input_data/create[@class=\'sujic_station\']/data[@name=\'' +
         lsn_h[:e_nc].attributes['name'] + '\']/..')]
    else
      lsn_h[:_ah][:e_nc] = nil
    end
    if lsn_h[:e_nxt] && lsn_h[:e_nc].attributes['name']
      lsn_h[:_ah][:e_nxt] = @doc.elements[
        ('rtss_input_data/create[@class=\'sujic_station\']/data[@name=\'' +
         lsn_h[:e_nxt].attributes['name'] + '\']/..')]
    else
      lsn_h[:_ah][:e_nxt] = nil
    end
    if lsn_h[:e_prv] && lsn_h[:e_prv].attributes['name']
      lsn_h[:_ah][:e_prv] = @doc.elements[
        ('rtss_input_data/create[@class=\'sujic_station\']/data[@name=\'' +
         lsn_h[:e_prv].attributes['name'] + '\']/..')]
    else
      lsn_h[:_ah][:e_prv] = nil
    end
    if lsn_h[:e_001] && lsn_h[:e_nc].attributes['name']
      lsn_h[:_ah][:e_001] = @doc.elements[
        ('rtss_input_data/create[@class=\'sujic_station\']/data[@name=\'' +
         lsn_h[:e_001].attributes['name'] + '\']/..')]
    else
      lsn_h[:_ah][:e_001] = nil
    end
    if lsn_h[:lef_nc]
      if lsn_h[:lef_nc].elements['diagram_info'].attributes[
           'inter_station_distance']
        lsn_h[:_ah][:isd] = lsn_h[:lef_nc].elements[
          'diagram_info'].attributes['inter_station_distance']
      else
        lsn_h[:_ah][:isd] = "0.0"
      end
      if lsn_h[:lef_nc].elements['diagram_info'].attributes[
           'show_from_platforms']
        lsn_h[:_ah][:sfp] = lsn_h[:lef_nc].elements[
          'diagram_info'].attributes['show_from_platforms']
      else
        lsn_h[:_ah][:sfp] = "false"
        lsn_h[:lef_nc].elements['diagram_info'].add_attribute(
          'show_from_platforms','false')
      end
    end
    if lsn_h[:let_nc] && lsn_h[:let_nc].elements['diagram_info'].attributes[
         'inter_station_distance']
      lsn_h[:_ah][:isd] = lsn_h[:let_nc].elements['diagram_info'].attributes[
        'inter_station_distance']
    else
      lsn_h[:_ah][:isd] = 0.0
    end

    ## (1) Line 1: Station for editing
    lsn_h[:lsm01] = TkFrame.new(lsn_h[:lsm]).pack(side: :top, fill: :x,
                                                  anchor: :w)
    lsn_h[:lsm01v] = TkVariable.new('orig')
    lsn_h[:lsm01_lbl] = TkLabel.new(lsn_h[:lsm01]).pack(side: :left)
    lsn_h[:lsm01_lbl].text = _("Station for editing")
    if lsn_h[:_ah][:e_nc]
      str_stf = lsn_h[:_ah][:e_nc].elements['data'].attributes['name']
    else
      str_stf = _("(To be selected)")
    end
    lsn_h[:lsm01_rb1] = TkRadioButton.new(lsn_h[:lsm01]) do
      text str_stf
      value "orig"
      variable lsn_h[:lsm01v]
      select
      pack(side: :left)
    end
    lsn_h[:lsm01_rb2] = TkRadioButton.new(lsn_h[:lsm01]) do
      text _("Select another")
      value "another"
      variable lsn_h[:lsm01v]
      pack(side: :left)
    end

    ## (2) Line 2: Line shape
    lsn_h[:lsm02] = TkFrame.new(lsn_h[:lsm]).pack(side: :top, fill: :x,
                                                  anchor: :w)
    lsn_h[:lsm02v] = TkVariable.new('orig')
    lsn_h[:lsm02_lbl] = TkLabel.new(lsn_h[:lsm02]).pack(side: :left)
    lsn_h[:lsm02_lbl].text = _("Line shape")
    str_stto = _("Straight line")
    str_sttf = _("Circular line")
    lsn_h[:lsm02_rb1] = TkRadioButton.new(lsn_h[:lsm02]) do
      text (str_stto)
      value "orig"
      variable lsn_h[:lsm02v]
      pack(side: :left)
    end
    lsn_h[:lsm02_rb2] = TkRadioButton.new(lsn_h[:lsm02]) do
      text (str_sttf)
      value "circ"
      variable lsn_h[:lsm02v]
      pack(side: :left)
    end
    lsn_h[:lsm02_rb1].state = "normal"
    lsn_h[:lsm02_rb2].state = "normal"
    if lsn_h[:e_nxt] || (! lsn_h[:e_001])
      lsn_h[:lsm02_rb1].state = "disabled"
      lsn_h[:lsm02_rb2].state = "disabled"
      lsn_h[:lsm02_rb1].select
    elsif lsn_h[:lef_nc].attributes['to'] &&
          lsn_h[:lef_nc].attributes['to'] != 'nil'
      lsn_h[:lsm02_rb2].select
    else
      lsn_h[:lsm02_rb1].select
    end

    ## (3) Line 3: Distance from last station
    lsn_h[:lsm03] = TkFrame.new(lsn_h[:lsm]).pack(side: :top, fill: :x,
                                                  anchor: :w)
    lsn_h[:lsm03_lbl] = TkLabel.new(lsn_h[:lsm03]).pack(side: :left)
    lsn_h[:lsm03_btn] = TkButton.new(lsn_h[:lsm03]).pack(side: :right)
    lsn_h[:lsm03_ent] = TkEntry.new(lsn_h[:lsm03]).pack(side: :right)
    lsn_h[:lsm03_ent].width(15)
    lsn_h[:lsm03_stt] = 'normal'
    lsn_h[:lsm03_lbl].text = sprintf(
      _("Distance from Station \"%1$s\" [km]: "),
      ((lsn_h[:let_nc] && lsn_h[:let_nc].attributes['from']) ?
         lsn_h[:let_nc].attributes['from'] : "(nil)" ))
    if lsn_h[:let_nc]
      if lsn_h[:let_nc].elements['diagram_info'].attributes[
           'inter_station_distance']
        lsn_h[:lsm03_ent].value = lsn_h[:let_nc].elements[
          'diagram_info'].attributes['inter_station_distance']
      else
        lsn_h[:lsm03_ent].value = "0.0"
      end
      if lsn_h[:let_nc] && lsn_h[:let_nc].attributes['from'] &&
         lsn_h[:let_nc].attributes['from'] == 'nil'
        lsn_h[:lsm03_stt] = 'disabled'
      else
        lsn_h[:lsm03_stt] = 'normal'
      end
    else
      lsn_h[:lsm03_ent].value = "0.0"
      lsn_h[:lsm03_stt] = 'disabled'
    end
    lsn_h[:lsm03_btn].text = _("Renew")
    # lsn_h[:lsm03_btn].command() ... to be defined later
    lsn_h[:lsm03_ent].state(lsn_h[:lsm03_stt])
    lsn_h[:lsm03_btn].state(lsn_h[:lsm03_stt])

    ## (4) Line 4: Position of "from" station
    lsn_h[:lsm04] = TkFrame.new(lsn_h[:lsm]).pack(side: :top, fill: :x,
                                                  anchor: :w)
    lsn_h[:lsm04_lbl] = TkLabel.new(lsn_h[:lsm04]).pack(side: :left)
    lsn_h[:lsm04_btn] = TkButton.new(lsn_h[:lsm04]).pack(side: :right)
    lsn_h[:lsm04_ent] = TkEntry.new(lsn_h[:lsm04]).pack(side: :right)
    lsn_h[:lsm04_ent].width(15)
    lsn_h[:lsm04_ent_min] = 0.0
    lsn_h[:lsm04_stt] = 'normal'
    lsn_h[:lsm04_lbl].text = sprintf(
      _("Position of Station \"%1$s\" [km]: "),
      ((lsn_h[:lef_nc] && lsn_h[:lef_nc].attributes['from']) ?
         lsn_h[:lef_nc].attributes['from'] : "(nil)" ))
    ## Setting lsm04_ent value
    if lsn_h[:let_nc]
      ## Second station and onwards
      if lsn_h[:lef_nc].elements['diagram_info'].attributes['from_position']
        ## lef_nc's attribute 'from_position' is set
        lsn_h[:lsm04_ent].value = lsn_h[:lef_nc].elements[
          'diagram_info'].attributes['from_position']
      else
        ## lef_nc's attribute 'from_position' is NOT set -- set 0.0
        lsn_h[:lsm04_ent].value = "0.0"
        lsn_h[:lef_nc].elements['diagram_info'].attributes[
          'from_position'] = lsn_h[:lsm04_ent].value
      end
      ## Status selection
      if lsn_h[:lef_nc] && lsn_h[:lef_nc].attributes['from'] &&
         lsn_h[:lef_nc].attributes['from'] == 'nil'
        ## lef_nc's attrubute 'from' is nil: disabled
        lsn_h[:lsm04_stt] = 'disabled'
      else
        ## lef_nc's attrubute 'from' is non-nil: normal
        lsn_h[:lsm04_stt] = 'normal'
      end
    else
      ## First station
      lsn_h[:lsm04_ent].value = "0.0"
      lsn_h[:lsm04_stt] = 'disabled'
      lsn_h[:lsm04_ent_min] = nil
    end
    if lsn_h[:lsm04_ent_min]
      lsn_h[:lsm04_ent_min] = lsn_h[:lsm04_ent].value.to_f
    end
    lsn_h[:lsm04_btn].text = _("Renew")
    # lsn_h[:lsm04_btn].command() ... to be defined later
    lsn_h[:lsm04_ent].state(lsn_h[:lsm04_stt])
    lsn_h[:lsm04_btn].state(lsn_h[:lsm04_stt])

    ## (5) Line 5: Distance to next station
    lsn_h[:lsm05] = TkFrame.new(lsn_h[:lsm]).pack(side: :top, fill: :x,
                                                  anchor: :w)
    lsn_h[:lsm05_lbl] = TkLabel.new(lsn_h[:lsm05]).pack(side: :left)
    lsn_h[:lsm05_btn] = TkButton.new(lsn_h[:lsm05]).pack(side: :right)
    lsn_h[:lsm05_ent] = TkEntry.new(lsn_h[:lsm05]).pack(side: :right)
    lsn_h[:lsm05_ent].width(15)
    lsn_h[:lsm05_stt] = 'normal'
    lsn_h[:lsm05_lbl].text = sprintf(
      _("Distance to Station \"%1$s\": "),
      ((lsn_h[:lef_nc] && lsn_h[:lef_nc].attributes['to']) ?
         lsn_h[:lef_nc].attributes['to'] : "(nil)" ))
    if lsn_h[:lef_nc]
      if lsn_h[:lef_nc].elements['diagram_info'].attributes[
           'inter_station_distance']
        lsn_h[:lsm05_ent].value = lsn_h[:lef_nc].elements[
          'diagram_info'].attributes['inter_station_distance']
      else
        lsn_h[:lsm05_ent].value = "0.0"
      end
      if ( lsn_h[:lef_nc] && lsn_h[:lef_nc].attributes['to'] &&
           lsn_h[:lef_nc].attributes['to'] == 'nil' ) ||
         ( lsn_h[:e_nxt] == nil && lsn_h[:lsm02v] == 'orig' )
        lsn_h[:lsm05_stt] = 'disabled'
      else
        lsn_h[:lsm05_stt] = 'normal'
      end
    else
      lsn_h[:lsm05_ent].value = "0.0"
      lsn_h[:lsm05_stt] = 'disabled'
    end
    lsn_h[:lsm05_btn].text = _("Renew")
    lsn_h[:lsm05_btn].command(
      proc do
        unless lsn_h[:lsm05_ent].value.is_float?
          # Value check lsm05_ent: length = float
          Tk.messageBox(
            "parent" => lsn_ew,
            "title" => _("Distance must be a real number"),
            "message" =>
            _("Distance between stations must be a real number " +
              "(Unit: km).") )
          return false
        else
          return true
        end; end)
    lsn_h[:lsm05_ent].state(lsn_h[:lsm05_stt])
    lsn_h[:lsm05_btn].state(lsn_h[:lsm05_stt])

    ## (6) Line 6: Position of "to" station
    lsn_h[:lsm06] = TkFrame.new(lsn_h[:lsm]).pack(side: :top, fill: :x,
                                                  anchor: :w)
    lsn_h[:lsm06_lbl] = TkLabel.new(lsn_h[:lsm06]).pack(side: :left)
    lsn_h[:lsm06_btn] = TkButton.new(lsn_h[:lsm06]).pack(side: :right)
    lsn_h[:lsm06_ent] = TkEntry.new(lsn_h[:lsm06]).pack(side: :right)
    lsn_h[:lsm06_stt] = "normal"
    lsn_h[:lsm06_ent].width(15)
    lsn_h[:lsm06_lbl].text = sprintf(
      _("Position of Station \"%1$s\" [km]: "),
      ((lsn_h[:lef_nc] && lsn_h[:lef_nc].attributes['to']) ?
         lsn_h[:lef_nc].attributes['to'] : "(nil)" ))
    if lsn_h[:lef_nc]
      if lsn_h[:lef_nc].elements['diagram_info'].attributes['to_position']
        lsn_h[:lsm06_ent].value = lsn_h[:lef_nc].elements[
          'diagram_info'].attributes['to_position']
      else
        lsn_h[:lsm06_ent].value = "0.0"
      end
      if ( lsn_h[:lef_nc] && lsn_h[:lef_nc].attributes['to'] &&
           lsn_h[:lef_nc].attributes['to'] == 'nil' ) ||
         ( lsn_h[:e_nxt] == nil && lsn_h[:lsm02v] == 'orig' )
        lsn_h[:lsm06_stt] = 'disabled'
      else
        lsn_h[:lsm06_stt] = 'normal'
      end
    else
      lsn_h[:lsm06_ent].value = "0.0"
      lsn_h[:lsm06_stt] = 'disabled'
    end
    lsn_h[:lsm06_ent].state(lsn_h[:lsm06_stt])
    lsn_h[:lsm06_btn].state(lsn_h[:lsm06_stt])
    lsn_h[:lsm06_btn].text = _("Renew")
    lsn_h[:lsm06_btn].command(
      proc do
        unless lsn_h[:lsm06_ent].value.is_float?
          # Value check lsm06_ent: position = float
          Tk.messageBox(
            "parent" => lsn_ew,
            "title" => _("Position must be a real number"),
            "message" =>
            _("Position of a station must be a real number (Unit: km).") )
        end; end)

    ## (7) Line 7: Show from station?
    lsn_h[:lsm07] = TkFrame.new(lsn_h[:lsm]).pack(side: :top, fill: :x,
                                                  anchor: :w)
    lsn_h[:lsm07v] = TkVariable.new(lsn_h[:_ah][:sfp])
    lsn_h[:lsm07_lbl] = TkLabel.new(lsn_h[:lsm07]).pack(side: :left)
    lsn_h[:lsm07_lbl].text = sprintf(
      _("Display platform departures/arrivals: "),
      (lsn_h[:_ah][:e_prv] ?
         lsn_h[:_ah][:e_prv].elements['data'].attributes['name'] :
       "(nil)"))
    lsn_h[:lsm07_rb1] = TkRadioButton.new(lsn_h[:lsm07]) do
      text _("Yes")
      value "true"
      variable lsn_h[:lsm07v]
      pack(side: :left)
    end
    lsn_h[:lsm07_rb2] = TkRadioButton.new(lsn_h[:lsm07]) do
      text _("No")
      value "false"
      variable lsn_h[:lsm07v]
      pack(side: :left)
    end
    
    ## (8) Line 8: Use patterns or not for "Up" side line element
    ##             (except when first station is pointed to)
    lsn_h[:lsm08] = TkFrame.new(lsn_h[:lsm]).pack(side: :top, fill: :x,
                                                  anchor: :w)
    lsn_h[:lsm08v] = TkVariable.new('yes')
    lsn_h[:lsm08_lbl] = TkLabel.new(lsn_h[:lsm08]).pack(side: :left)
    lsn_h[:lsm08_lbl].text = _("Use patterns?")
    lsn_h[:ptn_rv] = TkVariable.new('double')
    if lsn_h[:let_nc] &&
       lsn_h[:let_nc].elements['roads'].attributes['road_pattern']
      lsn_h[:ptn_rv].value =
        lsn_h[:let_nc].elements['roads'].attributes['road_pattern']
    end
    lsn_h[:ptn_sv] = TkVariable.new('uni')
    if lsn_h[:let_nc] &&
       lsn_h[:let_nc].elements['roads'].attributes['signal_pattern']
      lsn_h[:ptn_sv].value =
        lsn_h[:let_nc].elements['roads'].attributes['signal_pattern']
    end
    lsn_h[:lsm08_rb1] = TkRadioButton.new(lsn_h[:lsm08]) do
      text _("Yes")
      value "yes"
      variable lsn_h[:lsm08v]
      select
      command(
        proc{
          puts "lsn_h 3902: LineElem_Use_Pattern: Yes, R|S=" +
               lsn_h[:ptn_rv] + "|" + lsn_h[:ptn_sv]
          lsn_h[:ttn].itemconfigure(lsn_h[:ptn],:state => "normal")
          lsn_h[:ttn].itemconfigure(lsn_h[:rds],:state => "disabled")
          lsn_h[:ttn].itemconfigure(lsn_h[:rtc],:state => "disabled")
        })
      pack(side: :left)
    end
    lsn_h[:lsm08_rb2] = TkRadioButton.new(lsn_h[:lsm08]) do
      text _("No")
      value "no"
      variable lsn_h[:lsm08v]
      command(
        proc{
          puts "lsn_h 3920: LineElem_NOT_Use_Patterns"
          lsn_h[:ttn].itemconfigure(lsn_h[:ptn],:state => "disabled")
          lsn_h[:ttn].itemconfigure(lsn_h[:rds],:state => "normal")
          lsn_h[:ttn].itemconfigure(lsn_h[:rtc],:state => "normal")
        })
      pack(side: :left)
    end
    lsn_h[:ttn].itemconfigure(lsn_h[:rds],:state => "disabled")
    lsn_h[:ttn].itemconfigure(lsn_h[:rtc],:state => "disabled")
    if lsn_h[:let_nc]
      lsn_h[:ttn].itemconfigure(lsn_h[:ptn],:state => "normal")
    else
      lsn_h[:ttn].itemconfigure(lsn_h[:ptn],:state => "disabled")
      lsn_h[:lsm08_rb1].state('disabled')
      lsn_h[:lsm08_rb2].state('disabled')
    end

    ## (9) Line 9: Other edit operations
    if v_x == "edit"
      lsn_h[:lsm09] = TkFrame.new(lsn_h[:lsm]).pack(side: :top, fill: :x,
                                                    anchor: :w)
      lsn_h[:lsm09v] = TkVariable.new('none')
      lsn_h[:lsm09_lbl] = TkLabel.new(lsn_h[:lsm09]).pack(side: :left)
      lsn_h[:lsm09_lbl].text = _("Other edits: ")
      lsn_h[:lsm09_rb1] = TkRadioButton.new(lsn_h[:lsm09]) do
        text _("None")
        value 'none'
        variable lsn_h[:lsm09v]
        pack(side: :left)
      end
      lsn_h[:lsm09_rb2] = TkRadioButton.new(lsn_h[:lsm09]) do
        text _("Split section")
        value 'split'
        variable lsn_h[:lsm09v]
        pack(side: :left)
      end
    end

    ## (c1) Radiobutton [:lsm01_rb2] command.
    ##    * [:_ah][:e_nc] change -- pick one station window call
    ##    * [:let_nc] attr "to" change
    ##    * [:lef_nc] attr "from" change
    ##    * [:lsm01_rb1] text & selection status change
    ##    * [:lsm04_lbl] text change
    lsn_h[:lsm01_rb2].command(
      proc{
        p "Radiobutton LSM01_RB2"
        _lsm01h = Hash.new
        _lsm01h[:xpath_lb] =
          'rtss_input_data/create[@class=\'sujic_station\']'
        _lsm01h[:listlambda] = lambda{|e|
          e.elements['data'].attributes['name']}
        _lsm01h[:lb_otext] = _("Station")
        _lsm01h[:sw_nil] = false
        _lsm01h[:slambda] = lambda {|e,_org,_dx|
          if e == nil
            Tk.messageBox(
              "parent" => lsn_ew,
              "title" => _("Internal error") + " L03467",
              "message" =>
              sprintf(_("Internal error %1$s has happened. " +
                        "Report this as bug."), "L03467"))
            return false
          elsif e != _org
            unless _dx[:e_nc] &&
                   _dx[:e_nc].elements[
                     '../using[@name=\'' +
                     e.elements['data'].attributes['name'] + '\']'] == nil
              Tk.messageBox(
                "parent" => lsn_ew,
                "title" => _("Error: Station already in this line"),
                "message" =>
                sprintf(_("Station %1$s is already in this line"),
                        e.elements['data'].attributes['name']))
              return false
            end
            _dx[:_ah][:e_nc] = e
            _dx[:e_nc].add_attribute(
              'name',e.elements['data'].attributes['name'])
            _dx[:let_nc].add_attribute(
              "to",e.elements['data'].attributes['name']) if _dx[:let_nc]
            _dx[:lef_nc].add_attribute(
              "from",e.elements['data'].attributes['name']) if _dx[:lef_nc]
            _dx[:lsm01_rb1].text(e.elements['data'].attributes['name'])
            _dx[:lsm01_rb1].select
            _dx[:lsm04_stt] = 'normal'
            _dx[:lsm04_ent].state(_dx[:lsm04_stt])
            _dx[:lsm04_btn].state(_dx[:lsm04_stt])
            _dx[:lsm04_lbl].text(
              sprintf(
                _("Position of Station \"%1$s\" [km]: "),
                e.elements['data'].attributes['name']))
            return true
          end }
        stn_c = lsn_h[:_ah][:e_nc]
        pick_one_window(lsn_ew, stn_c, _lsm01h, lsn_h)
      })

    ## (c2) Radiobutton [:lsm02_rb1] command.
    ##    * [:lsm02v] to 'orig'
    ##    * [:lsm05_ent] status to 'disabled' unless lsn_h[:e_nxt]
    ##    * [:lsm06_ent] status to 'disabled' unless lsn_h[:e_nxt]
    lsn_h[:lsm02_rb1].command(
      proc{
        p "Radiobutton LSM02_RB1"
        unless lsn_h[:e_nxt]
          lsn_h[:lsm05_stt] = 'disabled'
          lsn_h[:lsm06_stt] = 'disabled'
          lsn_h[:lsm05_ent].state(lsn_h[:lsm05_stt])
          lsn_h[:lsm05_btn].state(lsn_h[:lsm05_stt])
          lsn_h[:lsm06_ent].state(lsn_h[:lsm06_stt])
          lsn_h[:lsm06_btn].state(lsn_h[:lsm06_stt])
        end
      })

    ## (c3) Radiobutton [:lsm02_rb2] command.
    ##    * [:lsm02v] to 'circ'
    ##    * [:lsm05_ent] status to 'normal' unless lsn_h[:e_nxt]
    ##    * [:lsm06_ent] status to 'normal' unless lsn_h[:e_nxt]
    lsn_h[:lsm02_rb1].command(
      proc{
        p "Radiobutton LSM02_RB1"
        unless lsn_h[:e_nxt]
          lsn_h[:lsm05_stt] = 'normal'
          lsn_h[:lsm06_stt] = 'normal'
          lsn_h[:lsm05_ent].state(lsn_h[:lsm05_stt])
          lsn_h[:lsm05_btn].state(lsn_h[:lsm05_stt])
          lsn_h[:lsm06_ent].state(lsn_h[:lsm06_stt])
          lsn_h[:lsm06_btn].state(lsn_h[:lsm06_stt])
        end
      })

    ## (c4) Button [:lsm03_btn] command.
    ##    * [:lsm03_ent].value check (is_float? / above zero)
    ##    * renew [:let_nc].inter_station_distance
    ##    * renew [:let_nc].to_position, [:lsm04_ent].value
    ##    * renew [:lef_nc].from_position
    ##    * renew [:lef_nc].to_position, [:lsm06_ent].value
    ##    * e_xcur=[:e_nc]; let_cur=[:let_nc]; lef_cur=[:lef_nc]
    ##    * while e_xcur != [:e_001] do
    ##    *  renew lef_cur.from_position/inter_station_distance/to_pos
    ##    *  let_cur=lef_cur,increment lef_cur, e_xcur
    ##    * end
    lsn_h[:lsm03_btn].command(
      proc{
        p "Button LSM03_BTN"
        if (! lsn_h[:lsm03_ent].value.is_float?)
          Tk.messageBox(
            "parent" => lsn_ew,
            "title" => _("Distance must be a real number"),
            "message" => sprintf(
              _("Distance from Station %1$s must be a real number " +
                "(Unit: km)." ),
              lsn_h[:let_nc].attributes['from']))
          return false
        elsif lsn_h[:lsm03_ent].value.to_f <= 0
          Tk.messageBox(
            "parent" => lsn_ew,
            "title" => _("Distance must be larger than zero"),
            "message" => sprintf(
              _("Distance from Station %1$s must be larger than zero."),
              lsn_h[:let_nc].attributes['from']))
          return false
        end
        if ! lsn_h[:let_nc].elements['diagram_info']
          lsn_h[:let_nc].add_element('diagram_info')
          lsn_h[:let_nc].elements['diagram_info'].add_attribute(
            'from_position','0.0')
        end
        lsn_h[:let_nc].elements['diagram_info'].add_attribute(
          'inter_station_distance',lsn_h[:lsm03_ent].value)
        x_to_pos = lsn_h[:let_nc].elements['diagram_info'].attributes[
          'from_position'].to_f + lsn_h[:lsm03_ent].value.to_f
        lsn_h[:let_nc].elements['diagram_info'].add_attribute(
          'to_position', x_to_pos.to_s)
        lsn_h[:lsm04_ent].state('normal')
        lsn_h[:lsm04_ent].value=
          lsn_h[:let_nc].elements['diagram_info'].attributes['to_position']
        lsn_h[:lsm04_ent].state(lsn_h[:lsm04_stt])
        lsn_h[:lef_nc].elements['diagram_info'].attributes['from_position'] =
          lsn_h[:lsm04_ent].value
        p "*=*=*=*=*=*=*=*=*=*=*=*=*=*=*"
        p lsn_h[:lef_nc].elements['diagram_info']
        p lsn_h[:lsm06_ent].value
        print lsn_h[:lef_nc]
        p ""
        print lsn_h[:lef_nc].parent
        p ""
        p "*=*=*=*=*=*=*=*=*=*=*=*=*=*=*"
        x_to_pos += lsn_h[:lef_nc].elements['diagram_info'].attributes[
          'inter_station_distance'].to_f
        lsn_h[:lef_nc].elements['diagram_info'].add_attribute(
          'to_position', x_to_pos.to_s)
        lsn_h[:lsm06_ent].state('normal')
        lsn_h[:lsm06_ent].value=
          lsn_h[:lef_nc].elements['diagram_info'].attributes['to_position']
        lsn_h[:lsm06_ent].state(lsn_h[:lsm06_stt])
        p "*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*"
        p lsn_h[:lef_nc].elements['diagram_info']
        p lsn_h[:lsm06_ent].value
        print lsn_h[:lef_nc]
        p ""
        print lsn_h[:lef_nc].parent
        p ""
        p "*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*"
        let_cur = lsn_h[:let_nc]
        lef_cur = lsn_h[:lef_nc]
        e_xcur = lsn_h[:e_nc]
        e_xparent = lsn_h[:e_nc].parent
        let_cur = lef_cur
        lef_cur = lef_cur.next_element
        e_xcur = e_xcur.next_element
        e_xcur = nil if e_xcur.name != "using"
        while e_xcur
          let_cur = lef_cur
          lef_cur = lef_cur.next_element
          e_xcur = e_xcur.next_element
          e_xcur = nil if e_xcur.name != "using"
          lef_cur.elements['diagram_info'].attributes['from_position'] =
            let_cur.elements['diagram_info'].attributes['to_position']
          x_to_pos += lef_cur.elements['diagram_info'].attributes[
            'inter_station_distance'].to_f
          lef_cur.elements['diagram_info'].add_attribute(
            'to_position',x_to_pos.to_s)
        end
      })

    ## (c5) Button [:lsm04_btn] command.
    ##    * [:lsm04_ent].value check (is_float?)
    ##    * renew [:lef_nc].from_position
    ##    * renew [:lef_nc].to_position, [:lsm06_ent].value
    ##    * e_xcur=[:e_nc]; let_cur=[:let_nc]; lef_cur=[:lef_nc]
    ##    * while e_xcur != [:e_001] do
    ##    *  renew lef_cur.from_position/inter_station_distance/to_pos
    ##    *  let_cur=lef_cur,increment lef_cur, e_xcur
    ##    * end
    ##    * e_xprv=[:e_prv]; let_cur=[:let_nc]; lef_cur=[:lef_nc]
    ##    * while e_xcur != [:e_001] do
    ##    *  renew lef_cur.from_position/inter_station_distance/to_pos
    ##    *  let_cur=lef_cur,increment lef_cur, e_xcur
    ##    * end
    lsn_h[:lsm04_btn].command(
      proc{
        p "Button LSM04_BTN"
        if (! lsn_h[:lsm04_ent].value.is_float?)
          Tk.messageBox(
            "parent" => lsn_ew,
            "title" => _("Position must be a real number"),
            "message" => sprintf(
              _("Position of Station %1$s must be a real number " +
                "(Unit: km)." ),
              lsn_h[:lef_nc].attributes['from']))
          return false
        elsif lsn_h[:lsm04_ent_min] &&
              lsn_h[:lsm04_ent].value.to_f <= lsn_h[:lsm04_ent_min]
          Tk.messageBox(
            "parent" => lsn_ew,
            "title" => _("Position value violation"),
            "message" => sprintf(
              _("Position of Station %1$s must have a value larger than " +
                "that of previous station, located at %2$s km." ),
              lsn_h[:lef_nc].attributes['from'],
              lsn_h[:lsm04_ent_min].to_s))
          return false
        end
        p "*=*=*=*=*=*=*=*=*=*=*=*=*=*=*"
        p lsn_h[:lef_nc].elements['diagram_info']
        p lsn_h[:lsm06_ent].value
        print lsn_h[:lef_nc]
        p ""
        print lsn_h[:lef_nc].parent
        p ""
        print lsn_h[:lsm04_ent].value
        p ""
        p "*=*=*=*=*=*=*=*=*=*=*=*=*=*=*"
        if lsn_h[:lsm04_ent_min]
          lsn_h[:lsm03_ent].value = (
            (lsn_h[:lsm04_ent].value.to_f) - lsn_h[:lsm04_ent_min]).to_s
          lsn_h[:let_nc].elements['diagram_info'].add_attribute(
            'inter_station_distance',lsn_h[:lsm03_ent].value)
        end
        lsn_h[:lef_nc].elements['diagram_info'].add_attribute(
          'from_position',lsn_h[:lsm04_ent].value)
        x_to_pos =
          lsn_h[:lsm04_ent].value.to_f +
          lsn_h[:lef_nc].elements['diagram_info'].attributes[
            'inter_station_distance'].to_f
        lsn_h[:lef_nc].elements['diagram_info'].add_attribute(
          'to_position', x_to_pos.to_s)
        lsn_h[:lsm06_ent].state('normal')
        lsn_h[:lsm06_ent].value=
          lsn_h[:lef_nc].elements['diagram_info'].attributes['to_position']
        lsn_h[:lsm06_ent].state(lsn_h[:lsm06_stt])
        p "*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*"
        p lsn_h[:lef_nc].elements['diagram_info']
        p lsn_h[:lsm06_ent].value
        print lsn_h[:lef_nc]
        p ""
        print lsn_h[:lef_nc].parent
        p ""
        p "*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*"
        if lsn_h[:let_nc] && lsn_h[:e_nc] != lsn_h[:e_001]
          if ! (lsn_h[:let_nc].elements['diagram_info'])
            lsn_h[:let_nc].add_element('diagram_info')
          end
          lsn_h[:let_nc].elements['diagram_info'].add_attribute(
            'to_position',x_to_pos.to_s)
        end
        let_cur = lsn_h[:let_nc]
        lef_cur = lsn_h[:lef_nc]
        e_xcur = lsn_h[:e_nc]
        e_xparent = lsn_h[:e_nc].parent
        let_cur = lef_cur
        lef_cur = lef_cur.next_element
        e_xcur = e_xcur.next_element
        e_xcur = nil if e_xcur.name != "using"
        while e_xcur
          let_cur = lef_cur
          lef_cur = lef_cur.next_element
          e_xcur = e_xcur.next_element
          e_xcur = nil if e_xcur.name != "using"
          lef_cur.elements['diagram_info'].attributes['from_position'] =
            let_cur.elements['diagram_info'].attributes['to_position']
          x_to_pos += lef_cur.elements['diagram_info'].attributes[
            'inter_station_distance'].to_f
          lef_cur.elements['diagram_info'].add_attribute(
            'to_position',x_to_pos.to_s)
        end
        let_cur = lsn_h[:let_nc]
        lef_cur = lsn_h[:lef_nc]
        e_xcur = lsn_h[:e_nc]
        x_to_pos_str = lef_cur.elements['diagram_info'].attributes[
          'from_position']
        x_to_pos = x_to_pos_str.to_f
        if let_cur && let_cur.elements['line_element'] && e_xcur
          x_to_pos -= let_cur.elements['line_element'].attributes[
            'inter_station_distance'].to_f
          let_cur.elements['line_element'].add_attribute(
            'from_position', x_to_pos.to_s)
        end
        e_xcur = e_xcur.previous_element
        while e_xcur
          let_cur = e_xcur.parent.elements[
            'data/line_element[@to=\'' + e_xcur.attributes['name'] + '\']']
          lef_cur = e_xcur.parent.elements[
            'data/line_element[@from=\'' + e_xcur.attributes['name'] + '\']']
          if let_cur
            if let_cur.elements['diagram_info']
              let_cur.elements['diagram_info'].add_attribute(
                'to_position',
                lef_cur.elements['diagram_info'].attributes['from_position'])
            end
          end
          e_xcur = e_xcur.previous_element
        end
      })

    ## (c6) Button [:lsm05_btn] command.
    ##    * [:lsm05_ent].value check (is_float? / above zero)
    ##    * renew [:lef_nc].inter_station_distance
    ##    * renew [:lef_nc].to_position, [:lsm06_ent].value
    ##    * e_xcur=[:e_nc]; let_cur=[:let_nc]; lef_cur=[:lef_nc]
    ##    * while e_xcur != [:e_001] do
    ##    *  renew lef_cur.from_position/inter_station_distance/to_pos
    ##    *  let_cur=lef_cur,increment lef_cur, e_xcur
    ##    * end
    lsn_h[:lsm05_btn].command(
      proc{
        p "Button LSM05_BTN"
        if (! lsn_h[:lsm05_ent].value.is_float?)
          Tk.messageBox(
            "parent" => lsn_ew,
            "title" => _("Distance must be a real number"),
            "message" => sprintf(
              _("Distance to Station %1$s must be a real number " +
                "(Unit: km)." ),
              lsn_h[:lef_nc].attributes['to']))
          return false
        elsif lsn_h[:lsm05_ent].value.to_f <= 0
          Tk.messageBox(
            "parent" => lsn_ew,
            "title" => _("Distance must be larger than zero"),
            "message" => sprintf(
              _("Distance to Station %1$s must be larger than zero."),
              lsn_h[:lef_nc].attributes['to']))
          return false
        end
        if ! lsn_h[:lef_nc].elements['diagram_info']
          lsn_h[:lef_nc].add_element('diagram_info')
          lsn_h[:lef_nc].elements['diagram_info'].add_attribute(
            'from_position','0.0')
        end
        lsn_h[:lef_nc].elements['diagram_info'].add_attribute(
          'inter_station_distance',lsn_h[:lsm05_ent].value)
        x_to_pos = lsn_h[:lef_nc].elements['diagram_info'].attributes[
          'from_position'].to_f + lsn_h[:lsm05_ent].value.to_f
        lsn_h[:lef_nc].elements['diagram_info'].add_attribute(
          'to_position', x_to_pos.to_s)
        lsn_h[:lsm06_ent].state('normal')
        lsn_h[:lsm06_ent].value=
          lsn_h[:lef_nc].elements['diagram_info'].attributes['to_position']
        lsn_h[:lsm06_ent].state(lsn_h[:lsm06_stt])
        let_cur = lsn_h[:let_nc]
        lef_cur = lsn_h[:lef_nc]
        e_xcur = lsn_h[:e_nc]
        while e_xcur != lsn_h[:e_001]
          let_cur = lef_cur
          lef_cur = e_cur.parent.elements[
            'data/line_element[@from=\'' + let_cur.attributes['to'] + '\']']
          e_xcur = e_xcur.next_element
          e_xcur = lsn_h[:e_001] if e_xcur.name != "using"
          lef_cur.elements['diagram_info'].attributes['from_position'] =
            let_cur.elements['diagram_info'].attributes['to_position']
          x_to_pos += lef_cur.elements['diagram_info'].attributes[
            'inter_station_distance'].to_f
          lef_cur.elements['diagram_info'].add_attribute(
            'to_position',x_to_pos.to_s)
        end
      })


    ## (c7) Button [:lsm06_btn] command.
    ##    * [:lsm06_ent].value check (is_float?)
    ##    * renew [:lef_nc].to_position
    ##    * e_xcur=[:e_nc]; let_cur=[:let_nc]; lef_cur=[:lef_nc]
    ##    * while e_xcur != [:e_001] do
    ##    *  renew lef_cur.from_position/inter_station_distance/to_pos
    ##    *  let_cur=lef_cur,increment lef_cur, e_xcur
    ##    * end
    ##    * e_xprv=[:e_prv]; let_cur=[:let_nc]; lef_cur=[:lef_nc]
    ##    * while e_xcur != [:e_001] do
    ##    *  renew lef_cur.from_position/inter_station_distance/to_pos
    ##    *  let_cur=lef_cur,increment lef_cur, e_xcur
    ##    * end
    lsn_h[:lsm06_btn].command(
      proc{
        p "Button LSM06_BTN"
        if (! lsn_h[:lsm06_ent].value.is_float?)
          Tk.messageBox(
            "parent" => lsn_ew,
            "title" => _("Position must be a real number"),
            "message" => sprintf(
              _("Position of Station %1$s must be a real number " +
                "(Unit: km)." ),
              lsn_h[:lef_nc].attributes['to']))
          return false
        end
        lsn_h[:lef_nc].elements['diagram_info'].add_attribute(
          'to_position',lsn_h[:lsm06_ent].value)
        x_to_pos = lsn_h[:lsm06_ent].value.to_f
        let_cur = lsn_h[:let_nc]
        lef_cur = lsn_h[:lef_nc]
        e_xcur = lsn_h[:e_nc]
        while e_xcur != lsn_h[:e_001]
          let_cur = lef_cur
          lef_cur = e_xcur.parent.elements[
            'data/line_element[@from=\'' + let_cur.attributes['to'] + '\']']
          e_xcur = e_xcur.next_element
          e_xcur = lsn_h[:e_001] if e_xcur.name != "using"
          lef_cur.elements['diagram_info'].attributes['from_position'] =
            let_cur.elements['diagram_info'].attributes['to_position']
          x_to_pos += lef_cur.elements['diagram_info'].attributes[
            'inter_station_distance'].to_f
          lef_cur.elements['diagram_info'].add_attribute(
            'to_position',x_to_pos.to_s)
        end
        let_cur = lsn_h[:let_nc]
        lef_cur = lsn_h[:lef_nc]
        e_xcur = lsn_h[:e_nc]
        x_to_pos = lsn_h[:lsm06_ent].value.to_f
        x_to_pos -= lef_cur.elements['line_element'].attributes[
          'inter_station_distance'].to_f
        lsn_h[:lsm04_ent].state('normal')
        lsn_h[:lsm04_ent].value = x_to_pos.to_s
        lsn_h[:lsm04_ent].state(lsn_h[:lsm04_stt])
        lsn_h[:lef_nc].elements['diagram_info'].add_attribute(
          'from_position',lsn_h[:lsm04_ent].value)
        if lsn_h[:let_nc] && lsn_h[:e_nc] != lsn_h[:e_001]
          if ! (lsn_h[:let_nc].elements['diagram_info'])
            lsn_h[:let_nc].add_element('diagram_info')
          end
          lsn_h[:let_nc].elements['diagram_info'].add_attribute(
            'to_position',lsn_h[:lsm04_ent].value)
          x_to_pos -= let_cur.elements['line_element'].attributes[
            'inter_station_distance'].to_f
          let_cur.elements['line_element'].add_attribute(
            'from_position', x_to_pos.to_s)
        end
        e_xcur = e_xcur.previous_element
        while e_xcur && e_xcur != lsn_h[:e_001]
          let_cur = e_xcur.parent.elements[
            'data/line_element[@to=\'' + e_xcur.attributes['name'] + '\']']
          lef_cur = e_xcur.parent.elements[
            'data/line_element[@from=\'' + e_xcur.attributes['name'] + '\']']
          if let_cur
            if let_cur.elements['diagram_info']
              let_cur.elements['diagram_info'].add_attribute(
                'to_position',
                lef_cur.elements['diagram_info'].attributes['from_position'])
            end
          end
          e_xcur = e_xcur.previous_element
        end
      })

    ## (c8a) Radiobutton [:lsm07_rb1] command.
    ##    * [:lef_nc].elements['diagram_info'].attributes['show_from_platforms']
    ##      to 'true'
    ##    * [:_ah][:sfp] to 'true'
    lsn_h[:lsm07_rb1].command(
      proc{
        p "Radiobutton LSM07_RB1"
        lsn_h[:lef_nc].elements['diagram_info'].add_attribute(
          'show_from_platforms','true')
        lsn_h[:_ah][:sfp] = 'true'
      })

    ## (c8b) Radiobutton [:lsm07_rb2] command.
    ##    * [:lef_nc].elements['diagram_info'].attributes['show_from_platforms']
    ##      to 'false'
    ##    * [:_ah][:sfp] to 'false'
    lsn_h[:lsm07_rb2].command(
      proc{
        p "Radiobutton LSM07_RB2"
        lsn_h[:lef_nc].elements['diagram_info'].add_attribute(
          'show_from_platforms','false')
        lsn_h[:_ah][:sfp] = 'false'
      })

    #---------------------------#
    # Populating patterns frame #
    #---------------------------#
    lsn_h[:ptn_ttn] = Tk::Tile::Notebook.new(lsn_h[:ptn]) do
      pack(fill: :x)
    end
    # lsn_h[:ptn_ttn].pack
    lsn_h[:ptn_t_r] = TkFrame.new(lsn_h[:ptn_ttn])
    lsn_h[:ptn_ttn].add(lsn_h[:ptn_t_r], :text => _('Inter-station roads'))
    lsn_h[:ptn_t_s] = TkFrame.new(lsn_h[:ptn_ttn])
    lsn_h[:ptn_ttn].add(lsn_h[:ptn_t_s], :text => _('Signalling'))
    # lsn_h[:ptn_t_c] = TkFrame.new(lsn_h[:ptn_ttn])
    # lsn_h[:ptn_ttn].add(lsn_h[:ptn_t_c], :text => _('Connectors'))
    lsn_h[:ptn_r01] = TkFrame.new(lsn_h[:ptn_t_r]).pack(side: :top, fill: :x)
    lsn_h[:ptn_r01_lbl] = TkLabel.new(lsn_h[:ptn_r01])
    lsn_h[:ptn_r01_lbl].text =
      _("Select one inter-station roads data set:")
    lsn_h[:ptn_r01_lbl].pack(side: :left)
    lsn_h[:ptn_r02] = TkFrame.new(lsn_h[:ptn_t_r]).pack(side: :top, fill: :x)
    lsn_h[:ptn_rttn] = Tk::Tile::Notebook.new(lsn_h[:ptn_r02]) do
      place('x'=>10, 'y'=>10)
    end
    lsn_h[:ptn_rttn].pack(side: :top, anchor: :nw, fill: :both)
    lsn_h[:ptn_rttn_p1] = TkFrame.new(lsn_h[:ptn_rttn])
    lsn_h[:ptn_rttn_p2] = TkFrame.new(lsn_h[:ptn_rttn])
    lsn_h[:ptn_rttn].add(lsn_h[:ptn_rttn_p1], :text => '1')
    lsn_h[:ptn_rttn].add(lsn_h[:ptn_rttn_p2], :text => '2')
    lsn_h[:ptn_rttn].itemconfigure(lsn_h[:ptn_rttn_p2], :state => 'disabled')
    lsn_h[:ptn_l_sgl] = TkFrame.new(lsn_h[:ptn_rttn_p1]).pack(
      side: :top, fill: :x)
    lsn_h[:ptn_l_sgl_rb] = TkRadioButton.new(lsn_h[:ptn_l_sgl]) do
      text _("Single-track")
      value "single"
      variable lsn_h[:ptn_rv]
      command(proc{puts "lsn_h 4031: LineElem_Pattern: " + lsn_h[:ptn_rv]})
      pack(side: :left)
    end
    lsn_h[:ptn_l_dbl] = TkFrame.new(lsn_h[:ptn_rttn_p1]).pack(
      side: :top, fill: :x)
    lsn_h[:ptn_l_dbl_rb] = TkRadioButton.new(lsn_h[:ptn_l_dbl]) do
      text _("Double-track")
      value "double"
      variable lsn_h[:ptn_rv]
      command(proc{puts "lsn_h 4040: LineElem_Pattern: " + lsn_h[:ptn_rv]})
      pack(side: :left)
    end
    lsn_h[:ptn_l_qpu] = TkFrame.new(lsn_h[:ptn_rttn_p1]).pack(
      side: :top, fill: :x)
    lsn_h[:ptn_l_qpu_rb] = TkRadioButton.new(lsn_h[:ptn_l_qpu]) do
      text _("Quadruple-track paired by use")
      value "quad_pu"
      variable lsn_h[:ptn_rv]
      command(proc{puts "lsn_h 4049: LineElem_Pattern: " + lsn_h[:ptn_rv]})
      pack(side: :left)
    end
    lsn_h[:ptn_l_qpdos] = TkFrame.new(lsn_h[:ptn_rttn_p1]).pack(
      side: :top, fill: :x)
    lsn_h[:ptn_l_qpdos_rb] = TkRadioButton.new(lsn_h[:ptn_l_qpdos]) do
      text _("Quadruple-track paired by direction (outer slow lines)")
      value "quad_pdos"
      variable lsn_h[:ptn_rv]
      command(proc{puts "lsn_h 4058: LineElem_Pattern: " + lsn_h[:ptn_rv]})
      pack(side: :left)
    end
    lsn_h[:ptn_l_qpdof] = TkFrame.new(lsn_h[:ptn_rttn_p1]).pack(
      side: :top, fill: :x)
    lsn_h[:ptn_l_qpdof_rb] = TkRadioButton.new(lsn_h[:ptn_l_qpdof]) do
      text _("Quadruple-track paired by direction (outer fast lines)")
      value "quad_pdof"
      variable lsn_h[:ptn_rv]
      command(proc{puts "lsn_h 4067: LineElem_Pattern: " + lsn_h[:ptn_rv]})
      pack(side: :left)
    end
    lsn_h[:ptn_s01] = TkFrame.new(lsn_h[:ptn_t_s]).pack(side: :top, fill: :x)
    lsn_h[:ptn_s01_lbl] = TkLabel.new(lsn_h[:ptn_s01])
    lsn_h[:ptn_s01_lbl].text = _("Select one connections data set:")
    lsn_h[:ptn_s01_lbl].pack(side: :left)
    lsn_h[:ptn_s02] = TkFrame.new(lsn_h[:ptn_t_s]).pack(side: :top, fill: :x)
    lsn_h[:ptn_sttn] = Tk::Tile::Notebook.new(lsn_h[:ptn_s02]) do
      place('x'=>10, 'y'=>10)
    end
    lsn_h[:ptn_sttn].pack(side: :top, anchor: :nw, fill: :both)
    lsn_h[:ptn_sttn_p1] = TkFrame.new(lsn_h[:ptn_sttn])
    lsn_h[:ptn_sttn_p2] = TkFrame.new(lsn_h[:ptn_sttn])
    lsn_h[:ptn_sttn].add(lsn_h[:ptn_sttn_p1], :text => '1')
    lsn_h[:ptn_sttn].add(lsn_h[:ptn_sttn_p2], :text => '2')
    lsn_h[:ptn_sttn].itemconfigure(lsn_h[:ptn_sttn_p2], :state => 'disabled')
    lsn_h[:ptn_l_uni] = TkFrame.new(lsn_h[:ptn_sttn_p1]).pack(
      side: :top, fill: :x)
    lsn_h[:ptn_l_uni_rb] = TkRadioButton.new(lsn_h[:ptn_l_uni]) do
      text _("Unidirectional")
      value "uni"
      variable lsn_h[:ptn_sv]
      command(proc{puts "lsn_h 4090: LineElem_Pattern: " + lsn_h[:ptn_sv]})
      pack(side: :left)
    end
    lsn_h[:ptn_l_bi] = TkFrame.new(lsn_h[:ptn_sttn_p1]).pack(
      side: :top, fill: :x)
    lsn_h[:ptn_l_bi_rb] = TkRadioButton.new(lsn_h[:ptn_l_bi]) do
      text _("Bidirectional")
      value "bi"
      variable lsn_h[:ptn_sv]
      command(proc{puts "lsn_h 4099: LineElem_Pattern: " + lsn_h[:ptn_sv]})
      pack(side: :left)
    end
    # lsn_h[:ptn_c01] = TkFrame.new(lsn_h[:ptn_t_c]).pack(side: :top, fill: :x)
    # lsn_h[:ptn_c01_lbl] = TkLabel.new(lsn_h[:ptn_c01])
    # lsn_h[:ptn_c01_lbl].text = _("Select connections at both stations:")
    # lsn_h[:ptn_c01_lbl].pack(side: :left)
    # lsn_h[:ptn_c02x] = TkFrame.new(lsn_h[:ptn_t_c]).pack(side: :top, fill: :x)
    # lsn_h[:ptn_c02_v] = TkVariable.new('false')
    # lsn_h[:ptn_c02_lbl] = TkLabel.new(lsn_h[:ptn_c02x]).pack(side: :left)
    # lsn_h[:ptn_c02_lbl].text = _("Up end station connector:")
    # lsn_h[:ptn_c02] = TkFrame.new(lsn_h[:ptn_t_c]).pack(side: :top, fill: :x)
    # lsn_h[:ptn_c02_rb1] = TkRadioButton.new(lsn_h[:ptn_c02]) do
    #   text _("Up side of the station")
    #   value "true"
    #   variable lsn_h[:ptn_c02_v]
    #   pack(side: :left)
    # end
    # lsn_h[:ptn_c02_rb2] = TkRadioButton.new(lsn_h[:ptn_c02]) do
    #   text _("Down side of the station")
    #   value "false"
    #   variable lsn_h[:ptn_c02_v]
    #   pack(side: :left)
    # end
    # lsn_h[:ptn_c03x] = TkFrame.new(lsn_h[:ptn_t_c]).pack(side: :top, fill: :x)
    # lsn_h[:ptn_c03_v] = TkVariable.new('true')
    # lsn_h[:ptn_c03_lbl] = TkLabel.new(lsn_h[:ptn_c03x]).pack(side: :left)
    # lsn_h[:ptn_c03_lbl].text = _("Down end station connector:")
    # lsn_h[:ptn_c03] = TkFrame.new(lsn_h[:ptn_t_c]).pack(side: :top, fill: :x)
    # lsn_h[:ptn_c03_rb1] = TkRadioButton.new(lsn_h[:ptn_c03]) do
    #   text _("Up side of the station")
    #   value "true"
    #   variable lsn_h[:ptn_c03_v]
    #   pack(side: :left)
    # end
    # lsn_h[:ptn_c03_rb2] = TkRadioButton.new(lsn_h[:ptn_c03]) do
    #   text _("Down side of the station")
    #   value "false"
    #   variable lsn_h[:ptn_c03_v]
    #   pack(side: :left)
    # end

    #-----------------------#
    # Populating road frame #
    #-----------------------#
    lsn_h[:rdh] = Hash.new
    lsn_h[:rdh]['new_e'] = lsn_h[:e_nc].elements['../data/line_element/roads']
    lsn_h[:rdh]['new_a'] = lsn_h[:rdh]['new_b'] = nil
    lsn_h[:rdh]['xpath_lb'] =
      'rtss_input_data/create[@class=\'sujic_line\']/data[@name=\'' +
      lsn_h[:e_nc].elements['../data'].attributes['name'] +
      '\']/line_element[@from=\'' +
      (lsn_h[:e_nc].attributes['name'] ? lsn_h[:e_nc].attributes['name'] :
         "nil") + '\']/roads/road'
    lsn_h[:rdh]['listlambda'] = lambda{|e| e.attributes['name']}
    lsn_h[:rdh]['lb_otext'] = _("Road")
    lsn_h[:rdh]['xref_cd'] = Hash.new
    lsn_h[:rdh]['xref_cd'][nil] = _("Road")
    ## Cross reference check too complicated... Check within deletor
    lsn_h[:rdh]['editor'] = lambda {|a,b,c,d,e,f,g,h|
      road_editor(a,b,c,d,e,f,g,h) }
    lsn_h[:rdh]['deletor'] = lambda {|a,b,c,d,e,f,g,h|
      road_deletor(a,b,c,d,e,f,g,h) }
    lsn_h[:rdm] = TkFrame.new(lsn_h[:rds]).pack
    editor_select_frame(parent_win,lsn_ew,lsn_h[:rdm],lsn_h[:rdh])

    #--------------------------------------#
    # Populating run time constraint frame #
    #--------------------------------------#
    lsn_h[:rth] = Hash.new
    lsn_h[:rth]['new_e'] =
      lsn_h[:e_nc].elements['data/line_element/run_time_constraints']
    lsn_h[:rth]['new_a'] = lsn_h[:rth]['new_b'] = nil
    lsn_h[:rth]['xpath_lb'] =
      'rtss_input_data/create[@class=\'sujic_line\']/data[@name=\'' +
      lsn_h[:e_nc].elements['../data'].attributes['name'] +
      '\']/line_element[@from=\'' +
      (lsn_h[:e_nc].attributes['name'] ? lsn_h[:e_nc].attributes['name'] :
         "nil") + '\']/run_time_constraints/run_time_constraint'
    lsn_h[:rth]['listlambda'] = lambda{|e|
      case e.attributes['direction']
      when "down"
        dir_sep = "=>>"
      when "up"
        dir_sep = "<<="
      else
        e.add_attribute('direction','nil')
        dir_sep = "<=>"
      end
      e.attributes['road'] + "::" + e.attributes['from_platform'] + ":" +
        e.attributes['from_status'] + dir_sep +
        e.attributes['to_platform'] + ":" + e.attributes['to_status'] +
        "(" + e.attributes['time'] + ")"}
    lsn_h[:rth]['lb_otext'] = _("Run time constraint")
    lsn_h[:rth]['xref_cd'] = Hash.new
    lsn_h[:rth]['xref_cd'][nil] = _("Run time constraint")
    lsn_h[:rth]['editor'] = lambda {|a,b,c,d,e,f,g,h|
      rtc_editor(a,b,c,d,e,f,g,h) }
    lsn_h[:rtm] = TkFrame.new(lsn_h[:rtc]).pack
    editor_select_frame(parent_win,lsn_ew,lsn_h[:rtm],lsn_h[:rth])

    #----------------#
    # Check and exit #
    #----------------#
    l_sc = lambda {
      if lsn_h[:lef_nc].attributes['from'] == 'nil'
        Tk.messageBox(
          "parent" => lsn_ew,
          "title" => _("Station not specified"),
          "message" => _("You must pick one station to add to this line") )
        return false
      elsif lsn_h[:let_nc] && lsn_h[:let_nc].elements['diagram_info'] &&
            lsn_h[:let_nc].elements['diagram_info'].attributes[
              'inter_station_distance'] &&
            lsn_h[:let_nc].elements['diagram_info'].attributes[
              'inter_station_distance'] != lsn_h[:lsm03_ent].value
        Tk.messageBox(
          "parent" => lsn_ew,
          "title" => sprintf(
            _("Distance from Station %1$s not set"),
            lsn_h[:let_nc].attributes['from']),
          "message" => sprintf(
            _("You must press \"Renew\" button to set distance from " +
              "Station %1$s."), lsn_h[:let_nc].attributes['from']))
        return false
      elsif lsn_h[:lef_nc] && lsn_h[:lef_nc].elements['diagram_info'] &&
            lsn_h[:lef_nc].elements['diagram_info'].attributes[
              'from_position'] &&
            lsn_h[:lef_nc].elements['diagram_info'].attributes[
              'from_position'] != lsn_h[:lsm04_ent].value
        p "LSM04_ENT != FROM_POSITION"
        p lsn_h[:lef_nc].elements['diagram_info']
        p lsn_h[:lsm04_ent].value
        print lsn_h[:lef_nc]
        p "***********************"
        Tk.messageBox(
          "parent" => lsn_ew,
          "title" => sprintf(
            _("Position of Station %1$s not set"),
            lsn_h[:lef_nc].attributes['from']),
          "message" => sprintf(
            _("You must press \"Renew\" button to set position of " +
              "Station %1$s."), lsn_h[:lef_nc].attributes['from']))
        return false
      elsif lsn_h[:lef_nc] && lsn_h[:lef_nc].elements['diagram_info'] &&
            lsn_h[:lef_nc].elements['diagram_info'].attributes[
              'inter_station_distance'] &&
            lsn_h[:lef_nc].elements['diagram_info'].attributes[
              'inter_station_distance'] != lsn_h[:lsm05_ent].value
        Tk.messageBox(
          "parent" => lsn_ew,
          "title" => sprintf(
            _("Distance to Station %1$s not set"),
            lsn_h[:lef_nc].attributes['to']),
          "message" => sprintf(
            _("You must press \"Renew\" button to set distance to " +
              "Station %1$s."), lsn_h[:lef_nc].attributes['to']))
        return false
      elsif lsn_h[:lef_nc] && lsn_h[:lef_nc].elements['diagram_info'] &&
            lsn_h[:lef_nc].elements['diagram_info'].attributes[
              'to_position'] &&
            lsn_h[:lef_nc].elements['diagram_info'].attributes[
              'to_position'] != lsn_h[:lsm06_ent].value
        p "LSM06_ENT != TO_POSITION"
        p lsn_h[:lef_nc].elements['diagram_info']
        p lsn_h[:lsm06_ent].value
        print lsn_h[:lef_nc]
        p ""
        print lsn_h[:lef_nc].parent
        p ""
        p "***********************"
        Tk.messageBox(
          "parent" => lsn_ew,
          "title" => sprintf(
            _("Position of Station %1$s not set"),
            lsn_h[:lef_nc].attributes['to']),
          "message" => sprintf(
            _("You must press \"Renew\" button to set position of " +
              "Station %1$s."), lsn_h[:lef_nc].attributes['to']))
        return false
      end
      if lsn_h[:let_nc] && lsn_h[:lsm08v] == "yes"
        if lsn_h[:ptn_rv] == "double"
          lsn_h[:let_nc].elements['roads'].each do |x|
            x.parent.delete(x)
          end
          lsn_h[:let_nc].elements['roads'].add_element(
            'road',
            {'from_connector'=>'DD','name'=>'DM','to_connector'=>'UD'})
          lsn_h[:let_nc].elements['roads'].add_element(
            'road',
            {'from_connector'=>'DU','name'=>'UM','to_connector'=>'UU'})
        else
          Tk.messageBox(
            "parent" => lsn_ew,
            "title" => _("Interstation road pattern not ready"),
            "message" =>
            _("The interstation road pattern you have selected " +
              "has not been coded yet."))
          return false
        end
        ## Second station onwards. 
      end
      return true
    }
    
    # Close this window
    last_of_editor_window(
      parent_win,lsn_ew,_de['xpath_lb'],_de['xpath_root'],_de['listlambda'],
      l_btn,lbox,ar_pel,l_sc,v_x,l_discard_edit)
    p 'line_editor end'

  end

  #============================================#
  # Creating a window for line content editing #
  #============================================#
  def create_line_content_editor (parent_win, line_name, station_name)
    p 'create_line_content_editor start'

    # Create a child window
    ln_edt = open_editor_window(parent_win, '路線::区間データ編集')

    # Create a frame
    lsn_l01 = TkFrame.new(ln_edt).pack
    lsn_l01_lbl = TkLabel.new(lsn_l01)
    lsn_l01_lbl.text="区間距離[km]: "
    lsn_l01_lbl.pack(side: :left)
    lsn_l01_ent=TkEntry.new(lsn_l01)
    lsn_l01_ent.width(15)
    lsn_l01_ent.state="normal"
    lsn_l01_ent.pack(side: :left)
    edn_xp = 'rtss_input_data/create[@class=\'sujic_line\']/data[@name=\''
    edn_xp += line_name
    edn_xp += '\']/line_element[@from=\''
    edn_xp += station_name
    edn_xp += '\']/diagram_info'
    isd_str="0.0"
    @doc.elements.each(edn_xp) do |elem|
      isd_str = elem.attributes['inter_station_distance']
      p isd_str
    end
    lsn_l01_ent.value=isd_str
    lsn_l01_be=TkButton.new(lsn_l01).pack(side: :right)
    lsn_l01_be.text="<== この値により変更"
    edn_yp = 'rtss_input_data/create[@class=\'sujic_line\']/data[@name=\''
    edn_yp += line_name
    edn_yp += '\']/line_element'
    isd_from=0.0
    isd_xx=0.0
    to_stn_str=""
    @doc.elements.each(edn_yp) do |elem|
      if elem.attributes["from"]==station_name
        isd_xx = ((elem.elements['./diagram_info']
                    .attributes['inter_station_distance']).to_f)
        to_stn_str=elem.attributes["to"]
        break
      end
      isd_from += ((elem.elements['./diagram_info']
                     .attributes['inter_station_distance']).to_f)
    end
    lsn_l02 = TkFrame.new(ln_edt).pack
    lsn_l02_lbl = TkLabel.new(lsn_l02)
    lsn_l02_lbl.text="駅位置[km]: " + station_name + ": " + isd_from.to_s +
                     " ==> " + to_stn_str + ": "
    lsn_l02_lbl.pack(side: :left)
    lsn_l02_ent=TkEntry.new(lsn_l02)
    lsn_l02_ent.width(15)
    lsn_l02_ent.state="normal"
    lsn_l02_ent.pack(side: :left)
    lsn_l02_ent.value=(isd_from+isd_xx).to_s
    lsn_l02_be=TkButton.new(lsn_l02).pack(side: :right)
    lsn_l02_be.text="<== この値により変更"
    lsn_l01_be.command(
      proc{
        if lsn_l01_ent.value.is_float?
          @doc.elements.each(edn_xp) do |elem|
            elem.add_attribute("inter_station_distance",lsn_l01_ent.value)
          end
          isd_str=lsn_l01_ent.value
          isd_xx=isd_str.to_f
          lsn_l02_ent.value=(isd_from+isd_xx).to_s
        else
          Tk.messageBox(
            "title" => "数字ではないものが入力されていませんか?",
            "message" => "数字ではないものが入力されていませんか?\n" +
                         "リセットします。")
          lsn_l01_ent.value=isd_str
        end
      })
    lsn_l02_be.command(
      proc{
        if lsn_l02_ent.value.is_float?
          isd_xx=lsn_l02_ent.value.to_f - isd_from
          isd_str=isd_xx.to_s
          @doc.elements.each(edn_xp) do |elem|
            elem.add_attribute("inter_station_distance", isd_str)
          end
          lsn_l01_ent.value=isd_str
        else
          Tk.messageBox(
            "title" => "数字ではないものが入力されていませんか?",
            "message" => "数字ではないものが入力されていませんか?\n" +
                         "リセットします。")
          lsn_l02_ent.value=(isd_from+isd_xx).to_s
        end
      })
    lsn_l03_bs=TkButton.new(ln_edt).pack(fill: :x)
    lsn_l03_bs.text="ファイルに保存"
    lsn_l03_bs.command(proc{output_xml_data_file(@str_etr,@doc)})

    # Close this window with tabs
    last_of_editor_window(
      parent_win,ln_edt, nil, nil, nil, nil, nil, nil, nil, v_x)
    p 'create_line_content_editor end'
  end


  # Checking the number of `<create class="sujic_schedule_set">` tags.
  # @return [int] Number of detected tags.
  # @param p_win [TkToplevel] Parent window.
  # @param ss_xp [string] XPath for element containing element(s) for
  #   duplication check
  # @param elm_n [string] Name of element to check duplication. Check
  #   elements at:  
  #   `ss_tg = ss_xp + '/' + elm_n`
  # @param a_n [string] If non-nil, attribute to add. `ss_tg` changed to:  
  #   `ss_tg = ss_xp + '/' + elm_n + '[@' + a_n + '=\'' + a_v + '\']'`
  # @param a_v [string] If non-nil, attribute to add. See `a_n`.
  # @param npth [string] Where to find the name of the element.
  #   Nil: no name.  
  #   Name: `@doc.elements[ss_tg].elements[npth].attributes['name']`
  # @param d_ck [Hash] Hash for use in `cross_reference_checker`  
  #   (see comments to the method for more details)
  # @param str_m_01 [string] String expressing element pointed to by `ss_xp`
  def duplicate_tag_processor(p_win,ss_xp,elm_n,a_n,a_v,npth,d_ck,str_m_01)
    tgt_x = 0
    tgn_dx = elm_n
    tgn_pr = elm_n
    if a_n != nil
      tgn_dx += '[@' + a_n + '=\'' + a_v + '\']'
      tgn_pr += ' ' + a_n + '="' + a_v + '"'
    end
    tgn_xp = ss_xp + '/' + tgn_dx
    tgt_c = 0
    @doc.elements.each(tgn_xp) do |elem|
      tgt_c += 1
      tgt_x += 1
    end
    if tgt_c > 1
      @doc.elements.each(ss_xp) do |elep|
        not_d = -1
        tgt_c = -1
        loop do
          elep.elements.each(tgn_dx) do |elem|
            tgt_c += 1
            str_msg = str_m_01 + "<" + tgn_pr + "> タグが複数あるようです。\n"
            str_msg += "この <" + tgn_pr + "> タグ(" + tgt_c.to_s
            str_msg += ")を削除しますか？\n\n(この <" + tgn_pr
            if npth == nil
              str_msg += "> タグの属性一覧:)\n" + elem.attributes.inspect
            else
              str_msg += "> タグの名称: "
              str_msg += elem.elements[npth].attributes['name'] + ")"
            end
            answer = Tk.messageBox( "parent" => p_win, "icon" => "error",
                                    "type" => "yesnocancel",
                                    "title" => "複数の <" + elm_n + "> タグ",
                                    "message" => str_msg)
            if answer == "yes"
              Tk.messageBox( "parent" => p_win,
                             "message" => "このタグをのちほど削除します" )
            elsif ( answer == "no" ) && ( not_d >= 0 )
              str_msg2 = "すでに別なタグ(" + not_d.to_s + ")を保存に"
              str_msg2 += "マークしています。\nそのタグを破棄し，"
              str_msg2 += "このタグを保存するように変更しますか？"
              answer2 = Tk.messageBox(
                "parent" => p_win, "icon" => "error",
                "type" => "yesnocancel",
                "title" => "複数の <" + elm_n + "> タグ",
                "message" => str_msg2 )
              if answer2 == "yes"
                str_msg2 = "以前保存にマークしたタグを破棄し"
                str_msg2 += "このタグを保存します。"
                Tk.messageBox( "parent" => p_win, "message" => str_msg2 )
                not_d = tgt_c
              elsif answer2 == "no"
                str_msg2 = "このタグは破棄し，以前保存にマークした"
                str_msg2 += "タグをそのまま保存します。"
                Tk.messageBox( "parent" => p_win, "message" => str_msg2 )
              else
                str_msg2 = "削除タグの選択をはじめからやりなおします。"
                Tk.messageBox( "parent" => p_win, "message" => str_msg2 )
                not_d = -1
                break
              end
            elsif answer == "no"
              not_d = tgt_c
              Tk.messageBox( "parent" => p_win,
                             "message" => "このタグ(" + not_d.to_s +
                                          ")は保存します。" )
            else
              Tk.messageBox( "parent" => p_win,
                             "message" =>
                             "削除タグの選択をはじめからやりなおします。" )
              not_d = -1
              break
            end
          end
          if not_d < 0
            Tk.messageBox( "parent" => p_win,
                           "message" => "保存にマークされたタグがありません。" +
                                        "削除タグの選択をやりなおします。" )
            tgt_c = -1
            next
          else
            str_msg2 = "削除を実行します。よろしいですか？"
            answer2 = Tk.messageBox(
              "parent" => p_win, "icon" => "error",
              "type" => "yesno",
              "title" => "複数の <" + elm_n + "> タグ",
              "message" => str_msg2 )
            if answer2 == "no"
              Tk.messageBox( "parent" => p_win,
                             "message" =>
                             "削除タグの選択をはじめからやりなおします。" )
              not_d = -1
              tgt_c = -1
              next
            end
            tgt_c = -1
            elep.elements.each(tgn_dx) do |elem|
              tgt_c += 1
              p "*****DUPLICATE_TAG*****"
              p tgt_c
              p elem
              if tgt_c != not_d
                p " ##### TO DELETE #####"
                tt_n = nil
                if npth != nil
                  tt_n = elem.elements[npth].attributes['name']
                end
                _lx = lambda {|elm| elm.elements['data'].attributes['name']}
                if cross_reference_checker(p_win,d_ck,elem,_lx)
                  elep.delete_element(elem)
                  tgt_x -= 1
                end
              else
                p " ##### TO SAVE #####"
              end
            end
            break
          end
        end
      end
    end
    if tgt_x == 1
      @ss_name = @doc.elements[
        'rtss_input_data/create[@class=\'sujic_schedule_set\']/data'
      ].attributes['name']
    end
    return tgt_x
  end

  #=========================================================#
  # Creating schedule set editing tab in edit select window #
  #=========================================================#
  def create_schedule_set_tab (parent_win, ss_fx, ss_obj)
    p 'create_schedule_set_tab start'
    str_tg = ""

    # Read cycle time and set it to @cycle_time.
    read_cycle_time(ss_obj)

    sch_out_from = ss_obj.elements['data/train_graph'].attributes[
      'out_from_time'].to_i_as_time
    sch_out_to = ss_obj.elements['data/train_graph'].attributes[
      'out_to_time'].to_i_as_time

    ss_xp = 'rtss_input_data/create[@class=\'sujic_schedule_set\']/data['
    ss_xp += '@name=\'' + @ss_name + '\']'
    p '*****CREATE_SCHEDULE_SET_TAB*****'
    p ss_xp
    p '****/CREATE_SCHEDULE_SET_TAB/****'
    elm_n = 'train_graph'
    tgn_xp = ss_xp + '/' + elm_n
    str_m_01 = sprintf(
      _("In the configuration of schedule set \"%1$s\", "),
      @ss_name)
    tgt_c = duplicate_tag_processor( parent_win, ss_xp, elm_n, nil, nil,
                                     nil, nil, str_m_01 )
    if tgt_c == 0
      @doc.elements[ss_xp].add_element(
        elm_n,{'outfile'=>'traingraph.svg', 'out_from_time'=>'10:00:00',
               'out_to_time'=>'11:00:00'})
    end      
    @doc.elements.each(tgn_xp) do |elem|
      p "Finding train graph output file name: " + elem.attributes['outfile']
      str_tg = elem.attributes['outfile']
    end

    ss_f_ax = Tk::Tile::Notebook.new(ss_fx) do
      pack(fill: :x)
    end
    ss_f = TkFrame.new(ss_f_ax)
    ss_f_ax.add(ss_f, :text => _('Edit'))
    ss_f2 = TkFrame.new(ss_f_ax)
    ss_f_ax.add(ss_f2, :text => _('Settings'))
    ss_f_ax.itemconfigure(ss_f, :state => 'normal')
    ss_f_ax.itemconfigure(ss_f2, :state => 'disabled')
    sch_lx1 = TkFrame.new(ss_f).pack(fill: :both)
    sch_lx1_lbl = TkLabel.new(sch_lx1)
    sch_lx1_lbl.text=_("Schedule set name: ")
    sch_lx1_lbl.pack(side: :left)
    sch_lx1_ent=TkEntry.new(sch_lx1)
    sch_lx1_ent.width(25)
    sch_lx1_ent.value= @ss_name
    sch_lx1_ent_stt = "readonly"
    sch_lx1_ent.state(sch_lx1_ent_stt)
    sch_lx1_btn=TkButton.new(sch_lx1)
    sch_lx1_btn.text=_("Set name")
    sch_lx1_btn.pack(side: :right)
    sch_lx1_ent.pack(anchor: :c, fill: :both)
    sch_lx1_btn.command {
      if sch_lx1_ent_stt == "readonly"
        sch_lx1_ent_stt = "normal"
        sch_lx1_ent.state(sch_lx1_ent_stt)
        return
      elsif sch_lx1_ent.value == "nil" || sch_lx1_ent.value == "niL" ||
         sch_lx1_ent.value == "nIl" || sch_lx1_ent.value == "nIL" ||
         sch_lx1_ent.value == "Nil" || sch_lx1_ent.value == "NiL" ||
         sch_lx1_ent.value == "NIl" || sch_lx1_ent.value == "NIL"
        Tk.messageBox(
          "parent" => parent_win,
          "title" => _("\"Nil\" not allowed as schedule set name"),
          "message" =>
          _("You cannot use \"nil\" as a name of a schedule set.") )
        sch_lx1_ent.value= @ss_name
        sch_lx1_ent_stt = "readonly"
        sch_lx1_ent.state(sch_lx1_ent_stt)
        return
      else
        @doc.elements.each(
          'rtss_input_data/create[@class=\'sujic_train\']/under' +
          '[@name=\'' + @ss_name + '\']') do |e|
          e.add_attribute('name',sch_lx1_ent.value)
        end
        @doc.elements.each(
          'rtss_input_data/create[@class=\'sujic_trainset\']/under' +
          '[@name=\'' + @ss_name + '\']') do |e|
          e.add_attribute('name',sch_lx1_ent.value)
        end
        @ss_name = sch_lx1_ent.value
        ss_obj.elements['data'].add_attribute('name',@ss_name)
        sch_lx1_ent_stt = "readonly"
        sch_lx1_ent.state(sch_lx1_ent_stt)
        return
      end
    }
    sch_lx2 = TkFrame.new(ss_f).pack(fill: :both)
    sch_lx2_lbl = TkLabel.new(sch_lx2)
    sch_lx2_lbl.text=_("Cycle time of this schedule: ")
    sch_lx2_lbl.pack(side: :left)
    sch_lx2_ent=TkEntry.new(sch_lx2)
    sch_lx2_ent.width(25)
    sch_lx2_ent.value= @cycle_time.to_s_as_time
    sch_lx2_ent_stt = "readonly"
    sch_lx2_ent.state(sch_lx2_ent_stt)
    sch_lx2_btn=TkButton.new(sch_lx2)
    sch_lx2_btn.text=_("Change")
    sch_lx2_btn.pack(side: :right)
    sch_lx2_ent.pack(anchor: :c, fill: :both)
    sch_lx2_btn.command {
      if sch_lx2_ent_stt == "readonly"
        sch_lx2_ent_stt = "normal"
        sch_lx2_ent.state(sch_lx2_ent_stt)
        return
      elsif !(sch_lx2_ent.value.is_i_as_time?)
        Tk.messageBox(
          "parent" => parent_win,
          "title" => _("Time not in proper format"),
          "message" =>
          _("The input value is not in proper format of time " +
            "(e.g. \"hh:mm:ss\")") )
        sch_lx2_ent.value= @cycle_time.to_s_as_time
        sch_lx2_ent_stt = "readonly"
        sch_lx2_ent.state(sch_lx2_ent_stt)
        return
      else
        @cycle_time = sch_lx2_ent.value.to_i_as_time
        ss_obj.elements['data'].add_attribute('cycletime', sch_lx2_ent.value)
        sch_lx2_ent_stt = "readonly"
        sch_lx2_ent.state(sch_lx2_ent_stt)
        return
      end
    }
    sch_l01 = TkFrame.new(ss_f).pack
    sch_l01_lbl = TkLabel.new(sch_l01)
    sch_l01_lbl.text=_("File name: ")
    sch_l01_lbl.pack(side: :left)
    sch_l01_ent=TkEntry.new(sch_l01)
    sch_l01_ent.width(50)
    sch_l01_ent.value=str_tg
    sch_l01_ent.state="readonly"
    sch_l01_ent.pack(side: :left)
    sch_l01_btn=TkButton.new(ss_f)
    sch_l01_btn.text=_("Choose train graph file")
    sch_l01_btn.pack(fill: :x)
    sch_l01_btn.command {
      str_tg=Tk.getSaveFile(
        'title' => _('Input train graph file name'),
        'defaultextension' => 'svg',
        'filetypes' => "{SVG {.svg}}" ) ;
      @doc.elements.each(tgn_xp) do |elem|
        p "Finding train graph output file name change: " + str_tg
        elem.add_attribute("outfile",str_tg)
        sch_l01_ent.state="normal"
        sch_l01_ent.value=str_tg
        sch_l01_ent.state="readonly"
      end
    }
    sch_lx3 = TkFrame.new(ss_f).pack(fill: :both)
    sch_lx3_lbl = TkLabel.new(sch_lx3)
    sch_lx3_lbl.text=_("Train graph start time: ")
    sch_lx3_lbl.pack(side: :left)
    sch_lx3_ent=TkEntry.new(sch_lx3)
    sch_lx3_ent.width(25)
    sch_lx3_ent.value= sch_out_from.to_s_as_time
    sch_lx3_ent_stt = "readonly"
    sch_lx3_ent.state(sch_lx3_ent_stt)
    sch_lx3_btn=TkButton.new(sch_lx3)
    sch_lx3_btn.text=_("Change")
    sch_lx3_btn.pack(side: :right)
    sch_lx3_ent.pack(side: :right)
    sch_lx3_btn.command {
      if sch_lx3_ent_stt == "readonly"
        sch_lx3_ent_stt = "normal"
        sch_lx3_ent.state(sch_lx3_ent_stt)
        return
      elsif !(sch_lx3_ent.value.is_i_as_time?)
        Tk.messageBox(
          "parent" => parent_win,
          "title" => _("Time not in proper format"),
          "message" =>
          _("The input value is not in proper format of time " +
            "(e.g. \"hh:mm:ss\")") )
        sch_lx3_ent.value= sch_out_from.to_s_as_time
        sch_lx3_ent_stt = "readonly"
        sch_lx3_ent.state(sch_lx3_ent_stt)
        return
      else
        tmp_out_from = sch_lx3_ent.value.to_i_as_time
        if tmp_out_from < 0
          Tk.messageBox(
            "parent" => parent_win,
            "title" => _("Train graph start time not allowed"),
            "message" =>
            _("This value is not allowed as train graph start time.") )
          sch_lx3_ent.value= sch_out_from.to_s_as_time
        elsif tmp_out_from >= sch_out_to
          Tk.messageBox(
            "parent" => parent_win,
            "title" => _("Train graph start time not allowed"),
            "message" =>
            _("Train graph start time must be before train graph end time.") )
          sch_lx3_ent.value= sch_out_from.to_s_as_time
        elsif sch_out_to - tmp_out_from > 24 * 60 * 60
          Tk.messageBox(
            "parent" => parent_win,
            "title" => _("Train graph start time not allowed"),
            "message" =>
            _("Train graph duration should be less than 24 hours.") )
          sch_lx3_ent.value= sch_out_from.to_s_as_time
        else          
          sch_out_from = sch_lx3_ent.value.to_i_as_time
          ss_obj.elements['data/train_graph'].add_attribute(
            'out_from_time', sch_lx3_ent.value)
        end
        sch_lx3_ent_stt = "readonly"
        sch_lx3_ent.state(sch_lx3_ent_stt)
        return
      end
    }
    sch_lx4 = TkFrame.new(ss_f).pack(fill: :both)
    sch_lx4_lbl = TkLabel.new(sch_lx4)
    sch_lx4_lbl.text=_("Train graph end time: ")
    sch_lx4_lbl.pack(side: :left)
    sch_lx4_ent=TkEntry.new(sch_lx4)
    sch_lx4_ent.width(25)
    sch_lx4_ent.value= sch_out_to.to_s_as_time
    sch_lx4_ent_stt = "readonly"
    sch_lx4_ent.state(sch_lx4_ent_stt)
    sch_lx4_btn=TkButton.new(sch_lx4)
    sch_lx4_btn.text=_("Change")
    sch_lx4_btn.pack(side: :right)
    sch_lx4_ent.pack(side: :right)
    sch_lx4_btn.command {
      if sch_lx4_ent_stt == "readonly"
        sch_lx4_ent_stt = "normal"
        sch_lx4_ent.state(sch_lx4_ent_stt)
        return
      elsif !(sch_lx4_ent.value.is_i_as_time?)
        Tk.messageBox(
          "parent" => parent_win,
          "title" => _("Time not in proper format"),
          "message" =>
          _("The input value is not in proper format of time " +
            "(e.g. \"hh:mm:ss\")") )
        sch_lx4_ent.value= sch_out_to.to_s_as_time
        sch_lx4_ent_stt = "readonly"
        sch_lx4_ent.state(sch_lx4_ent_stt)
        return
      else
        tmp_out_to = sch_lx4_ent.value.to_i_as_time
        if tmp_out_to >= 48 * 60 * 60
          Tk.messageBox(
            "parent" => parent_win,
            "title" => _("Train graph end time not allowed"),
            "message" =>
            _("This value is not allowed as train graph start time.") )
          sch_lx4_ent.value= sch_out_to.to_s_as_time
        elsif tmp_out_to <= sch_out_from
          Tk.messageBox(
            "parent" => parent_win,
            "title" => _("Train graph end time not allowed"),
            "message" =>
            _("Train graph end time must be after train graph start time.") )
          sch_lx4_ent.value= sch_out_to.to_s_as_time
        elsif tmp_out_to - sch_out_from > 24 * 60 * 60
          Tk.messageBox(
            "parent" => parent_win,
            "title" => _("Train graph end time not allowed"),
            "message" =>
            _("Train graph duration should be less than 24 hours.") )
          sch_lx4_ent.value= sch_out_to.to_s_as_time
        else          
          sch_out_to = sch_lx4_ent.value.to_i_as_time
          ss_obj.elements['data/train_graph'].add_attribute(
            'out_to_time', sch_lx4_ent.value)
        end
        sch_lx4_ent_stt = "readonly"
        sch_lx4_ent.state(sch_lx4_ent_stt)
        return
      end
    }
    sch_l0203 = TkFrame.new(ss_f).pack(side: :top, fill: :x)
    sch_l02_btn=TkButton.new(sch_l0203)
    sch_l02_btn.text=_("Output train graph")
    sch_l02_btn.pack(side: :left, fill: :x, expand: :yes)
    sch_l02_btn.command {
      ##--------------------------##
      ## SVG document creating... ##
      ##--------------------------##
      if !(parse_line_for_svg)
        return false
      end
      p @line_data
      if !(check_train_time_list())
        return false
      end
      if !(check_trainset_time_list())
        return false
      end
      p "C_DELAY: " + @trainset_times[@ss_name][nil][
          'minimum_cycle_delay'].to_s +
        " -- " + @trainset_times[@ss_name][nil][
          'maximum_cycle_delay'].to_s
      cdelay_base =
        @trainset_times[@ss_name][nil]['maximum_cycle_delay'] -
        @trainset_times[@ss_name][nil]['minimum_cycle_delay'] + 1
      @width = @tg_params['margin_left'] +
               @tg_params['text_station_name_width'] +
               @tg_params['text_mileage_figure_width'] +
               (@tg_params['hspace_name_mileage_traingraph'] * 2) +
               (@cycle_time * @tg_params['pixels_per_minute'] *
                (cdelay_base + @tg_params['showing_additional_cycles']) /
                60.0 ) + @tg_params['margin_right']
      @next_vspace_to_add = 0.0
      tgout = REXML::Document.new()
      tgout << REXML::XMLDecl.new('1.0','UTF-8')
      tgo_root = REXML::Element.new('svg')
      { 'xmlns' => 'http://www.w3.org/2000/svg',
        'xmlns:xlink' => 'http://www.w3.org/1999/xlink',
        'xml:lang' => 'ja', 'width' => @width.to_s,
        'height' => '2000' }.each { |key, val|
        tgo_root.add_attribute(key, val) }
      tgout.add_element(tgo_root)
      @height = @tg_params['margin_top']
      @next_vspace_to_add = @tg_params['vspace_around_traingraph']
      @sw_line_height_to_set = true
      @line_data.each { |lname, data|
        if lname != nil
          ## Line name (title)
          @height += @tg_params['vspace_line_above']
          @next_vspace_to_add = @tg_params['vspace_line_below'] +
                                @tg_params['vspace_around_traingraph']
          c_elm = REXML::Element.new('text')
          { 'x' => @tg_params['margin_left'].to_s, 'y' => @height.to_s,
            'textLength' => @tg_params['text_line_name_width'].to_s,
            'lengthAdjust' => @tg_params['text_line_name_lengthadjust'],
            'font-family' => @tg_params['text_line_name_font_family'],
            'font-size' => @tg_params['text_line_name_font_size'].to_s,
            'font-weight' => @tg_params['text_line_name_font_weight'],
            'fill' => @tg_params['text_line_name_fill'] }.each { |key, val|
            c_elm.add_attribute(key, val) }
          tgo_root.add_element(c_elm)
          c_elm.add_text(lname)
          for ix in data do
            ## Platforms of the starting station
            svg_write_platform_names_and_lines(tgo_root, ix['start_plats'] )
            ## Main part
            @height += @next_vspace_to_add
            if @sw_line_height_to_set
              @line_data[nil]['height_top'] = @height
              @sw_line_height_to_set = false
            end
            @line_data[nil]['height_bottom'] = @height
            @next_vspace_to_add = @tg_params['vspace_station_platform']
            ix['stations'].each {|sname, pos|
              pos << @height + ((pos[0] - ix['startpos']) *
                                @tg_params['pixels_per_kilometre'])
              c_elm = REXML::Element.new('text')
              { 'x' => @tg_params['margin_left'].to_s, 'y' => pos[1].to_s,
                'textLength' => @tg_params['text_station_name_width'].to_s,
                'lengthAdjust' => @tg_params['text_station_name_lengthadjust'],
                'font-family' => @tg_params['text_station_name_font_family'],
                'font-size' => @tg_params['text_station_name_font_size'].to_s,
                'font-weight' => @tg_params['text_station_name_font_weight'],
                'fill' => @tg_params['text_station_name_fill']
              }.each { |key, val| c_elm.add_attribute(key, val) }
              tgo_root.add_element(c_elm)
              c_elm.add_text(sname)
              c_elm = REXML::Element.new('text')
              { 'x' => (@tg_params['margin_left'] +
                        @tg_params['text_station_name_width'] +
                        @tg_params['hspace_name_mileage_traingraph'] +
                        @tg_params['text_mileage_figure_width']).to_s,
                'y' => pos[1].to_s, 'text-anchor' => 'end',
                'lengthAdjust' =>
                @tg_params['text_mileage_figure_lengthadjust'],
                'font-family' =>
                @tg_params['text_mileage_figure_font_family'],
                'font-size' =>
                @tg_params['text_mileage_figure_font_size'].to_s,
                'font-weight' =>
                @tg_params['text_mileage_figure_font_weight'],
                'fill' => @tg_params['text_mileage_figure_fill']
              }.each { |key, val| c_elm.add_attribute(key, val) }
              tgo_root.add_element(c_elm)
              c_elm.add_text("%.2f" % pos[0])
              c_elm = REXML::Element.new('line')
              { 'x1' => (@tg_params['margin_left'] +
                         @tg_params['text_station_name_width'] +
                         @tg_params['text_mileage_figure_width'] +
                         (@tg_params['hspace_name_mileage_traingraph'] *
                          2)).to_s,
                'x2' => (@width - @tg_params['margin_right']).to_s,
                'y1' => pos[1].to_s, 'y2' => pos[1].to_s,
                'stroke' => @tg_params['line_platform_stroke'],
                'stroke-width' => @tg_params['line_platform_stroke_width'],
                'stroke-dasharray' =>
                @tg_params['line_platform_stroke_dasharray']
              }.each { |key, val| c_elm.add_attribute(key, val) }
              tgo_root.add_element(c_elm)
            }
            @height += ix['length'] * @tg_params['pixels_per_kilometre']
            if @sw_line_height_to_set
              @line_data[nil]['height_top'] = @height
              @sw_line_height_to_set = false
            end
            @line_data[nil]['height_bottom'] = @height
            ## Platforms of the ending station
            svg_write_platform_names_and_lines(tgo_root, ix['end_plats'])
          end
        end
      }
      if @sw_line_height_to_set
        @line_data[nil]['height_top'] = @height
        @sw_line_height_to_set = false
      end
      @line_data[nil]['height_bottom'] = @height
      @height += @next_vspace_to_add + @tg_params['vspace_around_traingraph'] +
                 @tg_params['margin_bottom']
      p "*************************************************"
      p "width = " + @width.to_s
      p "height = " + @height.to_s
      p "*************************************************"
      p @line_data
      p "*************************************************"
      p @train_times
      p "*************************************************"
      p @trainset_times
      p "*************************************************"
      tgo_root.add_attribute('height', @height.to_s)
      ## Vertical grid lines
      draw_grid_lines_for_svg(tgo_root)
      ## Draw trains
      draw_trains_for_svg(tgo_root)
      output_xml_data_file(str_tg,tgout)
    }

    sch_l03_btn=TkButton.new(sch_l0203)
    sch_l03_btn.text=_("View train graph")
    sch_l03_btn.pack(side: :right, fill: :x, expand: :yes)
    sch_l03_btn.command {
      Open3.capture3("start #{str_tg}")
    }

    sch_l04_btn=TkButton.new(ss_f)
    sch_l04_btn.text=_("Open schedule editor")
    sch_l04_btn.pack(fill: :x)
    sch_l04_btn.command {
      schedule_editor(parent_win,ss_obj)
    }

    sch_l05_btn=TkButton.new(ss_f)
    sch_l05_btn.text=_("Check the current schedule")
    sch_l05_btn.pack(fill: :x)
    sch_l05_btn.command {
      # Do nothing...
      return true
    }
  end


  #============================================#
  # Create new window for station name editing #
  #============================================#
  def schedule_editor (parent_win, ss_obj)

    #-----------------#
    # Initializing... #
    #-----------------#
    # Create a child window
    es_win = open_editor_window(
      parent_win, _("\"Sujic\"") + ":: " + _("Editing schedule"))

    # Tab representation (notebook widget)
    es_tl = Tk::Tile::Notebook.new(es_win) do
      place('x' => 10, 'y' => 10)
    end
    es_tl.pack

    # Create a frame for Station objects
    es_f_tr = TkFrame.new(es_tl)

    # Create a frame for Line objects
    es_f_ts = TkFrame.new(es_tl)

    # Create a frame for train graph parameters
    es_f_v1 = TkFrame.new(es_tl)

    # Adding frames to the notebook widget
    es_tl.add(es_f_tr, :text => _("Trains"))
    es_tl.add(es_f_ts, :text => _("Trainsets"))
    es_tl.add(es_f_v1, :text => _("Settings"))

    #------------------------#
    # Populating train frame #
    #------------------------#
    f_tr_d = Hash.new
    f_tr_d['new_b'] = nil
    f_tr_d['new_a'] = @doc.elements[
      'rtss_input_data/create[@class=\'sujic_schedule_set\'][last()]']
    f_tr_d['new_e'] = nil
    f_tr_d['xpath_lb'] =
      'rtss_input_data/create[@class=\'sujic_train\']/under[@class=' +
      '\'sujic_schedule_set\' and @name=\'' + @ss_name + '\']/..'
    f_tr_d['listlambda'] = lambda {|elm|
      elm.elements['data'].attributes['name']}
    f_tr_d['lb_otext'] = _("Train")
    f_tr_d['xref_cd'] = Hash.new
    f_tr_d['xref_cd'][nil] = _("Train")
    _key1 = _("Trainset")
    _val0 = 'rtss_input_data/create[@class=\'sujic_trainset\']/' +
            'under[@class=\'sujic_schedule_set\' and @name=\'' +
            @ss_name + '\']/../data/train_time'
    _hash1 = Hash.new
    _hash1['train'] = ['.', 'name']
    f_tr_d['xref_cd'][_key1] = [[_val0,_hash1,'..']]
    f_tr_d['editor'] = lambda { |a,b,c,d,e,f,g,h|
      train_editor(a,b,c,d,e,f,g,h) }
    editor_select_frame(parent_win,es_win,es_f_tr,f_tr_d,ss_obj)

    #---------------------------#
    # Populating trainset frame #
    #---------------------------#
    f_ts_d = Hash.new
    f_ts_d['new_b'] = nil
    f_ts_d['new_a'] = 'rtss_input_data/create[@class=\'sujic_train\' or ' +
                      '@class=\'sujic_schedule_set\'][last()]'
    # unless f_ts_d['new_a']
    #   f_ts_d['new_a'] = @doc.elements[
    #     'rtss_input_data/create[@class=\'sujic_schedule_set\'][last()]']
    # end
    f_ts_d['new_e'] = nil
    f_ts_d['xpath_lb'] =
      'rtss_input_data/create[@class=\'sujic_trainset\']/under[@class=' +
      '\'sujic_schedule_set\' and @name=\'' + @ss_name + '\']/..'
    f_ts_d['listlambda'] = lambda {|elm|
      elm.elements['data'].attributes['name']}
    f_ts_d['lb_otext'] = _("Train")
    f_ts_d['xref_cd'] = Hash.new
    f_ts_d['xref_cd'][nil] = _("Trainset")
    # _key1 = _("Train")
    # _val0 = 'rtss_input_data/create[@class=\'sujic_trainset\']/' +
    #         'under[@class=\'sujic_schedule_set\' and @name=\'' +
    #         @ss_name + '\']/../data/train_time'
    # _hash1 = Hash.new
    # _hash1['train'] = ['.', 'name']
    # f_ts_d['xref_cd'][_key1] = [[_val0,_hash1,'..']]
    f_ts_d['editor'] = lambda { |a,b,c,d,e,f,g,h|
      trainset_editor(a,b,c,d,e,f,g,h) }
    editor_select_frame(parent_win,es_win,es_f_ts,f_ts_d,ss_obj)

    # Close this window with tabs
    last_of_editor_window(
      parent_win,es_win, nil, nil, nil, nil, nil, nil, nil, nil)

  end

  #=================================================#
  # Create new window for selecting an edit process #
  #=================================================#
  def edit_select_window (parent_win)

    #-----------------#
    # Initializing... #
    #-----------------#
    # Create a child window
    es_win = open_editor_window(
      parent_win, _("\"Sujic\"") + ":: " + _("Edit window"))

    # Tab representation (notebook widget)
    es_tl = Tk::Tile::Notebook.new(es_win) do
      place('x' => 10, 'y' => 10)
    end
    es_tl.pack
    
    # Create a frame for rolling stock data input
    es_f_rs = TkFrame.new(es_tl)
    es_rtl = Tk::Tile::Notebook.new(es_f_rs) do
      place('x' => 10, 'y' => 10)
    end
    es_rtl.pack

    # Create a frame for TrainType objects
    es_f_tt = TkFrame.new(es_rtl)
    
    # Create a frame for SpeedClass objects
    es_f_sc = TkFrame.new(es_rtl)
    
    # Create a frame for infrastructure data input
    es_f_is = TkFrame.new(es_tl)
    es_itl = Tk::Tile::Notebook.new(es_f_is) do
      place('x' => 10, 'y' => 10)
    end
    es_itl.pack

    # Create a frame for Station objects
    es_f_st = TkFrame.new(es_itl)

    # Create a frame for Line objects
    es_f_ln = TkFrame.new(es_itl)
    
    # Create a frame for ScheduleSet object(s)
    es_f_ss = TkFrame.new(es_tl)
    
    # Create a frame for demand input
    es_f_dm = TkFrame.new(es_tl)
    es_dtl = Tk::Tile::Notebook.new(es_f_dm) do
      place('x' => 10, 'y' => 10)
    end
    es_dtl.pack
    
    # Create a frame for PassengerActivity objects
    es_f_pc = TkFrame.new(es_dtl)
    
    # Create a frame for PassengerAppearance objects
    es_f_pp = TkFrame.new(es_dtl)
    
    # Create a frame for the Evaluator object
    es_f_ev = TkFrame.new(es_tl)

    # Adding frames to the notebook widget
    es_tl.add(es_f_rs, :text => _('Rolling stock'))
    es_rtl.add(es_f_tt, :text => _('Train types'))
    es_rtl.add(es_f_sc, :text => _('Speed classes'))
    es_tl.add(es_f_is, :text => _('Infrastructure'))
    es_itl.add(es_f_st, :text => _('Stations'))
    es_itl.add(es_f_ln, :text => _('Lines'))
    es_tl.add(es_f_ss, :text => _('Timetable'))
    es_tl.add(es_f_dm, :text => _('Demands'))
    es_dtl.add(es_f_pc, :text => _('Passenger activity'))
    es_dtl.add(es_f_pp, :text => _('Passenger appearance'))
    es_tl.add(es_f_ev, :text => _('Evaluation'))

    #-----------------------------#
    # Populating train type frame #
    #-----------------------------#
    f_l_tt = Hash.new
    f_l_tt['new_b'] = nil
    @doc.root.comments.each do |cmt|
      if cmt.string == "##SUJIC_TRAINTYPE##"
        f_l_tt['new_b'] = cmt
        break
      end
    end
    f_l_tt['new_a'] = f_l_tt['new_e'] = nil
    f_l_tt['xpath_lb'] =
      'rtss_input_data/create[@class=\'sujic_traintype\']'
    f_l_tt['listlambda'] = lambda{|e| e.elements['data'].attributes['name']}
    f_l_tt['lb_otext'] = _("Train type")
    f_l_tt['xref_cd'] = Hash.new
    f_l_tt['xref_cd'][nil] = _("Train type")
    _tt_key1 = _("Trainset")
    _tt1_val0 = 'rtss_input_data/create[@class=\'sujic_trainset\']/'
    _tt1_val0 += 'using[@class=\'sujic_traintype\']'
    _tt1_h = {['data','name'] => ['.','name']}
    f_l_tt['xref_cd'][_tt_key1] = [[_tt1_val0,_tt1_h,'../data']]
    _tt_key2 = _("Speed class")
    _tt2_val0 = 'rtss_input_data/create[@class=\'sujic_speedclass\']' +
                '/data/coupled_trainset/traintype'
    _tt2_h = {['data','name'] => ['.','name']}
    f_l_tt['xref_cd'][_tt_key2] = [[_tt2_val0,_tt2_h,'../..']]
    f_l_tt['editor'] = lambda { |a,b,c,d,e,f,g,h|
      train_type_editor(a,b,c,d,e,f,g,h) }
    estt_f1m = TkFrame.new(es_f_tt).pack
    editor_select_frame(parent_win,es_win,estt_f1m,f_l_tt)

    #------------------------------#
    # Populating speed class frame #
    #------------------------------#
    f_l_sc = Hash.new
    f_l_sc['new_b'] = nil
    @doc.root.comments.each do |cmt|
      if cmt.string == "##SUJIC_SPEEDCLASS##"
        f_l_sc['new_b'] = cmt
        break
      end
    end
    f_l_sc['new_a'] = f_l_sc['new_e'] = nil
    f_l_sc['xpath_lb'] =
      'rtss_input_data/create[@class=\'sujic_speedclass\']'
    f_l_sc['listlambda'] = lambda{|e| e.elements['data'].attributes['name']}
    f_l_sc['lb_otext'] = _("Speed class")
    f_l_sc['xref_cd'] = Hash.new
    f_l_sc['xref_cd'][nil] = _("Speed class")
    _sc_key1 = _("Station")
    _sc1_val00 = 'rtss_input_data/create[@class=\'sujic_station\']/data/' +
                 'shunting_movements/shunt/run'
    _sc1_0h = {['data','name'] => ['.','speedclass']}
    _sc1_val10 = 'rtss_input_data/create[@class=\'sujic_station\']/data/' +
                 'event_headways/event_headway'
    _sc1_1h = {['data','name'] => ['.','speedclass']}
    f_l_sc['xref_cd'][_sc_key1] = [[_sc1_val00,_sc1_0h,'../../../'],
                                           [_sc1_val10,_sc1_1h,'../../']]
    _sc_key3 = _("Line")
    _sc3_val0 = 'rtss_input_data/create[@class=\'sujic_line\']/data/' +
                'line_element/run_time_constraints/run_time_constraint'
    _sc3_h = {['data','name'] => ['.','speedclass']}
    f_l_sc['xref_cd'][_sc_key3] = [[_sc3_val0,_sc3_h,'../../../']]
    f_l_sc['editor'] = lambda { |a,b,c,d,e,f,g,h|
      speed_class_editor(a,b,c,d,e,f,g,h) }
    essc_f1m = TkFrame.new(es_f_sc).pack
    editor_select_frame(parent_win,es_win,essc_f1m,f_l_sc)

    #--------------------------#
    # Populating station frame #
    #--------------------------#
    f_l_st = Hash.new
    f_l_st['new_b'] = nil
    @doc.root.comments.each do |cmt|
      if cmt.string == "##SUJIC_STATION##"
        f_l_st['new_b'] = cmt
        break
      end
    end
    f_l_st['new_a'] = f_l_st['new_e'] = nil
    f_l_st['xpath_lb'] =
      'rtss_input_data/create[@class=\'sujic_station\']'
    f_l_st['listlambda'] = lambda{|e| e.elements['data'].attributes['name']}
    f_l_st['lb_otext'] = _("Station")
    f_l_st['xref_cd'] = Hash.new
    f_l_st['xref_cd'][nil] = _("Station")
    _st_key1 = _("Line")
    _st1_val00 = 'rtss_input_data/create[@class=\'sujic_line\']' +
                 '/using[@class=\'sujic_station\']'
    _st1_0h = {['data','name'] => ['.','name']}
    _st1_val10 = 'rtss_input_data/create[@class=\'sujic_line\']/data/' +
                 'line_element'
    _st1_1h = {['data','name'] => ['.','from']}
    _st1_2h = {['data','name'] => ['.','to']}
    f_l_st['xref_cd'][_st_key1] = [[_st1_val00,_st1_0h,'../data'],
                                           [_st1_val10,_st1_1h,'../'],
                                           [_st1_val10,_st1_2h,'../']]
    _st_key2 = _("Train")
    _st2_val00 = 'rtss_input_data/create[@class=\'sujic_train\']/data/' +
                 'train_time'
    _st2_0h = {['data','name'] => ['.','station']}
    f_l_st['xref_cd'][_st_key2] = [[_st2_val00,_st2_0h,'..']]
    _st_key3 = _("Trainset")
    _st3_val00 = 'rtss_input_data/create[@class=\'sujic_trainset\']/' +
                 'data/train_time'
    _st3_0h = {['data','name'] => ['.','station']}
    f_l_st['xref_cd'][_st_key3] = [[_st3_val00,_st3_0h,'..']]
    _st_key4 = _("Passenger appearance model")
    _st4_v0 = 'rtss_input_data/create[@class=\'sujic_passenger_appearance_' +
              'model\']/data/od_data'
    _st4_1h = {['data','name'] => ['.','from']}
    _st4_2h = {['data','name'] => ['.','to']}
    l_pp = lambda{|e| _ed = e.elements['..']
      _ed.attributes['class'] + "::" + _ed.attributes['name']}
    f_l_st['xref_cd'][_st_key4] = [[_st4_v0,_st4_1h,l_pp],
                                           [_st4_v0,_st4_2h,l_pp]]
    f_l_st['editor'] = lambda { |a,b,c,d,e,f,g,h|
      station_editor(a,b,c,d,e,f,g,h) }
    esst_f1m = TkFrame.new(es_f_st).pack
    editor_select_frame(parent_win,es_win,esst_f1m,f_l_st)

    #-----------------------#
    # Populating line frame #
    #-----------------------#
    f_l_ln = Hash.new
    f_l_ln['new_b'] = nil
    @doc.root.comments.each do |cmt|
      if cmt.string == "##SUJIC_LINE##"
        f_l_ln['new_b'] = cmt
        break
      end
    end
    f_l_ln['new_a'] = f_l_ln['new_e'] = nil
    f_l_ln['xpath_lb'] = 'rtss_input_data/create[@class=\'sujic_line\']'
    f_l_ln['listlambda'] = lambda{|e| e.elements['data'].attributes['name']}
    f_l_ln['lb_otext'] = _("Line")
    f_l_ln['xref_cd'] = Hash.new
    f_l_ln['xref_cd'][nil] = _("Line")
    _l_key1 = _("Train")
    _l1_val0 = 'rtss_input_data/create[@class=\'sujic_train\']/' +
               'data/train_time'
    _l_h = {['data','name'] => ['.','line']}
    f_l_ln['xref_cd'][_l_key1] = [[_l1_val0,_l_h,'..']]
    f_l_ln['editor'] = lambda { |a,b,c,d,e,f,g,h|
      line_editor(a,b,c,d,e,f,g,h) }
    esln_f1m = TkFrame.new(es_f_ln).pack
    editor_select_frame(parent_win,es_win,esln_f1m,f_l_ln,nil)

    #------------------------------------#
    # Schedule set... Draw diagram, etc. #
    #------------------------------------#
    esss_f1m = TkFrame.new(es_f_ss).pack

    #....................................
    # Find a schedule set.
    # (1) Duplicate <create class='sujic_schedule_set'> element.
    #....................................
    ss_xp = 'rtss_input_data'
    elm_n = 'create'
    a_n = 'class'
    a_v = 'sujic_schedule_set'
    npth = 'data'
    d_ck = Hash.new
    d_ck[nil] = _("Schedule set")
    dck_2 = {['./data','name'] => ['.','name']}
    d_ck[_("Train")] =
      [['rtss_input_data/create[@class=\'sujic_train\']/' +
        'under[@class=\'sujic_schedule_set\']',dck_2,'../data']]
    d_ck[_("Trainset")] =
      [['rtss_input_data/create[@class=\'sujic_trainset\']/' +
        'under[@class=\'sujic_schedule_set\']',dck_2,'../data']]
    msg_str = _("In the data file,")
    tgt_c = duplicate_tag_processor(es_win, ss_xp, elm_n, a_n, a_v, npth,
                                    d_ck, msg_str)
    if tgt_c > 1
      Tk.messageBox(
        "parent" => es_win,
        "title" => "削除失敗",
        "message" =>
        "<create class=\"sujic_schedule_set\"> タグが複数あり，" +
        "削除に失敗しました。最初のものを使います。")
    elsif tgt_c < 1
      c_s_s = REXML::Element.new('create')
      c_s_s.add_attribute('class', 'sujic_schedule_set')
      c_s_s.add_element('data', {'name' => 'RG', 'cycletime' => '00:10:00'})
      sw_not_inserted = true
      @doc.root.comments.each do |cmt|
        if cmt.string == "##SUJIC_SCHEDULE_SET##"
          @doc.root.insert_before(cmt,c_s_s)
          sw_not_inserted = false
          break
        end
      end
    end
    ss_xp += '/' + elm_n + '[@' + a_n + '=\'' + a_v + '\']'
    ss_obj = @doc.elements[ss_xp]

    p "*****CREATE_SCHEDULE_SET_TAB*****"
    p ss_xp
    p ss_obj
    p ss_obj.elements['data']
    p "****/CREATE_SCHEDULE_SET_TAB/****"

    create_schedule_set_tab(es_win, esss_f1m, ss_obj)

    #-------------------------------------#
    # Populating passenger activity frame #
    #-------------------------------------#
    f_l_pc = Hash.new
    f_l_pc['new_b'] = nil
    @doc.root.comments.each do |cmt|
      if cmt.string == "##SUJIC_PASSENGER_ACTIVITY##"
        f_l_pc['new_b'] = cmt
        break
      end
    end
    f_l_pc['new_a'] = f_l_pc['new_e'] = nil
    f_l_pc['xpath_lb'] =
      'rtss_input_data/create[@class=\'sujic_passenger_activity\']'
    f_l_pc['listlambda'] = lambda{|e| _ed = e.elements['data']
      _ed.attributes['class'] + "::" + _ed.attributes['name']}
    f_l_pc['lb_otext'] = _("Passenger activity model")
    f_l_pc['xref_cd'] = Hash.new
    f_l_pc['xref_cd'][nil] = _("Passenger activity model")
    _pc_key1 = _("Passenger appearance model")
    _pc1_v00 = 'rtss_input_data/create[@class=\'sujic_passenger_appearance' +
               '_model\']/under[@class=\'sujic_passenger_activity\']'
    _pc1_0h = {['data','name'] => ['.','name']}
    l_pc = lambda{|e| _ed = e.elements['../data']
      _ed.attributes['class'] + "::" + _ed.attributes['name']}
    f_l_pc['xref_cd'][_pc_key1] = [[_pc1_v00,_pc1_0h,l_pc]]
    f_l_pc['editor'] = lambda { |a,b,c,d,e,f,g,h|
      passenger_activity_editor(a,b,c,d,e,f,g,h) }
    espc_f1m = TkFrame.new(es_f_pc).pack
    editor_select_frame(parent_win,es_win,espc_f1m,f_l_pc)

    #--------------------------------------#
    # Populating passenger apparance frame #
    #--------------------------------------#
    f_l_pp = Hash.new
    f_l_pp['new_b'] = nil
    @doc.root.comments.each do |cmt|
      if cmt.string == "##SUJIC_PASSENGER_APPEARANCE##"
        f_l_pp['new_b'] = cmt
        break
      end
    end
    f_l_pp['new_a'] = f_l_pp['new_e'] = nil
    f_l_pp['xpath_lb'] = 'rtss_input_data/create[@class=\'' +
                              'sujic_passenger_appearance_model\']'
    f_l_pp['listlambda'] = lambda{|e| _ed = e.elements['data']
      _ed.attributes['class'] + "::" + _ed.attributes['name']}
    f_l_pp['lb_otext'] = _("Passenger appearance")
    f_l_pp['xref_cd'] = Hash.new
    f_l_pp['xref_cd'][nil] = _("Passenger appearance")
    f_l_pp['editor'] = lambda { |a,b,c,d,e,f,g,h|
      passenger_appearance_model_editor(a,b,c,d,e,f,g,h) }
    espp_f1m = TkFrame.new(es_f_pp).pack
    editor_select_frame(parent_win,es_win,espp_f1m,f_l_pp)

    #----------------------------#
    # Populating evaluator frame #
    #----------------------------#
    f_l_ev = Hash.new
    f_l_ev['new_e'] = @doc.root
    f_l_ev['new_a'] = f_l_ev['new_b'] = nil
    f_l_ev['xpath_lb'] = 'rtss_input_data/create[@class=\'sujic_evaluator\']'
    f_l_ev['listlambda'] = lambda{|e| e.elements['data'].attributes['name']}
    f_l_ev['lb_otext'] = _("Evaluator")
    f_l_ev['xref_cd'] = Hash.new
    f_l_ev['xref_cd'][nil] = _("Evaluator")
    f_l_ev['editor'] = lambda { |a,b,c,d,e,f,g,h|
      evaluator_editor(a,b,c,d,e,f,g,h) }
    esev_f1m = TkFrame.new(es_f_ev).pack
    editor_select_frame(parent_win,es_win,esev_f1m,f_l_ev)

    # Close this window with tabs
    last_of_editor_window(
      parent_win,es_win, nil, nil, nil, nil, nil, nil, nil, nil)

  end

  #=============#
  # Constructor #
  #=============#
  def initialize

    #--------------------#
    # train time parsing #
    #--------------------#
    ## @line_data: Hash.
    ## line_name => x: Array.
    ##  x[0], x[1], ..., x[i], ...: Hash
    ##   x[i]['start']: starting station (either the first station of the line,
    ##    or the station where platforms were last drawn)
    ##   x[i]['end']: ending station (either the last station of the line, or
    ##    the next station where platforms are to be drawn)
    ##   x[i]['startpos']: starting station position [km]
    ##   x[i]['endpos']: ending station position [km]
    ##   x[i]['start_plats']: Array. Platforms at start station.
    ##   x[i]['end_plats']: Array. Platforms at end station (for x[-1] only).
    ##   x[i]['length']: section length [km]
    ##   x[i]['stations']: Hash. Station name => position [km].
    ## nil => Hash
    ##  'total_length' => sum of lengths of all sections in all lines.
    ##  'line_separations' => number of line separations
    ##   (= number of lines - 1)
    ##  'section_separations' => number of section separations
    ##   (= number of sectoins - 1 - line_separations)
    @line_data = Hash.new

    ## Hash ... 'sch_name'
    ## @train_times[sch_name]: hash
    ##   train name => Hash
    ##    'data' => いろいろ
    ##    'times' => Hash
    ##      [CD, Time] => Array of hash
    ##      (line, station, plat, stat, in_service, road, trainset_set)]
    @train_times = Hash.new

    ## Hash ... 'sch_name'
    ## @trainset_times[sch_name]: hash
    ##   trainset name => Hash
    ##    'data' => いろいろ
    ##    'times' => Hash
    ##      [CD, Time] => Array of hash
    ##      (line, station, plat, stat, in_service, road, trainset_set)]
    @trainset_times = Hash.new

    ## Hash ... 'sch_name'
    ## @station_times[sch_name]: hash
    ##   [Time] => Array of Hash
    ##     (train, [trainsets], line, plat, stat, in_service, road)
    @station_times = Hash.new

    ## Hash ... 'sch_name'
    ## @events[sch_name]: hash
    ##   [Time] => Array of Hash
    ##    (train, ....)

    @events = Hash.new

    #------------------------#
    # SVG related parameters #
    #------------------------#
    @tg_params = Hash.new
    train_graph_defaults(@tg_params)

    # SVG temporary parameter
    @next_vspace_to_add = 0.0
    @height = 0.0
    @width = 0.0
    @cycle_time = 0
    @sw_line_height_to_set = true

    # Root window
    @root_win = TkRoot.new(title: _("\"Sujic\""))

    ## i_save: 1 if "Save file name" button has not been pressed.
    ##         0 otherwise.
    @i_save = 1

    ## i_xml: 1 if "Read XML file" button or "Create new XML file"
    ##          has not been pressed.
    ##        0 otherwise.
    @i_xml = 1

    @str_etr = TkVariable.new( "" )
    @str_xml = TkVariable.new( "" )

    @f_sfent = TkFrame.new(@root_win).grid(
      row: 0, column: 0, columnspan: 4, sticky: :ew)

    @lbl_save=TkLabel.new(@f_sfent)
    @lbl_save.text=_("File name: ")
    @lbl_save.pack(side: :left)

    @etr_save=TkEntry.new(@f_sfent)
    @etr_save.width(50)
    @etr_save.state("readonly")
    @etr_save.pack(side: :left)

    @btn_save_set=TkButton.new(@root_win)
    @btn_save_set.text = _("Save file Select")
    @btn_save_set.command {
      @str_etr =
        Tk.getSaveFile( 'title' => _('Enter file name for saving'),
                        'defaultextension' => 'xml',
                        'initialfile' => @str_etr ,
                        'filetypes' => "{XML {.xml}}" ) ;
      @etr_save.state = "normal" ;
      @etr_save.value = @str_etr ;
      @etr_save.state = "readonly" ;
      @i_save = 0 ;
    }
    @btn_save_set.grid(row: 1, column: 0, sticky: :ew)

    @btn_xml=TkButton.new(@root_win)
    @btn_xml.text = _("Open XML file")
    @btn_xml.command {
      if @i_xml == 1
        if @i_save == 1
          @str_xml = Tk.getOpenFile( 'title' => _('Open file'),
                                     'defaultextension' => 'xml',
                                     'filetypes' => "{XML {.xml}}" ) ;
        else
          @str_xml = Tk.getOpenFile( 'title' => _('Open file'),
                                     'defaultextension' => 'xml',
                                     'initialfile' => @etr_save.value ,
                                     'filetypes' => "{XML {.xml}}" ) ;
        end
      else
        @str_xml = Tk.getOpenFile( 'title' => _('Open file'),
                                   'defaultextension' => 'xml',
                                   'initialfile' => @str_xml ,
                                   'filetypes' => "{XML {.xml}}" ) ;
      end
      @f = File.open_for_read ( @str_xml ) ;
      if @f == 0
        Tk.messageBox( "title" => "File read error",
                       "message" => "ファイル読み込みエラー:\n" +
                                    "ファイルがないか，読み込み権限がありません。")
      else
        if @i_save == 1
          Tk.messageBox(
            "title" => _("File read success"),
            "message" => _("We recommend that you change the file name\n" +
                         "so the original file is preserved."))
          @etr_save.state = "normal" ;
          @etr_save.value = @str_xml ;
          @str_etr = @str_xml ;
          @etr_save.state = "readonly" ;
        end
        @doc = REXML::Document.new(@f)
        sw_err = false
        if @doc.elements.first.name != "rtss_input_data"
          emsg_root_incor = "Error: root element \"" +
                            @doc.elements.first.name +
                            "\" not correct"
          Tk.messageBox( "title" => "XML error",
                         "message" => emsg_root_incor )
          sw_err = true
        end
        if sw_err == false
          a_cs = ["##SUJIC_TRAINTYPE##", "##SUJIC_SPEEDCLASS##",
                  "##SUJIC_STATION##", "##SUJIC_LINE##",
                  "##SUJIC_SCHEDULE_SET##", "##SUJIC_PASSENGER_ACTIVITY##",
                  "##SUJIC_PASSENGER_APPEARANCE##"]
          ocmt_found = 0
          @doc.root.comments.each do |cmt|
            ocmt_now = -1
            for i in 0...a_cs.size do
              if cmt.string == a_cs[i]
                ocmt_now = i
                break
              end
            end
            next if ocmt_now < 0
            if ocmt_now != ocmt_found
              Tk.messageBox(
                "title" => "Data file error",
                "message" => "Data file not in proper format:\n Comment " +
                             a_cs[ocmt_found] + " not found before " +
                             a_cs[ocmt_now] )
              ocmt_found = a_cs.size
              break
            end
            ocmt_found += 1
          end
          if ocmt_found != a_cs.size
            Tk.messageBox(
              "title" => "Data file error",
              "message" => "Data file not in proper format:\n Comment " +
                           a_cs[ocmt_found] + " not found" )
          end
          @ss_name = @doc.elements[
            'rtss_input_data/create[@class=\'sujic_schedule_set\']/data'
          ].attributes['name']
        end
        @i_xml = 0
      end
    }
    @btn_xml.grid(row: 1, column: 1, columnspan: 2, sticky: :ew, ipadx: 30)

    @btn_shinki=TkButton.new(@root_win)
    @btn_shinki.text = _("Create new XML file")
    @btn_shinki.command {
      @shinki_ok = false
      if @i_save == 1
        Tk.messageBox(
          "icon" => "error",
          "type" => "ok",
          "title" => _("Save file not specified"),
          "message" => _("Please specify save file first!") )
      elsif File.exist?(@str_etr)
        Tk.messageBox(
          "icon" => "error",
          "type" => "ok",
          "title" => _("File already exists"),
          "message" => _("A file with the specified name already exists.") )
      elsif @i_xml == 0
        _mbspfm = _("Discard edits?")
        _str_m = _("Really discard all edits to the current file?")
        ans_ixml = Tk.messageBox(
          "icon" => "warning",
          "type" => "okcancel",
          "default" => "cancel",
          "title" => sprintf(_mbspfm),
          "message" => _str_m )
        @shinki_ok = true if ans_ixml == "ok"
      else
        @shinki_ok = true
      end
      if @shinki_ok == true
        @doc = REXML::Document.new(BLANKINPUT)
        @ss_name = @doc.elements[
          'rtss_input_data/create[@class=\'sujic_schedule_set\']/data'
        ].attributes['name']
        @i_xml = 0
      end
    }
    @btn_shinki.grid(row: 1, column: 3, sticky: :ew)

    # @f_ew = TkFrame.new(@root_win).pack(fill: :x)

    @btn_subw=TkButton.new(@root_win)
    @btn_subw.text = _("Start editing")
    @btn_subw.command { edit_select_window(@root_win) }
    @btn_subw.grid(row: 2, column: 0, columnspan: 2, sticky: :ew)

    @btn_outw=TkButton.new(@root_win)
    @btn_outw.text = _("Write XML file")
    @btn_outw.command { p @str_etr
      output_xml_data_file(@str_etr,@doc)
    }
    @btn_outw.grid(row: 2, column: 2, columnspan: 2, sticky: :ew)

    # @f_exec = TkFrame.new(@root_win).pack(fill: :x)

    @btn_pfsm=TkButton.new(@f_exec)
    @btn_pfsm.text = _("Invoke simulator")
    @btn_pfsm.command {
      sujic_path = File.expand_path("rtss-sujic.exe", __dir__)
      Open3.capture3("#{sujic_path} #{@str_etr}")
    }
    @btn_pfsm.grid(row: 3, column: 0, columnspan: 4, sticky: :ew)

    # @f_quit = TkFrame.new(@root_win).pack(fill: :x)

    @btn_about=TkButton.new(@root_win)
    @btn_about.text = _("About this program")
    @btn_about.command { p "Show \"About\""
      Tk.messageBox(
        "icon"=>"info","type"=>"ok","default"=>"ok",
        "title"=>sprintf(_("About \"Sujic\" FE %1$s"),"6.0.0b1"),
        "message"=>_("\"Sujic\"") + " FE 6.0.0b1\n" + _("\"Sujic\"") +
                   " 6.0.0b1\nby Ryo Takagi, (C)2009-2019")}
    @btn_about.grid(row: 4, column: 0, columnspan: 4, sticky: :ew)

    @btn_quit=TkButton.new(@root_win)
    @btn_quit.text = _("End the program (unsaved changes will be discarded)")
    @btn_quit.command { exit }
    @btn_quit.grid(row: 5, column: 0, columnspan: 4, sticky: :ew)

  end

end

x = Processor.new

Tk.mainloop
